@@ -43,6 +43,8 @@
 #include <kurllabel.h>
 #include <kgenericfactory.h>
 
+#include <kscreensaver_interface.h>
+
 #include <X11/X.h>
 #include <X11/Xlib.h>
 #include <X11/Xatom.h>
@@ -101,7 +103,7 @@ extern "C" {
 
     KDE_EXPORT void kcminit_energy() {
 #ifdef HAVE_DPMS
-        KConfig *_cfg = new KConfig( "kcmdisplayrc", KConfig::CascadeConfig );
+        KConfig *_cfg = new KConfig( "kcmdisplayrc", KConfig::NoGlobals );
         KConfigGroup cfg(_cfg, "DisplayEnergy");
 
 	Display *dpy = QX11Info::display();
@@ -235,7 +237,7 @@ KEnergy::KEnergy(QWidget *parent, const QVariantList &args)
     else
        setButtons( KCModule::Help );
 
-    m_pConfig = new KConfig("kcmdisplayrc", KConfig::CascadeConfig);
+    m_pConfig = new KConfig("kcmdisplayrc", KConfig::NoGlobals);
 
     load();
 }
@@ -364,6 +366,10 @@ void KEnergy::applySettings(bool enable, int standby, int suspend, int off)
 
     XFlush(dpy);
     XSetErrorHandler(defaultHandler);
+
+    // The screen saver depends on the DPMS settings
+    org::kde::screensaver kscreensaver("org.kde.screensaver", "/ScreenSaver", QDBusConnection::sessionBus());
+    kscreensaver.configure();
 #else
     /* keep gcc silent */
     if (enable | standby | suspend | off)
