@@ -12,7 +12,6 @@
 #include <QComboBox>
 #include <QDir>
 #include <QLabel>
-#include <QLayout>
 #include <QPushButton>
 #include <QtCore/QSettings>
 
@@ -328,7 +327,7 @@ bool FontAASettings::load()
 
   if(!xft.getHintStyle(hStyle) || KXftConfig::Hint::NotSet==hStyle)
   {
-    KConfig kglobals("kdeglobals", KConfig::CascadeConfig);
+    KConfig kglobals("kdeglobals", KConfig::NoGlobals);
 
     hStyle=KXftConfig::Hint::Medium;
     xft.setHintStyle(hStyle);
@@ -348,7 +347,7 @@ bool FontAASettings::load()
 bool FontAASettings::save( bool useAA )
 {
   KXftConfig   xft(KXftConfig::constStyleSettings);
-  KConfig      kglobals("kdeglobals", KConfig::CascadeConfig);
+  KConfig      kglobals("kdeglobals", KConfig::NoGlobals);
   KConfigGroup grp(&kglobals, "General");
 
   xft.setAntiAliasing( useAA );
@@ -495,20 +494,6 @@ int FontAASettings::exec()
 
 /**** KFonts ****/
 
-static QString desktopConfigName()
-{
-  int desktop=0;
-  if (QX11Info::display())
-    desktop = DefaultScreen(QX11Info::display());
-  QString name;
-  if (desktop == 0)
-    name = "kdesktoprc";
-  else
-    name.sprintf("kdesktop-screen-%drc", desktop);
-
-  return name;
-}
-
 KFonts::KFonts(QWidget *parent, const QVariantList &args)
     :   KCModule(FontFactory::componentData(), parent, args)
 {
@@ -517,29 +502,32 @@ KFonts::KFonts(QWidget *parent, const QVariantList &args)
   nameGroupKeyRc
     << i18n("General")        << "General"    << "font"         << ""
     << i18n("Fixed width")    << "General"    << "fixed"        << ""
+    << i18n("Small")          << "General"    << "smallestReadableFont" << ""
     << i18n("Toolbar")        << "General"    << "toolBarFont"  << ""
     << i18n("Menu")           << "General"    << "menuFont"     << ""
     << i18n("Window title")   << "WM"         << "activeFont"   << ""
     << i18n("Taskbar")        << "General"    << "taskbarFont"  << ""
-    << i18n("Desktop")        << "FMSettings" << "StandardFont" << desktopConfigName();
+    << i18n("Desktop")        << "General"    << "desktopFont" << "";
 
   QList<QFont> defaultFontList;
 
-  // Keep in sync with kdelibs/kdecore/kglobalsettings.cpp
+  // Keep in sync with kdelibs/kdeui/kernel/kglobalsettings.cpp
 
   QFont f0("Sans Serif", 10);
   QFont f1("Monospace", 10);
-  QFont f2("Sans Serif", 10);
+  QFont f2("Sans Serif", 8);
   QFont f3("Sans Serif", 9, QFont::Bold);
   QFont f4("Sans Serif", 10);
+  QFont f5("Sans Serif", 8); // smallestReadableFont
 
   f0.setPointSize(10);
   f1.setPointSize(10);
-  f2.setPointSize(10);
+  f2.setPointSize(8);
   f3.setPointSize(9);
   f4.setPointSize(10);
+  f5.setPointSize(8);
 
-  defaultFontList << f0 << f1 << f2 << f0 << f3 << f4 << f0;
+  defaultFontList << f0 << f1 << f5 << f2 << f0 << f3 << f4 << f0;
 
   QList<bool> fixedList;
 
@@ -550,6 +538,7 @@ KFonts::KFonts(QWidget *parent, const QVariantList &args)
     <<  false
     <<  false
     <<  false
+    <<  false
     <<  false;
 
   QStringList quickHelpList;
@@ -557,6 +546,7 @@ KFonts::KFonts(QWidget *parent, const QVariantList &args)
   quickHelpList
     << i18n("Used for normal text (e.g. button labels, list items).")
     << i18n("A non-proportional font (i.e. typewriter font).")
+    << i18n("Smallest font that is still readable well.")
     << i18n("Used to display text beside toolbar icons.")
     << i18n("Used by menu bars and popup menus.")
     << i18n("Used by the window titlebar.")
@@ -565,6 +555,7 @@ KFonts::KFonts(QWidget *parent, const QVariantList &args)
 
   QVBoxLayout * layout = new QVBoxLayout(this );
   layout->setSpacing( KDialog::spacingHint() );
+  layout->setMargin(0);
 
   QGridLayout * fontUseLayout = new QGridLayout( );
   layout->addItem( fontUseLayout );
@@ -817,7 +808,7 @@ void KFonts::slotApplyFontDiff()
 {
   QFont font = QFont(fontUseList.first()->font());
 	KFontChooser::FontDiffFlags fontDiffFlags = 0;
-  int ret = KFontDialog::getFontDiff(font,fontDiffFlags);
+  int ret = KFontDialog::getFontDiff(font,fontDiffFlags,KFontChooser::NoDisplayFlags,this);
 
   if (ret == KDialog::Accepted && fontDiffFlags)
   {
