@@ -62,6 +62,7 @@ KDMGeneralWidget::KDMGeneralWidget( QWidget *parent )
 	guicombo = new KBackedComboBox( box );
 	guicombo->insertItem( "", i18n("<placeholder>default</placeholder>") );
 	loadGuiStyles( guicombo );
+	guicombo->model()->sort( 0 );
 	QLabel *label = new QLabel( i18n("GUI s&tyle:"), box );
 	label->setBuddy( guicombo );
 	connect( guicombo, SIGNAL(activated( int )), SIGNAL(changed()) );
@@ -75,6 +76,7 @@ KDMGeneralWidget::KDMGeneralWidget( QWidget *parent )
 	colcombo = new KBackedComboBox( box );
 	colcombo->insertItem( "", i18n("<placeholder>default</placeholder>") );
 	loadColorSchemes( colcombo );
+	colcombo->model()->sort( 0 );
 	label = new QLabel( i18n("Color sche&me:"), box );
 	label->setBuddy( colcombo );
 	connect( colcombo, SIGNAL(activated( int )), SIGNAL(changed()) );
@@ -161,19 +163,18 @@ void KDMGeneralWidget::loadColorSchemes( KBackedComboBox *combo )
 {
 	// XXX: Global + local schemes
 	QStringList list = KGlobal::dirs()->
-		findAllResources( "data", "kdisplay/color-schemes/*.kcsrc", KStandardDirs::NoDuplicates );
+		findAllResources( "data", "color-schemes/*.colors", KStandardDirs::NoDuplicates );
 	for (QStringList::ConstIterator it = list.begin(); it != list.end(); ++it)
 	{
 		KConfig _config( *it, KConfig::SimpleConfig );
-		KConfigGroup config(&_config, "Color Scheme" );
+		KConfigGroup config(&_config, "General" );
 
 		QString str;
-		if (!(str = config.readEntry( "Name" )).isEmpty() ||
-			!(str = config.readEntry( "name" )).isEmpty())
+		if (!(str = config.readEntry( "Name" )).isEmpty())
 		{
 			QString str2 = (*it).mid( (*it).lastIndexOf( '/' ) + 1 ); // strip off path
-			str2.resize( str2.length() - 6 ); // strip off ".kcsrc
-				combo->insertItem( str2, str );
+			str2.chop( 7 ); // strip off ".colors"
+			combo->insertItem( str2, str );
 		}
 	}
 }
