@@ -5,9 +5,10 @@
  *
  * ----
  *
- * This program is free software; you can redistribute it and/or
- * modify it under the terms of the GNU General Public
- * License version 2 as published by the Free Software Foundation.
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -21,27 +22,54 @@
  */
 
 #include "FontFilter.h"
-#include <klocale.h>
-#include <kiconloader.h>
-#include <ktoggleaction.h>
-#include <kselectaction.h>
-#include <kicon.h>
-#include <QLabel>
-#include <QPen>
-#include <QPainter>
-#include <QStyleOption>
-#include <QSet>
-#include <QString>
-#include <QMenu>
-#include <QMouseEvent>
-#include <QApplication>
-#include <QActionGroup>
+#include "FontFilterProxyStyle.h"
+#include <KDE/KLocale>
+#include <KDE/KIconLoader>
+#include <KDE/KToggleAction>
+#include <KDE/KSelectAction>
+#include <KDE/KIcon>
+#include <QtGui/QLabel>
+#include <QtGui/QPen>
+#include <QtGui/QPainter>
+#include <QtGui/QStyleOption>
+#include <QtCore/QSet>
+#include <QtCore/QString>
+#include <QtGui/QMenu>
+#include <QtGui/QMouseEvent>
+#include <QtGui/QApplication>
+#include <QtGui/QActionGroup>
 
 namespace KFI
 {
 
 static const int constArrowPad(5);
 
+// FIXME: Go back to using StyleSheets instead of a proxy style
+// once Qt has been fixed not to mess with widget font when
+// using StyleSheets
+class CFontFilterStyle : public CFontFilterProxyStyle
+{
+    public:
+
+    CFontFilterStyle(CFontFilter *parent, int ol) : CFontFilterProxyStyle(parent), overlap(ol) {}
+
+    QRect subElementRect(SubElement element, const QStyleOption *option, const QWidget *widget) const;
+
+    int overlap;
+};
+
+QRect CFontFilterStyle::subElementRect(SubElement element, const QStyleOption *option, const QWidget *widget) const
+{
+    if (SE_LineEditContents==element)
+    {
+        QRect rect(style()->subElementRect(SE_LineEditContents, option, widget));
+
+        return rect.adjusted(overlap, 0, -overlap, 0);
+    }
+
+    return CFontFilterProxyStyle::subElementRect(element, option, widget);
+}
+
 CFontFilter::CFontFilter(QWidget *parent)
            : KLineEdit(parent)
 {
@@ -57,10 +85,10 @@ CFontFilter::CFontFilter(QWidget *parent)
     itsPixmaps[CRIT_FAMILY]=SmallIcon("text");
     itsPixmaps[CRIT_STYLE]=SmallIcon("format-text-bold");
     itsPixmaps[CRIT_FOUNDRY]=SmallIcon("user");
-    itsPixmaps[CRIT_FONTCONFIG]=SmallIcon("file-find");
-    itsPixmaps[CRIT_FILENAME]=SmallIcon("font-type1");
+    itsPixmaps[CRIT_FONTCONFIG]=SmallIcon("system-search");
+    itsPixmaps[CRIT_FILENAME]=SmallIcon("application-x-font-type1");
     itsPixmaps[CRIT_LOCATION]=SmallIcon("folder");
-    itsPixmaps[CRIT_WS]=SmallIcon("pencil");
+    itsPixmaps[CRIT_WS]=SmallIcon("character-set");
 
     itsActionGroup=new QActionGroup(this);
     addAction(CRIT_FAMILY, i18n("Family"), true, true);
@@ -97,6 +125,7 @@ CFontFilter::CFontFilter(QWidget *parent)
     connect(wsMenu, SIGNAL(triggered(const QString &)), SLOT(wsChanged()));
 
     setCriteria(CRIT_FAMILY);
+    setStyle(new CFontFilterStyle(this, itsMenuButton->width()));
 }
 
 void CFontFilter::setMgtMode(bool m)
@@ -122,7 +151,19 @@ void CFontFilter::setFoundries(const QSet<QString> &foundries)
     QStringList list(foundries.toList());
 
     list.sort();
-    ((KSelectAction *)itsActions[CRIT_FOUNDRY])->setItems(list);
+
+    // Add foundries to menu - replacing '&' with '&&', as '&' is taken to be
+    // a shortcut!
+    QStringList::ConstIterator it(list.begin()),
+                               end(list.end());
+
+    for(; it!=end; ++it)
+    {
+        QString foundry(*it);
+
+        foundry.replace("&", "&&");
+        ((KSelectAction *)itsActions[CRIT_FOUNDRY])->addAction(foundry);
+    }
 
     if(!prev.isEmpty())
     {
@@ -161,7 +202,6 @@ void CFontFilter::filterChanged()
             setCriteria(crit);
             setClickMessage(i18n("Type here to filter on %1", act->text()));
             setReadOnly(false);
-            modifyPadding();
         }
     }
 }
@@ -189,9 +229,8 @@ void CFontFilter::wsChanged()
             itsCurrentWs=ws;
             itsCurrentCriteria=CRIT_WS;
             setReadOnly(true);
-            modifyPadding();
             setCriteria(itsCurrentCriteria);
-            setText(i18n("%1 (Writing System)", act->text()));
+            setText(act->text());
             setClickMessage(text());
         }
     }
@@ -212,7 +251,6 @@ void CFontFilter::foundryChanged(const QString &foundry)
 
     itsCurrentCriteria=CRIT_FOUNDRY;
     setReadOnly(true);
-    modifyPadding();
     setText(foundry);
     setClickMessage(text());
     setCriteria(itsCurrentCriteria);
@@ -258,14 +296,14 @@ void CFontFilter::paintEvent(QPaintEvent *ev)
 void CFontFilter::resizeEvent(QResizeEvent *ev)
 {
     KLineEdit::resizeEvent(ev);
-    modifyPadding();
 
-    int frameWidth(style()->pixelMetric(QStyle::PM_DefaultFrameWidth));
+    int frameWidth(style()->pixelMetric(QStyle::PM_DefaultFrameWidth)),
+        y((height()-itsMenuButton->height())/2);
 
     if (qApp->isLeftToRight())
-        itsMenuButton->move(frameWidth + 1, frameWidth + 1);
+        itsMenuButton->move(frameWidth + 2, y);
     else
-        itsMenuButton->move(size().width() - frameWidth - itsMenuButton->width() - 1, frameWidth + 1);
+        itsMenuButton->move(size().width() - frameWidth - itsMenuButton->width() - 2, y);
 }
 
 void CFontFilter::mousePressEvent(QMouseEvent *ev)
@@ -280,14 +318,16 @@ void CFontFilter::setCriteria(ECriteria crit)
 {
     QPixmap arrowmap(itsPixmaps[crit].width()+constArrowPad, itsPixmaps[crit].height());
 
-    arrowmap.fill(palette().color(QPalette::Active, QPalette::Base));
+    QColor bgnd(palette().color(QPalette::Active, QPalette::Base));
+    bgnd.setAlphaF(0.0);
+    arrowmap.fill(bgnd);
 
     QPainter p(&arrowmap);
 
     p.drawPixmap(0, 0, itsPixmaps[crit]);
     QStyleOption opt;
     opt.state = QStyle::State_Enabled;
-    opt.rect = QRect(arrowmap.width()-(constArrowPad+1), arrowmap.height()-constArrowPad, constArrowPad+1, constArrowPad);
+    opt.rect = QRect(arrowmap.width()-(constArrowPad+1), arrowmap.height()-(constArrowPad+1), constArrowPad, constArrowPad);
     style()->drawPrimitive(QStyle::PE_IndicatorArrowDown, &opt, &p, itsMenuButton);
     p.end();
 
@@ -298,13 +338,6 @@ void CFontFilter::setCriteria(ECriteria crit)
     emit criteriaChanged(crit, ((qulonglong)1) << (int)itsCurrentWs);
 }
 
-void CFontFilter::modifyPadding()
-{
-    setStyleSheet(QString("QLineEdit { padding-left: %1; padding-right : %2; }")
-                  .arg(itsMenuButton->width())
-                  .arg(itsMenuButton->width()-constArrowPad));
-}
-
 }
 
 #include "FontFilter.moc"
