@@ -5,9 +5,10 @@
  *
  * ----
  *
- * This program is free software; you can redistribute it and/or
- * modify it under the terms of the GNU General Public
- * License version 2 as published by the Free Software Foundation.
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -21,31 +22,31 @@
  */
 
 #include "FontList.h"
-#include <kglobal.h>
-#include <kstandarddirs.h>
-#include <klocale.h>
-#include <kmimetype.h>
-#include <kicon.h>
-#include <kiconloader.h>
+#include <KDE/KGlobal>
+#include <KDE/KStandardDirs>
+#include <KDE/KLocale>
+#include <KDE/KMimeType>
+#include <KDE/KIcon>
+#include <KDE/KIconLoader>
 #include <kde_file.h>
-#include <kmessagebox.h>
-#include <QProcess>
-#include <QFont>
-#include <QMap>
-#include <QFile>
-#include <QImage>
-#include <QDir>
-#include <QFileInfo>
-#include <QPixmap>
-#include <QDropEvent>
-#include <QDateTime>
-#include <QX11Info>
-#include <QHeaderView>
-#include <QItemDelegate>
-#include <QPainter>
-#include <QMenu>
-#include <QTimer>
-#include <QApplication>
+#include <KDE/KMessageBox>
+#include <QtCore/QProcess>
+#include <QtGui/QFont>
+#include <QtCore/QMap>
+#include <QtCore/QFile>
+#include <QtGui/QImage>
+#include <QtCore/QDir>
+#include <QtCore/QFileInfo>
+#include <QtGui/QPixmap>
+#include <QtGui/QDropEvent>
+#include <QtCore/QDateTime>
+#include <QtGui/QX11Info>
+#include <QtGui/QHeaderView>
+#include <QtGui/QItemDelegate>
+#include <QtGui/QPainter>
+#include <QtGui/QMenu>
+#include <QtCore/QTimer>
+#include <QtGui/QApplication>
 #include <stdlib.h>
 #include <unistd.h>
 #include <utime.h>
@@ -60,6 +61,10 @@
 // cached to disk.
 #define KFI_SAVE_PIXMAPS
 
+#ifdef KFI_FONTLIST_DEBUG
+#include <kdebug.h>
+#endif
+
 namespace KFI
 {
 
@@ -239,6 +244,40 @@ static QString replaceChars(const QString &in)
     return rv;
 }
 
+QString capitaliseFoundry(const QString &foundry)
+{
+    QString f(foundry.toLower());
+
+    if(f==QString::fromLatin1("ibm"))
+        return QString::fromLatin1("IBM");
+    else if(f==QString::fromLatin1("urw"))
+        return QString::fromLatin1("URW");
+    else if(f==QString::fromLatin1("itc"))
+        return QString::fromLatin1("ITC");
+    else if(f==QString::fromLatin1("nec"))
+        return QString::fromLatin1("NEC");
+    else if(f==QString::fromLatin1("b&h"))
+        return QString::fromLatin1("B&H");
+    else
+    {
+        QChar   *ch(f.data());
+        int     len(f.length());
+        bool    isSpace(true);
+
+        while(len--)
+        {
+            if (isSpace)
+                *ch=ch->toUpper();
+
+            isSpace=ch->isSpace();
+            ++ch;
+        }
+
+    }
+
+    return f;
+}
+
 static void toggle(QString &file, bool enable)
 {
     QString newFile(enable
@@ -383,22 +422,21 @@ inline bool isSysFolder(const QString &sect)
 
 CFontItem::CFontItem(CFontModelItem *p, const KFileItem &item, const QString &style)
          : CFontModelItem(p),
-           itsItem(item),
            itsStyle(style),
            itsPixmap(NULL)
 {
-    const KIO::UDSEntry &udsEntry(entry());
+    const KIO::UDSEntry &udsEntry(item.entry());
 
-    updateStatus();
+    setUrl(item.url());
     itsName=udsEntry.stringValue(KIO::UDSEntry::UDS_NAME);
     itsFileName=udsEntry.stringValue((uint)UDS_EXTRA_FILE_NAME);
     itsStyleInfo=udsEntry.numberValue((uint)UDS_EXTRA_FC_STYLE);
     itsIndex=Misc::getIntQueryVal(KUrl(udsEntry.stringValue((uint)KIO::UDSEntry::UDS_URL)),
                                   KFI_KIO_FACE, 0);
     itsWritingSystems=udsEntry.numberValue((uint)UDS_EXTRA_WRITING_SYSTEMS);
-    QString mime(mimetype());
-
-    itsBitmap="application/x-font-pcf"==mime || "application/x-font-bdf"==mime;
+    itsMimeType=item.mimetype();
+    itsBitmap="application/x-font-pcf"==itsMimeType || "application/x-font-bdf"==itsMimeType;
+    itsSize=item.size();
 
     if(!Misc::root())
         setIsSystem(isSysFolder(url().path().section('/', 1, 1)));
@@ -421,9 +459,10 @@ void CFontItem::touchThumbnail()
 #endif
 }
 
-void CFontItem::updateStatus()
+void CFontItem::setUrl(const KUrl &url)
 {
-    itsEnabled=!Misc::isHidden(url());
+    itsUrl=url;
+    itsEnabled=!Misc::isHidden(itsUrl);
 
     if(!itsFiles.isEmpty()) // Then we changed state, so need to alter filename list...
     {
@@ -478,7 +517,7 @@ CFontItem * CFamilyItem::findFont(const KFileItem &i)
                                       fEnd(itsFonts.end());
 
     for(; fIt!=fEnd; ++fIt)
-        if((*(*fIt)).item()==i)
+        if((*(*fIt)).url()==i.url())
             return (*fIt);
 
     return NULL;
@@ -496,7 +535,7 @@ void CFamilyItem::getFoundries(QSet<QString> &foundries) const
 
         for(; fIt!=fEnd; ++fIt)
             if(!(*fIt).foundry.isEmpty())
-                foundries.insert((*fIt).foundry);
+                foundries.insert(capitaliseFoundry((*fIt).foundry));
     }
 }
 
@@ -586,7 +625,7 @@ bool CFamilyItem::updateStatus()
 
     itsIcon=1==mimeTypes.count()
                 ? KMimeType::mimeType(mimeTypes[0])->iconName()
-                : "font-truetype";
+                : "application-x-font-ttf";
 
     if(!root)
         setIsSystem(sys);
@@ -655,8 +694,8 @@ CFontList::CFontList(QWidget *parent)
             SLOT(newItems(const KFileItemList &)));
     connect(itsLister, SIGNAL(deleteItems(const KFileItemList &)),
             SLOT(deleteItems(const KFileItemList &)));
-    connect(itsLister, SIGNAL(refreshItems(const KFileItemList &)),
-            SLOT(refreshItems(const KFileItemList &)));
+    connect(itsLister, SIGNAL(renameItems(const RenameList &)),
+            SLOT(renameItems(const RenameList &)));
     connect(itsLister, SIGNAL(percent(int)), SIGNAL(percent(int)));
     connect(itsLister, SIGNAL(message(QString)), SIGNAL(status(QString)));
 }
@@ -913,10 +952,10 @@ void CFontList::newItems(const KFileItemList &items)
     emit layoutAboutToBeChanged();
 
 #ifdef KFI_FONTLIST_DEBUG
-    kDebug() << "************** newItems " << items.count();
+    kDebug() << "************** " << items.count();
 
-    for(KFileItemList::const_iterator it(items.begin()), end(items.end()) ; it!=end ; ++it)
-        kDebug() << "               " << (int)(*it);
+//     for(KFileItemList::const_iterator it(items.begin()), end(items.end()) ; it!=end ; ++it)
+//         kDebug() << "               " << (int)(*it);
 #endif
 
     for(KFileItemList::const_iterator it(items.begin()), end(items.end()) ; it!=end ; ++it)
@@ -928,7 +967,7 @@ void CFontList::newItems(const KFileItemList &items)
 void CFontList::clearItems()
 {
 #ifdef KFI_FONTLIST_DEBUG
-    kDebug() << "************** clearItems";
+    kDebug() << "**************";
 #endif
 
     beginRemoveRows(QModelIndex(), 0, itsFamilies.count());
@@ -938,35 +977,38 @@ void CFontList::clearItems()
     itsFonts.clear();
 }
 
-void CFontList::refreshItems(const KFileItemList &items)
+void CFontList::renameItems(const RenameList &items)
 {
     emit layoutAboutToBeChanged();
 
 #ifdef KFI_FONTLIST_DEBUG
-    kDebug() << "************** refreshItems " << items.count();
+    kDebug() << "************** " << items.count();
 
-    for(KFileItemList::const_iterator it(items.begin()), end(items.end()) ; it!=end ; ++it)
-        kDebug() << "               " << (int)(*it);
+    for(RenameList::const_iterator it(items.begin()), end(items.end()) ; it!=end ; ++it)
+        kDebug() << "               " << (*it).from.prettyUrl();
 #endif
 
     QSet<CFamilyItem *> families;
 
-    for(KFileItemList::const_iterator it(items.begin()), end(items.end()) ; it!=end ; ++it)
+    for(RenameList::const_iterator it(items.begin()), end(items.end()) ; it!=end ; ++it)
     {
-        CFontItem *font=findFont(*it);
+        CFontItem *font=findFont((*it).from);
 
         if(font)
         {
-            font->updateStatus();
+            font->setUrl((*it).to);
+            itsFonts.insert((*it).to, font);
+            itsFonts.erase(itsFonts.find((*it).from));
 #ifdef KFI_FONTLIST_DEBUG
             kDebug() << "               Found font, status now:" << font->isEnabled()
-                     << " url" << (*it)->url().prettyUrl();
+                     << " from:" << (*it).from.prettyUrl()
+                     << " to:" << (*it).to.prettyUrl();
 #endif
             families.insert(static_cast<CFamilyItem *>(font->parent()));
         }
 #ifdef KFI_FONTLIST_DEBUG
         else
-            kDebug() << "               Could not locate font :-( " << (int)(*it);
+            kDebug() << "               Could not locate font :-( " << (*it).from.prettyUrl();
 #endif
     }
 
@@ -984,7 +1026,7 @@ void CFontList::deleteItems(const KFileItemList &items)
     emit layoutAboutToBeChanged();
 
 #ifdef KFI_FONTLIST_DEBUG
-    kDebug() << "************** deleteItems " << items.count();
+    kDebug() << "************** " << items.count();
 #endif
 
     KFileItemList::ConstIterator it(items.begin()),
@@ -992,7 +1034,7 @@ void CFontList::deleteItems(const KFileItemList &items)
 
     for(; it!=end; ++it)
     {
-        CFontItem *font=findFont(*it);
+        CFontItem *font=findFont((*it).url());
 
         if(font)
         {
@@ -1002,7 +1044,7 @@ void CFontList::deleteItems(const KFileItemList &items)
                 itsFamilies.removeAll(fam);
             else
                 fam->removeFont(font);
-            itsFonts.remove(*it);
+            itsFonts.remove((*it).url());
         }
     }
 
@@ -1011,10 +1053,10 @@ void CFontList::deleteItems(const KFileItemList &items)
 
 void CFontList::addItem(const KFileItem &item)
 {
-    CFontItem *font=findFont(item);
+    CFontItem *font=findFont(item.url());
 
 #ifdef KFI_FONTLIST_DEBUG
-    kDebug() << "************** addItem " << item.url();
+    kDebug() << "************** " << item.url();
 #endif
     if(!font)
     {
@@ -1030,7 +1072,7 @@ void CFontList::addItem(const KFileItem &item)
             font=new CFontItem(fam, item, style);
 
             fam->addFont(font);
-            itsFonts.insert(item, font);
+            itsFonts.insert(item.url(), font);
         }
 #ifdef KFI_FONTLIST_DEBUG
         else
@@ -1062,10 +1104,10 @@ CFamilyItem * CFontList::findFamily(const QString &familyName, bool create)
     return fam;
 }
 
-CFontItem * CFontList::findFont(const KFileItem &item)
+CFontItem * CFontList::findFont(const KUrl &url)
 {
-    return itsFonts.contains(item)
-            ? itsFonts[item]
+    return itsFonts.contains(url)
+            ? itsFonts[url]
             : NULL;
 }
 
@@ -1266,7 +1308,7 @@ bool CFontListSortFilterProxy::acceptFont(CFontItem *fnt, bool checkFontText) co
                                                              end(fnt->files().end());
 
                     for(; it!=end && !fontMatch; ++it)
-                        fontMatch=-1!=(*it).foundry.indexOf(itsFilterText, 0, Qt::CaseInsensitive);
+                        fontMatch=0==(*it).foundry.compare(itsFilterText, Qt::CaseInsensitive);
                     break;
                 }
                 case CFontFilter::CRIT_FILENAME:
@@ -1980,10 +2022,10 @@ void CFontListView::view()
     {
         QSet<CFontItem *>::ConstIterator it(fonts.begin()),
                                          end(fonts.end());
+        QStringList                      args;
 
         for(; it!=end; ++it)
         {
-            QStringList args;
 
             // If we can, speed up font viewer by passing the font details...
             if((*it)->isEnabled())
@@ -2029,9 +2071,9 @@ void CFontListView::view()
                 if(!done)
                     args << (*it)->url().url();
             }
-
-            QProcess::startDetached(KFI_VIEWER, args);
         }
+
+        QProcess::startDetached(KFI_VIEWER, args);
     }
 }
 
@@ -2070,7 +2112,7 @@ void CFontListView::startDrag(Qt::DropActions supportedActions)
             return;
 
         QModelIndex index(itsProxy->mapToSource(indexes.first()));
-        const char  *icon="font-bitmap";
+        const char  *icon="application-x-font-pcf";
 
         if(index.isValid())
         {
@@ -2080,9 +2122,9 @@ void CFontListView::startDrag(Qt::DropActions supportedActions)
 
             if(font && !font->isBitmap())
                 if("application/x-font-type1"==font->mimetype())
-                    icon="font-type1";
+                    icon="application-x-font-type1";
                 else
-                    icon="font-truetype";
+                    icon="application-x-font-ttf";
         }
 
         QPoint  hotspot;
