@@ -5,9 +5,10 @@
  *
  * ----
  *
- * This program is free software; you can redistribute it and/or
- * modify it under the terms of the GNU General Public
- * License version 2 as published by the Free Software Foundation.
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -22,25 +23,25 @@
 
 #include "GroupList.h"
 #include "FontList.h"
-#include <kglobal.h>
-#include <kstandarddirs.h>
-#include <klocale.h>
-#include <kmimetype.h>
-#include <kiconloader.h>
+#include <KDE/KGlobal>
+#include <KDE/KStandardDirs>
+#include <KDE/KLocale>
+#include <KDE/KMimeType>
+#include <KDE/KIconLoader>
 #include <kde_file.h>
-#include <kmessagebox.h>
-#include <kinputdialog.h>
-#include <ksavefile.h>
-#include <QFont>
-#include <QFontDatabase>
-#include <QDropEvent>
-#include <QHeaderView>
-#include <QMenu>
-#include <QApplication>
-#include <QDomElement>
-#include <QTextStream>
-#include <QDir>
-#include <QPainter>
+#include <KDE/KMessageBox>
+#include <KDE/KInputDialog>
+#include <KDE/KSaveFile>
+#include <QtGui/QFont>
+#include <QtGui/QFontDatabase>
+#include <QtGui/QDropEvent>
+#include <QtGui/QHeaderView>
+#include <QtGui/QMenu>
+#include <QtGui/QApplication>
+#include <QtXml/QDomElement>
+#include <QtCore/QTextStream>
+#include <QtCore/QDir>
+#include <QtGui/QPainter>
 #include <stdlib.h>
 #include <unistd.h>
 #include <utime.h>
@@ -338,7 +339,7 @@ QVariant CGroupList::data(const QModelIndex &index, int role) const
                                 case CGroupListItem::PERSONAL: // Copying/moving
                                 case CGroupListItem::SYSTEM:   // Copying/moving
                                     return SmallIcon(Qt::LeftToRight==QApplication::layoutDirection()
-                                                        ? "go-next" : "go-next-rtl");
+                                                        ? "go-next" : "go-previous");
                                 case CGroupListItem::CUSTOM:   // Adding to a group
                                     return SmallIcon("list-add");
                                 default:
