@@ -5,9 +5,10 @@
  *
  * ----
  *
- * This program is free software; you can redistribute it and/or
- * modify it under the terms of the GNU General Public
- * License version 2 as published by the Free Software Foundation.
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -23,23 +24,22 @@
 #include "JobRunner.h"
 #include "KfiConstants.h"
 #include "Misc.h"
-#include <kio/jobuidelegate.h>
-#include <kio/skipdialog.h>
-#include <kio/netaccess.h>
-#include <kio/copyjob.h>
-#include <kio/deletejob.h>
-#include <kmessagebox.h>
-#include <kglobal.h>
-#include <kiconloader.h>
-#include <klocale.h>
-#include <kpassworddialog.h>
-#include <kdesu/su.h>
-#include <kdirnotify.h>
-#include <QGridLayout>
-#include <QProgressBar>
-#include <QLabel>
-#include <QX11Info>
-#include <QTimer>
+#include <KDE/KIO/JobUiDelegate>
+#include <KDE/KIO/NetAccess>
+#include <KDE/KIO/CopyJob>
+#include <KDE/KIO/DeleteJob>
+#include <KDE/KMessageBox>
+#include <KDE/KGlobal>
+#include <KDE/KIconLoader>
+#include <KDE/KLocale>
+#include <KDE/KPasswordDialog>
+#include <KDE/SuProcess>
+#include <KDE/OrgKdeKDirNotifyInterface>
+#include <QtGui/QGridLayout>
+#include <QtGui/QProgressBar>
+#include <QtGui/QLabel>
+#include <QtGui/QX11Info>
+#include <QtCore/QTimer>
 #include <X11/Xlib.h>
 #include <fixx11h.h>
 #include <sys/resource.h>
@@ -59,6 +59,58 @@ static KUrl toggle(const KUrl &orig, bool enable)
     return url;
 }
 
+class CSkipDialog : public KDialog
+{
+    public:
+
+    enum Result
+    {
+        SKIP,
+        AUTO_SKIP,
+        CANCEL
+    };
+
+    CSkipDialog(QWidget *parent, bool multi, const QString &errorText)
+        : KDialog(parent),
+          itsResult(CANCEL)
+    {
+        setCaption(i18n( "Information"));
+        setButtons(multi ? Cancel|User1|User2 : Cancel );
+        setButtonText(User1, i18n("Skip"));
+        setButtonText(User2, i18n("AutoSkip"));
+        setMainWidget(new QLabel(errorText, this));
+        resize(sizeHint());
+    }
+
+    Result go()
+    {
+        itsResult=CANCEL;
+        exec();
+        return itsResult;
+    }
+
+    void slotButtonClicked(int button)
+    {
+        switch(button)
+        {
+            case User1:
+                itsResult=SKIP;
+                break;
+            case User2:
+                itsResult=AUTO_SKIP;
+                break;
+            default:
+                itsResult=CANCEL;
+        }
+
+        KDialog::accept();
+    }
+
+    private:
+
+    Result itsResult;
+};
+
 class CPasswordDialog : public KPasswordDialog
 {
     public:
@@ -76,7 +128,7 @@ class CPasswordDialog : public KPasswordDialog
             setPrompt(i18n("The requested action requires administrator privileges.\n"
                            "Please enter the system administrator's password."));
 
-        setPixmap(DesktopIcon("password"));
+        setPixmap(DesktopIcon("dialog-password"));
     }
 
     bool checkPassword()
@@ -324,7 +376,7 @@ void CJobRunner::doNext()
                 break;
             case CMD_DISABLE:
                 itsStatusLabel->setText(i18n("Disabling %1", (*itsIt).displayName()));
-                job=KIO::rename(*itsIt, toggle(*itsIt, KIO::HideProgressInfo), KIO::HideProgressInfo);
+                job=KIO::rename(*itsIt, toggle(*itsIt, false), KIO::HideProgressInfo);
                 break;
             case CMD_MOVE:
             {
@@ -384,17 +436,17 @@ void CJobRunner::jobResult(KJob *job)
             }
             else
             {
-                KIO::SkipDialog dlg(this, true, job->errorString());
+                CSkipDialog dlg(this, true, job->errorString());
 
-                switch(dlg.exec())
+                switch(dlg.go())
                 {
-                    case KIO::S_SKIP:
+                    case CSkipDialog::SKIP:
                         cont=true;
                         break;
-                    case KIO::S_AUTO_SKIP:
+                    case CSkipDialog::AUTO_SKIP:
                         cont=itsAutoSkip=true;
                         break;
-                    case KIO::S_CANCEL:
+                    case CSkipDialog::CANCEL:
                         break;
                 }
             }
