@@ -5,9 +5,10 @@
  *
  * ----
  *
- * This program is free software; you can redistribute it and/or
- * modify it under the terms of the GNU General Public
- * License version 2 as published by the Free Software Foundation.
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -30,37 +31,38 @@
 #include "DuplicatesDialog.h"
 #include "PreviewSelectAction.h"
 #include "FontFilter.h"
-#include <QGridLayout>
-#include <QBoxLayout>
-#include <QLabel>
-#include <QSplitter>
-#include <QProgressBar>
-#include <QCoreApplication>
-#include <QApplication>
-#include <QTextStream>
-#include <QComboBox>
-#include <QProcess>
-#include <kaboutdata.h>
-#include <ktoolbar.h>
-#include <kfiledialog.h>
-#include <kmessagebox.h>
-#include <kio/job.h>
-#include <kio/netaccess.h>
-#include <kpushbutton.h>
-#include <kguiitem.h>
-#include <kinputdialog.h>
-#include <kiconloader.h>
-#include <kprogressdialog.h>
-#include <kzip.h>
-#include <ktempdir.h>
-#include <ktemporaryfile.h>
-#include <kicon.h>
-#include <kactionmenu.h>
-#include <ktoggleaction.h>
-#include <kstandarddirs.h>
-#include <kmenu.h>
-#include <KPluginFactory>
-#include <KPluginLoader>
+#include <QtGui/QGridLayout>
+#include <QtGui/QBoxLayout>
+#include <QtGui/QLabel>
+#include <QtGui/QSplitter>
+#include <QtGui/QProgressBar>
+#include <QtCore/QCoreApplication>
+#include <QtGui/QApplication>
+#include <QtCore/QTextStream>
+#include <QtGui/QComboBox>
+#include <QtCore/QProcess>
+#include <QtGui/QPainter>
+#include <KDE/KAboutData>
+#include <KDE/KToolBar>
+#include <KDE/KFileDialog>
+#include <KDE/KMessageBox>
+#include <KDE/KIO/Job>
+#include <KDE/KIO/NetAccess>
+#include <KDE/KPushButton>
+#include <KDE/KGuiItem>
+#include <KDE/KInputDialog>
+#include <KDE/KIconLoader>
+#include <KDE/KProgressDialog>
+#include <KDE/KZip>
+#include <KDE/KTempDir>
+#include <KDE/KTemporaryFile>
+#include <KDE/KIcon>
+#include <KDE/KActionMenu>
+#include <KDE/KToggleAction>
+#include <KDE/KStandardDirs>
+#include <KDE/KMenu>
+#include <KDE/KPluginFactory>
+#include <KDE/KPluginLoader>
 
 #define CFG_GROUP                  "Main Settings"
 #define CFG_PREVIEW_SPLITTER_SIZES "PreviewSplitterSizes"
@@ -111,6 +113,8 @@ class CPushButton : public KPushButton
         sh.setHeight(theirHeight);
         if(sh.width()<sh.height())
             sh.setWidth(sh.height());
+        else if(text().isEmpty())
+            sh.setWidth(theirHeight);
         return sh;
     }
 
@@ -127,6 +131,15 @@ class CToolBar : public KToolBar
         : KToolBar(parent)
     {
         theirHeight=qMax(theirHeight, height());
+        setMovable(false);
+        setFloatable(false);
+        setToolButtonStyle(Qt::ToolButtonTextBesideIcon);
+        setFont(QApplication::font());
+    }
+
+    void addSeparator()
+    {
+        addWidget(new QLabel(" ", this));
     }
 
     void link(CToolBar *l)
@@ -142,6 +155,14 @@ class CToolBar : public KToolBar
         return sh;
     }
 
+    void paintEvent(QPaintEvent *)
+    {
+        QColor col(palette().color(backgroundRole()));
+
+        col.setAlphaF(0.0);
+        QPainter(this).fillRect(rect(), col);
+    }
+
     void resizeEvent(QResizeEvent *ev)
     {
         KToolBar::resizeEvent(ev);
@@ -242,7 +263,6 @@ CKCmFontInst::CKCmFontInst(QWidget *parent, const QVariantList&)
     itsPreviewControl=new CPreviewSelectAction(itsPreviewWidget);
 
     previewToolbar->addAction(itsPreviewControl);
-    previewToolbar->setMovable(false);
 
     previewLayout->setMargin(0);
     previewLayout->setSpacing(KDialog::spacingHint());
@@ -264,14 +284,14 @@ CKCmFontInst::CKCmFontInst(QWidget *parent, const QVariantList&)
     // Toolbar...
     KActionMenu *settingsMenu=new KActionMenu(KIcon("configure"), i18n("Settings"), this);
     KAction     *changeTextAct=new KAction(KIcon("text"), i18n("Change Preview Text..."), this),
-                *duplicateFontsAct=new KAction(KIcon("file-find"), i18n("Scan For Duplicate Fonts..."), this);
+                *duplicateFontsAct=new KAction(KIcon("system-search"), i18n("Scan For Duplicate Fonts..."), this);
                 //*validateFontsAct=new KAction(KIcon("checkmark"), i18n("Validate Fonts..."), this);
                 //*downloadFontsAct=new KAction(KIcon("go-down"), i18n("Download Fonts..."), this);
 
-    itsToolsMenu=new KActionMenu(KIcon("tool2"), i18n("Tools"), this);
+    itsToolsMenu=new KActionMenu(KIcon("system-run"), i18n("Tools"), this);
     itsMgtMode=new KToggleAction(KIcon("preferences-desktop-font-installer"),
                                  i18n("Font Management Mode"), this),
-    itsShowPreview=new KToggleAction(KIcon("thumbnail-show"), i18n("Show Large Preview"), this);
+    itsShowPreview=new KToggleAction(KIcon("view-preview"), i18n("Show Large Preview"), this);
     settingsMenu->addAction(itsMgtMode);
     itsMgtMode->setChecked(true);
     settingsMenu->addSeparator();
@@ -284,8 +304,6 @@ CKCmFontInst::CKCmFontInst(QWidget *parent, const QVariantList&)
     itsToolsMenu->setDelayed(false);
     toolbar->addAction(settingsMenu);
     toolbar->addAction(itsToolsMenu);
-    toolbar->setToolButtonStyle(Qt::ToolButtonTextBesideIcon);
-    toolbar->setMovable(false);
     if(Misc::root())
         itsModeControl=NULL;
     else
@@ -310,7 +328,7 @@ CKCmFontInst::CKCmFontInst(QWidget *parent, const QVariantList&)
                                                       i18n("Create a new group")),
                                              itsGroupsWidget);
 
-    itsDeleteGroupControl=new CPushButton(KGuiItem(QString(), "list-remove",
+    itsDeleteGroupControl=new CPushButton(KGuiItem(QString(), "edit-delete",
                                                    i18n("Remove group")),
                                           itsGroupsWidget);
 
@@ -335,7 +353,7 @@ CKCmFontInst::CKCmFontInst(QWidget *parent, const QVariantList&)
     itsFontListView->setSizePolicy(QSizePolicy::MinimumExpanding, QSizePolicy::MinimumExpanding);
     itsFontListView->readConfig(cg);
 
-    itsAddFontControl=new CPushButton(KGuiItem(i18n("Add..."), "addfont",
+    itsAddFontControl=new CPushButton(KGuiItem(i18n("Add..."), "list-add",
                                                i18n("Install fonts")),
                                       itsFontsWidget);
 
@@ -688,84 +706,93 @@ void CKCmFontInst::print(bool all)
     // up to date.
     if(!working() && (!itsPrintProc || QProcess::NotRunning==itsPrintProc->state()))
     {
-        QSet<Misc::TFont> fonts;
+        QString exe(KStandardDirs::findExe(QLatin1String(KFI_PRINTER), KStandardDirs::installPath("libexec")));
 
-        itsFontListView->getPrintableFonts(fonts, !all);
-
-        if(fonts.count())
+        if(exe.isEmpty())
+            KMessageBox::error(this, i18n("Failed to locate font printer."));
+        else
         {
-            CPrintDialog dlg(this);
-            KConfigGroup cg(&itsConfig, CFG_GROUP);
+            QSet<Misc::TFont> fonts;
 
-            if(dlg.exec(cg.readEntry(CFG_FONT_SIZE, 1)))
+            itsFontListView->getPrintableFonts(fonts, !all);
+
+            if(fonts.count())
             {
-                static const int constSizes[]={0, 12, 18, 24, 36, 48};
-                QSet<Misc::TFont>::ConstIterator it(fonts.begin()),
-                                                 end(fonts.end());
-                KTemporaryFile                   tmpFile;
-                bool                             useFile(fonts.count()>16),
-                                                 startProc(true);
-                QStringList                      args;
+                CPrintDialog dlg(this);
+                KConfigGroup cg(&itsConfig, CFG_GROUP);
 
-                if(!itsPrintProc)
-                    itsPrintProc=new QProcess(this);
-                else
-                    itsPrintProc->kill();
-
-                //
-                // If we have lots of fonts to print, pass kfontinst a tempory groups file to print
-                // instead of passing font by font...
-                if(useFile)
+                if(dlg.exec(cg.readEntry(CFG_FONT_SIZE, 1)))
                 {
-                    if(tmpFile.open())
+                    static const int constSizes[]={0, 12, 18, 24, 36, 48};
+                    QSet<Misc::TFont>::ConstIterator it(fonts.begin()),
+                                                    end(fonts.end());
+                    KTemporaryFile                   tmpFile;
+                    bool                             useFile(fonts.count()>16),
+                                                    startProc(true);
+                    QStringList                      args;
+
+                    if(!itsPrintProc)
+                        itsPrintProc=new QProcess(this);
+                    else
+                        itsPrintProc->kill();
+
+                    //
+                    // If we have lots of fonts to print, pass kfontinst a tempory groups file to print
+                    // instead of passing font by font...
+                    if(useFile)
                     {
-                        QTextStream str(&tmpFile);
+                        if(tmpFile.open())
+                        {
+                            QTextStream str(&tmpFile);
+
+                            for(; it!=end; ++it)
+                                str << (*it).family << endl
+                                    << (*it).styleInfo << endl;
+
+                            args << "--embed" << QString().sprintf("0x%x", (unsigned int)topLevelWidget()->winId())
+                                << "--caption" << KGlobal::caption().toUtf8()
+                                << "--icon" << "preferences-desktop-font-installer"
+                                << "--size" << QString().setNum(constSizes[dlg.chosenSize() < 6 ? dlg.chosenSize() : 2])
+                                << "--listfile" << tmpFile.fileName()
+                                << "--deletefile";
+                        }
+                        else
+                        {
+                            KMessageBox::error(this, i18n("Failed to save list of fonts to print."));
+                            startProc=false;
+                        }
+                    }
+                    else
+                    {
+                        args << "--embed" << QString().sprintf("0x%x", (unsigned int)topLevelWidget()->winId())
+                            << "--caption" << KGlobal::caption().toUtf8()
+                            << "--icon" << "preferences-desktop-font-installer"
+                            << "--size" << QString().setNum(constSizes[dlg.chosenSize()<6 ? dlg.chosenSize() : 2]);
 
                         for(; it!=end; ++it)
-                            str << (*it).family << endl
-                                << (*it).styleInfo << endl;
-
-                        args << "--embed" << QString().sprintf("0x%x", (unsigned int)topLevelWidget()->winId())
-                             << "--caption" << KGlobal::caption().toUtf8()
-                             << "--size" << QString().setNum(constSizes[dlg.chosenSize() < 6 ? dlg.chosenSize() : 2])
-                             << "--listfile" << tmpFile.fileName()
-                             << "--deletefile";
+                            args << "--pfont" << QString((*it).family.toUtf8()+','+QString().setNum((*it).styleInfo));
                     }
-                    else
+
+                    if(startProc)
                     {
-                        KMessageBox::error(this, i18n("Failed to save list of fonts to print."));
-                        startProc=false;
+                        itsPrintProc->start(exe, args);
+
+                        if(itsPrintProc->waitForStarted(1000))
+                        {
+                            if(useFile)
+                                tmpFile.setAutoRemove(false);
+                        }
+                        else
+                            KMessageBox::error(this, i18n("Failed to start font printer."));
                     }
+                    cg.writeEntry(CFG_FONT_SIZE, dlg.chosenSize());
                 }
-                else
-                {
-                    args << "--embed" << QString().sprintf("0x%x", (unsigned int)topLevelWidget()->winId())
-                         << "--caption" << KGlobal::caption().toUtf8()
-                         << "--size" << QString().setNum(constSizes[dlg.chosenSize()<6 ? dlg.chosenSize() : 2]);
-
-                    for(; it!=end; ++it)
-                        args << "--pfont" << QString((*it).family.toUtf8()+','+QString().setNum((*it).styleInfo));
-                }
-
-                if(startProc)
-                {
-                    itsPrintProc->start(KFI_PRINTER, args);
-
-                    if(itsPrintProc->waitForStarted(1000))
-                    {
-                        if(useFile)
-                            tmpFile.setAutoRemove(false);
-                    }
-                    else
-                        KMessageBox::error(this, i18n("Failed to start font printer."));
-                }
-                cg.writeEntry(CFG_FONT_SIZE, dlg.chosenSize());
             }
+            else
+                KMessageBox::information(this, i18n("There are no printable fonts.\n"
+                                                    "You can only print non-bitmap and enabled fonts."),
+                                         i18n("Cannot Print"));
         }
-        else
-            KMessageBox::information(this, i18n("There are no printable fonts.\n"
-                                                "You can only print non-bitmap and enabled fonts."),
-                                     i18n("Cannot Print"));
     }
 }
 
