@@ -8,9 +8,10 @@
  *
  * ----
  *
- * This program is free software; you can redistribute it and/or
- * modify it under the terms of the GNU General Public
- * License version 2 as published by the Free Software Foundation.
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -25,15 +26,15 @@
 
 #include <fontconfig/fontconfig.h>
 #include <time.h>
-#include <kio/slavebase.h>
-#include <kurl.h>
-#include <klocale.h>
+#include <KDE/KIO/SlaveBase>
+#include <KDE/KUrl>
+#include <KDE/KLocale>
 
-#include <QByteArray>
-#include <QHash>
-#include <QSet>
-#include <QList>
-#include <QVariant>
+#include <QtCore/QByteArray>
+#include <QtCore/QHash>
+#include <QtCore/QSet>
+#include <QtCore/QList>
+#include <QtCore/QVariant>
 #include "Misc.h"
 #include "KfiConstants.h"
 #include "DisabledFonts.h"
@@ -97,7 +98,7 @@ class CKioFonts : public KIO::SlaveBase
     struct TFolder
     {
         TFolder() : disabled(NULL) { }
-        void  setLocation(const QString &l, const QString &d, bool sys);
+        void  setLocation(const QString &l, bool sys);
 
         QString        location;
         CDirList       modified;
