@@ -257,11 +257,7 @@ static void applyQtSettings( KConfigGroup kglobals, QSettings& settings )
 
   /* export widget style */
   kglobals.changeGroup("General");
-#ifdef __GNUC__
-#warning FIXME KDE4: need replacement for defaultStyle()
-#endif
-  // KStyle::defaultStyle()
-  QString style = kglobals.readEntry("widgetStyle", "plastique" );
+  QString style = kglobals.readEntry("widgetStyle", KStyle::defaultStyle());
   if (!style.isEmpty())
     settings.setValue("/qt/style", style);
 
@@ -408,6 +404,7 @@ static void createGtkrc( bool exportColors, const QPalette& cg, int version )
         t << "  fg[NORMAL] = " << color( group.color( QPalette::Active, QPalette::Foreground ) ) << endl;
         t << "}" << endl;
         t << endl;
+        t << "widget \"gtk-tooltip\" style \"ToolTip\"" << endl;
         t << "widget \"gtk-tooltips\" style \"ToolTip\"" << endl;
         t << endl;
 
@@ -435,7 +432,7 @@ void runRdb( uint flags )
   bool exportQtSettings  = flags & KRdbExportQtSettings;
   bool exportXftSettings = flags & KRdbExportXftSettings;
 
-  KConfig _kglobals( "kdeglobals", KConfig::CascadeConfig  );
+  KConfig _kglobals( "kdeglobals", KConfig::NoGlobals  );
   KConfigGroup kglobals(KSharedConfig::openConfig( "kdeglobals" ), "KDE");
 
   KTemporaryFile tmpFile;
@@ -585,7 +582,7 @@ void runRdb( uint flags )
   /* Qt exports */
   if ( exportQtColors || exportQtSettings )
   {
-    QSettings* settings = new QSettings;
+    QSettings* settings = new QSettings(QLatin1String("Trolltech"));
 
     if ( exportQtColors )
       applyQtColors( kglobals, *settings, newPal );    // For kcmcolors
