@@ -144,34 +144,34 @@ QString KTheme::createYourself( bool pack )
         QDomElement desktopElem = m_dom.createElement( "desktop" );
         desktopElem.setAttribute( "number", i );
         desktopElem.setAttribute( "common", common );
-
-        desktopConf.changeGroup( "Desktop" + QString::number( i ) );
+        
+        KConfigGroup group(&_desktopConf,"Desktop" + QString::number( i ));
 
         QDomElement modeElem = m_dom.createElement( "mode" );
-        modeElem.setAttribute( "id", desktopConf.readEntry( "BackgroundMode", "Flat" ) );
+        modeElem.setAttribute( "id", group.readEntry( "BackgroundMode", "Flat" ) );
         desktopElem.appendChild( modeElem );
 
         QDomElement c1Elem = m_dom.createElement( "color1" );
-        c1Elem.setAttribute( "rgb", desktopConf.readEntry( "Color1",QColor() ).name() );
+        c1Elem.setAttribute( "rgb", group.readEntry( "Color1",QColor() ).name() );
         desktopElem.appendChild( c1Elem );
 
         QDomElement c2Elem = m_dom.createElement( "color2" );
-        c2Elem.setAttribute( "rgb", desktopConf.readEntry( "Color2",QColor() ).name() );
+        c2Elem.setAttribute( "rgb", group.readEntry( "Color2",QColor() ).name() );
         desktopElem.appendChild( c2Elem );
 
         QDomElement blendElem = m_dom.createElement( "blending" );
-        blendElem.setAttribute( "mode", desktopConf.readEntry( "BlendMode", QString( "NoBlending" ) ) );
-        blendElem.setAttribute( "balance", desktopConf.readEntry( "BlendBalance", QString::number( 100 ) ) );
-        blendElem.setAttribute( "reverse", desktopConf.readEntry( "ReverseBlending", false ) );
+        blendElem.setAttribute( "mode", group.readEntry( "BlendMode", QString( "NoBlending" ) ) );
+        blendElem.setAttribute( "balance", group.readEntry( "BlendBalance", QString::number( 100 ) ) );
+        blendElem.setAttribute( "reverse", group.readEntry( "ReverseBlending", false ) );
         desktopElem.appendChild( blendElem );
 
         QDomElement patElem = m_dom.createElement( "pattern" );
-        patElem.setAttribute( "name", desktopConf.readEntry( "Pattern" ) );
+        patElem.setAttribute( "name", group.readEntry( "Pattern" ) );
         desktopElem.appendChild( patElem );
 
         QDomElement wallElem = m_dom.createElement( "wallpaper" );
-        wallElem.setAttribute( "url", processFilePath( "desktop", desktopConf.readPathEntry( "Wallpaper" ) ) );
-        wallElem.setAttribute( "mode", desktopConf.readEntry( "WallpaperMode" ) );
+        wallElem.setAttribute( "url", processFilePath( "desktop", group.readPathEntry( "Wallpaper", QString() ) ) );
+        wallElem.setAttribute( "mode", group.readEntry( "WallpaperMode" ) );
         desktopElem.appendChild( wallElem );
 
         // TODO handle multi wallpapers (aka slideshow)
@@ -183,9 +183,9 @@ QString KTheme::createYourself( bool pack )
     }
 
     // 11. Screensaver
-    desktopConf.changeGroup( "ScreenSaver" );
+    KConfigGroup saver(&_desktopConf,"ScreenSaver" );
     QDomElement saverElem = m_dom.createElement( "screensaver" );
-    saverElem.setAttribute( "name", desktopConf.readEntry( "Saver" ) );
+    saverElem.setAttribute( "name", saver.readEntry( "Saver" ) );
     m_root.appendChild( saverElem );
 
     // 3. Icons
@@ -278,7 +278,7 @@ QString KTheme::createYourself( bool pack )
     KConfigGroup konqConf(&_konqConf, "Settings" );
     QDomElement konqElem = m_dom.createElement( "konqueror" );
     QDomElement konqWallElem = m_dom.createElement( "wallpaper" );
-    QString bgImagePath = konqConf.readPathEntry( "BgImage" );
+    QString bgImagePath = konqConf.readPathEntry( "BgImage", QString() );
     konqWallElem.setAttribute( "url", processFilePath( "konqueror", bgImagePath ) );
     konqElem.appendChild( konqWallElem );
     QDomElement konqBgColorElem = m_dom.createElement( "bgcolor" );
@@ -295,7 +295,7 @@ QString KTheme::createYourself( bool pack )
     if ( kickerConf.readEntry( "UseBackgroundTheme" , false) )
     {
         QDomElement backElem = m_dom.createElement( "background" );
-        QString kbgPath = kickerConf.readPathEntry( "BackgroundTheme" );
+        QString kbgPath = kickerConf.readPathEntry( "BackgroundTheme", QString() );
         backElem.setAttribute( "url", processFilePath( "panel", kbgPath ) );
         backElem.setAttribute( "colorize", kickerConf.readEntry( "ColorizeBackground" , false) );
         panelElem.appendChild( backElem );
@@ -324,10 +324,7 @@ QString KTheme::createYourself( bool pack )
 
     // 10. Widget style
     QDomElement widgetsElem = m_dom.createElement( "widgets" );
-#ifdef __GNUC__
-#warning "qt4: FIXME KStyle::defaultStyle();";
-#endif
-    widgetsElem.setAttribute( "name", globalConf->group("General").readEntry( "widgetStyle"/*,KStyle::defaultStyle()*/  ) );
+    widgetsElem.setAttribute( "name", globalConf->group("General").readEntry( "widgetStyle", KStyle::defaultStyle() ) );
     m_root.appendChild( widgetsElem );
 
     // 12.
@@ -347,8 +344,8 @@ QString KTheme::createYourself( bool pack )
         QString value;
 
         if ( group == "FMSettings" ) {
-            desktopConf.changeGroup( group );
-            value = desktopConf.readEntry( key, QString() );
+            KConfigGroup grp(&_desktopConf,group );
+            value = grp.readEntry( key, QString() );
         }
         else {
             value = globalConf->group(group).readEntry( key, QString() );
@@ -411,19 +408,19 @@ void KTheme::apply()
             desktopConf.writeEntry( "CommonDesktop", common );
             desktopConf.writeEntry( "DeskNum", desktopElem.attribute( "number", "0" ).toUInt() );
 
-            desktopConf.changeGroup( QString( "Desktop%1" ).arg( i ) );
-            desktopConf.writeEntry( "BackgroundMode", getProperty( desktopElem, "mode", "id" ) );
-            desktopConf.writeEntry( "Color1", QColor( getProperty( desktopElem, "color1", "rgb" ) ) );
-            desktopConf.writeEntry( "Color2", QColor( getProperty( desktopElem, "color2", "rgb" ) ) );
-            desktopConf.writeEntry( "BlendMode", getProperty( desktopElem, "blending", "mode" ) );
-            desktopConf.writeEntry( "BlendBalance", getProperty( desktopElem, "blending", "balance" ) );
-            desktopConf.writeEntry( "ReverseBlending",
+            KConfigGroup grp(&_desktopConf, QString( "Desktop%1" ).arg( i ) );
+            grp.writeEntry( "BackgroundMode", getProperty( desktopElem, "mode", "id" ) );
+            grp.writeEntry( "Color1", QColor( getProperty( desktopElem, "color1", "rgb" ) ) );
+            grp.writeEntry( "Color2", QColor( getProperty( desktopElem, "color2", "rgb" ) ) );
+            grp.writeEntry( "BlendMode", getProperty( desktopElem, "blending", "mode" ) );
+            grp.writeEntry( "BlendBalance", getProperty( desktopElem, "blending", "balance" ) );
+            grp.writeEntry( "ReverseBlending",
                                     static_cast<bool>( getProperty( desktopElem, "blending", "reverse" ).toUInt() ) );
-            desktopConf.writeEntry( "Pattern", getProperty( desktopElem, "pattern", "name" ) );
-            desktopConf.writeEntry( "Wallpaper",
+            grp.writeEntry( "Pattern", getProperty( desktopElem, "pattern", "name" ) );
+            grp.writeEntry( "Wallpaper",
                                     unprocessFilePath( QLatin1String("desktop"),
                                                        getProperty( desktopElem, "wallpaper", "url" ) ) );
-            desktopConf.writeEntry( "WallpaperMode", getProperty( desktopElem, "wallpaper", "mode" ) );
+            grp.writeEntry( "WallpaperMode", getProperty( desktopElem, "wallpaper", "mode" ) );
 
             if ( common )
                 break;          // stop here
@@ -435,8 +432,8 @@ void KTheme::apply()
 
     if ( !saverElem.isNull() )
     {
-        desktopConf.changeGroup( "ScreenSaver" );
-        desktopConf.writeEntry( "Saver", saverElem.attribute( "name" ) );
+        KConfigGroup grp(&_desktopConf,"ScreenSaver" );
+        grp.writeEntry( "Saver", saverElem.attribute( "name" ) );
     }
 
     desktopConf.sync();         // TODO sync and signal only if <desktop> elem present
@@ -459,15 +456,15 @@ void KTheme::apply()
             QDomElement iconSubElem = iconList.item( i ).toElement();
             QString object = iconSubElem.attribute( "object" );
             if ( object == "desktop" )
-                iconConf.changeGroup( "DesktopIcons" );
+                iconConf=KConfigGroup(KGlobal::config(), "DesktopIcons" );
             else if ( object == "mainToolbar" )
-                iconConf.changeGroup( "MainToolbarIcons" );
+                iconConf=KConfigGroup(KGlobal::config(),"MainToolbarIcons" );
             else if ( object == "panel" )
-                iconConf.changeGroup( "PanelIcons" );
+                iconConf=KConfigGroup(KGlobal::config(),"PanelIcons" );
             else if ( object == "small" )
-                iconConf.changeGroup( "SmallIcons" );
+                iconConf=KConfigGroup(KGlobal::config(),"SmallIcons" );
             else if ( object == "toolbar" )
-                iconConf.changeGroup( "ToolbarIcons" );
+                iconConf=KConfigGroup(KGlobal::config(),"ToolbarIcons" );
 
             QString iconName = iconSubElem.tagName();
             if ( iconName.contains( "Color" ) )
@@ -493,8 +490,8 @@ void KTheme::apply()
     QDomElement soundsElem = m_dom.elementsByTagName( "sounds" ).item( 0 ).toElement();
     if ( !soundsElem.isNull() )
     {
-        KConfigGroup soundGroup(KSharedConfig::openConfig("knotify.eventsrc"), QByteArray(""));
-        KConfigGroup kwinSoundGroup(KSharedConfig::openConfig("kwin.eventsrc"), QByteArray(""));
+        KConfigGroup soundGroup(KSharedConfig::openConfig("knotify.eventsrc"), "");
+        KConfigGroup kwinSoundGroup(KSharedConfig::openConfig("kwin.eventsrc"), "");
         QDomNodeList eventList = soundsElem.elementsByTagName( "event" );
         for ( int i = 0; i < eventList.count(); i++ )
         {
@@ -503,13 +500,13 @@ void KTheme::apply()
 
             if ( object == "global" )
             {
-                soundGroup.changeGroup(eventElem.attribute("name"));
-                soundGroup.writeEntry( "soundfile", unprocessFilePath( QLatin1String("sounds"), eventElem.attribute( "url" ) ) );
-                soundGroup.writeEntry( "presentation", soundGroup.readEntry( "presentation" ,0) | 1 );
+                KConfigGroup grp(KSharedConfig::openConfig("knotify.eventsrc"),eventElem.attribute("name"));
+                grp.writeEntry( "soundfile", unprocessFilePath( QLatin1String("sounds"), eventElem.attribute( "url" ) ) );
+                grp.writeEntry( "presentation", soundGroup.readEntry( "presentation" ,0) | 1 );
             }
             else if ( object == "kwin" )
             {
-                kwinSoundGroup.changeGroup(eventElem.attribute("name"));
+                kwinSoundGroup=KConfigGroup(KSharedConfig::openConfig("kwin.eventsrc"),eventElem.attribute("name"));
                 kwinSoundGroup.writeEntry( "soundfile", unprocessFilePath( QLatin1String("sounds"), eventElem.attribute( "url" ) ) );
                 kwinSoundGroup.writeEntry( "presentation", soundGroup.readEntry( "presentation",0 ) | 1 );
             }
@@ -531,7 +528,7 @@ void KTheme::apply()
     if ( !colorsElem.isNull() )
     {
         QDomNodeList colorList = colorsElem.childNodes();
-        KConfigGroup colorGroup(KGlobal::config(), QByteArray(""));
+        KConfigGroup colorGroup(KGlobal::config(), "");
 
         QString sCurrentScheme = KStandardDirs::locateLocal("data", "kdisplay/color-schemes/thememgr.kcsrc");
         KConfigGroup colorScheme(KSharedConfig::openConfig( sCurrentScheme, KConfig::SimpleConfig), "Color Scheme" );
@@ -541,9 +538,9 @@ void KTheme::apply()
             QDomElement colorElem = colorList.item( i ).toElement();
             QString object = colorElem.attribute( "object" );
             if ( object == "global" )
-                colorGroup.changeGroup("General");
+                colorGroup=KConfigGroup(KGlobal::config(),"General");
             else if ( object == "kwin" )
-                colorGroup.changeGroup("WM");
+                colorGroup=KConfigGroup(KGlobal::config(),"WM");
 
             QString colName = colorElem.tagName();
             QColor curColor = QColor( colorElem.attribute( "rgb" ) );
@@ -551,7 +548,7 @@ void KTheme::apply()
             colorScheme.writeEntry( colName, curColor ); // thememgr.kcsrc
         }
 
-        colorGroup.changeGroup("KDE");
+        colorGroup=KConfigGroup(KGlobal::config(),"KDE");
         colorGroup.writeEntry( "colorScheme", "thememgr.kcsrc", KConfig::Persistent|KConfig::Global);
         colorGroup.writeEntry( "contrast", colorsElem.attribute( "contrast", "7" ), KConfig::Persistent|KConfig::Global);
         colorScheme.writeEntry( "contrast", colorsElem.attribute( "contrast", "7" ) );
@@ -661,7 +658,7 @@ void KTheme::apply()
     QDomElement fontsElem = m_dom.elementsByTagName( "fonts" ).item( 0 ).toElement();
     if ( !fontsElem.isNull() )
     {
-        KConfigGroup fontsGroup(KGlobal::config(), QByteArray(""));
+        KConfigGroup fontsGroup(KGlobal::config(), "");
         KConfigGroup kde1xGroup(KSharedConfig::openConfig(QDir::homePath() + "/.kderc"),
                                 "General");
 
@@ -785,7 +782,7 @@ void KTheme::createSoundList( const QStringList & events, const QString & object
         KConfigGroup group(cfg, *it);
         if ( group.exists() )
         {
-            QString soundURL = group.readPathEntry( "soundfile" );
+            QString soundURL = group.readPathEntry( "soundfile", QString() );
             int pres = group.readEntry( "presentation", 0 );
             if ( !soundURL.isEmpty() && ( ( pres & 1 ) == 1 ) )
             {
@@ -881,7 +878,7 @@ void KTheme::addPreview()
 bool KTheme::copyFile( const QString & from, const QString & to )
 {
     // we overwrite b/c of restoring the "original" theme
-    return KIO::file_copy( KUrl(from), KUrl(to), -1, KIO::Overwrite /*overwrite*/ );
+    return KIO::file_copy( KUrl(from), KUrl(to), -1, KIO::HideProgressInfo | KIO::Overwrite /*overwrite*/ );
 }
 
 QString KTheme::findResource( const QString & section, const QString & path )
