@@ -136,7 +136,7 @@ void kthememanager::load()
     listThemes();
 
     // Load the current theme name
-    KConfig _conf("kcmthememanagerrc", KConfig::CascadeConfig);
+    KConfig _conf("kcmthememanagerrc", KConfig::NoGlobals);
     KConfigGroup conf(&_conf, "General" );
     QString themeName = conf.readEntry( "CurrentTheme" );
     Q3ListViewItem * cur =  dlg->lvThemes->findItem( themeName, 0 );
@@ -166,7 +166,7 @@ void kthememanager::save()
         m_theme->apply();
 
         // Save the current theme name
-        KConfig _conf("kcmthememanagerrc", KConfig::CascadeConfig);
+        KConfig _conf("kcmthememanagerrc", KConfig::NoGlobals);
         KConfigGroup conf(&_conf, "General" );
         conf.writeEntry( "CurrentTheme", themeName );
         conf.sync();
@@ -393,12 +393,12 @@ void kthememanager::queryLNFModules()
     dlg->lvDetails->sort();*/
 
     // For now use a static list
-    dlg->btnBackground->setIcon( KIcon( "background" ) );
-    dlg->btnColors->setIcon( KIcon( "colorscm" ) );
-    dlg->btnStyle->setIcon( KIcon( "style" ) );
-    dlg->btnIcons->setIcon( KIcon( "icons" ) );
-    dlg->btnFonts->setIcon( KIcon( "fonts" ) );
-    dlg->btnSaver->setIcon( KIcon( "kscreensaver" ) );
+    dlg->btnBackground->setIcon( KIcon( "preferences-desktop-wallpaper" ) );
+    dlg->btnColors->setIcon( KIcon( "preferences-desktop-color" ) );
+    dlg->btnStyle->setIcon( KIcon( "preferences-desktop-theme-style" ) );
+    dlg->btnIcons->setIcon( KIcon( "preferences-desktop-icons" ) );
+    dlg->btnFonts->setIcon( KIcon( "preferences-desktop-font" ) );
+    dlg->btnSaver->setIcon( KIcon( "preferences-desktop-wallpaper" ) );
 }
 
 void kthememanager::updatePreview( const QString & pixFile )
