@@ -19,7 +19,6 @@
 
 #include <kservicetypetrader.h>
 #include <kstandarddirs.h>
-#include <Qt3Support/Q3ButtonGroup>
 #include <QCheckBox>
 #include <Qt3Support/Q3Header>
 #include <QLabel>
@@ -56,7 +55,7 @@
 #include "scrnsave.h"
 #include <QX11Info>
 #include <QDesktopWidget>
-#include <screensaver_interface.h>
+#include <kscreensaver_interface.h>
 #include <KPluginFactory>
 #include <KPluginLoader>
 
@@ -120,24 +119,18 @@ KScreenSaver::KScreenSaver(QWidget *parent, const QVariantList&)
             this, SLOT(slotPreviewExited(K3Process *)));
 
     QBoxLayout *topLayout = new QHBoxLayout(this);
-    topLayout->setSpacing(KDialog::spacingHint());
     topLayout->setMargin(0);
+    topLayout->setSpacing(KDialog::spacingHint());
 
     // left column
     QVBoxLayout *leftColumnLayout = new QVBoxLayout( );
     topLayout->addItem( leftColumnLayout );
     leftColumnLayout->setSpacing( KDialog::spacingHint() );
-    QBoxLayout *vLayout = new QVBoxLayout();
-    leftColumnLayout->addItem( vLayout );
-    vLayout->setSpacing( KDialog::spacingHint() );
 
-    mSaverGroup = new Q3GroupBox(i18n("Screen Saver"), this );
-    mSaverGroup->setColumnLayout( 0, Qt::Horizontal );
-    vLayout->addWidget(mSaverGroup);
-    vLayout->setStretchFactor( mSaverGroup, 10 );
-    QBoxLayout *groupLayout = new QVBoxLayout();
-    mSaverGroup->layout()->addItem( groupLayout );
-    groupLayout->setSpacing( KDialog::spacingHint() );
+    mSaverGroup = new QGroupBox(i18n("Screen Saver"), this );
+    QVBoxLayout *groupLayout = new QVBoxLayout( mSaverGroup );
+    leftColumnLayout->addWidget(mSaverGroup);
+    leftColumnLayout->setStretchFactor( mSaverGroup, 10 );
 
     mSaverListView = new Q3ListView( mSaverGroup );
     mSaverListView->setMinimumHeight( 120 );
@@ -150,8 +143,7 @@ KScreenSaver::KScreenSaver(QWidget *parent, const QVariantList&)
     mSaverListView->setWhatsThis( i18n("Select the screen saver to use.") );
 
     QBoxLayout* hlay = new QHBoxLayout();
-    groupLayout->addItem(hlay);
-    hlay->setSpacing(KDialog::spacingHint());
+    groupLayout->addLayout(hlay);
     mSetupBt = new QPushButton( i18n("&Setup..."), mSaverGroup );
     connect( mSetupBt, SIGNAL( clicked() ), SLOT( slotSetup() ) );
     mSetupBt->setEnabled(false);
@@ -164,12 +156,9 @@ KScreenSaver::KScreenSaver(QWidget *parent, const QVariantList&)
     hlay->addWidget( mTestBt );
     mTestBt->setWhatsThis( i18n("Show a full screen preview of the screen saver.") );
 
-    mSettingsGroup = new Q3GroupBox( i18n("Settings"), this );
-    mSettingsGroup->setColumnLayout( 0, Qt::Vertical );
+    mSettingsGroup = new QGroupBox( i18n("Settings"), this );
+    groupLayout = new QVBoxLayout( mSettingsGroup );
     leftColumnLayout->addWidget( mSettingsGroup );
-    groupLayout = new QVBoxLayout();
-    mSettingsGroup->layout()->addItem( groupLayout );
-    groupLayout->setSpacing( KDialog::spacingHint() );
 
     mEnabledCheckBox = new QCheckBox(i18n(
         "Start a&utomatically"), mSettingsGroup);
@@ -441,6 +430,7 @@ void KScreenSaver::defaults()
     slotTimeoutChanged( 5 );
     slotLockTimeoutChanged( 60 );
     slotLock( false );
+    mEnabledCheckBox->setChecked(false);
 
     updateValues();
 
@@ -465,11 +455,8 @@ void KScreenSaver::save()
         config.writeEntry("Saver", mSaver);
     config.sync();
 
-    // TODO (GJ): When you changed anything, these two lines will give a segfault
-    // on exit. I don't know why yet.
-
-    org::freedesktop::ScreenSaver desktop("org.freedesktop.ScreenSaver", "/Screensaver", QDBusConnection::sessionBus());
-    desktop.configure();
+    org::kde::screensaver kscreensaver("org.kde.screensaver", "/ScreenSaver", QDBusConnection::sessionBus());
+    kscreensaver.configure();
 
     mChanged = false;
     emit changed(false);
