@@ -20,8 +20,6 @@
  */
 
 #include <QCheckBox>
-#include <QLayout>
-#include <QRadioButton>
 //Added by qt3to4:
 #include <QVBoxLayout>
 
@@ -53,10 +51,11 @@ SMServerConfig::SMServerConfig( QWidget *parent, const QVariantList & )
     " exit by default."));
 
     QVBoxLayout *topLayout = new QVBoxLayout(this);
+    topLayout->setMargin(0);
+    topLayout->setSpacing(KDialog::spacingHint());
     dialog = new SMServerConfigImpl(this);
     connect(dialog, SIGNAL(changed()), SLOT(changed()));
 
-    dialog->show();
     topLayout->addWidget(dialog);
     load();
 
@@ -64,7 +63,7 @@ SMServerConfig::SMServerConfig( QWidget *parent, const QVariantList & )
 
 void SMServerConfig::load()
 {
-  KConfigGroup c(KSharedConfig::openConfig("ksmserverrc", KConfig::CascadeConfig), "General");
+  KConfigGroup c(KSharedConfig::openConfig("ksmserverrc", KConfig::NoGlobals), "General");
   dialog->confirmLogoutCheck->setChecked(c.readEntry("confirmLogout", true));
   bool en = c.readEntry("offerShutdown", true);
   dialog->offerShutdownCheck->setChecked(en);
@@ -96,7 +95,7 @@ void SMServerConfig::load()
 
 void SMServerConfig::save()
 {
-  KConfig *c = new KConfig("ksmserverrc", KConfig::CascadeConfig);
+  KConfig *c = new KConfig("ksmserverrc", KConfig::NoGlobals);
   KConfigGroup group = c->group("General");
   group.writeEntry( "confirmLogout", dialog->confirmLogoutCheck->isChecked());
   group.writeEntry( "offerShutdown", dialog->offerShutdownCheck->isChecked());
