@@ -80,6 +80,7 @@ extern int loginsuccess( const char *User, const char *Host, const char *Tty, ch
 #ifdef HAVE_GETSPNAM
 # include <shadow.h>
 #endif
+#include <signal.h>
 
 /*
  * Session data, mostly what struct verify_info was for
@@ -108,6 +109,7 @@ struct login_cap *lc;
 #endif
 #ifdef USE_PAM
 static pam_handle_t *pamh;
+static int inAuth;
 #elif defined(_AIX)
 static char tty[16], hostname[100];
 #else
@@ -145,18 +147,43 @@ displayMsg( int lv, const char *msg, ... )
 }
 #endif
 
+#ifdef _AIX
+# define _ENDUSERDB , enduserdb()
+#else
+# define _ENDUSERDB
+#endif
+#ifdef HAVE_GETSPNAM /* (sic!) - not USESHADOW */
+# define _ENDSPENT , endspent()
+#else
+# define _ENDSPENT
+#endif
+#define END_ENT endpwent() _ENDSPENT _ENDUSERDB
+
+#define V_RET_NP \
+		do { \
+			END_ENT; \
+			return False; \
+		} while(0)
+
+#define V_RET \
+		do { \
+			wipeStr( curpass ); \
+			curpass = 0; \
+			V_RET_NP; \
+		} while(0)
+
 #define V_RET_AUTH \
 		do { \
 			prepareErrorGreet(); \
 			gSendInt( V_AUTH ); \
-			return 0; \
+			V_RET; \
 		} while(0)
 
 #define V_RET_FAIL(m) \
 		do { \
 			displayStr( V_MSG_ERR, m ); \
 			gSendInt( V_FAIL ); \
-			return 0; \
+			V_RET; \
 		} while(0)
 
 #ifdef USE_PAM
@@ -194,7 +221,7 @@ PAM_conv( int num_msg,
 		switch (msg[count]->msg_style) {
 		case PAM_TEXT_INFO:
 			debug( " PAM_TEXT_INFO: %s\n", msg[count]->msg );
-			displayStr( V_MSG_INFO, msg[count]->msg );
+			displayStr( inAuth ? V_MSG_INFO_AUTH : V_MSG_INFO, msg[count]->msg );
 			continue;
 		case PAM_ERROR_MSG:
 			debug( " PAM_ERROR_MSG: %s\n", msg[count]->msg );
@@ -238,7 +265,7 @@ PAM_conv( int num_msg,
 			}
 			if (!reply[count].resp) {
 				debug( "  PAM_conv aborted\n" );
-				pd->abort = TRUE;
+				pd->abort = True;
 				goto conv_err;
 			}
 			reply[count].resp_retcode = PAM_SUCCESS; /* unused in linux-pam */
@@ -251,11 +278,13 @@ PAM_conv( int num_msg,
 	for (; count >= 0; count--)
 		if (reply[count].resp)
 			switch (msg[count]->msg_style) {
-			case PAM_PROMPT_ECHO_ON:
-			case PAM_PROMPT_ECHO_OFF: /* could wipe ... */
+			case PAM_PROMPT_ECHO_OFF:
+				wipeStr( reply[count].resp );
+				break;
 #ifdef PAM_BINARY_PROMPT
-			case PAM_BINARY_PROMPT: /* ... that too ... */
+			case PAM_BINARY_PROMPT: /* Don't know length, so how wipe? */
 #endif
+			case PAM_PROMPT_ECHO_ON:
 				free( reply[count].resp );
 				break;
 			}
@@ -277,7 +306,7 @@ doPAMAuth( const char *psrv, struct pam_data *pdata )
 	struct pam_conv pconv;
 	int pretc;
 
-	pdata->abort = FALSE;
+	pdata->abort = False;
 	pconv.conv = PAM_conv;
 	pconv.appdata_ptr = (void *)pdata;
 	debug( " PAM service %s\n", psrv );
@@ -309,15 +338,17 @@ doPAMAuth( const char *psrv, struct pam_data *pdata )
 # endif
 	reInitErrorLog();
 
+	inAuth = True;
 	debug( " pam_authenticate() ...\n" );
 	pretc = pam_authenticate( pamh,
 	                          td->allowNullPasswd ? 0 : PAM_DISALLOW_NULL_AUTHTOK );
 	reInitErrorLog();
 	debug( " pam_authenticate() returned: %s\n", pam_strerror( pamh, pretc ) );
+	inAuth = False;
 	if (pdata->abort) {
 		pam_end( pamh, PAM_SUCCESS );
 		pamh = 0;
-		return 0;
+		V_RET;
 	}
 	if (!curuser) {
 		debug( " asking PAM for user ...\n" );
@@ -342,7 +373,7 @@ doPAMAuth( const char *psrv, struct pam_data *pdata )
 			V_RET_FAIL( 0 );
 		}
 	}
-	return 1;
+	return True;
 }
 
 #endif /* USE_PAM */
@@ -351,73 +382,70 @@ static int
 #if defined(USE_PAM) || defined(_AIX)
 isNoPassAllowed( const char *un )
 {
-	struct passwd *pw = 0;
+	struct passwd *pw;
 # ifdef HAVE_GETSPNAM /* (sic!) - not USESHADOW */
 	struct spwd *spw;
 # endif
 #else
-isNoPassAllowed( const char *un, struct passwd *pw )
+isNoPassAllowed( struct passwd *pw )
 {
 #endif
 	struct group *gr;
 	char **fp;
 	int hg;
 
+#if defined(USE_PAM) || defined(_AIX)
 	if (!*un)
-		return 0;
+		return False;
+#endif
 
 	if (cursource != PWSRC_MANUAL)
-		return 1;
+		return True;
 
-	for (hg = 0, fp = td->noPassUsers; *fp; fp++)
-		if (**fp == '@')
-			hg = 1;
-		else if (!strcmp( un, *fp ))
-			return 1;
-		else if (!strcmp( "*", *fp )) {
 #if defined(USE_PAM) || defined(_AIX)
-			if (!(pw = getpwnam( un )))
-				return 0;
-			if (pw->pw_passwd[0] == '!' || pw->pw_passwd[0] == '*')
-				continue;
+	/* Give nss_ldap, etc. a chance to normalize (uppercase) the name. */
+	if (!(pw = getpwnam( un )) ||
+	    pw->pw_passwd[0] == '!' || pw->pw_passwd[0] == '*')
+		return False;
 # ifdef HAVE_GETSPNAM /* (sic!) - not USESHADOW */
-			if ((spw = getspnam( un )) &&
-			    (spw->sp_pwdp[0] == '!' || spw->sp_pwdp[0] == '*'))
-					continue;
+	if ((spw = getspnam( un )) &&
+	    (spw->sp_pwdp[0] == '!' || spw->sp_pwdp[0] == '*'))
+		return False;
 # endif
 #endif
-			if (pw->pw_uid)
-				return 1;
-		}
 
-#if defined(USE_PAM) || defined(_AIX)
-	if (hg && (pw || (pw = getpwnam( un )))) {
-#else
+	for (hg = False, fp = td->noPassUsers; *fp; fp++)
+		if (**fp == '@')
+			hg = True;
+		else if (!strcmp( pw->pw_name, *fp ))
+			return True;
+		else if (!strcmp( "*", *fp ) && pw->pw_uid)
+			return True;
+
 	if (hg) {
-#endif
 		for (setgrent(); (gr = getgrent()); )
 			for (fp = td->noPassUsers; *fp; fp++)
 				if (**fp == '@' && !strcmp( gr->gr_name, *fp + 1 )) {
 					if (pw->pw_gid == gr->gr_gid) {
 						endgrent();
-						return 1;
+						return True;
 					}
 					for (; *gr->gr_mem; gr->gr_mem++)
-						if (!strcmp( un, *gr->gr_mem )) {
+						if (!strcmp( pw->pw_name, *gr->gr_mem )) {
 							endgrent();
-							return 1;
+							return True;
 						}
 				}
 		endgrent();
 	}
 
-	return 0;
+	return False;
 }
 
 #if !defined(USE_PAM) && !defined(_AIX) && defined(HAVE_SETUSERCONTEXT)
-# define LC_RET0 do { login_close(lc); return 0; } while(0)
+# define LC_RET0 do { login_close(lc); V_RET; } while(0)
 #else
-# define LC_RET0 return 0
+# define LC_RET0 V_RET
 #endif
 
 int
@@ -448,29 +476,29 @@ verify( GConvFunc gconv, int rootok )
 
 #ifdef USE_PAM
 
-	pnopass = FALSE;
+	pnopass = False;
 	if (!strcmp( curtype, "classic" )) {
 		if (!gconv( GCONV_USER, 0 ))
-			return 0;
+			return False;
 		if (isNoPassAllowed( curuser )) {
 			gconv( GCONV_PASS_ND, 0 );
 			if (!*curpass) {
-				pnopass = TRUE;
+				pnopass = True;
 				sprintf( psrvb, "%.31s-np", PAMService );
 				psrv = psrvb;
 			} else
 				psrv = PAMService;
 		} else
 			psrv = PAMService;
-		pdata.usecur = TRUE;
+		pdata.usecur = True;
 	} else {
 		sprintf( psrvb, "%.31s-%.31s", PAMService, curtype );
 		psrv = psrvb;
-		pdata.usecur = FALSE;
+		pdata.usecur = False;
 	}
 	pdata.gconv = gconv;
 	if (!doPAMAuth( psrv, &pdata ))
-		return 0;
+		return False;
 
 #elif defined(_AIX)
 
@@ -505,7 +533,7 @@ verify( GConvFunc gconv, int rootok )
 
 	if (!strcmp( curtype, "classic" )) {
 		if (!gconv( GCONV_USER, 0 ))
-			return 0;
+			return False;
 		if (isNoPassAllowed( curuser )) {
 			gconv( GCONV_PASS_ND, 0 );
 			if (!*curpass) {
@@ -514,8 +542,7 @@ verify( GConvFunc gconv, int rootok )
 			}
 		} else
 			if (!gconv( GCONV_PASS, 0 ))
-				return 0;
-		enduserdb();
+				V_RET_NP;
 		msg = NULL;
 		if ((i = authenticate( curuser, curpass, &reenter, &msg ))) {
 			debug( "authenticate() failed: %s\n", msg );
@@ -534,7 +561,7 @@ verify( GConvFunc gconv, int rootok )
 		}
 	} else if (!strcmp( curtype, "generic" )) {
 		if (!gconv( GCONV_USER, 0 ))
-			return 0;
+			return False;
 		for (curret = 0;;) {
 			msg = NULL;
 			if ((i = authenticate( curuser, curret, &reenter, &msg ))) {
@@ -552,7 +579,7 @@ verify( GConvFunc gconv, int rootok )
 			if (!reenter)
 				break;
 			if (!(curret = gconv( GCONV_HIDDEN, msg )))
-				return 0;
+				return False;
 			free( msg );
 		}
 	} else {
@@ -574,7 +601,7 @@ verify( GConvFunc gconv, int rootok )
 	}
 
 	if (!gconv( GCONV_USER, 0 ))
-		return 0;
+		return False;
 
 	if (!(p = getpwnam( curuser ))) {
 		debug( "getpwnam() failed.\n" );
@@ -608,7 +635,7 @@ verify( GConvFunc gconv, int rootok )
 		goto nplogin;
 	}
 
-	if (isNoPassAllowed( curuser, p )) {
+	if (isNoPassAllowed( p )) {
 	  nplogin:
 		gconv( GCONV_PASS_ND, 0 );
 		if (!*curpass) {
@@ -617,7 +644,7 @@ verify( GConvFunc gconv, int rootok )
 		}
 	} else
 		if (!gconv( GCONV_PASS, 0 ))
-			return 0;
+			V_RET_NP;
 
 # ifdef KERBEROS
 	if (p->pw_uid) {
@@ -678,7 +705,9 @@ verify( GConvFunc gconv, int rootok )
 	if (!p->pw_uid) {
 		if (!rootok && !td->allowRootLogin)
 			V_RET_FAIL( "Root logins are not allowed" );
-		return 1; /* don't deny root to log in */
+		wipeStr( curpass );
+		curpass = 0;
+		return True; /* don't deny root to log in */
 	}
 
 #ifdef USE_PAM
@@ -688,7 +717,7 @@ verify( GConvFunc gconv, int rootok )
 	reInitErrorLog();
 	debug( " pam_acct_mgmt() returned: %s\n", pam_strerror( pamh, pretc ) );
 	if (pretc == PAM_NEW_AUTHTOK_REQD) {
-		pdata.usecur = FALSE;
+		pdata.usecur = False;
 		pdata.gconv = conv_interact;
 		/* pam will have output a message already, so no prepareErrorGreet() */
 		if (gconv != conv_interact || pnopass) {
@@ -698,27 +727,28 @@ verify( GConvFunc gconv, int rootok )
 			/* this cannot auth the wrong user, as only classic auths get here */
 			while (!doPAMAuth( PAMService, &pdata ))
 				if (pdata.abort)
-					return 0;
+					return False;
 			gSendInt( V_PRE_OK );
 		} else
 			gSendInt( V_CHTOK );
 		for (;;) {
+			inAuth = True;
 			debug( " pam_chauthtok() ...\n" );
 			pretc = pam_chauthtok( pamh, PAM_CHANGE_EXPIRED_AUTHTOK );
 			reInitErrorLog();
 			debug( " pam_chauthtok() returned: %s\n", pam_strerror( pamh, pretc ) );
+			inAuth = False;
 			if (pdata.abort) {
 				pam_end( pamh, PAM_SUCCESS );
 				pamh = 0;
-				return 0;
+				V_RET;
 			}
 			if (pretc == PAM_SUCCESS)
 				break;
 			/* effectively there is only PAM_AUTHTOK_ERR */
 			gSendInt( V_FAIL );
 		}
-		if (curpass)
-			free( curpass );
+		wipeStr( curpass );
 		curpass = newpass;
 		newpass = 0;
 	} else if (pretc != PAM_SUCCESS) {
@@ -742,7 +772,7 @@ verify( GConvFunc gconv, int rootok )
 			free( msg );
 		}
 		gSendInt( V_AUTH );
-		return 0;
+		V_RET;
 	}
 	if (msg)
 		free( (void *)msg );
@@ -778,12 +808,12 @@ verify( GConvFunc gconv, int rootok )
 		tim = time( NULL ) / 86400L;
 
 #   ifdef HAVE_SETUSERCONTEXT
-		quietlog = login_getcapbool( lc, "hushlogin", 0 );
+		quietlog = login_getcapbool( lc, "hushlogin", False );
 		warntime = login_getcaptime( lc, "warnexpire",
 		                             DEFAULT_WARN * 86400L,
 		                             DEFAULT_WARN * 86400L ) / 86400L;
 #   else
-		quietlog = 0;
+		quietlog = False;
 #    ifdef USESHADOW
 		warntime = sp->sp_warn != -1 ? sp->sp_warn : DEFAULT_WARN;
 #    else
@@ -858,7 +888,7 @@ verify( GConvFunc gconv, int rootok )
 	if ((
 #  ifdef HAVE_SETUSERCONTEXT
 	     /* Do we ignore a nologin file? */
-	     !login_getcapbool( lc, "ignorenologin", 0 )) &&
+	     !login_getcapbool( lc, "ignorenologin", False )) &&
 	    (!stat( (nolg = login_getcapstr( lc, "nologin", "", NULL )), &st ) ||
 #  endif
 		 !stat( (nolg = _PATH_NOLOGIN), &st )))
@@ -911,7 +941,7 @@ verify( GConvFunc gconv, int rootok )
 
 /* restrict_nohome */
 # ifdef HAVE_SETUSERCONTEXT
-	if (login_getcapbool( lc, "requirehome", 0 )) {
+	if (login_getcapbool( lc, "requirehome", False )) {
 		struct stat st;
 		if (!*p->pw_dir || stat( p->pw_dir, &st ) || st.st_uid != p->pw_uid) {
 			displayStr( V_MSG_ERR, "Home folder not available" );
@@ -923,7 +953,7 @@ verify( GConvFunc gconv, int rootok )
 
 #endif /* !_AIX */
 
-	return 1;
+	return True;
 
 }
 
@@ -946,12 +976,12 @@ saveGids( void )
 {
 	num_saved_gids = getgroups( 0, 0 );
 	if (!(saved_gids = Malloc( sizeof(gid_t) * num_saved_gids )))
-		return 0;
+		return False;
 	if (getgroups( num_saved_gids, saved_gids ) < 0) {
 		logError( "saving groups failed: %m\n" );
-		return 0;
+		return False;
 	}
-	return 1;
+	return True;
 }
 
 static int
@@ -959,13 +989,13 @@ restoreGids( void )
 {
 	if (setgroups( num_saved_gids, saved_gids ) < 0) {
 		logError( "restoring groups failed: %m\n" );
-		return 0;
+		return False;
 	}
 	if (setgid( p->pw_gid ) < 0) {
 		logError( "restoring gid failed: %m\n" );
-		return 0;
+		return False;
 	}
-	return 1;
+	return True;
 }
 #endif /* USE_PAM && HAVE_INITGROUPS */
 
@@ -975,14 +1005,14 @@ resetGids( void )
 #ifdef HAVE_INITGROUPS
 	if (setgroups( 0, &p->pw_gid /* anything */ ) < 0) {
 		logError( "restoring groups failed: %m\n" );
-		return 0;
+		return False;
 	}
 #endif
 	if (setgid( 0 ) < 0) {
 		logError( "restoring gid failed: %m\n" );
-		return 0;
+		return False;
 	}
-	return 1;
+	return True;
 }
 
 static int
@@ -990,16 +1020,16 @@ setGid( const char *name, int gid )
 {
 	if (setgid( gid ) < 0) {
 		logError( "setgid(%d) (user %s) failed: %m\n", gid, name );
-		return 0;
+		return False;
 	}
 #ifdef HAVE_INITGROUPS
 	if (initgroups( name, gid ) < 0) {
 		logError( "initgroups for %s failed: %m\n", name );
 		setgid( 0 );
-		return 0;
+		return False;
 	}
 #endif	 /* QNX4 doesn't support multi-groups, no initgroups() */
-	return 1;
+	return True;
 }
 
 static int
@@ -1007,9 +1037,9 @@ setUid( const char *name, int uid )
 {
 	if (setuid( uid ) < 0) {
 		logError( "setuid(%d) (user %s) failed: %m\n", uid, name );
-		return 0;
+		return False;
 	}
-	return 1;
+	return True;
 }
 
 static int
@@ -1017,10 +1047,10 @@ setUser( const char *name, int uid, int gid )
 {
 	if (setGid( name, gid )) {
 		if (setUid( name, uid ))
-			return 1;
+			return True;
 		resetGids();
 	}
-	return 0;
+	return False;
 }
 
 #if defined(SECURE_RPC) || defined(K5AUTH)
@@ -1052,17 +1082,17 @@ mergeSessionArgs( int cansave )
 	if ((!curdmrc || newdmrc) && *dmrcDir)
 		if (strApp( &mfname, dmrcDir, "/", curuser, fname, (char *)0 ))
 			fname = mfname;
-	needsave = 0;
+	needsave = False;
 	if (!curdmrc) {
 		curdmrc = iniLoad( fname );
 		if (!curdmrc) {
 			strDup( &curdmrc, "[Desktop]\nSession=default\n" );
-			needsave = 1;
+			needsave = True;
 		}
 	}
 	if (newdmrc) {
 		curdmrc = iniMerge( curdmrc, newdmrc );
-		needsave = 1;
+		needsave = True;
 	}
 	if (needsave && cansave)
 		if (!iniSave( curdmrc, fname ) && errno == ENOENT && mfname) {
@@ -1099,11 +1129,11 @@ createClientLog( const char *log )
 			dup2( lfd, 2 );
 			close( lfd );
 			free( lname );
-			return TRUE;
+			return True;
 		}
 		if (errno != EEXIST || !macros[2].uses) {
 			free( lname );
-			return FALSE;
+			return False;
 		}
 		logInfo( "Session log file %s not usable, trying another one.\n",
 		         lname );
@@ -1166,9 +1196,6 @@ startClient( volatile int *pid )
 	char **theenv;
 	extern char **newenv; /* from libs.a, this is set up by setpenv */
 # endif
-#endif
-#ifdef HAVE_SETUSERCONTEXT
-	extern char **environ;
 #endif
 	char *failsafeArgv[2];
 	char *buf, *buf2;
@@ -1184,10 +1211,12 @@ startClient( volatile int *pid )
 		logError( "getpwnam(%s) failed.\n", curuser );
 	  pError:
 		displayStr( V_MSG_ERR, 0 );
-		return 0;
+		V_RET;
 	}
 #endif
 
+	strcpy( curuser, p->pw_name ); /* Use normalized login name. */
+
 #ifndef USE_PAM
 # ifdef _AIX
 	msg = NULL;
@@ -1200,14 +1229,14 @@ startClient( volatile int *pid )
 #  if defined(KERBEROS) && defined(AFS)
 	if (krbtkfile[0] != '\0') {
 		if (k_hasafs()) {
-			int fail = 0;
+			int fail = False;
 			if (k_setpag() == -1) {
 				logError( "setpag() for %s failed\n", curuser );
-				fail = 1;
+				fail = True;
 			}
 			if ((ret = k_afsklog( NULL, NULL )) != KSUCCESS) {
 				logError( "AFS Warning: %s\n", krb_get_err_text( ret ) );
-				fail = 1;
+				fail = True;
 			}
 			if (fail)
 				displayMsg( V_MSG_ERR,
@@ -1221,7 +1250,7 @@ startClient( volatile int *pid )
 	curuid = p->pw_uid;
 	curgid = p->pw_gid;
 
-	env = baseEnv( curuser );
+	env = baseEnv( 0, curuser );
 	xma = 0;
 	strApp( &xma, "method=", curtype, (char *)0 );
 	if (td_setup)
@@ -1240,7 +1269,7 @@ startClient( volatile int *pid )
 		env = setEnv( env, "KRBTKFILE", krbtkfile );
 #endif
 	userEnviron = inheritEnv( env, envvars );
-	env = systemEnv( curuser );
+	env = systemEnv( 0, curuser );
 	systemEnviron = setEnv( env, "HOME", p->pw_dir );
 	debug( "user environment:\n%[|''>'\n's"
 	       "system environment:\n%[|''>'\n's"
@@ -1287,7 +1316,7 @@ startClient( volatile int *pid )
 	}
 
 	if (*dmrcDir)
-		mergeSessionArgs( TRUE );
+		mergeSessionArgs( True );
 
 	debug( "now starting the session\n" );
 
@@ -1312,7 +1341,7 @@ startClient( volatile int *pid )
 	saved_env = environ;
 	environ = pam_env;
 # endif
-	removeCreds = 1; /* set it first - i don't trust PAM's rollback */
+	removeCreds = True; /* set it first - i don't trust PAM's rollback */
 	pretc = pam_setcred( pamh, 0 );
 	reInitErrorLog();
 # ifndef HAVE_PAM_GETENVLIST
@@ -1332,17 +1361,17 @@ startClient( volatile int *pid )
 		logError( "pam_setcred() for %s failed: %s\n",
 		          curuser, pam_strerror( pamh, pretc ) );
 		resetGids();
-		return 0;
+		V_RET;
 	}
 
-	removeSession = 1; /* set it first - same as above */
+	removeSession = True; /* set it first - same as above */
 	pretc = pam_open_session( pamh, 0 );
 	reInitErrorLog();
 	if (pretc != PAM_SUCCESS) {
 		logError( "pam_open_session() for %s failed: %s\n",
 		          curuser, pam_strerror( pamh, pretc ) );
 		resetGids();
-		return 0;
+		V_RET;
 	}
 
 	/* we don't want sessreg and the startup/reset scripts run with user
@@ -1354,12 +1383,8 @@ startClient( volatile int *pid )
 # define D_LOGIN_SETGROUP 0
 #endif /* USE_PAM */
 
-	removeAuth = 1;
+	removeAuth = True;
 	chownCtrl( &td->ctrl, curuid );
-	endpwent();
-#if !defined(USE_PAM) && defined(USESHADOW) && !defined(_AIX)
-	endspent();
-#endif
 	ctltalk.pipe = &ctlpipe;
 	ASPrintf( &buf, "sub-daemon for display %s", td->name );
 	ASPrintf( &buf2, "client for display %s", td->name );
@@ -1476,6 +1501,8 @@ startClient( volatile int *pid )
 		 * obtain updated user environment list. This call will initialize
 		 * global 'newenv'.
 		 */
+		userEnviron = xCopyStrArr( 1, userEnviron );
+		userEnviron[0] = (char *)"USRENVIRON:";
 		if (setpenv( curuser, PENV_INIT | PENV_ARGV | PENV_NOEXEC,
 		             userEnviron, NULL ) != 0)
 		{
@@ -1498,7 +1525,7 @@ startClient( volatile int *pid )
 			char netname[MAXNETNAMELEN+1], secretkey[HEXKEYBYTES+1];
 			int nameret, keyret;
 			int len;
-			int key_set_ok = 0;
+			int key_set_ok = False;
 			struct key_netstarg netst;
 
 			nameret = getnetname( netname );
@@ -1522,7 +1549,7 @@ startClient( volatile int *pid )
 					if (keyret == -1)
 						logError( "Failed to set NIS secret key\n" );
 					else
-						key_set_ok = 1;
+						key_set_ok = True;
 				} else {
 					/* found a key, but couldn't interpret it */
 					logError( "Password incorrect for NIS principal %s\n",
@@ -1626,10 +1653,13 @@ startClient( volatile int *pid )
 		exit( 1 );
 	case -1:
 		free( buf );
-		return 0;
+		V_RET;
 	}
 	debug( "StartSession, fork succeeded %d\n", *pid );
 	free( buf );
+	wipeStr( curpass );
+	curpass = 0;
+	END_ENT;
 
 	gSet( &ctltalk );
 	if (!Setjmp( ctltalk.errjmp ))
@@ -1643,11 +1673,11 @@ startClient( volatile int *pid )
 	gClosen( ctltalk.pipe );
 	finishGreet();
 
-	return 1;
+	return True;
 }
 
 void
-sessionExit( int status )
+clientExited( void )
 {
 	int pid;
 #ifdef USE_PAM
@@ -1698,6 +1728,7 @@ sessionExit( int status )
 			Wait4( &pid );
 			break;
 		}
+		removeAuth = False;
 	}
 
 #ifdef USE_PAM
@@ -1718,12 +1749,22 @@ sessionExit( int status )
 			logError( "pam_setcred(DELETE_CRED) failed: %s\n",
 			          pam_strerror( pamh, pretc ) );
 		resetGids();
+		removeCreds = False;
 	}
 	if (pamh) {
 		pam_end( pamh, PAM_SUCCESS );
 		reInitErrorLog();
+		pamh = 0;
 	}
 #endif
+}
+
+void
+sessionExit( int status )
+{
+	Signal( SIGTERM, SIG_IGN );
+
+	clientExited();
 
 	finishGreet();
 
@@ -1743,11 +1784,12 @@ readDmrc()
 	char *data, *fname = 0;
 	int len, pid, pfd[2], err;
 
+	endpwent();
 	if (!dmrcuser || !dmrcuser[0] || !(p = getpwnam( dmrcuser )))
 		return GE_NoUser;
 
 	if (*dmrcDir) {
-		if (!strApp( &fname, dmrcDir, "/", dmrcuser, ".dmrc", (char *)0 ))
+		if (!strApp( &fname, dmrcDir, "/", p->pw_name, ".dmrc", (char *)0 ))
 			return GE_Error;
 		if (!(curdmrc = iniLoad( fname ))) {
 			free( fname );
