@@ -74,12 +74,18 @@ SdRec sdRec = { 0, 0, 0, TO_INF, TO_INF, 0, 0, 0 };
 
 time_t now;
 
-char *prog, *progpath;
+#if KDM_LIBEXEC_STRIP != -1
+char *progpath;
+#endif
+#if KDM_LIBEXEC_STRIP
+char *progname;
+#endif
+char *prog;
 
 int
 main( int argc, char **argv )
 {
-	int oldpid, oldumask, fd, noDaemonMode;
+	int oldpid, oldumask, fd, parentPid;
 	char *pt, *errorLogFile, **opts;
 
 	/* make sure at least world write access is disabled */
@@ -96,11 +102,15 @@ main( int argc, char **argv )
 	if (fcntl( 2, F_GETFD ) < 0)
 		dup2( 0, 2 );
 
+#if KDM_LIBEXEC_STRIP == -1
+	prog = strrchr( argv[0], '/' );
+	progname = prog = prog ? prog + 1 : argv[0];
+#else
 	if (argv[0][0] == '/') {
 		if (!strDup( &progpath, argv[0] ))
 			panic( "Out of memory" );
 	} else
-#ifdef __linux__
+# ifdef __linux__
 	{
 		/* note that this will resolve symlinks ... */
 		int len;
@@ -110,10 +120,10 @@ main( int argc, char **argv )
 		if (!strNDup( &progpath, fullpath, len ))
 			panic( "Out of memory" );
 	}
-#else
-# if 0
-		panic( "Must be invoked with full path specification" );
 # else
+#  if 0
+		panic( "Must be invoked with full path specification" );
+#  else
 	{
 		char directory[PATH_MAX+1];
 		if (!getcwd( directory, sizeof(directory) ))
@@ -152,9 +162,22 @@ main( int argc, char **argv )
 				panic( "Out of memory" );
 		}
 	}
+#  endif
+# endif
+	prog = strrchr( progpath, '/' ) + 1;
+# if KDM_LIBEXEC_STRIP
+	for (progname = pt = prog, fd = 0; fd < KDM_LIBEXEC_STRIP + 1; fd++) {
+		for (;;) {
+			pt--;
+			if (pt == progpath)
+				panic( "Executable is obviously located outside BINDIR" );
+			if (*pt == '/')
+				break;
+		}
+	}
+	*pt = 0;
 # endif
 #endif
-	prog = strrchr( progpath, '/' ) + 1;
 
 #if !defined(HAVE_SETPROCTITLE) && !defined(NOXDMTITLE)
 	title = argv[0];
@@ -164,7 +187,7 @@ main( int argc, char **argv )
 	/*
 	 * Parse command line options
 	 */
-	noDaemonMode = getppid();
+	parentPid = getppid();
 	errorLogFile = 0;
 	if (!(opts = Malloc( 2 * sizeof(char *) )))
 		return 1;
@@ -199,9 +222,9 @@ main( int argc, char **argv )
 			        , prog );
 			exit( 0 );
 		} else if (!strcmp( pt, "daemon" ))
-			noDaemonMode = 0;
+			parentPid = 0;
 		else if (!strcmp( pt, "nodaemon" ))
-			noDaemonMode = 1;
+			parentPid = 1;
 		else if (argv[1] && !strcmp( pt, "config" ))
 			strDup( opts, *++argv );
 		else if (argv[1] && !strcmp( pt, "xrm" ))
@@ -226,13 +249,13 @@ main( int argc, char **argv )
 
 	initErrorLog( errorLogFile );
 
-	if (noDaemonMode != 1)
+	if (parentPid != 1)
 		becomeDaemon();
 
 	/*
 	 * Step 1 - load configuration parameters
 	 */
-	if (!initResources( opts ) || scanConfigs( FALSE ) < 0)
+	if (!initResources( opts ) || scanConfigs( False ) < 0)
 		logPanic( "Config reader failed. Aborting ...\n" );
 
 	/* SUPPRESS 560 */
@@ -255,7 +278,7 @@ main( int argc, char **argv )
 	 */
 
 #ifdef XDMCP
-	init_session_id();
+	initXdmcp();
 #else
 	debug( "not compiled for XDMCP\n" );
 #endif
@@ -310,11 +333,11 @@ TTYtoVT( const char *tty )
 int
 activateVT( int vt )
 {
-	int ret = 0;
+	int ret = False;
 	int con = open( "/dev/console", O_RDONLY );
 	if (con >= 0) {
 		if (!ioctl( con, VT_ACTIVATE, vt ))
-			ret = 1;
+			ret = True;
 		close( con );
 	}
 	return ret;
@@ -424,7 +447,7 @@ checkUtmp( void )
 #endif
 					utp->state = UtWait;
 				} else {
-					utp->hadSess = 1;
+					utp->hadSess = True;
 					utp->state = UtActive;
 				}
 				if (utp->time < ut->ut_time) /* theoretically superfluous */
@@ -491,7 +514,7 @@ switchToTTY( struct display *d )
 	utp->d = d;
 #endif
 	utp->time = now;
-	utp->hadSess = 0;
+	utp->hadSess = False;
 	utp->next = utmpList;
 	utmpList = utp;
 	checkUtmp();
@@ -511,7 +534,7 @@ stopToTTY( struct display *d )
 	if ((d->displayType & d_location) == dLocal)
 		switch (d->status) {
 		default:
-			rStopDisplay( d, DS_TEXTMODE | 0x100 );
+			rStopDisplay( d, DS_TEXTMODE | DS_SCHEDULE );
 		case reserve:
 		case textMode:
 			break;
@@ -598,10 +621,22 @@ stoppen( int force )
 		forEachDisplay( stopDisplay );
 	else
 		forEachDisplay( stopInactiveDisplay );
-	stopping = 1;
+	stopping = True;
 }
 
 
+static void
+sessionDone( struct display *d )
+{
+	d->userSess = -1;
+	if (d->userName)
+		free( d->userName );
+	d->userName = 0;
+	if (d->sessName)
+		free( d->sessName );
+	d->sessName = 0;
+}
+
 void
 setNLogin( struct display *d,
            const char *nuser, const char *npass, char *nargs, int rl )
@@ -622,7 +657,7 @@ processDPipe( struct display *d )
 	GTalk dpytalk;
 #ifdef XDMCP
 	int ct, len;
-	ARRAY8 ca, ha;
+	ARRAY8 ca, cp, ha;
 #endif
 
 	dpytalk.pipe = &d->pipe;
@@ -642,6 +677,30 @@ processDPipe( struct display *d )
 		d->userName = gRecvStr();
 		d->sessName = gRecvStr();
 		break;
+	case D_UnUser:
+		sessionDone( d );
+		if (d->sdRec.how) {
+			if (d->sdRec.force == SHUT_ASK &&
+			    (anyUserLogins( -1 ) || d->allowShutdown == SHUT_ROOT))
+			{
+				gSendInt( True );
+			} else {
+				if (!sdRec.how || sdRec.force != SHUT_FORCE ||
+				    !((d->allowNuke == SHUT_NONE && sdRec.uid != d->sdRec.uid) ||
+				      (d->allowNuke == SHUT_ROOT && d->sdRec.uid)))
+				{
+					if (sdRec.osname)
+						free( sdRec.osname );
+					sdRec = d->sdRec;
+				} else if (d->sdRec.osname)
+					free( d->sdRec.osname );
+				d->sdRec.how = 0;
+				d->sdRec.osname = 0;
+				gSendInt( False );
+			}
+		} else
+			gSendInt( False );
+		break;
 	case D_ReLogin:
 		user = gRecvStr();
 		pass = gRecvStr();
@@ -655,11 +714,14 @@ processDPipe( struct display *d )
 	case D_ChooseHost:
 		ca.data = (unsigned char *)gRecvArr( &len );
 		ca.length = (CARD16)len;
+		cp.data = (unsigned char *)gRecvArr( &len );
+		cp.length = (CARD16)len;
 		ct = gRecvInt();
 		ha.data = (unsigned char *)gRecvArr( &len );
 		ha.length = (CARD16)len;
-		registerindirectChoice( &ca, ct, &ha );
+		registerIndirectChoice( &ca, &cp, ct, &ha );
 		XdmcpDisposeARRAY8( &ha );
+		XdmcpDisposeARRAY8( &cp );
 		XdmcpDisposeARRAY8( &ca );
 		break;
 	case D_RemoteHost:
@@ -764,6 +826,11 @@ processGPipe( struct display *d )
 		gSendInt( sdRec.uid );
 		gSendStr( sdRec.osname );
 		break;
+	case G_QryDpyShutdown:
+		gSendInt( d->sdRec.how );
+		gSendInt( d->sdRec.uid );
+		gSendStr( d->sdRec.osname );
+		break;
 	case G_List:
 		listSessions( gRecvInt(), d, 0, emitXSessG, emitTTYSessG );
 		gSendInt( 0 );
@@ -809,7 +876,7 @@ scanConfigs( int force )
 static void
 markDisplay( struct display *d )
 {
-	d->stillThere = 0;
+	d->stillThere = False;
 }
 
 static void
@@ -831,8 +898,8 @@ cancelShutdown( void )
 		free( sdRec.osname );
 		sdRec.osname = 0;
 	}
-	stopping = 0;
-	rescanConfigs( TRUE );
+	stopping = False;
+	rescanConfigs( True );
 }
 
 
@@ -859,26 +926,26 @@ reapChildren( void )
 #ifdef XDMCP
 			case EX_REMOTE:
 				debug( "display exited with EX_REMOTE\n" );
-				exitDisplay( d, DS_REMOTE, 0, 0 );
+				exitDisplay( d, DS_REMOTE, 0, False );
 				break;
 #endif
 			case EX_NORMAL:
 				/* (any type of) session ended */
 				debug( "display exited with EX_NORMAL\n" );
 				if ((d->displayType & d_lifetime) == dReserve)
-					exitDisplay( d, DS_RESERVE, 0, 0 );
+					exitDisplay( d, DS_RESERVE, 0, False );
 				else
-					exitDisplay( d, DS_RESTART, XS_KEEP, TRUE );
+					exitDisplay( d, DS_RESTART, XS_KEEP, True );
 				break;
 			case EX_RESERVE:
 				debug( "display exited with EX_RESERVE\n" );
-				exitDisplay( d, DS_RESERVE, 0, 0 );
+				exitDisplay( d, DS_RESERVE, 0, False );
 				break;
 #if 0
 			case EX_REMANAGE_DPY:
 				/* user session ended */
 				debug( "display exited with EX_REMANAGE_DPY\n" );
-				exitDisplay( d, DS_RESTART, XS_KEEP, TRUE );
+				exitDisplay( d, DS_RESTART, XS_KEEP, True );
 				break;
 #endif
 			case EX_OPENFAILED_DPY:
@@ -892,7 +959,7 @@ reapChildren( void )
 				if ((d->displayType & d_origin) == dFromXDMCP)
 					sendFailed( d, "cannot open display" );
 #endif
-				exitDisplay( d, DS_RESTART, XS_RETRY, FALSE );
+				exitDisplay( d, DS_RESTART, XS_RETRY, False );
 				break;
 			case wcCompose( SIGTERM,0,0 ):
 				/* killed before/during waitForServer()
@@ -901,14 +968,14 @@ reapChildren( void )
 				   - "login now" and "suicide" pipe commands (is raiser)
 				*/
 				debug( "display exited on SIGTERM\n" );
-				exitDisplay( d, DS_RESTART, XS_RETRY, FALSE );
+				exitDisplay( d, DS_RESTART, XS_RETRY, False );
 				break;
 			case EX_AL_RESERVER_DPY:
 				/* - killed after waitForServer()
 				   - Xserver dead after remote session exit
 				*/
 				debug( "display exited with EX_AL_RESERVER_DPY\n" );
-				exitDisplay( d, DS_RESTART, XS_RESTART, FALSE );
+				exitDisplay( d, DS_RESTART, XS_RESTART, False );
 				break;
 			case EX_RESERVER_DPY:
 				/* induced by greeter:
@@ -916,18 +983,18 @@ reapChildren( void )
 				   - requested by user
 				*/
 				debug( "display exited with EX_RESERVER_DPY\n" );
-				exitDisplay( d, DS_RESTART, XS_RESTART, TRUE );
+				exitDisplay( d, DS_RESTART, XS_RESTART, True );
 				break;
 			case EX_UNMANAGE_DPY:
 				/* some fatal error */
 				debug( "display exited with EX_UNMANAGE_DPY\n" );
-				exitDisplay( d, DS_REMOVE, 0, 0 );
+				exitDisplay( d, DS_REMOVE, 0, False );
 				break;
 			default:
 				/* prolly crash */
 				logError( "Unknown session exit code %d (sig %d) from manager process\n",
 				          waitCode( status ), waitSig( status ) );
-				exitDisplay( d, DS_REMOVE, 0, 0 );
+				exitDisplay( d, DS_REMOVE, 0, False );
 				break;
 			}
 		} else if ((d = findDisplayByServerPid( pid ))) {
@@ -1016,17 +1083,15 @@ reapChildren( void )
 static int
 wouldShutdown( void )
 {
-	struct display *d;
-
-	if (sdRec.force != SHUT_CANCEL) {
-		if (sdRec.force == SHUT_FORCEMY)
-			for (d = displays; d; d = d->next)
-				if (d->status == remoteLogin ||
-				    (d->userSess >= 0 && d->userSess != sdRec.uid))
-					return 0;
-		return 1;
+	switch (sdRec.force) {
+	case SHUT_FORCE:
+		return True;
+	case SHUT_FORCEMY:
+		return !anyUserLogins( sdRec.uid );
+	case SHUT_CANCEL:
+	default:
+		return !anyUserLogins( -1 );
 	}
-	return !anyActiveDisplays();
 }
 
 fd_set wellKnownSocketsMask;
@@ -1092,7 +1157,11 @@ mainLoop( void )
 	{
 		if (!stopping)
 			startDisplays();
+#ifdef XDMCP
+		to = disposeIndirectHosts();
+#else
 		to = TO_INF;
+#endif
 		if (sdRec.how) {
 			if (sdRec.start != TO_INF && now < sdRec.start) {
 				/*if (sdRec.start < to)*/
@@ -1102,11 +1171,11 @@ mainLoop( void )
 				if (now >= sdRec.timeout) {
 					sdRec.timeout = TO_INF;
 					if (wouldShutdown())
-						stoppen( TRUE );
+						stoppen( True );
 					else
 						cancelShutdown();
 				} else {
-					stoppen( FALSE );
+					stoppen( False );
 					/*if (sdRec.timeout < to)*/
 						to = sdRec.timeout;
 				}
@@ -1156,17 +1225,17 @@ mainLoop( void )
 				case SIGTERM:
 				case SIGINT:
 					debug( "shutting down entire manager\n" );
-					stoppen( TRUE );
+					stoppen( True );
 					break;
 				case SIGHUP:
 					logInfo( "Rescanning all config files\n" );
 					forEachDisplay( markDisplay );
-					rescanConfigs( TRUE );
+					rescanConfigs( True );
 					break;
 				case SIGCHLD:
 					reapChildren();
 					if (!stopping && autoRescan)
-						rescanConfigs( FALSE );
+						rescanConfigs( False );
 					break;
 				case SIGUSR1:
 					if (startingServer &&
@@ -1261,17 +1330,17 @@ allocateVT( struct display *d )
 			for (i = tvt = 0;;) {
 				if (serverVTs[i]) {
 					tvt = atoi( serverVTs[i++] );
-					volun = 0;
+					volun = False;
 					if (tvt < 0) {
 						tvt = -tvt;
-						volun = 1;
+						volun = True;
 					}
 					if (!tvt || tvt >= 16)
 						continue;
 				} else {
 					if (++tvt >= 16)
 						break;
-					volun = 1;
+					volun = True;
 				}
 				for (cd = displays; cd; cd = cd->next) {
 					if (cd->reqSrvVT == tvt && /* protect from lusers */
@@ -1398,7 +1467,7 @@ startDisplayP2( struct display *d )
 		registerInput( d->gpipe.fd.r );
 
 		d->hstent->lock = d->hstent->rLogin = d->hstent->goodExit =
-			d->hstent->sdRec.how = 0;
+			d->sdRec.how = 0;
 		d->lastStart = now;
 		break;
 	}
@@ -1420,13 +1489,13 @@ rStopDisplay( struct display *d, int endState )
 		if (d->serverPid != -1)
 			terminateProcess( d->serverPid, d->termSignal );
 		d->status = zombie;
-		d->zstatus = endState & 0xff;
+		d->zstatus = endState & DS_MASK;
 		debug( " zombiefied\n" );
 	} else if (endState == DS_TEXTMODE) {
 #ifdef HAVE_VTS
 		d->status = textMode;
 		checkTTYMode();
-	} else if (endState == (DS_TEXTMODE | 0x100)) {
+	} else if (endState == (DS_TEXTMODE | DS_SCHEDULE)) {
 		d->status = textMode;
 #else
 		switchToTTY( d );
@@ -1461,44 +1530,19 @@ exitDisplay( struct display *d,
 
 	if (d->status == raiser) {
 		serverCmd = XS_KEEP;
-		goodExit = TRUE;
+		goodExit = True;
 	}
 
 	debug( "exitDisplay %s, "
 	       "endState = %d, serverCmd = %d, GoodExit = %d\n",
 	       d->name, endState, serverCmd, goodExit );
 
-	d->userSess = -1;
-	if (d->userName)
-		free( d->userName );
-	d->userName = 0;
-	if (d->sessName)
-		free( d->sessName );
-	d->sessName = 0;
+	sessionDone( d );
 	he = d->hstent;
 	he->lastExit = now;
 	he->goodExit = goodExit;
-	if (he->sdRec.how) {
-		if (he->sdRec.force == SHUT_ASK &&
-		    (anyActiveDisplays() || d->allowShutdown == SHUT_ROOT))
-		{
-			endState = DS_RESTART;
-		} else {
-			if (!sdRec.how || sdRec.force != SHUT_FORCE ||
-			    !((d->allowNuke == SHUT_NONE && sdRec.uid != he->sdRec.uid) ||
-			      (d->allowNuke == SHUT_ROOT && he->sdRec.uid)))
-			{
-				if (sdRec.osname)
-					free( sdRec.osname );
-				sdRec = he->sdRec;
-				if (now < sdRec.timeout || wouldShutdown())
-					endState = DS_REMOVE;
-			} else if (he->sdRec.osname)
-				free( he->sdRec.osname );
-			he->sdRec.how = 0;
-			he->sdRec.osname = 0;
-		}
-	}
+	if (sdRec.how && sdRec.start == TO_INF)
+		endState = DS_REMOVE;
 	if (d->status == zombie)
 		rStopDisplay( d, d->zstatus );
 	else {
