@@ -38,6 +38,8 @@ from the copyright holder.
 #define _DM_H_ 1
 
 #include "greet.h"
+
+#define WANT_CORE_DECLS
 #include <config.ci>
 
 #include <X11/X.h> /* FamilyInternet6 */
@@ -56,6 +58,8 @@ from the copyright holder.
 #include <stdlib.h>
 #include <errno.h>
 
+extern char **environ;
+
 #ifdef XDMCP
 # if defined(__osf__)
 /* someone somewhere defines QUERY under Tru64 which confuses Xdmcp.h */
@@ -107,9 +111,7 @@ from the copyright holder.
 # define GETUTENT getutxent
 # define PUTUTLINE pututxline
 # define ENDUTENT endutxent
-# define LASTLOG lastlogx
 # define ut_time ut_tv.tv_sec
-# define ll_time ll_tv.tv_sec
 #else
 # define STRUCTUTMP struct utmp
 # define UTMPNAME utmpname
@@ -117,7 +119,6 @@ from the copyright holder.
 # define GETUTENT getutent
 # define PUTUTLINE pututline
 # define ENDUTENT endutent
-# define LASTLOG lastlog
 #endif
 #ifndef HAVE_STRUCT_UTMP_UT_USER
 # define ut_user ut_name
@@ -187,6 +188,17 @@ typedef enum displayStatus { notRunning = 0, running, zombie, phoenix, raiser,
 typedef enum serverStatus { ignore = 0, awaiting, starting,
                             terminated, killed, pausing } ServerStatus;
 
+typedef struct {
+	unsigned how:2,    /* 0=none 1=reboot 2=halt (SHUT_*) */
+	         force:2;
+	int uid;
+	int start;
+	int timeout;
+	char *osname;
+	time_t bmstamp;
+	int osindex;
+} SdRec;
+
 typedef struct RcStr {
 	struct RcStr *next;
 	char *str;
@@ -199,8 +211,8 @@ typedef struct CfgDep {
 } CfgDep;
 
 typedef struct CfgArr {
-	char *data;     /* config value array; allocated */
-	long *idx;      /* config index array; alias */
+	void *data;     /* config value array; allocated */
+	int *idx;      /* config index array; alias */
 	CfgDep dep;     /* filestamp */
 	int numCfgEnt;  /* number of config entries */
 } CfgArr;
@@ -250,6 +262,7 @@ struct display {
 	int userSess;               /* -1=nobody, otherwise uid */
 	char *userName;
 	char *sessName;
+	SdRec sdRec;                /* user session requested shutdown */
 	CtrlRec ctrl;               /* command socket */
 	GPipe pipe;                 /* comm master <-> slave */
 	GPipe gpipe;                /* comm master <-> greeter */
@@ -262,6 +275,7 @@ struct display {
 	unsigned displayNumber;     /* numerical part of name */
 	int useChooser;             /* Run the chooser for this display */
 	ARRAY8 clientAddr;          /* for chooser picking */
+	ARRAY8 clientPort;          /* ... */
 	unsigned connectionType;    /* ... */
 	int xdmcpFd;
 #endif
@@ -296,17 +310,6 @@ struct display {
 #define dFromXDMCP      8	/* started with XDMCP */
 #define dFromFile       0	/* started via entry in servers file */
 
-typedef struct {
-	unsigned how:2,    /* 0=none 1=reboot 2=halt (SHUT_*) */
-	         force:2;
-	int uid;
-	int start;
-	int timeout;
-	char *osname;
-	time_t bmstamp;
-	int osindex;
-} SdRec;
-
 struct disphist {
 	struct disphist *next;
 	char *name;
@@ -314,7 +317,6 @@ struct disphist {
 	unsigned rLogin:2,    /* 0=nothing 1=relogin 2=login */
 	         lock:1,      /* screen locker running */
 	         goodExit:1;  /* was the last exit "peaceful"? */
-	SdRec sdRec;
 	char *nuser, *npass, *nargs;
 };
 
@@ -339,12 +341,14 @@ struct protoDisplay {
 };
 #endif /* XDMCP */
 
-/* status code for RStopDisplay */
-#define DS_RESTART  0
-#define DS_TEXTMODE 1
-#define DS_RESERVE  2
-#define DS_REMOTE   3
-#define DS_REMOVE   4
+/* status code for rStopDisplay */
+#define DS_MASK      255
+# define DS_RESTART  0
+# define DS_TEXTMODE 1
+# define DS_RESERVE  2
+# define DS_REMOTE   3
+# define DS_REMOVE   4
+#define DS_SCHEDULE  256 /* flag for DS_TEXTMODE */
 
 /* command codes dpy process -> master process */
 #define D_User       1
@@ -352,6 +356,7 @@ struct protoDisplay {
 #define D_ChooseHost 4
 #define D_RemoteHost 5
 #define D_XConnOk    6
+#define D_UnUser     7
 
 extern int debugLevel;
 
@@ -361,7 +366,13 @@ CONF_CORE_GLOBAL_DECLS
 void becomeDaemon( void );
 
 /* in dm.c */
-extern char *prog, *progpath;
+#if KDM_LIBEXEC_STRIP != -1
+extern char *progpath;
+#endif
+#if KDM_LIBEXEC_STRIP
+extern char *progname;
+#endif
+extern char *prog;
 extern time_t now;
 extern SdRec sdRec;
 void startDisplay( struct display *d );
@@ -402,7 +413,6 @@ struct display
 	*findDisplayByPid( int pid ),
 	*findDisplayByServerPid( int serverPid ),
 	*newDisplay( const char *name );
-int anyActiveDisplays( void );
 int anyRunningDisplays( void );
 int anyReserveDisplays( void );
 int idleReserveDisplays( void );
@@ -412,7 +422,7 @@ int startReserveDisplay( int lt );
 void pseudoReset( void );
 
 /* in resource.c */
-char **findCfgEnt( struct display *d, int id );
+void **findCfgEnt( struct display *d, int id );
 int initResources( char **argv );
 int loadDMResources( int force );
 int loadDisplayResources( struct display *d );
@@ -426,9 +436,9 @@ extern GTalk cnftalk;
 /* in session.c */
 extern struct display *td;
 extern const char *td_setup;
-char **baseEnv( const char *user );
+char **baseEnv( char **env, const char *user );
 char **inheritEnv( char **env, const char **what );
-char **systemEnv( const char *user );
+char **systemEnv( char **env, const char *user );
 int source( char **env, const char *file, const char *arg );
 void manageSession( struct display *d );
 
@@ -495,6 +505,7 @@ char **gRecvArgv( void );
 typedef char *(*GConvFunc)( int what, const char *prompt );
 int verify( GConvFunc gconv, int rootok );
 int startClient( volatile int *pid );
+void clientExited( void );
 void sessionExit( int status ) ATTR_NORETURN;
 int readDmrc( void );
 extern char **userEnviron, **systemEnviron;
@@ -553,6 +564,7 @@ time_t mTime( const char *fn );
 void listSessions( int flags, struct display *d, void *ctx,
                    void (*emitXSess)( struct display *, struct display *, void * ),
                    void (*emitTTYSess)( STRUCTUTMP *, struct display *, void * ) );
+int anyUserLogins( int exclude_uid );
 
 struct expando {
 	char key;
@@ -573,9 +585,9 @@ int setBootOption( const char *opt, SdRec *sdr );
 void commitBootOption( void );
 
 /* in netaddr.c */
-char *netaddrAddress( char *netaddrp, int *lenp );
-char *netaddrPort( char *netaddrp, int *lenp );
-int convertAddr( char *saddr, int *len, char **addr );
+CARD8 *netaddrAddress( char *netaddrp, int *lenp );
+CARD8 *netaddrPort( char *netaddrp, int *lenp );
+int convertAddr( char *saddr, int *len, CARD8 **addr );
 int netaddrFamily( char *netaddrp );
 int addressEqual( char *a1, int len1, char *a2, int len2 );
 
@@ -584,7 +596,7 @@ int addressEqual( char *a1, int len1, char *a2, int len2 );
 /* in xdmcp.c */
 char *networkAddressToHostname( CARD16 connectionType, ARRAY8Ptr connectionAddress );
 void sendFailed( struct display *d, const char *reason );
-void init_session_id( void );
+void initXdmcp( void );
 
 /* in policy.c */
 struct sockaddr;
@@ -613,18 +625,19 @@ typedef void (*ListenFunc)( ARRAY8Ptr addr, void **closure );
 /* in access.c */
 ARRAY8Ptr getLocalAddress( void );
 int acceptableDisplayAddress( ARRAY8Ptr clientAddress, CARD16 connectionType, xdmOpCode type );
-int forEachMatchingIndirectHost( ARRAY8Ptr clientAddress, CARD16 connectionType, ChooserFunc function, char *closure );
+int forEachMatchingIndirectHost( ARRAY8Ptr clientAddress, ARRAY8Ptr clientPort, CARD16 connectionType,
+                                 ChooserFunc function, char *closure );
 void scanAccessDatabase( int force );
 int useChooser( ARRAY8Ptr clientAddress, CARD16 connectionType );
 void forEachChooserHost( ARRAY8Ptr clientAddress, CARD16 connectionType, ChooserFunc function, char *closure );
 void forEachListenAddr( ListenFunc listenfunction, ListenFunc mcastfcuntion, void **closure );
 
 /* in choose.c */
-ARRAY8Ptr indirectChoice( ARRAY8Ptr clientAddress, CARD16 connectionType );
-int isIndirectClient( ARRAY8Ptr clientAddress, CARD16 connectionType );
-int rememberIndirectClient( ARRAY8Ptr clientAddress, CARD16 connectionType );
-void forgetIndirectClient( ARRAY8Ptr clientAddress, CARD16 connectionType );
-int registerindirectChoice( ARRAY8Ptr clientAddress, CARD16 connectionType, ARRAY8Ptr choice );
+time_t disposeIndirectHosts( void );
+ARRAY8Ptr indirectChoice( ARRAY8Ptr clientAddress, ARRAY8Ptr clientPort, CARD16 connectionType );
+int checkIndirectChoice( ARRAY8Ptr clientAddress, ARRAY8Ptr clientPort, CARD16 connectionType );
+void registerIndirectChoice( ARRAY8Ptr clientAddress, ARRAY8Ptr clientPort, CARD16 connectionType,
+                             ARRAY8Ptr choice );
 int doChoose( void );
 
 /* socket.c or streams.c */
