@@ -38,20 +38,24 @@ Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 #include <KConfigGroup>
 
 #include <QAction>
+#include <QBuffer>
 #include <QDir>
 #include <QFile>
+#include <QImageReader>
 #include <QKeyEvent>
 #include <QLabel>
 #include <QListWidget>
 #include <QListWidgetItem>
 #include <QMenu>
 #include <QMovie>
+#include <QPainter>
 #include <QPushButton>
 #include <QStyle>
 
 #include <sys/types.h>
 #include <stdlib.h>
 #include <unistd.h>
+#include <fcntl.h>
 #include <pwd.h>
 #include <grp.h>
 
@@ -63,9 +67,11 @@ class UserListView : public QListWidget {
 		: QListWidget( parent )
 		, cachedSizeHint( -1, 0 )
 	{
-		setSizePolicy( QSizePolicy::Preferred, QSizePolicy::Ignored );
+		setSizePolicy( QSizePolicy::Minimum, QSizePolicy::Ignored );
 		setUniformItemSizes( true );
+		setVerticalScrollMode( ScrollPerPixel );
 		setIconSize( QSize( 48, 48 ) );
+		setAlternatingRowColors( true );
 	}
 
 	mutable QSize cachedSizeHint;
@@ -87,7 +93,10 @@ class UserListView : public QListWidget {
 			}
 			cachedSizeHint.setWidth(
 				style()->pixelMetric( QStyle::PM_ScrollBarExtent ) +
-				frameWidth() * 2 + maxw );
+				(frameWidth() +
+				 (style()->styleHint( QStyle::SH_ScrollView_FrameOnlyAroundContents ) ?
+					style()->pixelMetric(QStyle::PM_DefaultFrameWidth) : 0)) * 2 +
+				maxw );
 			cachedSizeHint.setHeight( frameWidth() * 2 + h );
 		}
 		return cachedSizeHint;
@@ -174,6 +183,59 @@ KGreeter::~KGreeter()
 	delete stsGroup;
 }
 
+#define FILE_LIMIT_ICON 20
+#define FILE_LIMIT_IMAGE 200
+#define PIXEL_LIMIT_ICON 100
+#define PIXEL_LIMIT_IMAGE 300
+
+static bool
+loadFace( QByteArray &fn, QImage &p )
+{
+	int fd, ico;
+	if ((fd = open( fn.data(), O_RDONLY | O_NONBLOCK )) < 0) {
+		fn.chop( 5 );
+		if ((fd = open( fn.data(), O_RDONLY | O_NONBLOCK )) < 0)
+			return false;
+		ico = 0;
+	} else
+		ico = 1;
+	QFile f;
+	f.open( fd, QFile::ReadOnly );
+	int fs = f.size();
+	if (fs > (ico ? FILE_LIMIT_ICON : FILE_LIMIT_IMAGE) * 1000) {
+		logWarn( "%s exceeds file size limit (%dkB)\n",
+		         fn.data(), ico ? FILE_LIMIT_ICON : FILE_LIMIT_IMAGE );
+		return false;
+	}
+	QByteArray fc = f.read( fs );
+	::close( fd );
+	QBuffer buf( &fc );
+	buf.open( QBuffer::ReadOnly );
+	QImageReader ir( &buf );
+	QSize sz = ir.size();
+	int lim = ico ? PIXEL_LIMIT_ICON : PIXEL_LIMIT_IMAGE;
+	if (sz.width() > lim || sz.height() > lim) {
+		logWarn( "%s exceeds image dimension limit (%dx%d)\n",
+		         fn.data(), lim, lim );
+		return false;
+	}
+	sz.scale( 48, 48, Qt::KeepAspectRatio );
+	ir.setScaledSize( sz );
+	p = ir.read();
+	if (p.isNull()) {
+		logInfo( "%s is no valid image\n", fn.data() );
+		return false;
+	}
+	if (p.width() < 48) {
+		QImage np( 48, p.height(), QImage::Format_ARGB32 );
+		np.fill( 0 );
+		QPainter pnt( &np );
+		pnt.drawImage( (48 - p.width()) / 2, 0, p );
+		p = np;
+	}
+	return true;
+}
+
 void
 KGreeter::insertUser( const QImage &default_pix,
                       const QString &username, struct passwd *ps )
@@ -192,16 +254,13 @@ KGreeter::insertUser( const QImage &default_pix,
 		nd = 1;
 	QImage p;
 	do {
-		QString fn = dp ?
-		             QFile::decodeName( ps->pw_dir ) + "/.face" :
-		             _faceDir + '/' + username + ".face";
-		if (p.load( fn + ".icon" ) || p.load( fn )) {
-			QSize ns( 48, 48 );
-			if (p.size() != ns)
-				p = p.convertToFormat( QImage::Format_RGB32 ).scaled( ns, Qt::KeepAspectRatio, Qt::SmoothTransformation );
+		dp ^= 1;
+		QByteArray fn = !dp ?
+		                QByteArray( ps->pw_dir ) + '/' :
+		                QFile::encodeName( _faceDir + '/' + username );
+		fn += ".face.icon";
+		if (loadFace( fn, p ))
 			goto gotit;
-		}
-		dp = 1 - dp;
 	} while (--nd >= 0);
 	p = default_pix;
   gotit:
@@ -259,13 +318,12 @@ KGreeter::insertUsers()
 
 	QImage default_pix;
 	if (userView) {
-		if (!default_pix.load( _faceDir + "/.default.face.icon" ))
-			if (!default_pix.load( _faceDir + "/.default.face" ))
-				logError( "Cannot open default user face\n" );
-		QSize ns( 48, 48 );
-		if (default_pix.size() != ns)
-			default_pix =
-			  default_pix.convertToFormat( QImage::Format_RGB32 ).scaled( ns, Qt::KeepAspectRatio, Qt::SmoothTransformation );
+		QByteArray fn = QFile::encodeName( _faceDir + "/.default.face.icon" );
+		if (!loadFace( fn, default_pix )) {
+			logError( "Cannot open default user face\n" );
+			default_pix = QImage( 48, 48, QImage::Format_ARGB32 );
+			default_pix.fill( 0 );
+		}
 	}
 	if (_showUsers == SHOW_ALL) {
 		UserList noUsers( _noUsers );
@@ -311,6 +369,7 @@ KGreeter::insertUsers()
 		}
 	}
 	endpwent();
+	endgrent();
 	if (_sortUsers) {
 		if (userView)
 			userView->sortItems();
@@ -373,13 +432,18 @@ KGreeter::insertSessions()
 void
 KGreeter::slotUserEntered()
 {
+	struct passwd *pw;
+
 	if (userView) {
-		for (int i = 0, rc = userView->model()->rowCount(); i < rc; i++) {
-			UserListViewItem *item =
-				static_cast<UserListViewItem *>(userView->item( i ));
-			if (item->login == curUser) {
-				userView->setCurrentItem( item );
-				goto oke;
+		if ((pw = getpwnam( curUser.toLocal8Bit().data() ))) {
+			QString theUser = QString::fromLocal8Bit( pw->pw_name );
+			for (int i = 0, rc = userView->model()->rowCount(); i < rc; i++) {
+				UserListViewItem *item =
+					static_cast<UserListViewItem *>(userView->item( i ));
+				if (item->login == theUser) {
+					userView->setCurrentItem( item );
+					goto oke;
+				}
 			}
 		}
 		userView->clearSelection();
@@ -495,8 +559,9 @@ KGreeter::slotLoadPrevWM()
 					return;
 				}
 			if (!curSel)
-				msgBox( sorrybox, i18n("Your saved session type '%1' is not valid any more.\n"
-				                       "Please select a new one, otherwise 'default' will be used.", sess ) );
+				KFMsgBox::box( this, sorrybox,
+				               i18n("Your saved session type '%1' is not valid any more.\n"
+				                    "Please select a new one, otherwise 'default' will be used.", sess ) );
 			free( sess );
 			prevValid = false;
 		}
@@ -609,10 +674,8 @@ KStdGreeter::KStdGreeter()
 	int rs = layout()->spacing();
 	main_box->setSpacing( layout()->margin() );
 
-	if (userView) {
-		userView->setAlternatingRowColors( true );
+	if (userView)
 		main_box->addWidget( userView );
-	}
 
 	QBoxLayout *inner_box = new QVBoxLayout();
 	main_box->addLayout( inner_box );
@@ -781,8 +844,14 @@ KThemedGreeter::KThemedGreeter( KdmThemer *_themer )
 	connect( themer, SIGNAL(activated( const QString & )),
 	         SLOT(slotThemeActivated( const QString & )) );
 
-	console_rect = themer->findNode( "xconsole" ); // kdm ext
-	userlist_rect = themer->findNode( "userlist" );
+	KdmItem *console_node = themer->findNode( "xconsole" ); // kdm ext
+	KdmItem *console_rect = themer->findNode( "xconsole-rect" ); // kdm ext
+	if (!console_rect)
+		console_rect = console_node;
+	userlist_node = themer->findNode( "userlist" );
+	userlist_rect = themer->findNode( "userlist-rect" );
+	if (!userlist_rect)
+		userlist_rect = userlist_node;
 	caps_warning = themer->findNode( "caps-lock-warning" );
 	xauth_warning = themer->findNode( "xauth-warning" ); // kdm ext
 	pam_error = themer->findNode( "pam-error" );
@@ -797,14 +866,14 @@ KThemedGreeter::KThemedGreeter( KdmThemer *_themer )
 	if ((itm = themer->findNode( "language_button" ))) // not implemented yet
 		itm->setVisible( false );
 
+	if (console_node) {
 #ifdef WITH_KDM_XCONSOLE
-	if (console_rect) {
 		if (consoleView)
-			console_rect->setWidget( consoleView );
+			console_node->setWidget( consoleView );
 		else
+#endif
 			console_rect->setVisible( false );
 	}
-#endif
 
 	if (xauth_warning && (_authorized || !_authComplain))
 		xauth_warning->setVisible( false );
@@ -883,8 +952,8 @@ KThemedGreeter::pluginSetup()
 {
 	inherited::pluginSetup();
 
-	if (userView && verify->entitiesLocal() && verify->entityPresettable() && userlist_rect) {
-		userlist_rect->setWidget( userView );
+	if (userView && verify->entitiesLocal() && verify->entityPresettable() && userlist_node) {
+		userlist_node->setWidget( userView );
 		userlist_rect->setVisible( true );
 	} else {
 		if (userView)
@@ -894,12 +963,11 @@ KThemedGreeter::pluginSetup()
 	}
 }
 
+#if 0
 void
 KThemedGreeter::verifyFailed()
 {
 //	goButton->setEnabled( false );
-	if (userView)
-		userView->setEnabled( false );
 	inherited::verifyFailed();
 }
 
@@ -907,10 +975,9 @@ void
 KThemedGreeter::verifyRetry()
 {
 //	goButton->setEnabled( true );
-	if (userView)
-		userView->setEnabled( true );
 	inherited::verifyRetry();
 }
+#endif
 
 void
 KThemedGreeter::updateStatus( bool fail, bool caps, int timedleft )
