@@ -13,6 +13,7 @@
 #include <config-khotkeys.h> // HAVE_ARTS
 
 #include "kcmkhotkeys.h"
+#include "khotkeysiface.h"
 
 #include <unistd.h>
 #include <stdlib.h>
@@ -66,7 +67,7 @@ extern "C"
         KToolInvocation::kdeinitExec( "khotkeys" );
     else
         {
-    QDBusInterface kded("org.kde.kded", "/kded", "org.kde.kded");
+        QDBusInterface kded("org.kde.kded", "/kded", "org.kde.kded");
 		QDBusReply<bool> reply = kded.call("loadModule",QString( "khotkeys" ) );
         if( !reply.isValid())
             {
@@ -89,12 +90,16 @@ Module::Module( QWidget* parent_P, const QVariantList & )
     init_global_data( false, this ); // don't grab keys
     init_arts();
     QVBoxLayout* vbox = new QVBoxLayout( this );
-    vbox->setSpacing( 6 );
-    vbox->setMargin( 11 );
+    vbox->setSpacing( KDialog::spacingHint() );
+    vbox->setMargin( 0 );
     QSplitter* splt = new QSplitter( this );
     actions_listview_widget = new Actions_listview_widget( splt );
+    actions_listview_widget->setSizePolicy( QSizePolicy::Preferred, QSizePolicy::Expanding );
     tab_widget = new Tab_widget( splt );
+    tab_widget->setSizePolicy( QSizePolicy::Expanding, QSizePolicy::Expanding );
     vbox->addWidget( splt );
+    splt->setStretchFactor( 0, 0 );
+    splt->setStretchFactor( 1, 1 );
     buttons_widget = new Main_buttons_widget( this );
     vbox->addWidget( buttons_widget );
     connect( actions_listview_widget, SIGNAL( current_action_changed()),
@@ -111,8 +116,8 @@ Module::Module( QWidget* parent_P, const QVariantList & )
       ki18n("(c) 1999-2005 Lubos Lunak"));
     about->addAuthor(ki18n("Lubos Lunak"), ki18n("Maintainer"), "l.lunak@kde.org");
     setAboutData( about );
-
-		}
+    load();
+}
 
 Module::~Module()
     {
@@ -140,29 +145,31 @@ void Module::save()
     {
     tab_widget->save_current_action_changes();
     settings.actions = _actions_root;
+    kDebug(1217) << "Storing actions" << _actions_root;
     settings.write_settings();
+    QDBusConnection bus = QDBusConnection::sessionBus();
     if( daemon_disabled())
         {
-#ifdef __GNUC__
-#warning port to DBUS signal quit
-#endif
-
-        //kapp->dcopClient()->send( "khotkeys*", "khotkeys", "quit()", data );
+        if( bus.interface()->isServiceRegistered( "org.kde.khotkeys" ))
+            {
+            // wait for it to finish
+            org::kde::khotkeys* iface = new org::kde::khotkeys("org.kde.khotkeys", "/KHotKeys", bus, this);
+            iface->quit();
+            sleep( 1 );
+            }
         kDebug( 1217 ) << "disabling khotkeys daemon";
         }
     else
         {
-        if( !QDBusConnection::sessionBus().interface()->isServiceRegistered( "org.kde.khotkeys" ))
+        if( !bus.interface()->isServiceRegistered( "org.kde.khotkeys" ))
             {
             kDebug( 1217 ) << "launching new khotkeys daemon";
             KToolInvocation::kdeinitExec( "khotkeys" );
             }
         else
             {
-#ifdef __GNUC__
-#warning port to DBUS signal reread_configuration
-#endif
-            //kapp->dcopClient()->send( "khotkeys*", "khotkeys", "reread_configuration()", data );
+            org::kde::khotkeys* iface = new org::kde::khotkeys("org.kde.khotkeys", "/KHotKeys", bus, this);
+            iface->reread_configuration();
             kDebug( 1217 ) << "telling khotkeys daemon to reread configuration";
             }
         }
@@ -324,6 +331,7 @@ void Module::changed()
 
 void Module::init_arts()
     {
+#warning Port to Phonon
 #ifdef HAVE_ARTS
     if( haveArts())
         {
