@@ -246,7 +246,7 @@ Shortcut_trigger_widget::Shortcut_trigger_widget( QWidget* parent_P, const char*
 
 void Shortcut_trigger_widget::clear_data()
     {
-    //bt->setShortcut( KShortcut() );
+    ksw->clearKeySequence();
     }
 
 void Shortcut_trigger_widget::capturedShortcut( const QKeySequence& s_P )
@@ -266,14 +266,12 @@ void Shortcut_trigger_widget::set_data( const Shortcut_trigger* data_P )
         clear_data();
         return;
         }
-    //bt->setShortcut( data_P->shortcut() );
+    ksw->setKeySequence( data_P->shortcut().primary() );
     }
 
 Shortcut_trigger* Shortcut_trigger_widget::get_data( Action_data* data_P ) const
     {
-    /*return !bt->shortcut().isEmpty()
-        ? new Shortcut_trigger( data_P, bt->shortcut()) : NULL;*/
-    return 0;
+    return new Shortcut_trigger( data_P, KShortcut( ksw->keySequence() ) );
     }
 
 // Shortcut_trigger_dialog
