@@ -44,7 +44,7 @@ Action_data_base::Action_data_base( KConfigGroup& cfg_P, Action_data_group* pare
     if( parent())
         parent()->add_child( this );
     }
-    
+
 Action_data_base::~Action_data_base()
     {
 //    kDebug( 1217 ) << "~Action_data_base() :" << this;
@@ -72,8 +72,8 @@ Action_data_base* Action_data_base::create_cfg_read( KConfigGroup& cfg_P, Action
         {
         if( cfg_P.readEntry( "AllowMerge", false ))
             {
-            for( Action_data_group::Iterator it = parent_P->first_child();
-                 it;
+            for( Action_data_group::ConstIterator it = parent_P->first_child();
+                 it != parent_P->after_last_child();
                  ++it )
                 {
                 if( Action_data_group* existing = dynamic_cast< Action_data_group* >( *it ))
@@ -125,6 +125,12 @@ bool Action_data_base::enabled( bool ignore_group_P ) const
         return _enabled && ( parent() == NULL || parent()->enabled( false ));
     }
 
+void Action_data_base::set_conditions( Condition_list* conditions_P )
+    {
+    assert( _conditions == NULL );
+    _conditions = conditions_P;
+    }
+
 bool Action_data_base::conditions_match() const
     {
     return ( conditions() ? conditions()->match() : true )
@@ -151,8 +157,8 @@ void Action_data_group::cfg_write( KConfigGroup& cfg_P ) const
     
 void Action_data_group::update_triggers()
     {
-    for( Action_data_group::Iterator it = first_child();
-         it;
+    for( Action_data_group::ConstIterator it = first_child();
+         it != after_last_child();
          ++it )
         ( *it )->update_triggers();
     }
@@ -163,7 +169,7 @@ Action_data::Action_data( KConfigGroup& cfg_P, Action_data_group* parent_P )
     : Action_data_base( cfg_P, parent_P )
     {
     KConfigGroup triggersGroup( cfg_P.config(), cfg_P.name() + "Triggers" );
-    _triggers = new Trigger_list( cfg_P, this );
+    _triggers = new Trigger_list( triggersGroup, this );
     KConfigGroup actionsGroup( cfg_P.config(), cfg_P.name() + "Actions" );
     _actions = new Action_list( actionsGroup, this );
     }
