@@ -26,6 +26,7 @@
 #include <QObject>
 #include <QTimer>
 #include <QtDBus/QtDBus>
+#include <QLineEdit>
 
 #include <KAction>
 #include <KActionCollection>
@@ -51,62 +52,63 @@
 Display* dpy = 0;
 Colormap colormap = 0;
 Visual *visual = 0;
-bool argbVisual = false;
 
-bool checkComposite()
+void checkComposite()
 {
+    // thanks to zack rusin and frederik for pointing me in the right direction
+    // for the following bits of X11 code
     dpy = XOpenDisplay(0); // open default display
     if (!dpy)
     {
         kError() << "Cannot connect to the X server" << endl;
-        return true;
+        return;
     }
 
-    KRunnerApp::s_haveCompositeManager = KWindowSystem::compositingActive();
+    int screen = DefaultScreen(dpy);
+    int eventBase, errorBase;
 
-    if (KRunnerApp::s_haveCompositeManager)
+    if (XRenderQueryExtension(dpy, &eventBase, &errorBase))
     {
-        int screen = DefaultScreen(dpy);
-        int eventBase, errorBase;
-
-        if (XRenderQueryExtension(dpy, &eventBase, &errorBase))
+        int nvi;
+        XVisualInfo templ;
+        templ.screen  = screen;
+        templ.depth   = 32;
+        templ.c_class = TrueColor;
+        XVisualInfo *xvi = XGetVisualInfo(dpy, VisualScreenMask |
+                                                VisualDepthMask |
+                                                VisualClassMask,
+                                            &templ, &nvi);
+        for (int i = 0; i < nvi; ++i)
         {
-            int nvi;
-            XVisualInfo templ;
-            templ.screen  = screen;
-            templ.depth   = 32;
-            templ.c_class = TrueColor;
-            XVisualInfo *xvi = XGetVisualInfo(dpy, VisualScreenMask |
-                                                   VisualDepthMask |
-                                                   VisualClassMask,
-                                              &templ, &nvi);
-            for (int i = 0; i < nvi; ++i)
+            XRenderPictFormat *format = XRenderFindVisualFormat(dpy,
+                                                                xvi[i].visual);
+            if (format->type == PictTypeDirect && format->direct.alphaMask)
             {
-                XRenderPictFormat *format = XRenderFindVisualFormat(dpy,
-                                                                    xvi[i].visual);
-                if (format->type == PictTypeDirect && format->direct.alphaMask)
-                {
-                    visual = xvi[i].visual;
-                    colormap = XCreateColormap(dpy, RootWindow(dpy, screen),
-                                               visual, AllocNone);
-                    argbVisual = true;
-                    break;
-                }
+                visual = xvi[i].visual;
+                colormap = XCreateColormap(dpy, RootWindow(dpy, screen),
+                                            visual, AllocNone);
+                break;
             }
         }
 
-        KRunnerApp::s_haveCompositeManager = argbVisual;
     }
-
-    kDebug() << "KRunnerApp::s_haveCompositeManager: " << KRunnerApp::s_haveCompositeManager;
-    return true;
 }
 
-KRunnerApp::KRunnerApp()
-    : RestartingApplication(checkComposite() ? dpy : dpy, dpy ? Qt::HANDLE(visual) : 0, dpy ? Qt::HANDLE(colormap) : 0),
+KRunnerApp* KRunnerApp::self()
+{
+    if (!kapp) {
+        checkComposite();
+        return new KRunnerApp(dpy, visual ? Qt::HANDLE(visual) : 0, colormap ? Qt::HANDLE(colormap) : 0);
+    }
+
+    return qobject_cast<KRunnerApp*>(kapp);
+}
+
+KRunnerApp::KRunnerApp(Display *display, Qt::HANDLE visual, Qt::HANDLE colormap)
+    : RestartingApplication(display, visual, colormap),
       m_interface(0),
       m_tasks(0),
-      m_startupId( NULL )
+      m_startupId(NULL)
 {
     initialize();
 }
@@ -211,6 +213,7 @@ void KRunnerApp::showWindowList()
 void KRunnerApp::showTaskManager()
 {
     //kDebug(1204) << "Launching KSysGuard...";
+    KSysGuardProcessList* w = NULL;
     if (!m_tasks) {
         //TODO: move this dialog into its own KDialog subclass
         //      add an expander widget (as seen in the main
@@ -222,27 +225,48 @@ void KRunnerApp::showTaskManager()
 
         m_tasks = new KDialog(0);
         m_tasks->setWindowTitle(i18n("System Activity"));
+        m_tasks->setWindowIcon(KIcon("utilities-system-monitor"));
         connect(m_tasks, SIGNAL(finished()),
                 this, SLOT(taskDialogFinished()));
         m_tasks->setButtons(KDialog::Close);
-        QWidget* w = new KSysGuardProcessList(m_tasks);
+        w = new KSysGuardProcessList(m_tasks);
         m_tasks->setMainWidget(w);
 
         m_tasks->setInitialSize(QSize(650, 420));
         KConfigGroup cg = KGlobal::config()->group("TaskDialog");
         m_tasks->restoreDialogSize(cg);
-    }
+	w->loadSettings(cg);
+        // Since we default to forcing the window to be KeepAbove, if the user turns this off, remember this
+        bool keepAbove = cg.readEntry("KeepAbove", true);
+	if(keepAbove) {
+            KWindowSystem::setState( m_tasks->winId(), NET::KeepAbove );
+	}
+
+    } else
+        w = static_cast<KSysGuardProcessList *> (m_tasks->mainWidget());
+
 
     m_tasks->show();
     m_tasks->raise();
     KWindowSystem::setOnDesktop(m_tasks->winId(), KWindowSystem::currentDesktop());
     KWindowSystem::forceActiveWindow(m_tasks->winId());
+
+    if(w) w->filterLineEdit()->setFocus();
 }
 
 void KRunnerApp::taskDialogFinished()
 {
     KConfigGroup cg = KGlobal::config()->group("TaskDialog");
     m_tasks->saveDialogSize(cg);
+    KSysGuardProcessList *w = static_cast<KSysGuardProcessList *> (m_tasks->mainWidget());
+    if(w) w->saveSettings(cg);
+    // Since we default to forcing the window to be KeepAbove, if the user turns this off, remember this
+    bool keepAbove = KWindowSystem::windowInfo(m_tasks->winId(), NET::WMState).hasState(NET::KeepAbove);
+    if(keepAbove)
+        cg.revertToDefault("KeepAbove");
+    else
+        cg.writeEntry("KeepAbove", false);
+
     KGlobal::config()->sync();
     m_tasks->deleteLater();
     m_tasks = 0;
@@ -297,4 +321,9 @@ int KRunnerApp::newInstance()
     //return 0;
 }
 
+bool KRunnerApp::hasCompositeManager() const
+{
+    return colormap && KWindowSystem::compositingActive();
+}
+
 #include "krunnerapp.moc"
