@@ -322,9 +322,7 @@ void LockProcess::configure()
     }
 
 #ifdef HAVE_DPMS
-    //if the user  decided that the screensaver should run independent from
-    //dpms, we shouldn't check for it, aleXXX
-    mDPMSDepend = KScreenSaverSettings::dpmsDependent();
+    mDPMSDepend = KScreenSaverSettings::suspendWhenInvisible();
 #endif
 
     mPriority = KScreenSaverSettings::priority();
@@ -396,7 +394,7 @@ void LockProcess::readSaver()
 
         if (config.hasActionGroup("Root"))
         {
-            mSaverExec = config.actionGroup("Root").readPathEntry("Exec");
+            mSaverExec = config.actionGroup("Root").readPathEntry("Exec", QString());
         }
     }
 }
@@ -738,7 +736,7 @@ QVariant LockProcess::getConf(void *ctx, const char *key, const QVariant &dflt)
 
 void LockProcess::cantLock( const QString &txt)
 {
-    msgBox( QMessageBox::Critical, i18n("Will not lock the session, as unlocking would be impossible:\n") + txt );
+    msgBox( 0, QMessageBox::Critical, i18n("Will not lock the session, as unlocking would be impossible:\n") + txt );
 }
 
 #if 0 // placeholders for later
@@ -947,6 +945,12 @@ int LockProcess::execDialog( QDialog *dlg )
 {
     dlg->adjustSize();
 
+    QFrame *winFrame = new QFrame( dlg );
+    winFrame->setFrameStyle( QFrame::WinPanel | QFrame::Raised );
+    winFrame->setLineWidth( 2 );
+    winFrame->resize( dlg->size() );
+    winFrame->lower();
+
     QRect rect = dlg->geometry();
     rect.moveCenter(KGlobalSettings::desktopGeometry(QCursor::pos()).center());
     dlg->move( rect.topLeft() );
@@ -1149,24 +1153,20 @@ void LockProcess::unlockXF86()
 }
 #endif
 
-void LockProcess::msgBox( QMessageBox::Icon type, const QString &txt )
+void LockProcess::msgBox( QWidget *parent, QMessageBox::Icon type, const QString &txt )
 {
-    QDialog box( 0, Qt::X11BypassWindowManagerHint );
+    QDialog box( parent, Qt::X11BypassWindowManagerHint );
     box.setModal( true );
-    QFrame *winFrame = new QFrame( &box );
-    winFrame->setFrameStyle( QFrame::WinPanel | QFrame::Raised );
-    winFrame->setLineWidth( 2 );
-    QLabel *label1 = new QLabel( winFrame );
+
+    QLabel *label1 = new QLabel( &box );
     label1->setPixmap( QMessageBox::standardIcon( type ) );
-    QLabel *label2 = new QLabel( txt, winFrame );
-    KPushButton *button = new KPushButton( KStandardGuiItem::ok(), winFrame );
+    QLabel *label2 = new QLabel( txt, &box );
+    KPushButton *button = new KPushButton( KStandardGuiItem::ok(), &box );
     button->setDefault( true );
     button->setSizePolicy( QSizePolicy( QSizePolicy::Preferred, QSizePolicy::Preferred ) );
     connect( button, SIGNAL( clicked() ), &box, SLOT( accept() ) );
 
-    QVBoxLayout *vbox = new QVBoxLayout( &box );
-    vbox->addWidget( winFrame );
-    QGridLayout *grid = new QGridLayout( winFrame );
+    QGridLayout *grid = new QGridLayout( &box );
     grid->setSpacing( 10 );
     grid->addWidget( label1, 0, 0, Qt::AlignCenter );
     grid->addWidget( label2, 0, 1, Qt::AlignCenter );
