@@ -41,80 +41,56 @@ QString formattedName( KService::Ptr service )
     return name;
 }
 
-ServiceAction::ServiceAction(KService::Ptr service, QObject* parent)
-    : QAction(KIcon(service->icon()), formattedName(service), parent),
-      m_service(service)
-{
-}
-
 ServiceRunner::ServiceRunner( QObject* parent )
     : Plasma::AbstractRunner( parent )
 {
-    setObjectName( i18n( "Application" ) );
+    setObjectName(i18n("Application"));
 }
 
 ServiceRunner::~ServiceRunner()
 {
 }
 
-QAction* ServiceRunner::accepts(const QString& term)
+void ServiceRunner::match(Plasma::SearchContext *search)
 {
-    KService::Ptr service = KService::serviceByName(term);
-    QAction* action = 0;
-
-    if ( service && !service->exec().isEmpty() ) {
-        action = new ServiceAction(service, this);
-    }
-
-    return action;
-}
-
-void ServiceRunner::fillMatches( KActionCollection* matches,
-                                 const QString& term,
-                                 int max, int offset )
-{
-    Q_UNUSED( max )
-    Q_UNUSED( offset )
-
-    QString query = QString("exist Exec and '%1' ~in Keywords and Name != '%2'").arg(term, term);
-
-    const KService::List services = KServiceTypeTrader::self()->query( "Application", query );
-
-    //kDebug() << "got " << services.count() << " services from " << query;
-
-    foreach ( const KService::Ptr service, services ) {
-        ServiceAction* action = new ServiceAction(service, matches);
-        matches->addAction(service->name(), action);
-        connect(action, SIGNAL(triggered()), SLOT(launchService()));
-    }
-}
-
-bool ServiceRunner::exec(QAction* action, const QString& term)
-{
-    KService::Ptr service;
-    ServiceAction* serviceAction = qobject_cast<ServiceAction*>(action);
-
-    if (serviceAction) {
-        service = serviceAction->m_service;
-    }
-
-    if (!service && !term.isEmpty()) {
-        service = KService::serviceByName(term);
-    }
-
-    return service &&
-           KRun::run(*service, KUrl::List(), 0) != 0;
-}
-
-void ServiceRunner::launchService()
-{
-    ServiceAction* action = qobject_cast<ServiceAction*>(sender());
-
-    if (!action) {
+    QString term = search->searchTerm();
+    if (term.length() <  3) {
         return;
     }
 
-    KRun::run(*action->m_service, KUrl::List(), 0);
+    KService::Ptr service = KService::serviceByName(term);
+
+    if (service && !service->exec().isEmpty()) {
+        setupAction(service, search->addExactMatch(this));
+    }
+
+    QString query = QString("exist Exec and ('%1' ~in Keywords or '%2' ~~ GenericName or '%3' ~~ Name) and Name != '%4'").arg(term, term, term, term);
+    const KService::List services = KServiceTypeTrader::self()->query("Application", query);
+
+    //kDebug() << "got " << services.count() << " services from " << query;
+
+    foreach (const KService::Ptr service, services) {
+        setupAction(service, search->addPossibleMatch(this));
+    }
+}
+
+void ServiceRunner::exec(Plasma::SearchMatch* action)
+{
+    KService::Ptr service = KService::serviceByStorageId(action->data().toString());
+    if (service) {
+        KRun::run(*service, KUrl::List(), 0);
+    }
+}
+
+void ServiceRunner::setupAction(const KService::Ptr &service, Plasma::SearchMatch *action)
+{
+    action->setText(service->name());
+    action->setData(service->storageId());
+
+    if (!service->icon().isEmpty()) {
+        action->setIcon(KIcon(service->icon()));
+    }
 }
 
 #include "servicerunner.moc"
+
