@@ -57,21 +57,16 @@ CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 #endif
 
 #include <QFile>
-#include <QTextStream>
-#include <QDataStream>
 #include <QPushButton>
-#include <QMessageBox>
-#include <QTimer>
-#include <QDesktopWidget>
 #include <QRegExp>
 #include <QtDBus/QtDBus>
+#include <QSocketNotifier>
 
 #include <klocale.h>
 #include <kglobal.h>
 #include <kconfig.h>
 #include <kstandarddirs.h>
 #include <kapplication.h>
-#include <k3staticdeleter.h>
 #include <ktemporaryfile.h>
 #include <kconfiggroup.h>
 #include <kprocess.h>
@@ -88,7 +83,6 @@ CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 
 #include <dmctl.h>
 #include <QX11Info>
-#include <QApplication>
 #include <krandom.h>
 #include "klauncher_interface.h"
 
@@ -497,6 +491,7 @@ static int Xio_ErrorHandler( Display * )
     }
 
     exit(0); // Don't report error, it's not our fault.
+    return 0; // Bogus return value, notreached
 }
 
 void KSMServer::setupXIOErrorHandler()
@@ -585,6 +580,7 @@ extern "C" int _IceTransNoListen(const char * protocol);
 
 KSMServer::KSMServer( const QString& windowManager, bool _only_local )
   : sessionGroup( "" )
+  , logoutEffectWidget( NULL )
 {
     new KSMServerInterfaceAdaptor( this );
     QDBusConnection::sessionBus().registerObject("/KSMServer", this);
@@ -825,7 +821,7 @@ void KSMServer::discardSession()
         // case up to KDE and Qt < 3.1
         int i = 1;
         while ( i <= count &&
-                config.readPathListEntry( QString("discardCommand") + QString::number(i) ) != discardCommand )
+                config.readPathEntry( QString("discardCommand") + QString::number(i), QStringList() ) != discardCommand )
             i++;
         if ( i <= count )
             executeCommand( discardCommand );
@@ -841,7 +837,7 @@ void KSMServer::storeSession()
     KConfigGroup configSessionGroup(config, sessionGroup);
     int count =  configSessionGroup.readEntry( "count", 0 );
     for ( int i = 1; i <= count; i++ ) {
-        QStringList discardCommand = configSessionGroup.readPathListEntry( QLatin1String("discardCommand") + QString::number(i) );
+        QStringList discardCommand = configSessionGroup.readPathEntry( QLatin1String("discardCommand") + QString::number(i), QStringList() );
         if ( discardCommand.isEmpty())
             continue;
         // check that non of the new clients uses the exactly same
@@ -910,13 +906,17 @@ QStringList KSMServer::sessionList()
 }
 
 bool KSMServer::isWM( const KSMClient* client ) const
+{
+    return isWM( client->program());
+}
+
+bool KSMServer::isWM( const QString& command ) const
 {
     // KWin relies on ksmserver's special treatment in phase1,
     // therefore make sure it's recognized even if ksmserver
     // was initially started with different WM, and kwin replaced
     // it later
-    return client->program() == wm
-        || client->program() == "kwin";
+    return command == wm || command == "kwin";
 }
 
 bool KSMServer::defaultSession() const
