@@ -54,13 +54,8 @@ CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 #include <limits.h>
 #endif
 
-#include <QFile>
-#include <QTextStream>
-#include <QDataStream>
 #include <QPushButton>
-#include <QMessageBox>
 #include <QTimer>
-#include <QDesktopWidget>
 #include <QtDBus/QtDBus>
 
 #include <klocale.h>
@@ -68,7 +63,6 @@ CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 #include <kconfig.h>
 #include <kstandarddirs.h>
 #include <kapplication.h>
-#include <k3staticdeleter.h>
 #include <ktemporaryfile.h>
 #include <knotification.h>
 #include <kconfiggroup.h>
@@ -79,7 +73,6 @@ CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 #include <kdebug.h>
 
 #include <QX11Info>
-#include <QApplication>
 
 //#include "kdesktop_interface.h"
 #include "klauncher_interface.h"
@@ -180,7 +173,6 @@ void KSMServer::autoStart0Done()
         return;
     kDebug( 1218 ) << "Autostart 0 done";
     upAndRunning( "desktop" );
-    upAndRunning( "ready" ); // desktop is more or less ready at this point, tell ksplash to go away
     kcminitSignals = new QDBusInterface("org.kde.kcminit", "/kcminit", "org.kde.KCMInit", QDBusConnection::sessionBus(), this );
     if( !kcminitSignals->isValid())
         kWarning() << "kcminit not running?" ;
@@ -342,6 +334,8 @@ void KSMServer::finishStartup()
     if( waitAutoStart2 || waitKcmInit2 )
         return;
 
+    upAndRunning( "ready" );
+
     state = Idle;
     setupXIOErrorHandler(); // From now on handle X errors as normal shutdown.
 }
