@@ -1,12 +1,34 @@
+/********************************************************************
+
+Copyright (C) 2007 Lubos Lunak <l.lunak@kde.org>
+
+Please see file LICENSE for the licensing terms of ksplashx as a whole.
+
+This program is free software; you can redistribute it and/or modify
+it under the terms of the GNU General Public License as published by
+the Free Software Foundation; either version 2 of the License, or
+(at your option) any later version.
+
+This program is distributed in the hope that it will be useful,
+but WITHOUT ANY WARRANTY; without even the implied warranty of
+MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+GNU General Public License for more details.
+
+You should have received a copy of the GNU General Public License
+along with this program.  If not, see <http://www.gnu.org/licenses/>.
+*********************************************************************/
+
 //#define DEBUG
 
 const int MAX_ITEMS = 100;
 const int ANIM_IMAGES_ROW = 10;
 
-// for convering from startup states to (internal) numbers
+// for converting from startup states to (internal) numbers
+// these are also in the simple splash and in krunner
 const char states[][ 12 ] =
-    { "initial", "kded", "confupdate", "kcminit", "ksmserver", "wm", "desktop", "ready" };
-const int LAST_STATE = 7;
+    { "initial", "kded", "confupdate", "kcminit", "ksmserver", "wm", "desktop" };
+// State "ready" isn't used, make splash go away as soon as desktop is ready.
+const int LAST_STATE = 6;
 
 #include <config-workspace.h>
 
@@ -23,6 +45,8 @@ const int LAST_STATE = 7;
 #include <assert.h>
 #include <dirent.h>
 
+#include <X11/Xutil.h>
+
 # ifdef HAVE_SYS_TIME_H
 #  include <sys/time.h>
 # endif
@@ -448,6 +472,10 @@ static void createWindow()
         geometry.x(), geometry.y(), geometry.width(), geometry.height(),
         0, CopyFromParent, CopyFromParent, CopyFromParent, CWOverrideRedirect | CWBackPixmap, &attrs );
     XSelectInput( qt_xdisplay(), window, ButtonPressMask | ExposureMask );
+    XClassHint class_hint;
+    class_hint.res_name = const_cast< char* >( "ksplashx" );
+    class_hint.res_class = const_cast< char* >( "ksplashx" );
+    XSetWMProperties( qt_xdisplay(), window, NULL, NULL, NULL, NULL, NULL, NULL, &class_hint );
     XMapRaised( qt_xdisplay(), window );
     }
 
@@ -471,7 +499,7 @@ static bool waitState( int expected_state )
         fprintf( stderr, "No window contents\n" );
         exit( 3 );
         }
-    time_t test_time = time( NULL ) + 5;
+    time_t test_time = time( NULL ) + 2;
 #ifdef DEBUG
     fprintf( stderr,"AWATING STATE: %d (%s)\n", expected_state, states[ expected_state ] );
 #endif
@@ -534,7 +562,7 @@ static bool waitState( int expected_state )
         if( test && time( NULL ) >= test_time )
             {
             ++state;
-            test_time = time( NULL ) + 5;
+            test_time = time( NULL ) + 2;
             }
         if( expected_state <= state )
             return false;
