@@ -4,8 +4,9 @@
     Copyright (c) 1999 - 2002 Chris Schlaeger <cs@kde.org>
 
     This program is free software; you can redistribute it and/or
-    modify it under the terms of version 2 of the GNU General Public
-    License as published by the Free Software Foundation.
+    modify it under the terms of the GNU General Public
+    License version 2 or at your option version 3 as published by
+    the Free Software Foundation.
 
     This program is distributed in the hope that it will be useful,
     but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -21,6 +22,8 @@
 #include <QtXml/qdom.h>
 #include <QtGui/QImage>
 #include <QtGui/QToolTip>
+#include <QtGui/QHBoxLayout>
+
 
 #include <kdebug.h>
 #include <klocale.h>
@@ -43,7 +46,9 @@ FancyPlotter::FancyPlotter( QWidget* parent,
   mSettingsDialog = 0;
   mSensorReportedMax = 0;
   mSensorReportedMin = 0;
+  QLayout *layout = new QHBoxLayout(this);
   mPlotter = new KSignalPlotter( this );
+  layout->addWidget(mPlotter);
   mPlotter->setVerticalLinesColor(KSGRD::Style->firstForegroundColor());
   mPlotter->setHorizontalLinesColor(KSGRD::Style->secondForegroundColor());
   mPlotter->setBackgroundColor(KSGRD::Style->backgroundColor());
@@ -55,16 +60,13 @@ FancyPlotter::FancyPlotter( QWidget* parent,
 
   mPlotter->setThinFrame(!(workSheetSettings && workSheetSettings->isApplet));
  
-  setTitle(title);
-
   /* All RMB clicks to the mPlotter widget will be handled by 
    * SensorDisplay::eventFilter. */
   mPlotter->installEventFilter( this );
 
   setPlotterWidget( mPlotter );
+  mPlotter->setTranslatedTitle( translatedTitle() );
 
-  //Force a resize so that mPlotter's size gets set
-  FancyPlotter::resizeEvent( 0 );
 }
 
 FancyPlotter::~FancyPlotter()
@@ -72,9 +74,11 @@ FancyPlotter::~FancyPlotter()
 }
 
 void FancyPlotter::setTitle( const QString &title ) { //virtual
-  if(mPlotter)
-    mPlotter->setTitle( title );
   KSGRD::SensorDisplay::setTitle( title );
+  if(mPlotter) {
+    mPlotter->setTranslatedTitle( translatedTitle() );
+    mPlotter->setTranslatedUnit( KSGRD::SensorMgr->translateUnit( mUnit ) );
+  }
 }
 
 bool FancyPlotter::eventFilter( QObject* object, QEvent* event ) {	//virtual
@@ -125,7 +129,7 @@ void FancyPlotter::configureSettings()
     entry.setSensorName( sensors().at( i )->name() );
     entry.setUnit( KSGRD::SensorMgr->translateUnit( sensors().at( i )->unit() ) );
     entry.setStatus( sensors().at( i )->isOk() ? i18n( "OK" ) : i18n( "Error" ) );
-    entry.setColor( mPlotter->beamColors()[ i ] );
+    entry.setColor( mPlotter->beamColor( i ) );
 
     list.append( entry );
   }
@@ -157,9 +161,6 @@ void FancyPlotter::applySettings() {
 
     if ( mPlotter->horizontalScale() != mSettingsDialog->horizontalScale() ) {
       mPlotter->setHorizontalScale( mSettingsDialog->horizontalScale() );
-      // Can someone think of a useful QResizeEvent to pass?
-      // It doesn't really matter anyway because it's not used.
-      emit resizeEvent( 0 );
     }
 
     mPlotter->setStackBeams( mSettingsDialog->stackBeams());
@@ -197,7 +198,7 @@ void FancyPlotter::applySettings() {
     SensorModelEntry::List list = mSettingsDialog->sensors();
 
     for ( int i = 0; i < sensors().count(); ++i ) {
-          mPlotter->beamColors()[ i ] = list[ i ].color();
+          mPlotter->setBeamColor( i, list[ i ].color() );
     }
 
     mPlotter->update();
@@ -211,9 +212,9 @@ void FancyPlotter::applyStyle()
   QFont font = mPlotter->font();
   font.setPointSize(KSGRD::Style->fontSize() );
   mPlotter->setFont( font );
-  for ( int i = 0; i < mPlotter->beamColors().count() &&
+  for ( int i = 0; i < mPlotter->numBeams() &&
         (unsigned int)i < KSGRD::Style->numSensorColors(); ++i )
-    mPlotter->beamColors()[ i ] = KSGRD::Style->sensorColor( i );
+    mPlotter->setBeamColor( i, KSGRD::Style->sensorColor( i ) );
 
   mPlotter->update();
 }
@@ -290,15 +291,15 @@ void FancyPlotter::setTooltip()
 
     if(sensors().at( i)->isLocalhost()) {
       tooltip += QString( "%1%2 %3 (%4)" ).arg( i != 0 ? "<br>" : "<qt>")
-            .arg("<font color=\"" + mPlotter->beamColors()[ i ].name() + "\">"+indicatorSymbol+"</font>")
-            .arg( description )
+            .arg("<font color=\"" + mPlotter->beamColor( i ).name() + "\">"+indicatorSymbol+"</font>")
+            .arg( i18n(description.toLatin1()) )
 	    .arg( lastValue );
 
     } else {
       tooltip += QString( "%1%2 %3:%4 (%5)" ).arg( i != 0 ? "<br>" : "<qt>" )
-                 .arg("<font color=\"" + mPlotter->beamColors()[ i ].name() + "\">"+indicatorSymbol+"</font>")
+                 .arg("<font color=\"" + mPlotter->beamColor( i ).name() + "\">"+indicatorSymbol+"</font>")
                  .arg( sensors().at( i )->hostName() )
-                 .arg( description )
+                 .arg( i18n(description.toLatin1()) )
 	         .arg( lastValue );
     }
   }
@@ -306,16 +307,6 @@ void FancyPlotter::setTooltip()
   mPlotter->setToolTip( tooltip );
 }
 
-void FancyPlotter::resizeEvent( QResizeEvent* )
-{
-  mPlotter->setGeometry( 0, 0, width(), height() );
-}
-
-QSize FancyPlotter::sizeHint() const
-{
-  return mPlotter->sizeHint();
-}
-
 void FancyPlotter::timerTick( ) //virtual
 {
   if(!mSampleBuf.isEmpty() && mBeams != 0) {
@@ -346,14 +337,14 @@ void FancyPlotter::answerReceived( int id, const QList<QByteArray> &answerlist )
     sensorError( id, false );
   } else if ( id >= 100 ) {
     KSGRD::SensorFloatInfo info( answer );
-    QString unit = info.unit();
-    if(unit.toUpper() == "KB" || unit.toUpper() == "KiB") {
+    mUnit = info.unit();
+    if(mUnit.toUpper() == "KB" || mUnit.toUpper() == "KiB") {
       if(info.max() >= 1024*1024*0.7) {  //If it's over 0.7GiB, then set the scale to gigabytes
         mPlotter->setScaleDownBy(1024*1024);
-	unit = "GiB";
+	mUnit = "GiB";
       } else if(info.max() > 1024) {
         mPlotter->setScaleDownBy(1024);
-	unit = "MiB";
+	mUnit = "MiB";
       }
     }
 
@@ -371,7 +362,7 @@ void FancyPlotter::answerReceived( int id, const QList<QByteArray> &answerlist )
     }
     sensors().at( id - 100 )->setUnit( info.unit() );
 
-    mPlotter->setTranslatedUnit( KSGRD::SensorMgr->translateUnit( unit ) );
+    mPlotter->setTranslatedUnit( KSGRD::SensorMgr->translateUnit( mUnit ) );
     sensors().at( id - 100 )->setDescription( info.name() );
   }
 }
@@ -467,7 +458,7 @@ bool FancyPlotter::saveSettings( QDomDocument &doc, QDomElement &element)
     beam.setAttribute( "hostName", sensors().at( i )->hostName() );
     beam.setAttribute( "sensorName", sensors().at( i )->name() );
     beam.setAttribute( "sensorType", sensors().at( i )->type() );
-    saveColor( beam, "color", mPlotter->beamColors()[ i ] );
+    saveColor( beam, "color", mPlotter->beamColor( i ) );
   }
 
   SensorDisplay::saveSettings( doc, element );
