@@ -4,8 +4,9 @@
     Copyright (c) 1999 - 2002 Chris Schlaeger <cs@kde.org>
 
     This program is free software; you can redistribute it and/or
-    modify it under the terms of version 2 of the GNU General Public
-    License as published by the Free Software Foundation.
+    modify it under the terms of the GNU General Public
+    License version 2 or at your option version 3 as published by
+    the Free Software Foundation.
 
     This program is distributed in the hope that it will be useful,
     but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -26,7 +27,6 @@
 #include <QBitmap>
 #include <QPixmap>
 #include <QEvent>
-#include <QResizeEvent>
 #include <QMouseEvent>
 #include <QCustomEvent>
 
@@ -107,11 +107,8 @@ void SensorDisplay::timerTick()
  }
 }
 
-bool SensorDisplay::eventFilter( QObject *object, QEvent *event )
+void SensorDisplay::showContextMenu(const QPoint &pos)
 {
-  if ( event->type() == QEvent::MouseButtonPress &&
-     ( (QMouseEvent*)event)->button() == Qt::RightButton ) {
-
     QMenu pm;
     QAction *action = 0;
     bool menuEmpty = true;
@@ -133,8 +130,8 @@ bool SensorDisplay::eventFilter( QObject *object, QEvent *event )
       menuEmpty = false;
     }
 
-    if(menuEmpty) return true;
-    action = pm.exec( QCursor::pos() );
+    if(menuEmpty) return;
+    action = pm.exec( mapToGlobal(pos) );
     if ( action ) {
       switch ( action->data().toInt() ) {
         case 1:
@@ -152,8 +149,16 @@ bool SensorDisplay::eventFilter( QObject *object, QEvent *event )
           break;
       }
     }
+}
 
-    return true;
+bool SensorDisplay::eventFilter( QObject *object, QEvent *event )
+{
+  if ( event->type() == QEvent::MouseButtonPress) {
+    QMouseEvent *e = static_cast<QMouseEvent *> (event);
+    if( e->button() == Qt::RightButton ) {
+      showContextMenu( e->pos() );
+      return true;
+    }
   } 
 
   return QWidget::eventFilter( object, event );
@@ -322,10 +327,23 @@ void SensorDisplay::setSensorOk( bool ok )
   }
 }
 
+void SensorDisplay::changeEvent( QEvent * event ) {
+  if (event->type() == QEvent::LanguageChange) {
+    setTitle(mTitle);  //retranslate
+  }
+}
+
 void SensorDisplay::setTitle( const QString &title )
 {
   mTitle = title;
-  emit changeTitle(title);
+  mTranslatedTitle = i18n(title.toLatin1());
+  emit titleChanged(mTitle);
+  emit translatedTitleChanged(mTranslatedTitle);
+}
+
+QString SensorDisplay::translatedTitle() const
+{
+  return mTranslatedTitle;
 }
 
 QString SensorDisplay::title() const
