@@ -1,12 +1,13 @@
 /*
     KSysGuard, the KDE System Guard
 
-    Copyright (c) 2006 John Tapsell <john.tapsell@kdemail.org>
+    Copyright (c) 2006 - 2008 John Tapsell <john.tapsell@kde.org>
     Copyright (c) 1999 - 2001 Chris Schlaeger <cs@kde.org>
 
     This program is free software; you can redistribute it and/or
-    modify it under the terms of version 2 of the GNU General Public
-    License as published by the Free Software Foundation.
+    modify it under the terms of the GNU General Public
+    License version 2 or at your option version 3 as published by
+    the Free Software Foundation.
 
     This program is distributed in the hope that it will be useful,
     but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -60,6 +61,7 @@
 #include "WorkSheet.h"
 #include "StyleEngine.h"
 #include "HostConnector.h"
+#include "ProcessController.h"
 
 #include "ksysguard.h"
 
@@ -76,14 +78,11 @@ TopLevel* topLevel;
   TaskMan widget.
  */
 TopLevel::TopLevel()
-  : KXmlGuiWindow( 0 )
+  : KXmlGuiWindow( NULL )
 {
   QDBusConnection::sessionBus().registerObject("/", this, QDBusConnection::ExportScriptableSlots);
-  setPlainCaption( i18n( "System Monitor" ) );
   mTimerId = -1;
 
-  setWindowIcon(KIcon("utilities-system-monitor"));
-
   mSplitter = new QSplitter( this );
   mSplitter->setOrientation( Qt::Horizontal );
   mSplitter->setOpaqueResize( KGlobalSettings::opaqueResize() );
@@ -101,51 +100,73 @@ TopLevel::TopLevel()
    * number of processes and the memory consumption of the local
    * host. */
   const int STATUSBAR_STRETCH=1;
-  statusBar()->insertItem( i18n( "Loading Processes Count.." ), 0, STATUSBAR_STRETCH );
-  statusBar()->insertItem( i18n( "Loading CPU Stat.." ), 1, STATUSBAR_STRETCH );
-  statusBar()->insertItem( i18n( "Loading Memory Totals.." ), 2, STATUSBAR_STRETCH );
-  statusBar()->insertItem( i18n( "Loading Swap Totals.." ), 3, STATUSBAR_STRETCH);
+
+  sbProcessCount = new QLabel();
+  statusBar()->addWidget( sbProcessCount, STATUSBAR_STRETCH );
+
+  sbCpuStat = new QLabel();
+  statusBar()->addWidget( sbCpuStat, STATUSBAR_STRETCH );
+
+  sbMemTotal = new QLabel();
+  statusBar()->addWidget( sbMemTotal, STATUSBAR_STRETCH );
+
+  sbSwapTotal = new QLabel();
+  statusBar()->addWidget( sbSwapTotal, STATUSBAR_STRETCH );
+
   statusBar()->hide();
 
   // create actions for menu entries
-  QAction *action = actionCollection()->addAction("new_worksheet");
-  action->setIcon(KIcon("tab-new"));
-  action->setText(i18n( "&New Worksheet..." ));
-  connect(action, SIGNAL(triggered(bool)), mWorkSpace, SLOT( newWorkSheet() ));
-  action = actionCollection()->addAction("import_worksheet");
-  action->setIcon(KIcon("document-open") );
-  action->setText(i18n( "Import Worksheet..." ));
-  connect(action, SIGNAL(triggered(bool)), mWorkSpace, SLOT( importWorkSheet() ));
+  mNewWorksheetAction = actionCollection()->addAction("new_worksheet");
+  mNewWorksheetAction->setIcon(KIcon("tab-new"));
+  connect(mNewWorksheetAction, SIGNAL(triggered(bool)), mWorkSpace, SLOT( newWorkSheet() ));
+  mInsertWorksheetAction = actionCollection()->addAction("import_worksheet");
+  mInsertWorksheetAction->setIcon(KIcon("document-open") );
+  connect(mInsertWorksheetAction, SIGNAL(triggered(bool)), mWorkSpace, SLOT( importWorkSheet() ));
   mTabRemoveAction = actionCollection()->addAction( "remove_worksheet" );
-  mTabRemoveAction->setIcon( KIcon("tab-remove") );
-  mTabRemoveAction->setText( i18n( "&Remove Worksheet" ) );
+  mTabRemoveAction->setIcon( KIcon("tab-close") );
   connect(mTabRemoveAction, SIGNAL(triggered(bool)), mWorkSpace, SLOT( removeWorkSheet() ));
   mTabExportAction = actionCollection()->addAction( "export_worksheet" );
   mTabExportAction->setIcon( KIcon("document-save-as") );
-  mTabExportAction->setText( i18n( "&Export Worksheet..." ) );
   connect(mTabExportAction, SIGNAL(triggered(bool)), mWorkSpace, SLOT( exportWorkSheet() ));
 
-  KStandardAction::quit( this, SLOT( close() ), actionCollection() );
+  mQuitAction = NULL;
 
   mMonitorRemoteAction = actionCollection()->addAction( "connect_host" );
-  mMonitorRemoteAction->setIcon( KIcon("connection-established") );
-  mMonitorRemoteAction->setText( i18n( "Monitor remote machine..." ) );
+  mMonitorRemoteAction->setIcon( KIcon("network-connect") );
   connect(mMonitorRemoteAction, SIGNAL(triggered(bool)), SLOT( connectHost() ));
 
-  action = actionCollection()->addAction( "configure_sheet" );
-  action->setIcon( KIcon("configure") );
-  action->setText( i18n( "&Worksheet Properties" ) );
-  connect(action, SIGNAL(triggered(bool)), mWorkSpace, SLOT( configure() ));
+  mConfigureSheetAction = actionCollection()->addAction( "configure_sheet" );
+  mConfigureSheetAction->setIcon( KIcon("configure") );
+  connect(mConfigureSheetAction, SIGNAL(triggered(bool)), mWorkSpace, SLOT( configure() ));
 
   if (!initialGeometrySet())
     resize( QSize(700, 480).expandedTo(minimumSizeHint()));
 
-  setupGUI(ToolBar | Keys | StatusBar | Create);
+  retranslateUi();
 }
 
+void TopLevel::retranslateUi()
+{
+  setPlainCaption( i18n( "System Monitor" ) );
+  mNewWorksheetAction->setText(i18n( "&New Worksheet..." ));
+  mInsertWorksheetAction->setText(i18n( "Import Worksheet..." ));
+  mTabExportAction->setText( i18n( "&Export Worksheet..." ) );
+  mTabRemoveAction->setText( i18n( "&Remove Worksheet" ) );
+  mMonitorRemoteAction->setText( i18n( "Monitor remote machine..." ) );
+  mConfigureSheetAction->setText( i18n( "&Worksheet Properties" ) );
+  if(mQuitAction) {
+    KAction *tmpQuitAction = KStandardAction::quit( NULL, NULL, NULL );
+    mQuitAction->setText(tmpQuitAction->text());
+    mQuitAction->setWhatsThis(tmpQuitAction->whatsThis());
+    mQuitAction->setToolTip(tmpQuitAction->toolTip());
+    delete tmpQuitAction;
+  } else
+    mQuitAction = KStandardAction::quit( this, SLOT( close() ), actionCollection() );
+}
+
+
 void TopLevel::currentTabChanged(int index)
 {
-  kDebug() << "Current tab changed to " << index;
   QWidget *wdg = mWorkSpace->widget(index);
   WorkSheet *sheet = (WorkSheet *)(wdg);
   Q_ASSERT(sheet);
@@ -167,7 +188,6 @@ void TopLevel::currentTabChanged(int index)
 void TopLevel::startSensorBrowserWidget()
 {
   if(mSensorBrowser) return;
-  kDebug() << "Creating sensor browser";
   mSensorBrowser = new SensorBrowserWidget( 0, KSGRD::SensorMgr );
   mSplitter->insertWidget(2,mSensorBrowser);
   mSplitter->setSizes( mSplitterSize );
@@ -196,13 +216,23 @@ void TopLevel::removeWorkSheet( const QString &fileName )
 
 QStringList TopLevel::listSensors( const QString &hostName )
 {
-  startSensorBrowserWidget();
+  if(!mSensorBrowser) {
+    setUpdatesEnabled(false);
+    startSensorBrowserWidget();
+    mSensorBrowser->setVisible(false);
+    setUpdatesEnabled(true);
+  }
   return mSensorBrowser->listSensors( hostName );
 }
 
 QStringList TopLevel::listHosts()
 {
-  startSensorBrowserWidget();
+  if(!mSensorBrowser) {
+    setUpdatesEnabled(false);
+    startSensorBrowserWidget();
+    mSensorBrowser->setVisible(false);
+    setUpdatesEnabled(true);
+  }
   return mSensorBrowser->listHosts();
 }
 
@@ -214,11 +244,11 @@ void TopLevel::initStatusBar()
    * answerReceived(). */
   KSGRD::SensorMgr->sendRequest( "localhost", "mem/swap/used?",
                                  (KSGRD::SensorClient*)this, 7 );
-  updateStatusBar();
 
   KToggleAction *sb = dynamic_cast<KToggleAction*>(action("options_show_statusbar"));
   if (sb)
      connect(sb, SIGNAL(toggled(bool)), this, SLOT(updateStatusBar()));
+  setupGUI(ToolBar | Keys | StatusBar | Save | Create);
 }
 
 void TopLevel::updateStatusBar()
@@ -273,7 +303,7 @@ void TopLevel::editToolbars()
 {
   saveMainWindowSettings( KConfigGroup( KGlobal::config(), "MainWindow" ) );
   KEditToolBar dlg( actionCollection() );
-  connect( &dlg, SIGNAL( newToolbarConfig() ), this,
+  connect( &dlg, SIGNAL( newToolBarConfig() ), this,
            SLOT( slotNewToolbarConfig() ) );
 
   dlg.exec();
@@ -306,7 +336,7 @@ void TopLevel::timerEvent( QTimerEvent* )
      * information will be received by answerReceived(). */
     KSGRD::SensorMgr->sendRequest( "localhost", "pscount",
                                    (KSGRD::SensorClient*)this, 0 );
-    KSGRD::SensorMgr->sendRequest( "localhost", "cpu/system/idle",
+    KSGRD::SensorMgr->sendRequest( "localhost", "cpu/idle",
                                    (KSGRD::SensorClient*)this, 1 );
     KSGRD::SensorMgr->sendRequest( "localhost", "mem/physical/free",
                                    (KSGRD::SensorClient*)this, 2 );
@@ -321,6 +351,18 @@ void TopLevel::timerEvent( QTimerEvent* )
   }
 }
 
+void TopLevel::changeEvent( QEvent * event ) 
+{
+  if (event->type() == QEvent::LanguageChange) {
+    KSGRD::SensorMgr->retranslate();
+    setUpdatesEnabled(false);
+    setupGUI(ToolBar | Keys | StatusBar | Create);
+    retranslateUi();
+    setUpdatesEnabled(true);
+  }
+  KXmlGuiWindow::changeEvent(event);
+}
+
 bool TopLevel::queryClose()
 {
   if ( !mWorkSpace->saveOnQuit() )
@@ -353,7 +395,18 @@ void TopLevel::readProperties( const KConfigGroup& cfg )
 
   mWorkSpace->readProperties( cfg );
 
-  applyMainWindowSettings( cfg );
+  QList<WorkSheet *> workSheets = mWorkSpace->getWorkSheets();
+  ProcessController *processController = NULL;
+  foreach(WorkSheet *sheet, workSheets) {
+    processController = sheet->getLocalProcessController();
+    if(processController != NULL) {
+      for(int i = 0; i < processController->actions().size(); i++) {
+        actionCollection()->addAction("processAction" + QString::number(i), processController->actions().at(i));
+      }
+    }
+  }
+
+  updateStatusBar();
 }
 
 void TopLevel::saveProperties( KConfigGroup& cfg )
@@ -388,12 +441,13 @@ void TopLevel::answerReceived( int id, const QList<QByteArray> &answerList )
   switch ( id ) {
     case 0:
       s = i18n( " %1 processes ", answer.toInt() );
-      statusBar()->changeItem( s, 0 );
+      sbProcessCount->setText( s );
+
       break;
 
     case 1:
       s = i18n( " CPU: %1% ", (int) (100 - answer.toFloat()) );
-      statusBar()->changeItem( s, 1 );
+      sbCpuStat->setText( s );
       break;
 
     case 2:
@@ -409,7 +463,7 @@ void TopLevel::answerReceived( int id, const QList<QByteArray> &answerList )
       s = i18n( " Memory: %1 / %2 " ,
                 KGlobal::locale()->formatByteSize( mUsedApplication*1024),
                 KGlobal::locale()->formatByteSize( (mFree+mUsedTotal)*1024 ) );
-      statusBar()->changeItem( s, 2 );
+      sbMemTotal->setText( s );
       break;
 
     case 5:
@@ -440,7 +494,7 @@ void TopLevel::setSwapInfo( long used, long free, const QString & )
                 KGlobal::locale()->formatByteSize( free*1024) );
   }
 
-  statusBar()->changeItem( msg, 3 );
+  sbSwapTotal->setText( msg );
 }
 
 /*
@@ -482,7 +536,7 @@ extern "C" KDE_EXPORT int kdemain( int argc, char** argv )
 
   KAboutData aboutData( "ksysguard", 0, ki18n( "System Monitor" ),
                         KSYSGUARD_VERSION, ki18n(Description), KAboutData::License_GPL,
-                        ki18n( "(c) 1996-2006 The KDE System Monitor Developers" ) );
+                        ki18n( "(c) 1996-2008 The KDE System Monitor Developers" ) );
   aboutData.addAuthor( ki18n("John Tapsell"), ki18n("Current Maintainer"), "john.tapsell@kde.org" );
   aboutData.addAuthor( ki18n("Chris Schlaeger"), ki18n("Previous Maintainer"), "cs@kde.org" );
   aboutData.addAuthor( ki18n("Greg Martyn"), KLocalizedString(), "greg.martyn@gmail.com" );
@@ -504,11 +558,11 @@ extern "C" KDE_EXPORT int kdemain( int argc, char** argv )
   KCmdLineArgs::addCmdLineOptions( options );
   // initialize KDE application
   KApplication *app = new KApplication;
+  QApplication::setWindowIcon(KIcon("utilities-system-monitor"));
 
   KSGRD::SensorMgr = new KSGRD::SensorManager();
   KSGRD::Style = new KSGRD::StyleEngine();
 
-
 #ifdef FORK_KSYSGUARD
   char c = 0;
   write( initpipe[ 1 ], &c, 1 );
@@ -516,24 +570,23 @@ extern "C" KDE_EXPORT int kdemain( int argc, char** argv )
 #endif
   topLevel = new TopLevel();
 
+  topLevel->initStatusBar();
+
   // create top-level widget
+  topLevel->readProperties( KConfigGroup( KGlobal::config(), "MainWindow" ) );
+
   if ( app->isSessionRestored() )
     topLevel->restore( 1 );
-  else
-  {
-    topLevel->readProperties( KConfigGroup( KGlobal::config(), "MainWindow" ) );
-  }
 
-  topLevel->initStatusBar();
   topLevel->show();
-  KSGRD::SensorMgr->setBroadcaster( topLevel );
+  KSGRD::SensorMgr->setBroadcaster( topLevel );  // SensorMgr uses a QPointer for toplevel, so it is okay if topLevel is deleted first
 
   // run the application
   int result = app->exec();
 
-  delete KSGRD::Style;
-  delete KSGRD::SensorMgr;
   delete app;
+  delete KSGRD::SensorMgr;
+  delete KSGRD::Style;
 
   return result;
 }
