@@ -4,7 +4,7 @@
 	Copyright (c) 2007 Greg Martyn <greg.martyn@gmail.com>, John Tapsell <tapsell@kde.org>
 	
 	This program is free software; you can redistribute it and/or
-	modify it under the terms of version 2 of the GNU General Public
+	modify it under the terms of version 2 or later of the GNU General Public
 	License as published by the Free Software Foundation.
 	
 	This program is distributed in the hope that it will be useful,
@@ -214,7 +214,11 @@ void getMdadmDetail( ArrayInfo* MyArray ) {
 	char* mdadmStatBufP;
 
 	/* Create a pipe */
-	pipe(fd);
+	if(pipe(fd) == -1)
+	{
+		perror("Could not create a pipe to launch mdadm.");
+		exit(1);
+	}
 
 	/* Fork */
 	if((ChildPID = fork()) == -1)
