@@ -123,7 +123,7 @@ static void printWelcome( FILE* out )
   fprintf( out, "ksysguardd %s\n"
            "(c) 1999, 2000, 2001, 2002 Chris Schlaeger <cs@kde.org>\n"
            "(c) 2001 Tobias Koenig <tokoe@kde.org>\n"
-           "(c) 2006 Greg Martyn <greg.martyn@gmail.com>\n"
+           "(c) 2006-2008 Greg Martyn <greg.martyn@gmail.com>\n"
            "This program is part of the KDE Project and licensed under\n"
            "the GNU GPL version 2. See http://www.kde.org for details.\n", 
            KSYSGUARD_VERSION );
@@ -154,7 +154,11 @@ static int createLockFile()
     fseek( file, 0, SEEK_SET );
     fprintf( file, "%d\n", getpid() );
     fflush( file );
-    ftruncate( fileno( file ), ftell( file ) );
+    if (ftruncate( fileno( file ), ftell( file ) ) == -1) {
+      log_error( "Cannot set size of lockfile '%s'", LockFile );
+      fprintf( stderr, "Cannot set size of lockfile '%s'\n", LockFile );
+      return -2;
+    }
   } else {
     log_error( "Cannot create lockfile '%s'", LockFile );
     fprintf( stderr, "Cannot create lockfile '%s'\n", LockFile );
