@@ -1,13 +1,23 @@
-/*****************************************************************
+/********************************************************************
  KWin - the KDE window manager
  This file is part of the KDE project.
 
 Copyright (C) 1999, 2000 Matthias Ettrich <ettrich@kde.org>
 Copyright (C) 2003 Lubos Lunak <l.lunak@kde.org>
 
-You can Freely distribute this program under the GNU General Public
-License. See the file "COPYING" for the exact licensing terms.
-******************************************************************/
+This program is free software; you can redistribute it and/or modify
+it under the terms of the GNU General Public License as published by
+the Free Software Foundation; either version 2 of the License, or
+(at your option) any later version.
+
+This program is distributed in the hope that it will be useful,
+but WITHOUT ANY WARRANTY; without even the implied warranty of
+MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+GNU General Public License for more details.
+
+You should have received a copy of the GNU General Public License
+along with this program.  If not, see <http://www.gnu.org/licenses/>.
+*********************************************************************/
 
 #include "client.h"
 
@@ -15,13 +25,13 @@ License. See the file "COPYING" for the exact licensing terms.
 #include <QPainter>
 #include <QDateTime>
 #include <QProcess>
-#include <k3process.h>
 #include <unistd.h>
 #include <kstandarddirs.h>
 #include <QWhatsThis>
 #include <kwindowsystem.h>
 #include <kiconloader.h>
 #include <stdlib.h>
+#include <signal.h>
 
 #include "bridge.h"
 #include "group.h"
@@ -141,6 +151,7 @@ Client::Client( Workspace *ws )
     ignore_focus_stealing = false;
     demands_attention = false;
     hidden_preview = false;
+    raw_shown = false;
     check_active_modal = false;
 
     Pdeletewindow = 0;
@@ -423,7 +434,7 @@ void Client::detectNoBorder()
     // NET::Override is some strange beast without clear definition, usually
     // just meaning "noborder", so let's treat it only as such flag, and ignore it as
     // a window type otherwise (SUPPORTED_WINDOW_TYPES_MASK doesn't include it)
-    if( info->windowType( SUPPORTED_WINDOW_TYPES_MASK | NET::OverrideMask ) == NET::Override )
+    if( info->windowType( SUPPORTED_MANAGED_WINDOW_TYPES_MASK | NET::OverrideMask ) == NET::Override )
         noborder = true;
     }
 
@@ -521,6 +532,8 @@ void Client::updateInputShape()
           // that after the second step there's a hole in the input shape
           // until the real shape of the client is added and that can make
           // the window lose focus (which is a problem with mouse focus policies)
+          
+          // TODO it seems there is, after all - XShapeGetRectangles()
         static Window helper_window = None;
         if( helper_window == None )
             helper_window = XCreateSimpleWindow( display(), rootWindow(),
@@ -866,6 +879,9 @@ void Client::setMappingState(int s)
  */
 void Client::rawShow()
     {
+    if( raw_shown ) // this flag is used to purely avoid repeated calls to rawShow(),
+        return;     // it doesn't say anything more about the state
+    raw_shown = true;
     if( decoration != NULL )
         decoration->widget()->show(); // not really necessary, but let it know the state
     XMapWindow( display(), frameId());
@@ -896,6 +912,9 @@ void Client::rawShow()
 */
 void Client::rawHide()
     {
+    if( !raw_shown )
+        return;
+    raw_shown = false;
     StackingUpdatesBlocker blocker( workspace());
     addWorkspaceRepaint( geometry());
     if( options->hiddenPreviews == HiddenPreviewsNever )
@@ -1098,21 +1117,15 @@ void Client::killProcess( bool ask, Time timestamp )
         }
     else
         {
-        process_killer = new K3Process( this );
-        *process_killer << KStandardDirs::findExe( "kwin_killer_helper" )
-            << "--pid" << QByteArray().setNum( pid ) << "--hostname" << machine
+        process_killer = new QProcess( this );
+        connect( process_killer, SIGNAL( error( QProcess::ProcessError )), SLOT( processKillerExited()));
+        connect( process_killer, SIGNAL( finished( int, QProcess::ExitStatus )), SLOT( processKillerExited()));
+        process_killer->start( KStandardDirs::findExe( "kwin_killer_helper" ),
+            QStringList() << "--pid" << QByteArray().setNum( (unsigned)pid ) << "--hostname" << machine
             << "--windowname" << caption().toUtf8()
             << "--applicationname" << resourceClass()
             << "--wid" << QString::number( window() )
-            << "--timestamp" << QString::number( timestamp );
-        connect( process_killer, SIGNAL( processExited( K3Process* )),
-            SLOT( processKillerExited()));
-        if( !process_killer->start( K3Process::NotifyOnExit ))
-            {
-            delete process_killer;
-            process_killer = NULL;
-            return;
-            }
+            << "--timestamp" << QString::number( timestamp ));
         }
     }
 
