@@ -1,13 +1,23 @@
-/*****************************************************************
+/********************************************************************
  KWin - the KDE window manager
  This file is part of the KDE project.
 
 Copyright (C) 1999, 2000 Matthias Ettrich <ettrich@kde.org>
 Copyright (C) 2003 Lubos Lunak <l.lunak@kde.org>
 
-You can Freely distribute this program under the GNU General Public
-License. See the file "COPYING" for the exact licensing terms.
-******************************************************************/
+This program is free software; you can redistribute it and/or modify
+it under the terms of the GNU General Public License as published by
+the Free Software Foundation; either version 2 of the License, or
+(at your option) any later version.
+
+This program is distributed in the hope that it will be useful,
+but WITHOUT ANY WARRANTY; without even the implied warranty of
+MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+GNU General Public License for more details.
+
+You should have received a copy of the GNU General Public License
+along with this program.  If not, see <http://www.gnu.org/licenses/>.
+*********************************************************************/
 
 #ifndef KWIN_CLIENT_H
 #define KWIN_CLIENT_H
@@ -36,8 +46,8 @@ License. See the file "COPYING" for the exact licensing terms.
 #include <X11/extensions/sync.h>
 #endif
 
+class QProcess;
 class QTimer;
-class K3Process;
 class KStartupInfoData;
 
 namespace KWin
@@ -449,6 +459,7 @@ class Client
         uint ignore_focus_stealing : 1; // don't apply focus stealing prevention to this client
         uint demands_attention : 1;
         uint hidden_preview : 1; // mapped only to get a window pixmap for compositing
+        uint raw_shown : 1; // for use in rawShow()/rawHide()
         WindowRules client_rules;
         void getWMHints();
         void readIcons();
@@ -474,7 +485,7 @@ class Client
         Window window_group;
         Layer in_layer;
         QTimer* ping_timer;
-        K3Process* process_killer;
+        QProcess* process_killer;
         Time ping_timestamp;
         Time user_time;
         unsigned long allowed_actions;
