@@ -1,21 +1,35 @@
+/*********************************************************************
+  B-II KWin Client
+ 
+  Changes:
+    Customizable button positions by Karol Szwed <gallium@kde.org>
+ 
+    Thin frame in fixed size windows, titlebar gradient support, accessibility
+    improvements, customizable menu double click action and button hover
+    effects are
+    Copyright (c) 2003, 2004, 2006 Luciano Montanaro <mikelima@cirulla.net>
+
+This program is free software; you can redistribute it and/or modify
+it under the terms of the GNU General Public License as published by
+the Free Software Foundation; either version 2 of the License, or
+(at your option) any later version.
+
+This program is distributed in the hope that it will be useful,
+but WITHOUT ANY WARRANTY; without even the implied warranty of
+MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+GNU General Public License for more details.
+
+You should have received a copy of the GNU General Public License
+along with this program.  If not, see <http://www.gnu.org/licenses/>.
+*********************************************************************/
+
 #ifndef CLIENTS_B2_B2CLIENT
 #define CLIENTS_B2_B2CLIENT
-/*
- * B-II KWin Client
- *
- * Changes:
- *   Customizable button positions by Karol Szwed <gallium@kde.org>
- *
- *   Thin frame in fixed size windows, titlebar gradient support, accessibility
- *   improvements, customizable menu double click action and button hover
- *   effects are
- *   Copyright (c) 2003, 2004, 2006 Luciano Montanaro <mikelima@cirulla.net>
- */
 
 #include "b2client.h"
+
 #include <QApplication>
 #include <QLayout>
-#include <qdrawutil.h>
 #include <QPixmap>
 #include <QPaintEvent>
 #include <QPolygon>
@@ -31,7 +45,6 @@
 #include <QTextStream>
 #include <kicontheme.h>
 #include <kiconeffect.h>
-//#include <kdrawutil.h>
 #include <klocale.h>
 #include <kconfig.h>
 #include <QBitmap>
@@ -55,26 +68,31 @@ enum {
 
 enum {
     P_CLOSE = 0,
-    P_MAX, P_NORMALIZE, P_ICONIFY, P_PINUP, P_MENU, P_HELP, P_SHADE, P_RESIZE,
-    P_NUM_BUTTON_TYPES
+    P_MAX = NumStates, 
+    P_NORMALIZE = P_MAX + NumStates, 
+    P_ICONIFY = P_NORMALIZE + NumStates, 
+    P_PINUP = P_ICONIFY + NumStates, 
+    P_MENU = P_PINUP + NumStates, 
+    P_HELP = P_MENU + NumStates, 
+    P_SHADE = P_HELP + NumStates, 
+    P_RESIZE = P_SHADE + NumStates,
+    P_NUM_PIXMAPS = P_RESIZE + NumStates
 };
 
-#define NUM_PIXMAPS (P_NUM_BUTTON_TYPES * NumStates)
-
-static QPixmap *pixmap[NUM_PIXMAPS];
+static QPixmap *pixmap[P_NUM_PIXMAPS];
 
 // active
-#define PIXMAP_A(i)  (pixmap[(i) * NumStates + Norm])
+#define PIXMAP_A(i)  (pixmap[(i) + Norm])
 // active, hover
-#define PIXMAP_AH(i) (pixmap[(i) * NumStates + Hover])
+#define PIXMAP_AH(i) (pixmap[(i) + Hover])
 // active, down
-#define PIXMAP_AD(i) (pixmap[(i) * NumStates + Down])
+#define PIXMAP_AD(i) (pixmap[(i) + Down])
 // inactive
-#define PIXMAP_I(i)  (pixmap[(i) * NumStates + INorm])
+#define PIXMAP_I(i)  (pixmap[(i) + INorm])
 // inactive, hover
-#define PIXMAP_IH(i) (pixmap[(i) * NumStates + IHover])
+#define PIXMAP_IH(i) (pixmap[(i) + IHover])
 // inactive, down
-#define PIXMAP_ID(i) (pixmap[(i) * NumStates + IDown])
+#define PIXMAP_ID(i) (pixmap[(i) + IDown])
 
 static QPixmap* titleGradient[2] = {0, 0};
 
@@ -115,10 +133,10 @@ static void read_config(B2ClientFactory *f)
 {
     // Force button size to be in a reasonable range.
     // If the frame width is large, the button size must be large too.
-    buttonSize = (QFontMetrics(options()->font(true)).height() + 1) & 0x3e;
+    buttonSize = (QFontMetrics(options()->font(true)).height() - 1) & 0x3e;
     if (buttonSize < 16) buttonSize = 16;
 
-    KConfig _conf( "kwinb2rc" );
+    KConfig _conf("kwinb2rc");
     KConfigGroup conf(&_conf, "General");
     colored_frame = conf.readEntry("UseTitleBarBorderColors", false);
     do_draw_handle = conf.readEntry("DrawGrabHandle", true);
@@ -203,10 +221,9 @@ static void create_pixmaps()
     int bsize = buttonSize - 2;
     if (bsize < 16) bsize = 16;
 
-    for (i = 0; i < NUM_PIXMAPS; i++) {
-
-        switch (i / NumStates) {
-	case P_MAX: // will be initialized by copying P_CLOSE
+    for (i = 0; i < P_NUM_PIXMAPS; i++) {
+        switch (NumStates *(i / NumStates)) {
+	case P_CLOSE: // will be initialized by copying P_MAX
 	case P_RESIZE:
 	    pixmap[i] = new QPixmap();
 	    break;
@@ -214,7 +231,8 @@ static void create_pixmaps()
 	    pixmap[i] = new QPixmap(10, 10);
 	    break;
 	case P_SHADE:
-	case P_CLOSE:
+	case P_MAX:
+	case P_HELP:
 	    pixmap[i] = new QPixmap(bsize, bsize);
 	    break;
 	default:
@@ -236,12 +254,10 @@ static void create_pixmaps()
     QBitmap pinupMask = QBitmap::fromData(QSize(16, 16), pinup_mask_bits);
     QBitmap pindownMask = QBitmap::fromData(QSize(16, 16), pindown_mask_bits);
     QBitmap menuMask = QBitmap::fromData(QSize(16, 16), menu_mask_bits);
-    QBitmap helpMask = QBitmap::fromData(QSize(16, 16), help_mask_bits);
     for (i = 0; i < NumStates; i++) {
 	bool isDown = (i == Down) || (i == IDown);
-	pixmap[P_MENU * NumStates + i]->setMask(menuMask);
-	pixmap[P_HELP * NumStates + i]->setMask(helpMask);
-	pixmap[P_PINUP * NumStates + i]->setMask(isDown ? pindownMask: pinupMask);
+	pixmap[P_MENU + i]->setMask(menuMask);
+	pixmap[P_PINUP + i]->setMask(isDown ? pindownMask: pinupMask);
     }
 
     QBitmap normalizeMask(16, 16);
@@ -258,7 +274,7 @@ static void create_pixmaps()
     mask.end();
 
     for (i = 0; i < NumStates; i++)
-	pixmap[P_NORMALIZE * NumStates + i]->setMask(normalizeMask);
+	pixmap[P_NORMALIZE + i]->setMask(normalizeMask);
 
     QBitmap shadeMask(bsize, bsize);
     shadeMask.clear();
@@ -266,12 +282,12 @@ static void create_pixmaps()
     mask.fillRect(0, 0, bsize, 6, one);
     mask.end();
     for (i = 0; i < NumStates; i++)
-	pixmap[P_SHADE * NumStates + i]->setMask(shadeMask);
+	pixmap[P_SHADE + i]->setMask(shadeMask);
 }
 
 static void delete_pixmaps()
 {
-    for (int i = 0; i < NUM_PIXMAPS; i++) {
+    for (int i = 0; i < P_NUM_PIXMAPS; i++) {
     	delete pixmap[i];
 	pixmap[i] = 0;
     }
@@ -316,24 +332,34 @@ bool B2ClientFactory::reset(unsigned long changed)
     return needsReset;
 }
 
-bool B2ClientFactory::supports(Ability ability)
+bool B2ClientFactory::supports(Ability ability) const
 {
     switch (ability) {
-        case AbilityAnnounceButtons:
-        case AbilityButtonMenu:
-        case AbilityButtonOnAllDesktops:
-        case AbilityButtonSpacer:
-        case AbilityButtonHelp:
-        case AbilityButtonMinimize:
-        case AbilityButtonMaximize:
-        case AbilityButtonClose:
-        case AbilityButtonAboveOthers:
-        case AbilityButtonBelowOthers:
-        case AbilityButtonShade:
-        case AbilityButtonResize:
-            return true;
-        default:
-            return false;
+    // announce
+    case AbilityAnnounceButtons:
+    case AbilityAnnounceColors:
+    // buttons
+    case AbilityButtonMenu:
+    case AbilityButtonOnAllDesktops:
+    case AbilityButtonSpacer:
+    case AbilityButtonHelp:
+    case AbilityButtonMinimize:
+    case AbilityButtonMaximize:
+    case AbilityButtonClose:
+    case AbilityButtonShade:
+    case AbilityButtonResize:
+    // colors
+    case AbilityColorTitleBack:
+    case AbilityColorTitleBlend:
+    case AbilityColorTitleFore:
+    case AbilityColorFrame:
+    case AbilityColorButtonBack:
+	return true;
+    // These are not (yet) supported.
+    case AbilityButtonAboveOthers:
+    case AbilityButtonBelowOthers:
+    default:
+	return false;
     };
 }
 
@@ -382,10 +408,9 @@ void B2Client::init()
     // Check this early, otherwise the preview will be rendered badly.
     resizable = isResizable();
 
-    createMainWidget(Qt::WResizeNoErase | Qt::WRepaintNoErase);
-    widget()->installEventFilter(this);
-
+    createMainWidget();
     widget()->setAttribute(Qt::WA_NoSystemBackground);
+    widget()->installEventFilter(this);
 
     // Set button pointers to NULL so we know what has been created
     for (int i = 0; i < BtnCount; i++)
@@ -976,21 +1001,21 @@ static void redraw_pixmaps()
     QColor inactiveColor = iPal.color(QPalette::Button);
     QColor activeColor = aPal.color(QPalette::Button);
 
-    // close
-    drawB2Rect(PIXMAP_A(P_CLOSE), activeColor, false);
-    drawB2Rect(PIXMAP_AH(P_CLOSE), activeColor, true);
-    drawB2Rect(PIXMAP_AD(P_CLOSE), activeColor, true);
-
-    drawB2Rect(PIXMAP_I(P_CLOSE), inactiveColor, false);
-    drawB2Rect(PIXMAP_IH(P_CLOSE), inactiveColor, true);
-    drawB2Rect(PIXMAP_ID(P_CLOSE), inactiveColor, true);
+    // maximize
+    for (int i = 0; i < NumStates; i++) {
+	bool is_act = (i < 2);
+	bool is_down = ((i & 1) == 1);
+	QPixmap *pix = pixmap[P_MAX + i];
+	QColor color = is_act ? activeColor : inactiveColor;
+	drawB2Rect(pix, color, is_down);
+    }
 
     // shade
     QPixmap thinBox(buttonSize - 2, 6);
     for (int i = 0; i < NumStates; i++) {
 	bool is_act = (i < 2);
 	bool is_down = ((i & 1) == 1);
-	QPixmap *pix = pixmap[P_SHADE * NumStates + i];
+	QPixmap *pix = pixmap[P_SHADE + i];
 	QColor color = is_act ? activeColor : inactiveColor;
 	drawB2Rect(&thinBox, color, is_down);
 	pix->fill(Qt::black);
@@ -998,20 +1023,14 @@ static void redraw_pixmaps()
 		0, 0, thinBox.width(), thinBox.height());
     }
 
-    // maximize
-    for (int i = 0; i < NumStates; i++) {
-	*pixmap[P_MAX * NumStates + i] = *pixmap[P_CLOSE * NumStates + i];
-	pixmap[P_MAX * NumStates + i]->detach();
-    }
-
     // normalize + iconify
-    QPixmap smallBox( 10, 10 );
-    QPixmap largeBox( 12, 12 );
+    QPixmap smallBox(10, 10);
+    QPixmap largeBox(12, 12);
 
     for (int i = 0; i < NumStates; i++) {
 	bool is_act = (i < 3);
 	bool is_down = (i == Down || i == IDown);
-	QPixmap *pix = pixmap[P_NORMALIZE * NumStates + i];
+	QPixmap *pix = pixmap[P_NORMALIZE + i];
 	QColor color = is_act ? activeColor : inactiveColor;
 	drawB2Rect(&smallBox, color, is_down);
 	drawB2Rect(&largeBox, color, is_down);
@@ -1020,7 +1039,7 @@ static void redraw_pixmaps()
 	       0, 0, 12, 12);
 	bitBlt(pix, 0, 0, &smallBox, 0, 0, 10, 10);
 
-	bitBlt(pixmap[P_ICONIFY * NumStates + i], 0, 0,
+	bitBlt(pixmap[P_ICONIFY + i], 0, 0,
 	       &smallBox, 0, 0, 10, 10);
     }
 
@@ -1028,42 +1047,53 @@ static void redraw_pixmaps()
     for (int i = 0; i < NumStates; i++) {
 	bool is_act = (i < 3);
 	bool is_down = (i == Down || i == IDown);
-	*pixmap[P_RESIZE * NumStates + i] = *pixmap[P_CLOSE * NumStates + i];
-	pixmap[P_RESIZE * NumStates + i]->detach();
+	*pixmap[P_RESIZE + i] = *pixmap[P_MAX + i];
+	pixmap[P_RESIZE + i]->detach();
 	drawB2Rect(&smallBox, is_act ? activeColor : inactiveColor, is_down);
-	bitBlt(pixmap[P_RESIZE * NumStates + i],
-		0, 0, &smallBox, 0, 0, 10, 10);
+	bitBlt(pixmap[P_RESIZE + i], 0, 0, &smallBox, 0, 0, 10, 10);
     }
 
-
     QPainter p;
-    // x for close + menu + help
-    for (int j = 0; j < 3; j++) {
-        int pix;
-        unsigned const char *light, *dark;
-        switch (j) {
-        case 0:
-            pix = P_CLOSE; light = close_white_bits; dark = close_dgray_bits;
-            break;
-        case 1:
-            pix = P_MENU; light = menu_white_bits; dark = menu_dgray_bits;
-            break;
-        default:
-            pix = P_HELP; light = help_light_bits; dark = help_dark_bits;
-            break;
-        }
-	int off = (pixmap[pix * NumStates]->width() - 16) / 2;
+    // close: copy the maximize image, then add the X
+    for (int i = 0; i < NumStates; i++) {
+	*pixmap[P_CLOSE + i] = *pixmap[P_MAX + i];
+	pixmap[P_CLOSE + i]->detach();
+    }
+
+    for (int i = 0; i < NumStates; i++) {
+	bool isAct = (i < 3);
+	QPixmap *pixm = pixmap[P_CLOSE + i];
+	p.begin(pixm);
+	QColor color = isAct ? activeColor : inactiveColor;
+	QRect r = QRect(3, 3, pixm->width() - 6, pixm->height() - 6);
+	for (int j = 0; j < 2; j++) {
+	    r.moveTo(j + 3, 3);
+	    p.setPen(j == 0 ? color.light(150) : color.dark(150));
+	    p.drawLine(r.left(), r.top(), r.right() - 1, r.bottom() - 1);
+	    p.drawLine(r.left(), r.top() + 1, r.right() - 1, r.bottom());
+	    p.drawLine(r.right() - 1, r.top(), r.left(), r.bottom() - 1);
+	    p.drawLine(r.right() - 1, r.top() + 1, r.left(), r.bottom());
+	}
+	p.end();
+    }
+    for (int i = 0; i < 2; i++) {
+	
+    }
+
+    // menu 
+    {
+	int off = (pixmap[P_MENU]->width() - 16) / 2;
 	QSize bSize(16, 16);
 	QBitmap lightBitmap = QBitmap::fromData(bSize, 
-		    light, QImage::Format_MonoLSB);
+		    menu_white_bits, QImage::Format_MonoLSB);
 	//lightBitmap.setMask(lightBitmap);
 	QBitmap darkBitmap = QBitmap::fromData(bSize, 
-		    dark, QImage::Format_MonoLSB);
+		    menu_dgray_bits, QImage::Format_MonoLSB);
 	//darkBitmap.setMask(darkBitmap);
 
         for (int i = 0; i < NumStates; i++) {
 	    bool isAct = (i < 3);
-	    QPixmap *pixm = pixmap[pix* NumStates + i];
+	    QPixmap *pixm = pixmap[P_MENU + i];
 	    p.begin(pixm);
 	    QColor color = isAct ? activeColor : inactiveColor;
 	    p.setPen(color.light(150));
@@ -1074,49 +1104,77 @@ static void redraw_pixmaps()
         }
     }
 
+    // Help button: a question mark.
+    {
+	QFont font = options()->font(true);
+	font.setWeight(QFont::Black);
+	font.setStretch(110);
+	font.setPointSizeF(font.pointSizeF() * 1.1);
+	for (int i = 0; i < NumStates; i++) {
+	    bool isAct = (i < 3);
+	    QPixmap *pixm = pixmap[P_HELP + i];
+	    pixm->fill(QColor(qRgba(0, 0, 0, 0)));
+	    pixm->setAlphaChannel(*pixm);
+	    p.begin(pixm);
+	    QColor color = isAct ? activeColor : inactiveColor;
+	    QRect r = QRect(0, 0, pixm->width(), pixm->height());
+	    p.setFont(font);
+	    QString label = i18nc("Help button label, one character", "?");
+	    r.moveTo(1, 2);
+	    p.setPen(color.light(150));
+	    p.drawText(r, Qt::AlignCenter | Qt::AlignVCenter, label);
+	    r.moveTo(0, 1);
+	    p.setPen(color.dark(150));
+	    p.drawText(r, Qt::AlignCenter | Qt::AlignVCenter, label);
+	    p.end();
+	}
+    }
+
+    // Help button: a question mark.
     // pin
     for (int i = 0; i < NumStates; i++) {
 	const bool isDown = (i == Down || i == IDown);
 	bool isAct = (i < 3);
 
 	QSize pinSize(16, 16);
-	QBitmap white = QBitmap::fromData(pinSize, 
-		    isDown ? pindown_white_bits : pinup_white_bits, 
-		    QImage::Format_MonoLSB);
-	QBitmap gray = QBitmap::fromData(pinSize, 
-		    isDown ? pindown_gray_bits : pinup_gray_bits, 
-		    QImage::Format_MonoLSB);
-	QBitmap dgray = QBitmap::fromData(pinSize, 
-		    isDown ? pindown_dgray_bits : pinup_dgray_bits, 
-		    QImage::Format_MonoLSB);
+	QBitmap white = QBitmap::fromData(pinSize,
+		isDown ? pindown_white_bits : pinup_white_bits, 
+		QImage::Format_MonoLSB);
+	QBitmap gray = QBitmap::fromData(pinSize,
+		isDown ? pindown_gray_bits : pinup_gray_bits, 
+		QImage::Format_MonoLSB);
+	QBitmap dgray = QBitmap::fromData(pinSize,
+		isDown ? pindown_dgray_bits : pinup_dgray_bits, 
+		QImage::Format_MonoLSB);
 
-	QPixmap *pix = pixmap[P_PINUP * NumStates + i];
+	QPixmap *pix = pixmap[P_PINUP + i];
 	QColor color = isAct ? activeColor : inactiveColor;
-        p.begin(pix);
+	QPoint origin(0, 0);
+	QImage pin(16, 16, QImage::Format_ARGB32_Premultiplied);
+        p.begin(&pin);
 	p.setPen(color.light(150));
-	p.drawPixmap(0, 0, white);
+	p.drawPixmap(origin, white);
 	p.setPen(color);
-	p.drawPixmap(0, 0, gray);
+	p.drawPixmap(origin, gray);
 	p.setPen(color.dark(150));
-	p.drawPixmap(0, 0, dgray);
+	p.drawPixmap(origin, dgray);
 	p.end();
+	*pix = QPixmap::fromImage(pin);
     }
 
     // Apply the hilight effect to the 'Hover' icons
     KIconEffect ie;
     QPixmap hilighted;
-    for (int i = 0; i < P_NUM_BUTTON_TYPES; i++) {
-	int offset = i * NumStates;
-	hilighted = ie.apply(*pixmap[offset + Norm],
+    for (int i = 0; i < P_NUM_PIXMAPS; i += NumStates) {
+	hilighted = ie.apply(*pixmap[i + Norm],
 		KIconLoader::Small, KIconLoader::ActiveState);
-	*pixmap[offset + Hover] = hilighted;
+	*pixmap[i + Hover] = hilighted;
 
-	hilighted = ie.apply(*pixmap[offset + INorm],
+	hilighted = ie.apply(*pixmap[i + INorm],
 		KIconLoader::Small, KIconLoader::ActiveState);
-	*pixmap[offset + IHover] = hilighted;
+	*pixmap[i + IHover] = hilighted;
     }
 
-
     // Create the titlebar gradients
     if (QPixmap::defaultDepth() > 8) {
 	QColor titleColor[4] = {
@@ -1171,12 +1229,15 @@ void B2Client::positionButtons()
 }
 
 // Transparent bound stuff.
-
 static QRect *visible_bound;
 static QPolygon bound_shape;
 
+
 bool B2Client::drawbound(const QRect& geom, bool clear)
 {
+    // Let kwin draw the bounds, for now.
+    return false;
+#if 0
     if (clear) {
 	if (!visible_bound) return true;
     }
@@ -1184,18 +1245,13 @@ bool B2Client::drawbound(const QRect& geom, bool clear)
     if (!visible_bound) {
 	visible_bound = new QRect(geom);
 	QRect t = titlebar->geometry();
-	int frameTop = geom.top() + t.bottom();
-	int barLeft = geom.left() + bar_x_ofs;
+        QRect g = geom;
+        // line width is 5 pixels, so compensate for the 2 outer pixels (#88657)
+	g.adjust(2, 2, -2, -2);
+	int frameTop = geom.top() + t.bottom() + 2;
+	int barLeft = geom.left() + bar_x_ofs + 2;
 	int barRight = barLeft + t.width() - 1;
 	if (barRight > geom.right()) barRight = geom.right();
-        // line width is 5 pixels, so compensate for the 2 outer pixels (#88657)
-        QRect g = geom;
-        g.setLeft(g.left() + 2);
-        g.setTop(g.top() + 2);
-        g.setRight(g.right() - 2);
-        g.setBottom(g.bottom() - 2);
-        frameTop += 2;
-        barLeft += 2;
         barRight -= 2;
 
 	bound_shape.putPoints(0, 8,
@@ -1210,17 +1266,18 @@ bool B2Client::drawbound(const QRect& geom, bool clear)
     } else {
 	*visible_bound = geom;
     }
-/**
- * TODO: Replace by QRubberBand
- *   QPainter p(workspaceWidget());
- *   p.setPen(QPen(Qt::white, 5));
- *   p.setRasterOp(Qt::XorROP);
- *   p.drawPolygon(bound_shape);
- */
-    if (clear) {
-	delete visible_bound;
-	visible_bound = 0;
+    QPainter p;
+    if (p.begin(workspaceWidget())) {
+	p.setPen(QPen(Qt::white, 5));
+	p.setCompositionMode(QPainter::CompositionMode_Xor);
+	p.drawPolygon(bound_shape);
+	if (clear) {
+	    delete visible_bound;
+	    visible_bound = 0;
+	}
+	p.end();
     }
+#endif
     return true;
 }
 
@@ -1318,7 +1375,6 @@ void B2Button::paintEvent(QPaintEvent *)
 
 void B2Button::setPixmaps(int button_id)
 {
-    button_id *= NumStates;
     for (int i = 0; i < NumStates; i++) {
 	icon[i] = B2::pixmap[button_id + i];
     }
@@ -1367,7 +1423,7 @@ B2Titlebar::B2Titlebar(B2Client *parent)
       set_x11mask(false), isfullyobscured(false), shift_move(false)
 {
     setAttribute(Qt::WA_NoSystemBackground);
-    captionSpacer = new QSpacerItem(buttonSize, buttonSize + 4,
+    captionSpacer = new QSpacerItem(buttonSize, buttonSize + 3,
 	    QSizePolicy::Expanding, QSizePolicy::Fixed);
 }
 
@@ -1423,7 +1479,7 @@ void B2Titlebar::drawTitlebar(QPainter &p, bool state)
     p.setPen(options()->color(KDecoration::ColorFont, state));
     p.setFont(options()->font(state));
     t = captionSpacer->geometry();
-    p.drawText(t, Qt::AlignCenter | Qt::AlignVCenter, client->caption());
+    p.drawText(t.translated(0, 1), Qt::AlignCenter | Qt::AlignVCenter, client->caption());
 }
 
 void B2Titlebar::recalcBuffer()
