@@ -3,19 +3,35 @@
 // -------------------
 // Oxygen window decoration for KDE
 // -------------------
-// Copyright (c) 2003, 2004 David Johnson
 // Copyright (c) 2006, 2007 Casper Boemann <cbr@boemann.dk>
-// Copyright (c) 2006, 2007 Riccardo Iaconelli <ruphy@fsfe.org>
+// Copyright (c) 2006, 2007 Riccardo Iaconelli <riccardo@kde.org>
 //
-// Please see the header file for copyright and license information.
+// Permission is hereby granted, free of charge, to any person obtaining a copy
+// of this software and associated documentation files (the "Software"), to
+// deal in the Software without restriction, including without limitation the
+// rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
+// sell copies of the Software, and to permit persons to whom the Software is
+// furnished to do so, subject to the following conditions:
+//
+// The above copyright notice and this permission notice shall be included in
+// all copies or substantial portions of the Software.
+//
+// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
+// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
+// FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
+// IN THE SOFTWARE.
 //////////////////////////////////////////////////////////////////////////////
 // #ifndef OXYGENCLIENT_H
 // #define OXYGENCLIENT_H
 
-#include <kconfig.h>
-#include <kglobal.h>
-#include <klocale.h>
-#include <kdebug.h>
+#include <KConfig>
+#include <KGlobal>
+#include <KLocale>
+#include <KDebug>
+#include <KColorUtils>
 
 #include <qbitmap.h>
 #include <qlabel.h>
@@ -34,29 +50,16 @@
 #include <QTimer>
 #include <QCache>
 
-#include "lib/helper.h"
+#include "math.h"
 
 #include "oxygenclient.h"
 #include "oxygenclient.moc"
 #include "oxygenbutton.h"
 #include "oxygen.h"
 
-#include "definitions.cpp"
-
 namespace Oxygen
 {
 
-// global constants
-
-// static const int BUTTONSIZE      = 18;
-// static const int DECOSIZE        = 8;
-// static const int TITLESIZE       = 18;
-// static const int TFRAMESIZE       = 8;
-// static const int BFRAMESIZE       = 7;
-// static const int LFRAMESIZE       = 8;
-// static const int RFRAMESIZE       = 7;BUTTONSIZE
-// static const int FRAMEBUTTONSPACE       = 67;
-
 K_GLOBAL_STATIC_WITH_ARGS(OxygenHelper, globalHelper, ("OxygenDeco"))
 
 void renderDot(QPainter *p, const QPointF &point, qreal diameter)
@@ -66,7 +69,11 @@ void renderDot(QPainter *p, const QPointF &point, qreal diameter)
 
 
 OxygenClient::OxygenClient(KDecorationBridge *b, KDecorationFactory *f)
-    : KCommonDecoration(b, f) { ; }
+    : KCommonDecoration(b, f)
+    , colorCacheInvalid_(true)
+    , helper_(*globalHelper)
+{
+}
 
 OxygenClient::~OxygenClient()
 {
@@ -76,23 +83,13 @@ QString OxygenClient::visibleName() const
     return i18n("Oxygen");
 }
 
-QString OxygenClient::defaultButtonsLeft() const
-{
-    return "M";
-}
-
-QString OxygenClient::defaultButtonsRight() const
-{
-    return "HIAX";
-}
-
 void OxygenClient::init()
 {
     KCommonDecoration::init();
 
     widget()->setAutoFillBackground(false);
     widget()->setAttribute(Qt::WA_OpaquePaintEvent);
-    widget()->setAttribute( Qt::WA_PaintOnScreen, false);
+    widget()->setAttribute(Qt::WA_PaintOnScreen, false);
 }
 
 bool OxygenClient::decorationBehaviour(DecorationBehaviour behaviour) const
@@ -170,9 +167,6 @@ int OxygenClient::layoutMetric(LayoutMetric lm, bool respectWindowState, const K
         case LM_ButtonMarginTop:
             return 0;
 
-        case LM_ExplicitButtonSpacer:
-            return 3;
-
         default:
             return KCommonDecoration::layoutMetric(lm, respectWindowState, btn);
     }
@@ -183,26 +177,75 @@ KCommonDecorationButton *OxygenClient::createButton(::ButtonType type)
 {
     switch (type) {
         case MenuButton:
-            return new OxygenButton(this, i18n("Menu"), ButtonMenu);
+            return new OxygenButton(*this, i18n("Menu"), ButtonMenu);
 
         case HelpButton:
-            return new OxygenButton(this, i18n("Help"), ButtonHelp);
+            return new OxygenButton(*this, i18n("Help"), ButtonHelp);
 
         case MinButton:
-            return new OxygenButton(this, i18n("Minimize"), ButtonMin);
+            return new OxygenButton(*this, i18n("Minimize"), ButtonMin);
 
         case MaxButton:
-            return new OxygenButton(this, i18n("Minimize"), ButtonMax);
+            return new OxygenButton(*this, i18n("Maximize"), ButtonMax);
 
         case CloseButton:
-            return new OxygenButton(this, i18n("Minimize"), ButtonClose);
-
+            return new OxygenButton(*this, i18n("Close"), ButtonClose);
+/*
+        case OnAllDesktopsButton:
+            return new OxygenButton(*this, i18n("All Desktops"), ButtonSticky);
+*/
         default:
             return 0;
     }
 }
 
 
+// c0 - background
+// c1 - foreground
+// t - target contrast ratio
+QColor reduceContrast(const QColor &c0, const QColor &c1, double t)
+{
+    double s = KColorUtils::contrastRatio(c0, c1);
+    if (s < t)
+        return c1;
+
+    double l = 0.0, h = 1.0;
+    double x = s, a;
+    QColor r = c1;
+    for (int maxiter = 16; maxiter; --maxiter) {
+        a = 0.5 * (l + h);
+        r = KColorUtils::mix(c0, c1, a);
+        x = KColorUtils::contrastRatio(c0, r);
+        if (fabs(x - t) < 0.01)
+            break;
+        if (x > t)
+            h = a;
+        else
+            l = a;
+    }
+    return r;
+}
+
+
+QColor OxygenClient::titlebarTextColor(const QPalette &palette)
+{
+    if (isActive())
+        return palette.color(QPalette::Active, QPalette::WindowText);
+    else {
+        if(colorCacheInvalid_) {
+            QColor ab = palette.color(QPalette::Active, QPalette::Window);
+            QColor af = palette.color(QPalette::Active, QPalette::WindowText);
+            QColor nb = palette.color(QPalette::Inactive, QPalette::Window);
+            QColor nf = palette.color(QPalette::Inactive, QPalette::WindowText);
+
+            colorCacheInvalid_ = false;
+            cachedTitlebarTextColor_ = reduceContrast(nb, nf, qMax(qreal(2.5), KColorUtils::contrastRatio(ab, KColorUtils::mix(ab, af, 0.4))));
+        }
+        return cachedTitlebarTextColor_;
+    }
+}
+
+
 void OxygenClient::paintEvent(QPaintEvent *e)
 {
     Q_UNUSED(e)
@@ -213,10 +256,8 @@ void OxygenClient::paintEvent(QPaintEvent *e)
     QPalette palette = widget()->palette();
     QPainter painter(widget());
 
-    // ### - This feels like a kwin bug; the palette we get back always seems
-    // to be (incorrectly) using the Inactive group, which is wrong; active
-    // windows should have currentColorGroup() == Active. So, hack around it...
-    // I don't think a window can be disabled?
+    // Set palette to the right group.
+    // TODO - fix KWin to do this for us :-).
     if (isActive())
         palette.setCurrentColorGroup(QPalette::Active);
     else
@@ -224,7 +265,7 @@ void OxygenClient::paintEvent(QPaintEvent *e)
 
     int x,y,w,h;
     QRect frame = widget()->frameGeometry();
-    QColor color = palette.window();
+    QColor color = palette.window().color();
 
     const int titleHeight = layoutMetric(LM_TitleHeight);
     const int titleTop = layoutMetric(LM_TitleEdgeTop) + frame.top();
@@ -241,72 +282,90 @@ void OxygenClient::paintEvent(QPaintEvent *e)
 
     int splitY = qMin(300, 3*frame.height()/4);
 
-    QPixmap tile = globalHelper->verticalGradient(color, splitY);
+    QPixmap tile = helper_.verticalGradient(color, splitY);
     painter.drawTiledPixmap(QRect(0, 0, frame.width(), titleHeight + TFRAMESIZE), tile);
 
     painter.drawTiledPixmap(QRect(0, 0, LFRAMESIZE, splitY), tile);
-    painter.fillRect(0, splitY, LFRAMESIZE, frame.height() - splitY, globalHelper->backgroundBottomColor(color));
+    painter.fillRect(0, splitY, LFRAMESIZE, frame.height() - splitY, helper_.backgroundBottomColor(color));
 
     painter.drawTiledPixmap(QRect(frame.width()-RFRAMESIZE, 0,
                                                         RFRAMESIZE, splitY), tile,
                                                         QPoint(frame.width()-RFRAMESIZE, 0));
-    painter.fillRect(frame.width()-RFRAMESIZE, splitY, RFRAMESIZE, frame.height() - splitY, globalHelper->backgroundBottomColor(color));
+    painter.fillRect(frame.width()-RFRAMESIZE, splitY, RFRAMESIZE, frame.height() - splitY, helper_.backgroundBottomColor(color));
 
-    painter.fillRect(0, frame.height() - BFRAMESIZE, frame.width(), BFRAMESIZE, globalHelper->backgroundBottomColor(color));
+    painter.fillRect(0, frame.height() - BFRAMESIZE, frame.width(), BFRAMESIZE, helper_.backgroundBottomColor(color));
 
-    int radialW = qMin(600, frame.width());
-    tile = globalHelper->radialGradient(color, radialW);
-    QRect radialRect = QRect((frame.width() - radialW) / 2, 0, radialW, 64);
+    int radialW = qMin(600, frame.width() - LFRAMESIZE - RFRAMESIZE);
+    tile = helper_.radialGradient(color, radialW);
+    // In the style we assume the top frame to have a height of 32 this we need offset here with the
+    // actual value
+    int frameTopFixer = titleTop+titleHeight - 32;
+    QRect radialRect = QRect((frame.width() - radialW) / 2, frameTopFixer, radialW, 64);
     painter.drawPixmap(radialRect, tile);
 
-    //painter.setRenderHint(QPainter::Antialiasing,true);
-
     // draw title text
     painter.setFont(options()->font(isActive(), false));
-    painter.setPen(palette.windowText());
-    painter.drawText(titleLeft, titleTop, titleWidth, titleHeight,
+    painter.setPen(titlebarTextColor(palette));
+    painter.drawText(titleLeft, titleTop-1, titleWidth, titleHeight,  // -1 is to go into top resizearea
               OxygenFactory::titleAlign() | Qt::AlignVCenter, caption());
 
     painter.setRenderHint(QPainter::Antialiasing);
 
-    // shadows of the frame
+    // Draw dividing line
     frame = widget()->rect();
+    frame.adjust(1,1,-1,-1);
     frame.getRect(&x, &y, &w, &h);
 
-    painter.setBrush(Qt::NoBrush);
-    QLinearGradient lg(0, 0, 0, 10);
-    QGradientStops stops;
-    stops << QGradientStop( 0, QColor(255,255,255, 110) )
-           << QGradientStop( 1, QColor(128,128,128, 60) );
-    lg.setStops(stops);
-    painter.setPen(QPen(QBrush(lg),1));
-    painter.drawLine(QPointF(6.3, 0.5), QPointF(w-6.3, 0.5));
-    painter.drawArc(QRectF(0.5, 0.5, 9.5, 9.5),90*16, 90*16);
-    painter.drawArc(QRectF(w-9.5-0.5, 0.5, 9.5, 9.5), 0, 90*16);
+    QColor light = helper_.calcLightColor(color);
+    QColor dark = helper_.calcDarkColor(color);
+    dark.setAlpha(120);
 
-    painter.setPen(QColor(128,128,128, 60));
-    painter.drawLine(QPointF(0.5, 6.3), QPointF(0.5, h-6.3));
-    painter.drawLine(QPointF(w-0.5, 6.3), QPointF(w-0.5, h-6.3));
+    if(!isActive()) {
+        light.setAlpha(120);
+        dark.setAlpha(50);
+    }
 
-    lg = QLinearGradient(0, h-10, 0, h);
-    stops.clear();
-    stops << QGradientStop( 0, QColor(128,128,128, 60) )
-           << QGradientStop( 1, QColor(0,0,0, 50) );
-    lg.setStops(stops);
-    painter.setPen(QPen(QBrush(lg),1));
-    painter.drawArc(QRectF(0.5, h-9.5-0.5, 9.5, 9.5),180*16, 90*16);
-    painter.drawArc(QRectF(w-9.5-0.5, h-9.5-0.5, 9.5, 9.5), 270*16, 90*16);
-    painter.drawLine(QPointF(6.3, h-0.5), QPointF(w-6.3, h-0.5));
+    QLinearGradient lg(x,0,x+w,0);
+    lg.setColorAt(0.5, dark);
+    dark.setAlpha(0);
+    lg.setColorAt(0.0, dark);
+    lg.setColorAt(1.0, dark);
+    painter.setPen(QPen(lg,1));
 
-    qreal cenY = frame.height() / 2 + 0.5;
-    qreal posX = frame.width() - 2.5;
+    painter.drawLine(QPointF(x, titleTop+titleHeight-1.5),
+                            QPointF(x+w, titleTop+titleHeight-1.5));
+
+    lg = QLinearGradient(x,0,x+w,0);
+    lg.setColorAt(0.5, light);
+    light.setAlpha(0);
+    lg.setColorAt(0.0, light);
+    lg.setColorAt(1.0, light);
+    painter.setPen(QPen(lg,1));
+
+    painter.drawLine(QPointF(x, titleTop+titleHeight-0.5),
+                           QPointF(x+w, titleTop+titleHeight-0.5));
+
+    // Draw shadows of the frame
+    bool maximized = maximizeMode()==MaximizeFull && !options()->moveResizeMaximizedWindows();
+
+    if(maximized)
+        return;
+
+    helper_.drawFloatFrame(&painter, widget()->rect(), color);
+
+    if(!isResizable())
+        return;
+
+    // Draw the 3-dots resize handles
+    qreal cenY = frame.height() / 2 + x + 0.5;
+    qreal posX = frame.width() + y - 2.5;
     painter.setPen(Qt::NoPen);
     painter.setBrush(QColor(0, 0, 0, 66));
     renderDot(&painter, QPointF(posX, cenY - 3), 1.8);
     renderDot(&painter, QPointF(posX, cenY), 1.8);
     renderDot(&painter, QPointF(posX, cenY + 3), 1.8);
 
-    painter.translate(frame.width()-9, frame.height()-9);
+    painter.translate(x + frame.width()-9, y + frame.height()-9);
     renderDot(&painter, QPointF(2.5, 6.5), 1.8);
     renderDot(&painter, QPointF(5.5, 5.5), 1.8);
     renderDot(&painter, QPointF(6.5, 2.5), 1.8);
@@ -314,28 +373,20 @@ void OxygenClient::paintEvent(QPaintEvent *e)
 
 void OxygenClient::doShape()
 {
-  int r=widget()->width();
-  int b=widget()->height();
-QRegion mask(0,0,r,b);
-    // Remove top-left corner.
-    mask -= QRegion(0, 0, 3, 1);
-    mask -= QRegion(0, 1, 2, 1);
-    mask -= QRegion(0, 2, 1, 1);
+    bool maximized = maximizeMode()==MaximizeFull && !options()->moveResizeMaximizedWindows();
+    int w=widget()->width();
+    int h=widget()->height();
 
-    // Remove top-right corner.
-    mask -= QRegion(r - 3, 0, 3, 1);
-    mask -= QRegion(r - 2, 1, 2, 1);
-    mask -= QRegion(r - 1, 2, 1, 1);
+    if(maximized) {
+        QRegion mask(0,0,w,h);
+        setMask(mask);
+        return;
+    }
 
-    // Remove bottom-left corner.
-    mask -= QRegion(0, b-1-0, 3, b-1-1);
-    mask -= QRegion(0, b-1-1, 2, b-1-1);
-    mask -= QRegion(0, b-1-2, 1, b-1-1);
-
-    // Remove bottom-right corner.
-    mask -= QRegion(r - 3, b-1-0, 3, b-1-1);
-    mask -= QRegion(r - 2, b-1-1, 2, b-1-1);
-    mask -= QRegion(r - 1, b-1-2, 1, b-1-1);
+    QRegion mask(4, 0, w-8, h);
+    mask += QRegion(0, 4, w, h-8);
+    mask += QRegion(2, 1, w-4, h-2);
+    mask += QRegion(1, 2, w-2, h-4);
 
     setMask(mask);
 }
