@@ -31,7 +31,7 @@
 
 #include <kcommondecoration.h>
 
-#include "oxygenbutton.h"
+#include "lib/helper.h"
 
 class QPoint;
 
@@ -45,8 +45,6 @@ public:
     virtual ~OxygenClient();
 
     virtual QString visibleName() const;
-    virtual QString defaultButtonsLeft() const;
-    virtual QString defaultButtonsRight() const;
     virtual KCommonDecorationButton *createButton(::ButtonType type);
     virtual bool decorationBehaviour(DecorationBehaviour behaviour) const;
     virtual int layoutMetric(LayoutMetric lm, bool respectWindowState = true, const KCommonDecorationButton * = 0) const;
@@ -56,6 +54,13 @@ public:
 private:
     void paintEvent(QPaintEvent *e);
     void doShape();
+    QColor titlebarTextColor(const QPalette &palette);
+    bool colorCacheInvalid_;
+    QColor cachedTitlebarTextColor_;
+
+protected:
+    friend class OxygenButton;
+    OxygenHelper &helper_;
 };
 
 
