@@ -33,7 +33,10 @@
 
 #include "plastikbutton.moc"
 #include "plastikclient.h"
-#include "misc.h"
+
+#include <KGlobalSettings>
+#include <KColorScheme>
+#include <KColorUtils>
 
 namespace KWinPlastik
 {
@@ -174,34 +177,34 @@ void PlastikButton::drawButton(QPainter *painter)
     bool active = m_client->isActive();
     QPixmap tempPixmap;
 
-    QColor highlightColor;
-    if(type() == CloseButton) {
-        highlightColor = QColor(255,64,0);
-    } else {
-        highlightColor = Qt::white;
+
+    double c = KGlobalSettings::contrastF();
+    QColor titleBar = KDecoration::options()->color(KDecoration::ColorTitleBar, active);
+    QColor contourTop = KColorScheme::shade(titleBar, KColorScheme::DarkShade, c - 0.4);
+    QColor contourBottom = KColorScheme::shade(titleBar, KColorScheme::MidShade, c);
+    QColor surfaceTop = KColorScheme::shade(titleBar, KColorScheme::MidlightShade, c - 0.4);
+    QColor surfaceBottom = KColorScheme::shade(titleBar, KColorScheme::LightShade, c - 0.4);
+
+    QColor highlightColor = titleBar;
+    double alpha;
+    if (type() == CloseButton) {
+        KColorScheme kcs(active ? QPalette::Active : QPalette::Inactive, KColorScheme::Button);
+        highlightColor = kcs.foreground(KColorScheme::NegativeText).color();
+    }
+    if (isDown()) {
+        highlightColor = KColorScheme::shade(highlightColor, KColorScheme::ShadowShade);
+        alpha = 0.3;
+    }
+    else if (animProgress > 0) {
+        alpha = 0.6 * (double)animProgress / (double)ANIMATIONSTEPS;
+        highlightColor = KColorScheme::shade(highlightColor, KColorScheme::LightShade, qMin(1.0, c + 0.4));
     }
 
-    QColor contourTop = alphaBlendColors(Handler()->getColor(TitleGradient2, active),
-            Qt::black, 215);
-    QColor contourBottom = alphaBlendColors(Handler()->getColor(TitleGradient3, active),
-            Qt::black, 215);
-    QColor sourfaceTop = alphaBlendColors(Handler()->getColor(TitleGradient2, active),
-            Qt::white, 210);
-    QColor sourfaceBottom = alphaBlendColors(Handler()->getColor(TitleGradient3, active),
-            Qt::white, 210);
-
-    int highlightAlpha = static_cast<int>(255-((60/static_cast<double>(ANIMATIONSTEPS))*
-                                          static_cast<double>(animProgress) ) );
-    contourTop = alphaBlendColors(contourTop, highlightColor, highlightAlpha );
-    contourBottom = alphaBlendColors(contourBottom, highlightColor, highlightAlpha);
-    sourfaceTop = alphaBlendColors(sourfaceTop, highlightColor, highlightAlpha);
-    sourfaceBottom = alphaBlendColors(sourfaceBottom, highlightColor, highlightAlpha);
-
-    if (isDown() ) {
-        contourTop = alphaBlendColors(contourTop, Qt::black, 200);
-        contourBottom = alphaBlendColors(contourBottom, Qt::black, 200);
-        sourfaceTop = alphaBlendColors(sourfaceTop, Qt::black, 200);
-        sourfaceBottom = alphaBlendColors(sourfaceBottom, Qt::black, 200);
+    if (alpha > 0.0) {
+        contourTop    = KColorUtils::mix(contourTop,    highlightColor, alpha*0.4);
+        contourBottom = KColorUtils::mix(contourBottom, highlightColor, alpha*0.4);
+        surfaceTop    = KColorUtils::mix(surfaceTop,    highlightColor, alpha);
+        surfaceBottom = KColorUtils::mix(surfaceBottom, highlightColor, alpha);
     }
 
     QPixmap buffer(width(), height());
@@ -211,8 +214,10 @@ void PlastikButton::drawButton(QPainter *painter)
     bP.drawTiledPixmap(0, 0, width(), width(), m_client->getTitleBarTile(active) );
 
     if (type() != MenuButton || hover || animProgress != 0) {
-        int rx = 200/width();
-        int ry = 200/height();
+        int rxo = 600/width();
+        int ryo = 600/height();
+        int rxi = 500/width();
+        int ryi = 500/height();
         bP.setPen(Qt::NoPen);
         bP.setRenderHints(QPainter::Antialiasing);
         // contour
@@ -220,13 +225,13 @@ void PlastikButton::drawButton(QPainter *painter)
         outlineGradient.setColorAt(0.0, contourTop);
         outlineGradient.setColorAt(1.0, contourBottom);
         bP.setBrush(outlineGradient);
-        bP.drawRoundRect(r, rx, ry);
+        bP.drawRoundRect(r, rxo, ryo);
         // surface
         QLinearGradient surfaceGradient(0, 0, 0, r.height());
-        surfaceGradient.setColorAt(0.0, sourfaceTop);
-        surfaceGradient.setColorAt(1.0, sourfaceBottom);
+        surfaceGradient.setColorAt(0.0, surfaceTop);
+        surfaceGradient.setColorAt(1.0, surfaceBottom);
         bP.setBrush(surfaceGradient);
-        bP.drawRoundRect(r.adjusted(1,1,-1,-1), rx, ry);
+        bP.drawRoundRect(r.adjusted(1,1,-1,-1), rxi, ryi);
     }
 
     if (type() == MenuButton)
@@ -247,17 +252,15 @@ void PlastikButton::drawButton(QPainter *painter)
             dY++;
         }
 
+        QColor fontColor = Handler()->getColor(TitleFont,active);
         if(!isDown() && Handler()->titleShadow() ) {
-            QColor shadowColor;
-            if (qGray(Handler()->getColor(TitleFont,active).rgb()) < 100)
-                shadowColor = QColor(255, 255, 255, 75);
-            else
-                shadowColor = QColor(0,0,0,75);
+            QColor shadowColor = KColorScheme::shade(fontColor, KColorScheme::ShadowShade);
+            shadowColor.setAlphaF(shadowColor.alphaF() * 0.3);
             bP.setPen(shadowColor);
             bP.drawPixmap(dX+1, dY+1, icon);
         }
 
-        bP.setPen(Handler()->getColor(TitleFont,active) );
+        bP.setPen(fontColor );
         bP.drawPixmap(dX, dY, icon);
     }
 
