@@ -1,16 +1,28 @@
-/*
- *
- * Redmond KWin client
- *
- * Copyright 2001
- *   Karol Szwed <gallium@kde.org>
- *   http://gallium.n3.net/
- *
- * Based on the default KWin client.
- *
- * Updated to support toolwindows 3/2001 (KS)
- *
- */
+/********************************************************************
+ 
+  Redmond KWin client
+ 
+  Copyright 2001
+    Karol Szwed <gallium@kde.org>
+    http://gallium.n3.net/
+ 
+  Based on the default KWin client.
+ 
+  Updated to support toolwindows 3/2001 (KS)
+
+This program is free software; you can redistribute it and/or modify
+it under the terms of the GNU General Public License as published by
+the Free Software Foundation; either version 2 of the License, or
+(at your option) any later version.
+
+This program is distributed in the hope that it will be useful,
+but WITHOUT ANY WARRANTY; without even the implied warranty of
+MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+GNU General Public License for more details.
+
+You should have received a copy of the GNU General Public License
+along with this program.  If not, see <http://www.gnu.org/licenses/>.
+*********************************************************************/
 
 #include "redmond.h"
 
@@ -431,9 +443,6 @@ int RedmondDeco::layoutMetric(LayoutMetric lm, bool respectWindowState, const KC
 		case LM_ButtonSpacing:
 			return 0;
 
-		case LM_ExplicitButtonSpacer:
-			return 2;
-
 		default:
 			return KCommonDecoration::layoutMetric(lm, respectWindowState, btn);
 	}
@@ -462,9 +471,9 @@ void RedmondDeco::init()
 {
 //	Finally, toolwindows look small
 	if ( isToolWindow() ) {
-		titleHeight = toolTitleHeight+2;
+		titleHeight = toolTitleHeight;
 	} else {
-		titleHeight = normalTitleHeight+2;
+		titleHeight = normalTitleHeight;
 	}
 
     KCommonDecoration::init();
@@ -640,7 +649,7 @@ RedmondDecoFactory::~RedmondDecoFactory()
 
 KDecoration *RedmondDecoFactory::createDecoration( KDecorationBridge *b )
 {
-	return new RedmondDeco(b, this);
+	return (new RedmondDeco(b, this))->decoration();
 }
 
 bool RedmondDecoFactory::reset( unsigned long changed )
@@ -658,17 +667,24 @@ bool RedmondDecoFactory::reset( unsigned long changed )
 	}
 }
 
-bool RedmondDecoFactory::supports( Ability ability )
+bool RedmondDecoFactory::supports( Ability ability ) const
 {
 	switch( ability )
 	{
+		// announce
 		case AbilityAnnounceButtons:
+		case AbilityAnnounceColors:
+		// buttons
 		case AbilityButtonMenu:
 		case AbilityButtonHelp:
 		case AbilityButtonMinimize:
 		case AbilityButtonMaximize:
 		case AbilityButtonClose:
 		case AbilityButtonSpacer:
+		// colors
+		case AbilityColorTitleBack:
+		case AbilityColorTitleBlend:
+		case AbilityColorTitleFore:
 			return true;
 		default:
 			return false;
