@@ -1,12 +1,22 @@
-/*****************************************************************
+/********************************************************************
  KWin - the KDE window manager
  This file is part of the KDE project.
 
 Copyright (C) 2006 Lubos Lunak <l.lunak@kde.org>
 
-You can Freely distribute this program under the GNU General Public
-License. See the file "COPYING" for the exact licensing terms.
-******************************************************************/
+This program is free software; you can redistribute it and/or modify
+it under the terms of the GNU General Public License as published by
+the Free Software Foundation; either version 2 of the License, or
+(at your option) any later version.
+
+This program is distributed in the hope that it will be useful,
+but WITHOUT ANY WARRANTY; without even the implied warranty of
+MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+GNU General Public License for more details.
+
+You should have received a copy of the GNU General Public License
+along with this program.  If not, see <http://www.gnu.org/licenses/>.
+*********************************************************************/
 
 /*
  Code related to compositing (redirecting windows to pixmaps and tracking
@@ -66,7 +76,7 @@ namespace KWin
 
 void Workspace::setupCompositing()
     {
-#if defined( HAVE_XCOMPOSITE ) && defined( HAVE_XDAMAGE )
+#ifdef KWIN_HAVE_COMPOSITING
     if( !options->useCompositing )
         {
         kDebug( 1212 ) << "Compositing is turned off in options";
@@ -99,6 +109,7 @@ void Workspace::setupCompositing()
                 break;
             default:
                 kDebug( 1212 ) << "No compositing";
+                delete cm_selection;
                 return;
             }
         }
@@ -109,7 +120,7 @@ void Workspace::setupCompositing()
             kDebug( 1212 ) << "X compositing";
             scene = new SceneBasic( this );
           break; // don't fall through (this is a testing one) */
-#ifdef HAVE_OPENGL
+#ifdef KWIN_HAVE_OPENGL_COMPOSITING
         case OpenGLCompositing:
             kDebug( 1212 ) << "OpenGL compositing";
             scene = new SceneOpenGL( this );
@@ -117,16 +128,21 @@ void Workspace::setupCompositing()
                 break; // -->
             delete scene;
             scene = NULL;
-            // fall through, try XRender
+            break; // do not fall back to XRender for now, maybe in the future
 #endif
-#if defined(HAVE_XRENDER) && defined(HAVE_XFIXES)
+#ifdef KWIN_HAVE_XRENDER_COMPOSITING
         case XRenderCompositing:
             kDebug( 1212 ) << "XRender compositing";
             scene = new SceneXrender( this );
           break;
 #endif
         default:
-          kDebug( 1212 ) << "No compositing";
+#ifndef KWIN_HAVE_COMPOSITING
+            kDebug( 1212 ) << "Compositing was not available at compile time";
+#else
+            kDebug( 1212 ) << "No compositing";
+#endif
+            delete cm_selection;
           return;
         }
     if( scene == NULL || scene->initFailed())
@@ -191,7 +207,7 @@ void Workspace::setupCompositing()
 
 void Workspace::finishCompositing()
     {
-#if defined( HAVE_XCOMPOSITE ) && defined( HAVE_XDAMAGE )
+#ifdef KWIN_HAVE_COMPOSITING
     if( scene == NULL )
         return;
     delete cm_selection;
@@ -262,16 +278,17 @@ void Workspace::addRepaintFull()
 
 void Workspace::performCompositing()
     {
-#if defined( HAVE_XCOMPOSITE ) && defined( HAVE_XDAMAGE )
+#ifdef KWIN_HAVE_COMPOSITING
     // The event loop apparently tries to fire a QTimer as often as possible, even
     // at the expense of not processing many X events. This means that the composite
     // repaints can seriously impact performance of everything else, therefore throttle
-    // them - leave at least 5msec time after one repaint is finished and next one
+    // them - leave at least 1msec time after one repaint is finished and next one
     // is started.
-    if( lastCompositePaint.elapsed() < 5 )
+    if( lastCompositePaint.elapsed() < 1 )
         return;
     checkCursorPos();
-    if( repaints_region.isEmpty() && !windowRepaintsPending()) // no damage
+    if(( repaints_region.isEmpty() && !windowRepaintsPending()) // no damage
+        || !overlay_visible ) // nothing is visible anyway
         {
         scene->idle();
         return;
@@ -365,6 +382,7 @@ void Workspace::setupOverlay( Window w )
         XMapWindow( display(), w );
         }
     XMapRaised( display(), overlay );
+    XSelectInput( display(), overlay, VisibilityChangeMask );
     }
 
 void Workspace::destroyOverlay()
@@ -383,7 +401,7 @@ void Workspace::destroyOverlay()
 
 void Toplevel::setupCompositing()
     {
-#if defined( HAVE_XCOMPOSITE ) && defined( HAVE_XDAMAGE )
+#ifdef KWIN_HAVE_COMPOSITING
     if( !compositing())
         return;
     if( damage_handle != None )
@@ -397,7 +415,7 @@ void Toplevel::setupCompositing()
 
 void Toplevel::finishCompositing()
     {
-#if defined( HAVE_XCOMPOSITE ) && defined( HAVE_XDAMAGE )
+#ifdef KWIN_HAVE_COMPOSITING
     if( damage_handle == None )
         return;
     if( effect_window->window() == this ) // otherwise it's already passed to Deleted, don't free data
@@ -426,7 +444,7 @@ void Toplevel::discardWindowPixmap()
 
 Pixmap Toplevel::createWindowPixmap()
     {
-#ifdef HAVE_XCOMPOSITE
+#ifdef KWIN_HAVE_COMPOSITING
     assert( compositing());
     grabXServer();
     KXErrorHandler err;
@@ -546,6 +564,8 @@ void Toplevel::addWorkspaceRepaint( const QRect& r2 )
     {
     if( !compositing())
         return;
+    if( effectWindow() == NULL ) // TODO - this can happen during window destruction
+        return workspace()->addRepaint( r2 );
     QRect r = effects->transformWindowDamage( effectWindow(), r2 );
     workspace()->addRepaint( r );
     }
