@@ -1,12 +1,22 @@
-/*****************************************************************
+/********************************************************************
  KWin - the KDE window manager
  This file is part of the KDE project.
 
 Copyright (C) 2006 Lubos Lunak <l.lunak@kde.org>
 
-You can Freely distribute this program under the GNU General Public
-License. See the file "COPYING" for the exact licensing terms.
-******************************************************************/
+This program is free software; you can redistribute it and/or modify
+it under the terms of the GNU General Public License as published by
+the Free Software Foundation; either version 2 of the License, or
+(at your option) any later version.
+
+This program is distributed in the hope that it will be useful,
+but WITHOUT ANY WARRANTY; without even the implied warranty of
+MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+GNU General Public License for more details.
+
+You should have received a copy of the GNU General Public License
+along with this program.  If not, see <http://www.gnu.org/licenses/>.
+*********************************************************************/
 
 #include "effects.h"
 
@@ -15,6 +25,7 @@ License. See the file "COPYING" for the exact licensing terms.
 #include "group.h"
 #include "scene_xrender.h"
 #include "scene_opengl.h"
+#include "unmanaged.h"
 #include "workspace.h"
 #include "kwinglutils.h"
 
@@ -39,6 +50,7 @@ namespace KWin
 EffectsHandlerImpl::EffectsHandlerImpl(CompositingType type)
     : EffectsHandler(type)
     , keyboard_grab_effect( NULL )
+    , fullscreen_effect( 0 )
     {
     reconfigure();
     }
@@ -258,6 +270,16 @@ void EffectsHandlerImpl::tabBoxUpdated()
         ep.second->tabBoxUpdated();
     }
 
+void EffectsHandlerImpl::setActiveFullScreenEffect( Effect* e )
+    {
+    fullscreen_effect = e;
+    }
+
+Effect* EffectsHandlerImpl::activeFullScreenEffect() const
+    {
+    return fullscreen_effect;
+    }
+
 bool EffectsHandlerImpl::borderActivated( ElectricBorder border )
     {
     bool ret = false;
@@ -267,11 +289,12 @@ bool EffectsHandlerImpl::borderActivated( ElectricBorder border )
     return ret;
     }
 
-void EffectsHandlerImpl::mouseChanged( const QPoint& pos, const QPoint& old,
-    Qt::MouseButtons buttons, Qt::KeyboardModifiers modifiers )
+void EffectsHandlerImpl::mouseChanged( const QPoint& pos, const QPoint& oldpos,
+    Qt::MouseButtons buttons, Qt::MouseButtons oldbuttons,
+    Qt::KeyboardModifiers modifiers, Qt::KeyboardModifiers oldmodifiers )
     {
     foreach( EffectPair ep, loaded_effects )
-        ep.second->mouseChanged( pos, old, buttons, modifiers );
+        ep.second->mouseChanged( pos, oldpos, buttons, oldbuttons, modifiers, oldmodifiers );
     }
 
 bool EffectsHandlerImpl::grabKeyboard( Effect* effect )
@@ -303,6 +326,25 @@ bool EffectsHandlerImpl::hasKeyboardGrab() const
     return keyboard_grab_effect != NULL;
     }
 
+void EffectsHandlerImpl::propertyNotify( EffectWindow* c, long atom )
+    {
+    if( !registered_atoms.contains( atom ))
+        return;
+    foreach( EffectPair ep, loaded_effects )
+        ep.second->propertyNotify( c, atom );
+    }
+
+void EffectsHandlerImpl::registerPropertyType( long atom, bool reg )
+    {
+    if( reg )
+        ++registered_atoms[ atom ]; // initialized to 0 if not present yet
+    else
+        {
+        if( --registered_atoms[ atom ] == 0 )
+            registered_atoms.remove( atom );
+        }
+    }
+
 void EffectsHandlerImpl::activateWindow( EffectWindow* c )
     {
     if( Client* cl = dynamic_cast< Client* >( static_cast<EffectWindowImpl*>(c)->window()))
@@ -324,7 +366,7 @@ void EffectsHandlerImpl::moveWindow( EffectWindow* w, const QPoint& pos )
 void EffectsHandlerImpl::windowToDesktop( EffectWindow* w, int desktop )
     {
     Client* cl = dynamic_cast< Client* >( static_cast<EffectWindowImpl*>(w)->window());
-    if( cl && cl->isMovable())
+    if( cl && !cl->isDesktop() && !cl->isDock() && !cl->isTopMenu())
         Workspace::self()->sendClientToDesktop( cl, desktop, true );
     }
 
@@ -358,6 +400,26 @@ bool EffectsHandlerImpl::optionRollOverDesktops() const
     return options->rollOverDesktops;
     }
 
+int EffectsHandlerImpl::desktopToLeft( int desktop, bool wrap ) const
+    {
+    return Workspace::self()->desktopToLeft( desktop, wrap );
+    }
+
+int EffectsHandlerImpl::desktopToRight( int desktop, bool wrap ) const
+    {
+    return Workspace::self()->desktopToRight( desktop, wrap );
+    }
+
+int EffectsHandlerImpl::desktopUp( int desktop, bool wrap ) const
+    {
+    return Workspace::self()->desktopUp( desktop, wrap );
+    }
+
+int EffectsHandlerImpl::desktopDown( int desktop, bool wrap ) const
+    {
+    return Workspace::self()->desktopDown( desktop, wrap );
+    }
+
 int EffectsHandlerImpl::displayWidth() const
     {
     return KWin::displayWidth();
@@ -368,6 +430,15 @@ int EffectsHandlerImpl::displayHeight() const
     return KWin::displayWidth();
     }
 
+EffectWindow* EffectsHandlerImpl::findWindow( WId id ) const
+    {
+    if( Client* w = Workspace::self()->findClient( WindowMatchPredicate( id )))
+        return w->effectWindow();
+    if( Unmanaged* w = Workspace::self()->findUnmanaged( WindowMatchPredicate( id )))
+        return w->effectWindow();
+    return NULL;
+    }
+
 EffectWindowList EffectsHandlerImpl::stackingOrder() const
     {
     ClientList list = Workspace::self()->stackingOrder();
@@ -439,7 +510,7 @@ EffectWindow* EffectsHandlerImpl::currentTabBoxWindow() const
 
 void EffectsHandlerImpl::pushRenderTarget(GLRenderTarget* target)
 {
-#ifdef HAVE_OPENGL
+#ifdef KWIN_HAVE_OPENGL_COMPOSITING
     target->enable();
     render_targets.push(target);
 #endif
@@ -447,7 +518,7 @@ void EffectsHandlerImpl::pushRenderTarget(GLRenderTarget* target)
 
 GLRenderTarget* EffectsHandlerImpl::popRenderTarget()
 {
-#ifdef HAVE_OPENGL
+#ifdef KWIN_HAVE_OPENGL_COMPOSITING
     GLRenderTarget* ret = render_targets.pop();
     ret->disable();
     if( !render_targets.isEmpty() )
@@ -473,6 +544,25 @@ void EffectsHandlerImpl::addRepaint( int x, int y, int w, int h )
     Workspace::self()->addRepaint( x, y, w, h );
     }
 
+int EffectsHandlerImpl::activeScreen() const
+    {
+    return Workspace::self()->activeScreen();
+    }
+
+QRect EffectsHandlerImpl::clientArea( clientAreaOption opt, int screen, int desktop ) const
+    {
+    return Workspace::self()->clientArea( opt, screen, desktop );
+    }
+
+QRect EffectsHandlerImpl::clientArea( clientAreaOption opt, const EffectWindow* c ) const
+    {
+    const Toplevel* t = static_cast< const EffectWindowImpl* >(c)->window();
+    if( const Client* cl = dynamic_cast< const Client* >( t ))
+        return Workspace::self()->clientArea( opt, cl );
+    else
+        return Workspace::self()->clientArea( opt, t->geometry().center(), Workspace::self()->currentDesktop());
+    }
+
 QRect EffectsHandlerImpl::clientArea( clientAreaOption opt, const QPoint& p, int desktop ) const
     {
     return Workspace::self()->clientArea( opt, p, desktop );
@@ -594,7 +684,7 @@ void EffectsHandlerImpl::reserveElectricBorderSwitching( bool reserve )
 
 unsigned long EffectsHandlerImpl::xrenderBufferPicture()
     {
-#if defined(HAVE_XRENDER) && defined(HAVE_XFIXES)
+#ifdef KWIN_HAVE_XRENDER_COMPOSITING
     if( SceneXrender* s = dynamic_cast< SceneXrender* >( scene ))
         return s->bufferPicture();
 #endif
@@ -663,6 +753,24 @@ bool EffectsHandlerImpl::loadEffect( const QString& name )
         return false;
         }
 
+    QString version_symbol = "effect_version_" + name;
+    KLibrary::void_function_ptr version_func = library->resolveFunction(version_symbol.toAscii());
+    if( version_func == NULL )
+        {
+        kWarning( 1212 ) << "Effect " << name << " does not provide required API version, ignoring.";
+        return false;
+        }
+    typedef int (*t_versionfunc)();
+    int version = reinterpret_cast< t_versionfunc >( version_func )(); // call it
+    // Version must be the same or less, but major must be the same.
+    // With major 0 minor must match exactly.
+    if( version > KWIN_EFFECT_API_VERSION
+        || ( version >> 8 ) != KWIN_EFFECT_API_VERSION_MAJOR
+        || ( KWIN_EFFECT_API_VERSION_MAJOR == 0 && version != KWIN_EFFECT_API_VERSION ))
+        {
+        kWarning( 1212 ) << "Effect " << name << " requires unsupported API version " << version;
+        return false;
+        }
     QString supported_symbol = "effect_supported_" + name;
     KLibrary::void_function_ptr supported_func = library->resolveFunction(supported_symbol.toAscii().data());
     QString create_symbol = "effect_create_" + name;
@@ -723,6 +831,10 @@ void EffectsHandlerImpl::unloadEffect( const QString& name )
         if ( it.value().first == name )
             {
             kDebug( 1212 ) << "EffectsHandler::unloadEffect : Unloading Effect : " << name;
+            if( activeFullScreenEffect() == it.value().second )
+                {
+                setActiveFullScreenEffect( 0 );
+                }
             delete it.value().second;
             effect_order.erase(it);
             effectsChanged();
@@ -755,10 +867,10 @@ bool EffectsHandlerImpl::isEffectLoaded( const QString& name )
 void EffectsHandlerImpl::effectsChanged()
     {
     loaded_effects.clear();
-    kDebug() << "Recreating effects' list:";
+    kDebug(1212) << "Recreating effects' list:";
     foreach( EffectPair effect, effect_order )
         {
-        kDebug() << effect.first;
+        kDebug(1212) << effect.first;
         loaded_effects.append( effect );
         }
     }
@@ -831,6 +943,11 @@ QString EffectWindowImpl::windowClass() const
     return toplevel->resourceName() + ' ' + toplevel->resourceClass();
     }
 
+QString EffectWindowImpl::windowRole() const
+    {
+    return toplevel->windowRole();
+    }
+
 QPixmap EffectWindowImpl::icon() const
     {
     if( Client* c = dynamic_cast<Client*>( toplevel ))
@@ -912,6 +1029,16 @@ QRect EffectWindowImpl::geometry() const
     return toplevel->geometry();
     }
 
+QRegion EffectWindowImpl::shape() const
+    {
+    return sw ? sw->shape() : geometry();
+    }
+
+bool EffectWindowImpl::hasOwnShape() const
+    {
+    return toplevel->shape();
+    }
+
 QSize EffectWindowImpl::size() const
     {
     return toplevel->size();
@@ -932,6 +1059,42 @@ QRect EffectWindowImpl::contentsRect() const
     return QRect( toplevel->clientPos(), toplevel->clientSize());
     }
 
+QByteArray EffectWindowImpl::readProperty( long atom, long type, int format ) const
+    {
+    int len = 32768;
+    for(;;)
+        {
+        unsigned char* data;
+        Atom rtype;
+        int rformat;
+        unsigned long nitems, after;
+        if( XGetWindowProperty( QX11Info::display(), window()->window(),
+            atom, 0, len, False, AnyPropertyType,
+            &rtype, &rformat, &nitems, &after, &data ) == Success )
+            {
+            if( after > 0 )
+                {
+                XFree( data );
+                len *= 2;
+                continue;
+                }
+            if( long( rtype ) == type && rformat == format )
+                {
+                QByteArray ret( reinterpret_cast< const char* >( data ), nitems * ( format / 8 ));
+                XFree( data );
+                return ret;
+                }
+            else // wrong format, type or something
+                {
+                XFree( data );
+                return QByteArray();
+                }
+            }
+        else // XGetWindowProperty() failed
+            return QByteArray();
+        }
+    }
+
 bool EffectWindowImpl::isMovable() const
     {
     if( Client* c = dynamic_cast< Client* >( toplevel ))
@@ -995,7 +1158,7 @@ bool EffectWindowImpl::isSpecialWindow() const
     if( Client* c = dynamic_cast<Client*>( toplevel ))
         return c->isSpecialWindow();
     else
-        return false;
+        return true;
     }
 
 bool EffectWindowImpl::isDialog() const
@@ -1043,6 +1206,11 @@ bool EffectWindowImpl::isDNDIcon() const
     return toplevel->isDNDIcon();
     }
 
+bool EffectWindowImpl::isManaged() const
+    {
+    return dynamic_cast< const Client* >( toplevel ) != NULL;
+    }
+
 bool EffectWindowImpl::isModal() const
     {
     if( Client* c = dynamic_cast< Client* >( toplevel ))
