@@ -1,12 +1,22 @@
-/*****************************************************************
+/********************************************************************
  KWin - the KDE window manager
  This file is part of the KDE project.
 
 Copyright (C) 2006 Lubos Lunak <l.lunak@kde.org>
 
-You can Freely distribute this program under the GNU General Public
-License. See the file "COPYING" for the exact licensing terms.
-******************************************************************/
+This program is free software; you can redistribute it and/or modify
+it under the terms of the GNU General Public License as published by
+the Free Software Foundation; either version 2 of the License, or
+(at your option) any later version.
+
+This program is distributed in the hope that it will be useful,
+but WITHOUT ANY WARRANTY; without even the implied warranty of
+MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+GNU General Public License for more details.
+
+You should have received a copy of the GNU General Public License
+along with this program.  If not, see <http://www.gnu.org/licenses/>.
+*********************************************************************/
 
 #ifndef KWIN_EFFECTSIMPL_H
 #define KWIN_EFFECTSIMPL_H
@@ -53,6 +63,7 @@ class EffectsHandlerImpl : public EffectsHandler
         virtual QPoint cursorPos() const;
         virtual bool grabKeyboard( Effect* effect );
         virtual void ungrabKeyboard();
+        virtual EffectWindow* findWindow( WId id ) const;
         virtual EffectWindowList stackingOrder() const;
         virtual void setElevatedWindow( EffectWindow* w, bool set );
 
@@ -66,17 +77,28 @@ class EffectsHandlerImpl : public EffectsHandler
         virtual int currentTabBoxDesktop() const;
         virtual EffectWindow* currentTabBoxWindow() const;
 
+        virtual void setActiveFullScreenEffect( Effect* e );
+        virtual Effect* activeFullScreenEffect() const;
+
         virtual void pushRenderTarget(GLRenderTarget* target);
         virtual GLRenderTarget* popRenderTarget();
 
         virtual void addRepaintFull();
         virtual void addRepaint( const QRect& r );
         virtual void addRepaint( int x, int y, int w, int h );
-        virtual QRect clientArea( clientAreaOption opt, const QPoint& p, int desktop ) const;
+        virtual int activeScreen() const;
+        virtual QRect clientArea( clientAreaOption, int screen, int desktop ) const;
+        virtual QRect clientArea( clientAreaOption, const EffectWindow* c ) const;
+        virtual QRect clientArea( clientAreaOption, const QPoint& p, int desktop ) const;
         virtual void calcDesktopLayout(int* x, int* y, Qt::Orientation* orientation) const;
         virtual bool optionRollOverDesktops() const;
+        virtual int desktopToLeft( int desktop, bool wrap ) const;
+        virtual int desktopToRight( int desktop, bool wrap ) const;
+        virtual int desktopUp( int desktop, bool wrap ) const;
+        virtual int desktopDown( int desktop, bool wrap ) const;
 
         virtual Window createInputWindow( Effect* e, int x, int y, int w, int h, const QCursor& cursor );
+        using EffectsHandler::createInputWindow;
         virtual void destroyInputWindow( Window w );
         virtual bool checkInputWindowEvent( XEvent* e );
         virtual void checkInputWindowStacking();
@@ -88,6 +110,7 @@ class EffectsHandlerImpl : public EffectsHandler
 
         virtual unsigned long xrenderBufferPicture();
         virtual void reconfigure();
+        virtual void registerPropertyType( long atom, bool reg );
 
         // internal (used by kwin core or compositing code)
         void startPaint();
@@ -106,10 +129,12 @@ class EffectsHandlerImpl : public EffectsHandler
         void tabBoxClosed();
         void tabBoxUpdated();
         bool borderActivated( ElectricBorder border );
-        void mouseChanged( const QPoint& pos, const QPoint& old,
-            Qt::MouseButtons buttons, Qt::KeyboardModifiers modifiers );
+        void mouseChanged( const QPoint& pos, const QPoint& oldpos,
+            Qt::MouseButtons buttons, Qt::MouseButtons oldbuttons,
+            Qt::KeyboardModifiers modifiers, Qt::KeyboardModifiers oldmodifiers );
         void grabbedKeyboardEvent( QKeyEvent* e );
         bool hasKeyboardGrab() const;
+        void propertyNotify( EffectWindow* c, long atom );
 
         bool loadEffect( const QString& name );
         void toggleEffect( const QString& name );
@@ -125,8 +150,10 @@ class EffectsHandlerImpl : public EffectsHandler
 
         Effect* keyboard_grab_effect;
         QStack<GLRenderTarget*> render_targets;
+        Effect* fullscreen_effect;
         ToplevelList elevated_windows;
         QMultiMap< int, EffectPair > effect_order;
+        QHash< long, int > registered_atoms;
 };
 
 class EffectWindowImpl : public EffectWindow
@@ -153,6 +180,7 @@ class EffectWindowImpl : public EffectWindow
         virtual QString caption() const;
         virtual QPixmap icon() const;
         virtual QString windowClass() const;
+        virtual QString windowRole() const;
         virtual const EffectWindowGroup* group() const;
 
         virtual int x() const;
@@ -160,6 +188,8 @@ class EffectWindowImpl : public EffectWindow
         virtual int width() const;
         virtual int height() const;
         virtual QRect geometry() const;
+        virtual QRegion shape() const;
+        virtual bool hasOwnShape() const;
         virtual QPoint pos() const;
         virtual QSize size() const;
         virtual QRect rect() const;
@@ -168,6 +198,7 @@ class EffectWindowImpl : public EffectWindow
         virtual bool isUserResize() const;
         virtual QRect iconGeometry() const;
         virtual QRect contentsRect() const;
+        virtual QByteArray readProperty( long atom, long type, int format ) const;
 
         virtual bool isDesktop() const;
         virtual bool isDock() const;
@@ -185,6 +216,7 @@ class EffectWindowImpl : public EffectWindow
         virtual bool isNotification() const;
         virtual bool isComboBox() const;
         virtual bool isDNDIcon() const;
+        virtual bool isManaged() const; // managed or override-redirect
 
         virtual bool isModal() const;
         virtual EffectWindow* findModal();
