@@ -1,14 +1,24 @@
-/*****************************************************************
+/********************************************************************
  KWin - the KDE window manager
  This file is part of the KDE project.
 
 Copyright (C) 2007 Philip Falkner <philip.falkner@gmail.com>
 
-You can Freely distribute this program under the GNU General Public
-License. See the file "COPYING" for the exact licensing terms.
-******************************************************************/
+This program is free software; you can redistribute it and/or modify
+it under the terms of the GNU General Public License as published by
+the Free Software Foundation; either version 2 of the License, or
+(at your option) any later version.
 
-#include <config-X11.h>
+This program is distributed in the hope that it will be useful,
+but WITHOUT ANY WARRANTY; without even the implied warranty of
+MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+GNU General Public License for more details.
+
+You should have received a copy of the GNU General Public License
+along with this program.  If not, see <http://www.gnu.org/licenses/>.
+*********************************************************************/
+
+#include <kwinconfig.h>
 
 #include "boxswitch.h"
 
@@ -20,7 +30,7 @@ License. See the file "COPYING" for the exact licensing terms.
 #include <kapplication.h>
 #include <kcolorscheme.h>
 
-#ifdef HAVE_OPENGL
+#ifdef KWIN_HAVE_OPENGL_COMPOSITING
 #include <GL/gl.h>
 #endif
 
@@ -36,33 +46,32 @@ BoxSwitchEffect::BoxSwitchEffect()
     {
     frame_margin = 10;
     highlight_margin = 5;
-#ifdef HAVE_XRENDER
+#ifdef KWIN_HAVE_XRENDER_COMPOSITING
     alphaFormat = XRenderFindStandardFormat( display(), PictStandardARGB32 );
 #endif
-    color_frame = KColorScheme( QPalette::Active, KColorScheme::Window ).background();
+    color_frame = KColorScheme( QPalette::Active, KColorScheme::Window ).background().color();
     color_frame.setAlphaF( 0.9 );
-    color_highlight = KColorScheme( QPalette::Active, KColorScheme::Selection ).background();
+    color_highlight = KColorScheme( QPalette::Active, KColorScheme::Selection ).background().color();
     color_highlight.setAlphaF( 0.9 );
-    color_text = KColorScheme( QPalette::Active, KColorScheme::Window ).foreground();
+    color_text = KColorScheme( QPalette::Active, KColorScheme::Window ).foreground().color();
     }
 
 BoxSwitchEffect::~BoxSwitchEffect()
     {
     }
 
-void BoxSwitchEffect::prePaintScreen( ScreenPrePaintData& data, int time )
-    {
-    effects->prePaintScreen( data, time );
-    }
-
 void BoxSwitchEffect::prePaintWindow( EffectWindow* w, WindowPrePaintData& data, int time )
     {
     if( mActivated )
         {
         if( mMode == TabBoxWindowsMode )
             {
-            if( windows.contains( w ) && w != selected_window )
-                data.setTranslucent();
+            if( windows.contains( w ))
+                {
+                if( w != selected_window )
+                    data.setTranslucent();
+                w->enablePainting( EffectWindow::PAINT_DISABLED_BY_MINIMIZE );
+                }
             }
         else
             {
@@ -313,8 +322,7 @@ void BoxSwitchEffect::setActive()
         {
         foreach( EffectWindow* w, windows.keys())
             {
-            if( w != selected_window )
-                w->addRepaintFull();
+            w->addRepaintFull();
             }
         }
     }
@@ -337,7 +345,7 @@ void BoxSwitchEffect::setInactive()
             }
         foreach( ItemInfo* i, windows )
             {
-#ifdef HAVE_XRENDER
+#ifdef KWIN_HAVE_XRENDER_COMPOSITING
         if( effects->compositingType() == XRenderCompositing )
                 {
                 if( i->iconPicture != None )
@@ -389,7 +397,8 @@ void BoxSwitchEffect::calculateFrameSize()
     const int separator_height = 6;
     // Shrink the size until all windows/desktops can fit onscreen
     frame_area.setWidth( frame_margin * 2 + itemcount * item_max_size.width());
-    while( frame_area.width() > displayWidth())
+    QRect screenr = effects->clientArea( PlacementArea, effects->activeScreen(), effects->currentDesktop());
+    while( frame_area.width() > screenr.width())
         {
         item_max_size /= 2;
         frame_area.setWidth( frame_margin * 2 + itemcount * item_max_size.width());
@@ -398,7 +407,8 @@ void BoxSwitchEffect::calculateFrameSize()
             separator_height + text_area.height());
     text_area.setWidth( frame_area.width() - frame_margin * 2 );
 
-    frame_area.moveTo(( displayWidth() - frame_area.width()) / 2, ( displayHeight() - frame_area.height()) / 2 );
+    frame_area.moveTo( screenr.x() + ( screenr.width() - frame_area.width()) / 2,
+        screenr.y() + ( screenr.height() - frame_area.height()) / 2 );
     text_area.moveTo( frame_area.x() + frame_margin,
                       frame_area.y() + frame_margin + item_max_size.height() + separator_height);
     }
@@ -439,7 +449,7 @@ void BoxSwitchEffect::calculateItemSizes()
 
 void BoxSwitchEffect::paintFrame()
     {
-#ifdef HAVE_OPENGL
+#ifdef KWIN_HAVE_OPENGL_COMPOSITING
     if( effects->compositingType() == OpenGLCompositing )
         {
         glPushAttrib( GL_CURRENT_BIT );
@@ -448,7 +458,7 @@ void BoxSwitchEffect::paintFrame()
         glPopAttrib();
         }
 #endif
-#ifdef HAVE_XRENDER
+#ifdef KWIN_HAVE_XRENDER_COMPOSITING
     if( effects->compositingType() == XRenderCompositing )
         {
         Pixmap pixmap = XCreatePixmap( display(), rootWindow(),
@@ -472,7 +482,7 @@ void BoxSwitchEffect::paintFrame()
 
 void BoxSwitchEffect::paintHighlight( QRect area )
     {
-#ifdef HAVE_OPENGL
+#ifdef KWIN_HAVE_OPENGL_COMPOSITING
     if( effects->compositingType() == OpenGLCompositing )
         {
         glPushAttrib( GL_CURRENT_BIT );
@@ -481,7 +491,7 @@ void BoxSwitchEffect::paintHighlight( QRect area )
         glPopAttrib();
         }
 #endif
-#ifdef HAVE_XRENDER
+#ifdef KWIN_HAVE_XRENDER_COMPOSITING
     if( effects->compositingType() == XRenderCompositing )
         {
         Pixmap pixmap = XCreatePixmap( display(), rootWindow(),
@@ -555,17 +565,17 @@ void BoxSwitchEffect::paintWindowIcon( EffectWindow* w )
         return;
         }
 
-    if( windows[ w ]->icon.serialNumber() != w->icon().serialNumber())
+    if( windows[ w ]->icon.cacheKey() != w->icon().cacheKey())
         { // make sure windows[ w ]->icon is the right QPixmap, and rebind
         windows[ w ]->icon = w->icon();
-#ifdef HAVE_OPENGL
+#ifdef KWIN_HAVE_OPENGL_COMPOSITING
         if( effects->compositingType() == OpenGLCompositing )
             {
             windows[ w ]->iconTexture.load( windows[ w ]->icon );
             windows[ w ]->iconTexture.setFilter( GL_LINEAR );
             }
 #endif
-#ifdef HAVE_XRENDER
+#ifdef KWIN_HAVE_XRENDER_COMPOSITING
         if( effects->compositingType() == XRenderCompositing )
             {
             if( windows[ w ]->iconPicture != None )
@@ -579,7 +589,7 @@ void BoxSwitchEffect::paintWindowIcon( EffectWindow* w )
     int height = windows[ w ]->icon.height();
     int x = windows[ w ]->area.x() + windows[ w ]->area.width() - width - highlight_margin;
     int y = windows[ w ]->area.y() + windows[ w ]->area.height() - height - highlight_margin;
-#ifdef HAVE_OPENGL
+#ifdef KWIN_HAVE_OPENGL_COMPOSITING
     if( effects->compositingType() == OpenGLCompositing )
         {
         glPushAttrib( GL_CURRENT_BIT | GL_ENABLE_BIT );
@@ -610,7 +620,7 @@ void BoxSwitchEffect::paintWindowIcon( EffectWindow* w )
         glPopAttrib();
         }
 #endif
-#ifdef HAVE_XRENDER
+#ifdef KWIN_HAVE_XRENDER_COMPOSITING
     if( effects->compositingType() == XRenderCompositing )
         {
         XRenderComposite( display(),
