@@ -1,12 +1,22 @@
-/*****************************************************************
+/********************************************************************
  KWin - the KDE window manager
  This file is part of the KDE project.
 
 Copyright (C) 2007 Lubos Lunak <l.lunak@kde.org>
 
-You can Freely distribute this program under the GNU General Public
-License. See the file "COPYING" for the exact licensing terms.
-******************************************************************/
+This program is free software; you can redistribute it and/or modify
+it under the terms of the GNU General Public License as published by
+the Free Software Foundation; either version 2 of the License, or
+(at your option) any later version.
+
+This program is distributed in the hope that it will be useful,
+but WITHOUT ANY WARRANTY; without even the implied warranty of
+MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+GNU General Public License for more details.
+
+You should have received a copy of the GNU General Public License
+along with this program.  If not, see <http://www.gnu.org/licenses/>.
+*********************************************************************/
 
 #include "desktopgrid.h"
 
@@ -70,13 +80,6 @@ void DesktopGridEffect::prePaintScreen( ScreenPrePaintData& data, int time )
             data.mask |= PAINT_SCREEN_TRANSFORMED | PAINT_SCREEN_BACKGROUND_FIRST;
         if( !activated && progress == 0 )
             finish();
-        int d = posToDesktop( cursorPos());
-        if( d != hover_desktop )
-            {
-            data.paint |= desktopRect( hover_desktop, true );
-            hover_desktop = d;
-            data.paint |= desktopRect( hover_desktop, true );
-            }
         }
     effects->prePaintScreen( data, time );
     }
@@ -249,7 +252,7 @@ void DesktopGridEffect::paintWindow( EffectWindow* w, int mask, QRegion region,
             data.xTranslate += window_move_pos.x() * x - ( desktop.x() + w->x());
             data.yTranslate += window_move_pos.y() * y - ( desktop.y() + w->y());
             }
-        else if( painting_desktop != hover_desktop )
+        else if( painting_desktop != highlighted_desktop )
             data.brightness *= 0.7;
         }
     effects->paintWindow( w, mask, region, data );    
@@ -350,6 +353,8 @@ EffectWindow* DesktopGridEffect::windowAt( const QPoint& pos, QRect* rect ) cons
 
 void DesktopGridEffect::desktopChanged( int old )
     {
+    if( effects->activeFullScreenEffect() && effects->activeFullScreenEffect() != this )
+        return;
     if( activated )
         setActive( false );
     else
@@ -403,8 +408,16 @@ void DesktopGridEffect::slideDesktopChanged( int old )
         else // current position is not on current desktop, do full progress
             progress = 0;
         diffPos = rect.topLeft() - currentPos;
-        // Compute starting point for this new move (given current and end positions)
-        slide_start_pos = rect.topLeft() - diffPos * 1 / ( 1 - progress );
+        if( progress <= 0 )
+            {
+            // Compute starting point for this new move (given current and end positions)
+            slide_start_pos = rect.topLeft() - diffPos * 1 / ( 1 - progress );
+            }
+        else
+            { // at the end, stop
+            slide = false;
+            progress = 0;
+            }
         }
     else
         {
@@ -422,6 +435,8 @@ void DesktopGridEffect::toggle()
     
 void DesktopGridEffect::setActive( bool active )
     {
+    if( effects->activeFullScreenEffect() && effects->activeFullScreenEffect() != this )
+        return;
     if( activated == active )
         return;
     activated = active;
@@ -435,7 +450,8 @@ void DesktopGridEffect::setup()
     keyboard_grab = effects->grabKeyboard( this );
     input = effects->createInputWindow( this, 0, 0, displayWidth(), displayHeight(),
         Qt::PointingHandCursor );
-    hover_desktop = effects->currentDesktop();
+    effects->setActiveFullScreenEffect( this );
+    setHighlightedDesktop( effects->currentDesktop());
     }
 
 void DesktopGridEffect::finish()
@@ -447,7 +463,8 @@ void DesktopGridEffect::finish()
         effects->setElevatedWindow( window_move, false );
     window_move = NULL;
     effects->destroyInputWindow( input );
-    effects->addRepaintFull(); // to get rid of hover
+    effects->setActiveFullScreenEffect( 0 );
+    effects->addRepaintFull(); // to get rid of highlight
     }
 
 void DesktopGridEffect::windowInputMouseEvent( Window, QEvent* e )
@@ -461,12 +478,8 @@ void DesktopGridEffect::windowInputMouseEvent( Window, QEvent* e )
         {
         // highlight desktop under mouse
         int d = posToDesktop( me->pos());
-        if( d != hover_desktop )
-            {
-            effects->addRepaint( desktopRect( hover_desktop, true ));
-            hover_desktop = d;
-            effects->addRepaint( desktopRect( hover_desktop, true ));
-            }
+        if( d != highlighted_desktop )
+            setHighlightedDesktop( d );
         if( window_move != NULL ) // handle window moving
             {
             was_window_move = true;
@@ -490,10 +503,10 @@ void DesktopGridEffect::windowInputMouseEvent( Window, QEvent* e )
                 effects->setElevatedWindow( window_move, true );
                 }
             }
-        else if( me->buttons() == Qt::MidButton && window_move == NULL )
+        else if(( me->buttons() == Qt::MidButton || me->buttons() == Qt::RightButton ) && window_move == NULL )
             {
             EffectWindow* w = windowAt( me->pos());
-            if( w != NULL && w->isMovable())
+            if( w != NULL )
                 {
                 if( w->isOnAllDesktops())
                     effects->windowToDesktop( w, posToDesktop( me->pos()));
@@ -518,6 +531,7 @@ void DesktopGridEffect::windowInputMouseEvent( Window, QEvent* e )
                 effects->calcDesktopLayout( &x, &y, &orientation );
                 effects->moveWindow( window_move, QPoint( rect.x() * x, rect.y() * y ));
                 effects->windowToDesktop( window_move, desktop );
+                effects->setElevatedWindow( window_move, false );
                 window_move = NULL;
                 }
             }
@@ -541,9 +555,65 @@ void DesktopGridEffect::windowClosed( EffectWindow* w )
 
 void DesktopGridEffect::grabbedKeyboardEvent( QKeyEvent* e )
     {
-    // TODO
+    if( e->type() == QEvent::KeyPress )
+        {
+        int desktop = -1;
+        // switch by F<number> or just <number>
+        if( e->key() >= Qt::Key_F1 && e->key() <= Qt::Key_F35 )
+            desktop = e->key() - Qt::Key_F1 + 1;
+        else if( e->key() >= Qt::Key_0 && e->key() <= Qt::Key_9 )
+            desktop = e->key() == Qt::Key_0 ? 10 : e->key() - Qt::Key_0;
+        if( desktop != -1 )
+            {
+            if( desktop <= effects->numberOfDesktops())
+                {
+                setHighlightedDesktop( desktop );
+                effects->setCurrentDesktop( desktop );
+                setActive( false );
+                }
+            return;
+            }
+        switch( e->key())
+            { // wrap only on autorepeat
+            case Qt::Key_Left:
+                setHighlightedDesktop( effects->desktopToLeft( highlighted_desktop,
+                    !e->isAutoRepeat()));
+                break;
+            case Qt::Key_Right:
+                setHighlightedDesktop( effects->desktopToRight( highlighted_desktop,
+                    !e->isAutoRepeat()));
+                break;
+            case Qt::Key_Up:
+                setHighlightedDesktop( effects->desktopUp( highlighted_desktop,
+                    !e->isAutoRepeat()));
+                break;
+            case Qt::Key_Down:
+                setHighlightedDesktop( effects->desktopDown( highlighted_desktop,
+                    !e->isAutoRepeat()));
+                break;
+            case Qt::Key_Escape:
+                setActive( false );
+                return;
+            case Qt::Key_Enter:
+            case Qt::Key_Return:
+            case Qt::Key_Space:
+                effects->setCurrentDesktop( highlighted_desktop );
+                setActive( false );
+                return;
+            default:
+                break;
+            }
+        }
     }
 
+void DesktopGridEffect::setHighlightedDesktop( int d )
+    {
+    if( d == highlighted_desktop || d <= 0 || d > effects->numberOfDesktops())
+        return;
+    effects->addRepaint( desktopRect( highlighted_desktop, true ));
+    highlighted_desktop = d;
+    effects->addRepaint( desktopRect( highlighted_desktop, true ));
+    }
 
 } // namespace
 
