@@ -1,13 +1,22 @@
-/*****************************************************************
+/********************************************************************
  KWin - the KDE window manager
  This file is part of the KDE project.
 
 Copyright (C) 2007 Rivo Laks <rivolaks@hot.ee>
 
-You can Freely distribute this program under the GNU General Public
-License. See the file "COPYING" for the exact licensing terms.
-******************************************************************/
+This program is free software; you can redistribute it and/or modify
+it under the terms of the GNU General Public License as published by
+the Free Software Foundation; either version 2 of the License, or
+(at your option) any later version.
 
+This program is distributed in the hope that it will be useful,
+but WITHOUT ANY WARRANTY; without even the implied warranty of
+MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+GNU General Public License for more details.
+
+You should have received a copy of the GNU General Public License
+along with this program.  If not, see <http://www.gnu.org/licenses/>.
+*********************************************************************/
 
 #include "presentwindows.h"
 
@@ -36,8 +45,8 @@ PresentWindowsEffect::PresentWindowsEffect()
     , mActiveness( 0.0 )
     , mRearranging( 1.0 )
     , hasKeyboardGrab( false )
-    , mHoverWindow( NULL )
-#ifdef HAVE_OPENGL
+    , mHighlightedWindow( NULL )
+#ifdef KWIN_HAVE_OPENGL_COMPOSITING
     , filterTexture( NULL )
 #endif
     {
@@ -45,21 +54,21 @@ PresentWindowsEffect::PresentWindowsEffect()
 
     KActionCollection* actionCollection = new KActionCollection( this );
     KAction* a = (KAction*)actionCollection->addAction( "Expose" );
-    a->setText( i18n("Toggle Expose effect" ));
-    a->setGlobalShortcut(KShortcut(Qt::CTRL + Qt::Key_F10));
+    a->setText( i18n("Toggle Expose Effect" ));
+    a->setGlobalShortcut(KShortcut(Qt::CTRL + Qt::Key_F9));
     connect(a, SIGNAL(triggered(bool)), this, SLOT(toggleActive()));
     KAction* b = (KAction*)actionCollection->addAction( "ExposeAll" );
     b->setText( i18n("Toggle Expose effect (incl other desktops)" ));
-    b->setGlobalShortcut(KShortcut(Qt::CTRL + Qt::Key_F11));
+    b->setGlobalShortcut(KShortcut(Qt::CTRL + Qt::Key_F10));
     connect(b, SIGNAL(triggered(bool)), this, SLOT(toggleActiveAllDesktops()));
 
-    borderActivate = (ElectricBorder)conf.readEntry("BorderActivate", (int)ElectricTopRight);
-    borderActivateAll = (ElectricBorder)conf.readEntry("BorderActivateAll", (int)ElectricNone);
+    borderActivate = (ElectricBorder)conf.readEntry("BorderActivate", (int)ElectricNone);
+    borderActivateAll = (ElectricBorder)conf.readEntry("BorderActivateAll", (int)ElectricTopLeft);
 
     effects->reserveElectricBorder( borderActivate );
     effects->reserveElectricBorder( borderActivateAll );
 
-#ifdef HAVE_XRENDER
+#ifdef KWIN_HAVE_XRENDER_COMPOSITING
     alphaFormat = XRenderFindStandardFormat( display(), PictStandardARGB32 );
 #endif
     }
@@ -111,13 +120,13 @@ void PresentWindowsEffect::prePaintWindow( EffectWindow* w, WindowPrePaintData&
             //  fully active, then apply some transparency
             if( mActiveness < 1.0f && (w->isMinimized() || !w->isOnCurrentDesktop() ))
                 data.setTranslucent();
-            // Change window's hover according to cursor pos
+            // Change window's highlight
             WindowData& windata = mWindowData[w];
-            const double hoverchangetime = 200;
-            if( windata.area.contains(cursorPos()) )
-                windata.hover = qMin(1.0, windata.hover + time / hoverchangetime);
+            const double highlightchangetime = 200;
+            if( w == mHighlightedWindow )
+                windata.highlight = qMin(1.0, windata.highlight + time / highlightchangetime);
             else
-                windata.hover = qMax(0.0, windata.hover - time / hoverchangetime);
+                windata.highlight = qMax(0.0, windata.highlight - time / highlightchangetime);
             }
         else if( !w->isDesktop())
             w->disablePainting( EffectWindow::PAINT_DISABLED );
@@ -128,14 +137,14 @@ void PresentWindowsEffect::prePaintWindow( EffectWindow* w, WindowPrePaintData&
 void PresentWindowsEffect::paintScreen( int mask, QRegion region, ScreenPaintData& data )
     {
     effects->paintScreen( mask, region, data );
-#ifdef HAVE_OPENGL
+#ifdef KWIN_HAVE_OPENGL_COMPOSITING
     if( filterTexture && region.intersects( filterFrameRect ))
         {
         glPushAttrib( GL_CURRENT_BIT | GL_ENABLE_BIT );
         glEnable( GL_BLEND );
         glBlendFunc( GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA );
         // First render the frame
-        QColor color = QPalette().color( QPalette::Highlight );
+        QColor color = QPalette().color( QPalette::Active, QPalette::Highlight );
         glColor4f( color.redF(), color.greenF(), color.blueF(), 0.75f );
         renderRoundBoxWithEdge( filterFrameRect );
         // And then the text on top of it
@@ -181,7 +190,7 @@ void PresentWindowsEffect::paintWindow( EffectWindow* w, int mask, QRegion regio
             data.yTranslate = (int)interpolate(data.yTranslate, windata.area.top() - w->y(), mActiveness);
             }
         // Darken all windows except for the one under the cursor
-        data.brightness *= interpolate(1.0, 0.7, mActiveness * (1.0f - windata.hover));
+        data.brightness *= interpolate(1.0, 0.7, mActiveness * (1.0f - windata.highlight));
         // If it's minimized window or on another desktop and effect is not
         //  fully active, then apply some transparency
         if( mActiveness < 1.0f && (w->isMinimized() || !w->isOnCurrentDesktop() ))
@@ -200,7 +209,7 @@ void PresentWindowsEffect::paintWindow( EffectWindow* w, int mask, QRegion regio
         double centerx = w->x() + data.xTranslate + w->width() * data.xScale * 0.5f;
         double centery = w->y() + data.yTranslate + w->height() * data.yScale * 0.5f;
         int maxwidth = (int)(w->width() * data.xScale - 20);
-        double opacity = (0.7 + 0.2*windata.hover) * data.opacity * mActiveness;
+        double opacity = (0.7 + 0.2*windata.highlight) * data.opacity * mActiveness;
         QColor textcolor( 255, 255, 255, (int)(255*opacity) );
         QColor bgcolor( 0, 0, 0, (int)(255*opacity) );
         QFont f;
@@ -221,7 +230,7 @@ void PresentWindowsEffect::postPaintScreen()
         effects->addRepaintFull();
     foreach( const WindowData& d, mWindowData )
         {
-        if( d.hover > 0 && d.hover < 1 ) // changing highlight
+        if( d.highlight > 0 && d.highlight < 1 ) // changing highlight
             effects->addRepaintFull();
         }
     // Call the next effect.
@@ -232,7 +241,7 @@ void PresentWindowsEffect::windowInputMouseEvent( Window w, QEvent* e )
     {
     assert( w == mInput );
     if( e->type() == QEvent::MouseMove )
-        { // Repaint if the hovered-over window changed.
+        { // Repaint if the highlighted window changed.
           // (No need to use cursorMoved(), this takes care of it as well)
         for( DataHash::ConstIterator it = mWindowData.begin();
              it != mWindowData.end();
@@ -240,11 +249,8 @@ void PresentWindowsEffect::windowInputMouseEvent( Window w, QEvent* e )
             {
             if( (*it).area.contains( cursorPos()))
                 {
-                if( mHoverWindow != it.key())
-                    {
-                    mHoverWindow = it.key();
-                    effects->addRepaintFull(); // screen is transformed, so paint all
-                    }
+                if( mHighlightedWindow != it.key())
+                    setHighlightedWindow( it.key());
                 return;
                 }
             }
@@ -278,18 +284,19 @@ void PresentWindowsEffect::windowInputMouseEvent( Window w, QEvent* e )
 
 void PresentWindowsEffect::windowClosed( EffectWindow* w )
     {
-    if( mHoverWindow == w )
-        mHoverWindow = NULL;
+    if( mHighlightedWindow == w )
+        setHighlightedWindow( findFirstWindow());
     mWindowsToPresent.removeAll( w );
     rearrangeWindows();
     }
 
 void PresentWindowsEffect::setActive(bool active)
     {
+    if( effects->activeFullScreenEffect() && effects->activeFullScreenEffect() != this )
+        return;
     if( mActivated == active )
         return;
     mActivated = active;
-    mHoverWindow = NULL;
     if( mActivated )
         {
         mWindowData.clear();
@@ -310,6 +317,7 @@ void PresentWindowsEffect::setActive(bool active)
             mWindowsToPresent.append(window);
             }
         rearrangeWindows();
+        setHighlightedWindow( effects->activeWindow());
         }
     else
         {
@@ -317,6 +325,7 @@ void PresentWindowsEffect::setActive(bool active)
         mRearranging = 1; // turn off
         mActiveness = 1; // go back from arranged position
         discardFilterTexture();
+        mHighlightedWindow = NULL;
         }
     effects->addRepaintFull(); // trigger next animation repaint
     }
@@ -326,6 +335,8 @@ void PresentWindowsEffect::effectActivated()
     // Create temporary input window to catch mouse events
     mInput = effects->createFullScreenInputWindow( this, Qt::PointingHandCursor );
     hasKeyboardGrab = effects->grabKeyboard( this );
+    effects->setActiveFullScreenEffect( this );
+    setHighlightedWindow( effects->activeWindow());
     }
 
 void PresentWindowsEffect::effectTerminated()
@@ -335,6 +346,8 @@ void PresentWindowsEffect::effectTerminated()
     if( hasKeyboardGrab )
         effects->ungrabKeyboard();
     hasKeyboardGrab = false;
+    effects->setActiveFullScreenEffect( 0 );
+    effects->addRepaintFull(); // to get rid of highlight
     }
 
 void PresentWindowsEffect::rearrangeWindows()
@@ -350,13 +363,15 @@ void PresentWindowsEffect::rearrangeWindows()
         foreach( EffectWindow* w, mWindowsToPresent )
             {
             if( w->caption().contains( windowFilter, Qt::CaseInsensitive )
-                || w->windowClass().contains( windowFilter, Qt::CaseInsensitive ))
+                || w->windowClass().contains( windowFilter, Qt::CaseInsensitive )
+                || w->windowRole().contains( windowFilter, Qt::CaseInsensitive ))
                 windowlist.append( w );
             }
         }
     if( windowlist.isEmpty())
         {
         mWindowData.clear();
+        setHighlightedWindow( NULL );
         effects->addRepaintFull();
         return;
         }
@@ -377,6 +392,8 @@ void PresentWindowsEffect::rearrangeWindows()
         mWindowData = newdata;
         if( !rearrange && newlist == oldlist )
             return;
+        if( mHighlightedWindow != NULL && !mWindowData.contains( mHighlightedWindow ))
+            setHighlightedWindow( NULL );
         for( DataHash::Iterator it = mWindowData.begin();
              it != mWindowData.end();
              ++it )
@@ -388,7 +405,7 @@ void PresentWindowsEffect::rearrangeWindows()
         foreach( EffectWindow* w, windowlist )
             if( !mWindowData.contains( w ))
                 {
-                mWindowData[ w ].hover = 0;
+                mWindowData[ w ].highlight = 0;
                 }
         mRearranging = 0; // start animation again
         }
@@ -398,6 +415,9 @@ void PresentWindowsEffect::rearrangeWindows()
 //    calculateWindowTransformationsKompose( windowlist );
     calculateWindowTransformationsClosest( windowlist );
 
+    if( !mWindowData.isEmpty() && mHighlightedWindow == NULL )
+        setHighlightedWindow( findFirstWindow());
+
     // Schedule entire desktop to be repainted
     effects->addRepaintFull();
     }
@@ -408,7 +428,7 @@ void PresentWindowsEffect::calculateWindowTransformationsDumb(EffectWindowList w
     int rows = windowlist.count() / 4 + 1;
     int cols = windowlist.count() / rows + windowlist.count() % rows;
     // Get rect which we can use on current desktop. This excludes e.g. panels
-    QRect placementRect = effects->clientArea( PlacementArea, QPoint( 0, 0 ), 0 );
+    QRect placementRect = effects->clientArea( PlacementArea, effects->activeScreen(), effects->currentDesktop());
     // Size of one cell
     int cellwidth = placementRect.width() / cols;
     int cellheight = placementRect.height() / rows;
@@ -422,7 +442,10 @@ void PresentWindowsEffect::calculateWindowTransformationsDumb(EffectWindowList w
         // Row/Col of this window
         int r = i / cols;
         int c = i % cols;
-        mWindowData[window].hover = 0.0f;
+        mWindowData[window].slot = i;
+        mWindowData[window].x = c;
+        mWindowData[window].y = r;
+        mWindowData[window].highlight = 0.0f;
         mWindowData[window].scale = qMin(cellwidth / (double)window->width(), cellheight / (double)window->height());
         mWindowData[window].area.setLeft(placementRect.left() + cellwidth * c);
         mWindowData[window].area.setTop(placementRect.top() + cellheight * r);
@@ -454,7 +477,7 @@ int PresentWindowsEffect::windowHeightForWidth(EffectWindow* c, int w)
 void PresentWindowsEffect::calculateWindowTransformationsKompose(EffectWindowList windowlist)
     {
      // Get rect which we can use on current desktop. This excludes e.g. panels
-    QRect availRect = effects->clientArea( PlacementArea, QPoint( 0, 0 ), effects->currentDesktop());
+    QRect availRect = effects->clientArea( PlacementArea, effects->activeScreen(), effects->currentDesktop());
 
     // Following code is taken from Kompose 0.5.4, src/komposelayout.cpp
 
@@ -464,12 +487,12 @@ void PresentWindowsEffect::calculateWindowTransformationsKompose(EffectWindowLis
     // Use more columns than rows when parent's width > parent's height
     if ( parentRatio > 1 )
     {
-        columns = (int)ceil( sqrt(windowlist.count()) );
+        columns = (int)ceil( sqrt((double)windowlist.count()) );
         rows = (int)ceil( (double)windowlist.count() / (double)columns );
     }
     else
     {
-        rows = (int)ceil( sqrt(windowlist.count()) );
+        rows = (int)ceil( sqrt((double)windowlist.count()) );
         columns = (int)ceil( (double)windowlist.count() / (double)rows );
     }
     kDebug() << "Using " << rows << " rows & " << columns << " columns for " << windowlist.count() << " clients";
@@ -577,9 +600,12 @@ void PresentWindowsEffect::calculateWindowTransformationsKompose(EffectWindowLis
             EffectWindow* window = windowlist[pos];
             QRect geom = geometryRects[pos];
             geom.setY( geom.y() + topOffset );
+            mWindowData[window].slot = pos;
+            mWindowData[window].x = j;
+            mWindowData[window].y = i;
             mWindowData[window].area = geom;
             mWindowData[window].scale = geom.width() / (double)window->width();
-            mWindowData[window].hover = 0.0f;
+            mWindowData[window].highlight = 0.0f;
 
             kDebug() << "Window '" << window->caption() << "' gets moved to (" <<
                     mWindowData[window].area.left() << "; " << mWindowData[window].area.right() <<
@@ -592,8 +618,8 @@ void PresentWindowsEffect::calculateWindowTransformationsKompose(EffectWindowLis
 
 void PresentWindowsEffect::calculateWindowTransformationsClosest(EffectWindowList windowlist)
     {
-    QRect area = effects->clientArea( PlacementArea, QPoint( 0, 0 ), effects->currentDesktop());
-    int columns = int( ceil( sqrt( windowlist.count())));
+    QRect area = effects->clientArea( PlacementArea, effects->activeScreen(), effects->currentDesktop());
+    int columns = int( ceil( sqrt( (double)windowlist.count())));
     int rows = int( ceil( windowlist.count() / double( columns )));
     foreach( EffectWindow* w, windowlist )
         mWindowData[ w ].slot = -1;
@@ -689,11 +715,13 @@ void PresentWindowsEffect::assignSlots( const QRect& area, int columns, int rows
                     continue;
                 int xdiff = pos.x() - ( area.x() + slotwidth * x + slotwidth / 2 ); // slotwidth/2 for center
                 int ydiff = pos.y() - ( area.y() + slotheight * y + slotheight / 2 );
-                int dist = int( sqrt( xdiff * xdiff + ydiff * ydiff ));
+                int dist = int( sqrt( (double)(xdiff * xdiff + ydiff * ydiff) ));
                 if( dist < distance )
                     {
                     distance = dist;
                     (*it).slot = slot;
+                    (*it).x = x;
+                    (*it).y = y;
                     (*it).slot_distance = distance;
                     }
                 }
@@ -721,16 +749,18 @@ void PresentWindowsEffect::getBestAssignments()
 
 bool PresentWindowsEffect::canRearrangeClosest(EffectWindowList windowlist)
     {
-    QRect area = effects->clientArea( PlacementArea, QPoint( 0, 0 ), effects->currentDesktop());
-    int columns = int( ceil( sqrt( windowlist.count())));
+    QRect area = effects->clientArea( PlacementArea, effects->activeScreen(), effects->currentDesktop());
+    int columns = int( ceil( sqrt( (double)windowlist.count())));
     int rows = int( ceil( windowlist.count() / double( columns )));
-    int old_columns = int( ceil( sqrt( mWindowData.count())));
+    int old_columns = int( ceil( sqrt( (double)mWindowData.count())));
     int old_rows = int( ceil( mWindowData.count() / double( columns )));
     return old_columns != columns || old_rows != rows;
     }
 
 bool PresentWindowsEffect::borderActivated( ElectricBorder border )
     {
+    if( effects->activeFullScreenEffect() && effects->activeFullScreenEffect() != this )
+        return false;
     if( border == borderActivate && !mActivated )
         {
         toggleActive();
@@ -746,50 +776,197 @@ bool PresentWindowsEffect::borderActivated( ElectricBorder border )
 
 void PresentWindowsEffect::grabbedKeyboardEvent( QKeyEvent* e )
     {
-    if( e->type() != QEvent::KeyPress )
-        return;
-    if( e->key() == Qt::Key_Escape )
+    if( e->type() == QEvent::KeyPress )
         {
-        setActive( false );
-        return;
-        }
-    if( e->key() == Qt::Key_Backspace )
-        {
-        if( !windowFilter.isEmpty())
-            {
-            windowFilter.remove( windowFilter.length() - 1, 1 );
-            updateFilterTexture();
-            rearrangeWindows();
+        switch( e->key())
+            { // wrap only on autorepeat
+            case Qt::Key_Left:
+                setHighlightedWindow( relativeWindow( mHighlightedWindow, -1, 0, !e->isAutoRepeat()));
+                break;
+            case Qt::Key_Right:
+                setHighlightedWindow( relativeWindow( mHighlightedWindow, 1, 0, !e->isAutoRepeat()));
+                break;
+            case Qt::Key_Up:
+                setHighlightedWindow( relativeWindow( mHighlightedWindow, 0, -1, !e->isAutoRepeat()));
+                break;
+            case Qt::Key_Down:
+                setHighlightedWindow( relativeWindow( mHighlightedWindow, 0, 1, !e->isAutoRepeat()));
+                break;
+            case Qt::Key_Home:
+                setHighlightedWindow( relativeWindow( mHighlightedWindow, -1000, 0, false ));
+                break;
+            case Qt::Key_End:
+                setHighlightedWindow( relativeWindow( mHighlightedWindow, 1000, 0, false ));
+                break;
+            case Qt::Key_PageUp:
+                setHighlightedWindow( relativeWindow( mHighlightedWindow, 0, -1000, false ));
+                break;
+            case Qt::Key_PageDown:
+                setHighlightedWindow( relativeWindow( mHighlightedWindow, 0, 1000, false ));
+                break;
+            case Qt::Key_Backspace:
+                if( !windowFilter.isEmpty())
+                    {
+                    windowFilter.remove( windowFilter.length() - 1, 1 );
+                    updateFilterTexture();
+                    rearrangeWindows();
+                    }
+                return;
+            case Qt::Key_Escape:
+                setActive( false );
+                return;
+            case Qt::Key_Return:
+            case Qt::Key_Enter:
+                if( mHighlightedWindow != NULL )
+                    {
+                    effects->activateWindow( mHighlightedWindow );
+                    setActive( false );
+                    return;
+                    }
+                if( mWindowData.count() == 1 ) // only one window shown
+                    {
+                    effects->activateWindow( mWindowData.begin().key());
+                    setActive( false );
+                    }
+                return;
+            default:
+                if( !e->text().isEmpty())
+                    {
+                    windowFilter.append( e->text());
+                    updateFilterTexture();
+                    rearrangeWindows();
+                    return;
+                    }
+                break;
             }
-        return;
         }
-    if( e->key() == Qt::Key_Return || e->key() == Qt::Key_Enter )
+    }
+
+void PresentWindowsEffect::setHighlightedWindow( EffectWindow* w )
+    {
+    if( w == mHighlightedWindow || ( w != NULL && !mWindowData.contains( w )))
+        return;
+    effects->addRepaintFull(); // everything is transformed anyway
+    mHighlightedWindow = w;
+    }
+
+// returns a window which is to relative position <xdiff,ydiff> from the given window
+EffectWindow* PresentWindowsEffect::relativeWindow( EffectWindow* w, int xdiff, int ydiff, bool wrap ) const
+    {
+    if( mWindowData.count() == 0 )
+        return NULL;
+    if( w == NULL )
+        return findFirstWindow();
+    int columns = int( ceil( sqrt( (double)mWindowData.count())));
+    int rows = int( ceil( mWindowData.count() / double( columns )));
+    QVector< QVector< EffectWindow* > > grid;
+    grid.resize( columns );
+    for( int i = 0;
+         i < columns;
+         ++i )
+        grid[ i ].resize( rows );
+    for( DataHash::ConstIterator it = mWindowData.begin();
+         it != mWindowData.end();
+         ++it )
+        grid[ it->x ][ it->y ] = it.key();
+    int x = mWindowData[ w ].x;
+    int y = mWindowData[ w ].y;
+    while( xdiff > 0 )
         {
-        if( mHoverWindow != NULL )
+        ++x;
+        if( x == columns )
             {
-            effects->activateWindow( mHoverWindow );
-            setActive( false );
-            return;
+            if( !wrap )
+                { // make sure to find the leftmost (or 'w', which is guaranteed)
+                --x;
+                while( x >= 0 && grid[ x ][ y ] == NULL )
+                    --x;
+                break;
+                }
+            else
+                x = 0;
             }
-        if( mWindowData.count() == 1 ) // only one window shown
-            {
-            effects->activateWindow( mWindowData.begin().key());
-            setActive( false );
-            }
-        return;
+        if( grid[ x ][ y ] != NULL )
+            --xdiff;
         }
-    if( !e->text().isEmpty())
+    while( xdiff < 0 )
         {
-        windowFilter.append( e->text());
-        updateFilterTexture();
-        rearrangeWindows();
-        return;
+        --x;
+        if( x < 0 )
+            {
+            if( !wrap )
+                {
+                ++x;
+                while( x <= columns - 1 && grid[ x ][ y ] == NULL )
+                    ++x;
+                break;
+                }
+            else
+                x = columns - 1;
+            }
+        if( grid[ x ][ y ] != NULL )
+            ++xdiff;
         }
+    while( ydiff > 0 )
+        {
+        ++y;
+        if( y == rows )
+            {
+            if( !wrap )
+                {
+                --y;
+                while( y >= 0 && grid[ x ][ y ] == NULL )
+                    --y;
+                break;
+                }
+            else
+                y = 0;
+            }
+        if( grid[ x ][ y ] != NULL )
+            --ydiff;
+        }
+    while( ydiff < 0 )
+        {
+        --y;
+        if( y < 0 )
+            {
+            if( !wrap )
+                {
+                ++y;
+                while( y <= rows - 1 && grid[ x ][ y ] == NULL )
+                    ++y;
+                break;
+                }
+            else
+                y = rows - 1;
+            }
+        if( grid[ x ][ y ] != NULL )
+            ++ydiff;
+        }
+    return grid[ x ][ y ];
+    }
+
+// returns the window that is the most to the topleft, if any
+EffectWindow* PresentWindowsEffect::findFirstWindow() const
+    {
+    int minslot = INT_MAX;
+    EffectWindow* ret = NULL;
+    for( DataHash::ConstIterator it = mWindowData.begin();
+         it != mWindowData.end();
+         ++it )
+        {
+        if( (*it).slot < minslot )
+            {
+            minslot = (*it).slot;
+            ret = it.key();
+            }
+        }
+    return ret;
     }
 
 void PresentWindowsEffect::discardFilterTexture()
     {
-#ifdef HAVE_OPENGL
+#ifdef KWIN_HAVE_OPENGL_COMPOSITING
     delete filterTexture;
     filterTexture = NULL;
 #endif
@@ -797,10 +974,13 @@ void PresentWindowsEffect::discardFilterTexture()
 
 void PresentWindowsEffect::updateFilterTexture()
     {
-#ifdef HAVE_OPENGL
+#ifdef KWIN_HAVE_OPENGL_COMPOSITING
     discardFilterTexture();
     if( windowFilter.isEmpty())
+        {
+        effects->addRepaint( filterTextureRect );
         return;
+        }
     // Create font for filter text
     QFont font;
     font.setPointSize( font.pointSize() * 2 );
@@ -810,14 +990,14 @@ void PresentWindowsEffect::updateFilterTexture()
     QRect rect;
     QString translatedString = i18n( "Filter:\n%1", windowFilter );
     rect.setSize( fm.size( 0, translatedString ));
-    QRect area = effects->clientArea( PlacementArea, QPoint( 0, 0 ), effects->currentDesktop());
+    QRect area = effects->clientArea( PlacementArea, effects->activeScreen(), effects->currentDesktop());
     // Create image
     QImage im( rect.width(), rect.height(), QImage::Format_ARGB32 );
     im.fill( Qt::transparent );
     // Paint the filter text to it
     QPainter p( &im );
     p.setFont( font );
-    p.setPen( QPalette().color( QPalette::HighlightedText ) );
+    p.setPen( QPalette().color( QPalette::Active, QPalette::HighlightedText ) );
     p.drawText( rect, Qt::AlignCenter, translatedString );
     p.end();
     // Create GL texture
@@ -842,17 +1022,18 @@ void PresentWindowsEffect::paintWindowIcon( EffectWindow* w, WindowPaintData& pa
         }
 
     WindowData& data = mWindowData[ w ];
-    if( data.icon.serialNumber() != w->icon().serialNumber())
+    // TODO: find out why this doesn't work properly
+    // if( data.icon.cacheKey() != w->icon().cacheKey())
         { // make sure data.icon is the right QPixmap, and rebind
         data.icon = w->icon();
-#ifdef HAVE_OPENGL
+#ifdef KWIN_HAVE_OPENGL_COMPOSITING
         if( effects->compositingType() == OpenGLCompositing )
             {
             data.iconTexture.load( data.icon );
             data.iconTexture.setFilter( GL_LINEAR );
             }
 #endif
-#ifdef HAVE_XRENDER
+#ifdef KWIN_HAVE_XRENDER_COMPOSITING
         if( effects->compositingType() == XRenderCompositing )
             {
             if( data.iconPicture != None )
@@ -867,7 +1048,7 @@ void PresentWindowsEffect::paintWindowIcon( EffectWindow* w, WindowPaintData& pa
     int height = data.icon.height();
     int x = w->x() + paintdata.xTranslate + w->width() * paintdata.xScale * 0.95 - width - icon_margin;
     int y = w->y() + paintdata.yTranslate + w->height() * paintdata.yScale * 0.95 - height - icon_margin;
-#ifdef HAVE_OPENGL
+#ifdef KWIN_HAVE_OPENGL_COMPOSITING
     if( effects->compositingType() == OpenGLCompositing )
         {
         glPushAttrib( GL_CURRENT_BIT | GL_ENABLE_BIT | GL_TEXTURE_BIT );
@@ -899,7 +1080,7 @@ void PresentWindowsEffect::paintWindowIcon( EffectWindow* w, WindowPaintData& pa
         glPopAttrib();
         }
 #endif
-#ifdef HAVE_XRENDER
+#ifdef KWIN_HAVE_XRENDER_COMPOSITING
     if( effects->compositingType() == XRenderCompositing )
         {
         XRenderComposite( display(),
