@@ -1,12 +1,22 @@
-/*****************************************************************
+/********************************************************************
  KWin - the KDE window manager
  This file is part of the KDE project.
 
 Copyright (C) 2007 Rivo Laks <rivolaks@hot.ee>
 
-You can Freely distribute this program under the GNU General Public
-License. See the file "COPYING" for the exact licensing terms.
-******************************************************************/
+This program is free software; you can redistribute it and/or modify
+it under the terms of the GNU General Public License as published by
+the Free Software Foundation; either version 2 of the License, or
+(at your option) any later version.
+
+This program is distributed in the hope that it will be useful,
+but WITHOUT ANY WARRANTY; without even the implied warranty of
+MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+GNU General Public License for more details.
+
+You should have received a copy of the GNU General Public License
+along with this program.  If not, see <http://www.gnu.org/licenses/>.
+*********************************************************************/
 
 #ifndef KWIN_PRESENTWINDOWS_H
 #define KWIN_PRESENTWINDOWS_H
@@ -17,7 +27,7 @@ License. See the file "COPYING" for the exact licensing terms.
 
 #include <QPixmap>
 
-#ifdef HAVE_XRENDER
+#ifdef KWIN_HAVE_XRENDER_COMPOSITING
 #include <X11/extensions/Xrender.h>
 #endif
 
@@ -73,6 +83,10 @@ class PresentWindowsEffect
 
         void paintWindowIcon( EffectWindow* w, WindowPaintData& data );
 
+        void setHighlightedWindow( EffectWindow* w );
+        EffectWindow* relativeWindow( EffectWindow* w, int xdiff, int ydiff, bool wrap ) const;
+        EffectWindow* findFirstWindow() const;
+
         // Called once the effect is activated (and wasn't activated before)
         void effectActivated();
         // Called once the effect has terminated
@@ -98,23 +112,24 @@ class PresentWindowsEffect
             QRect old_area; // when rearranging, otherwise unset
             double scale;
             double old_scale; // when rearranging, otherwise unset
-            double hover;
+            double highlight;
             int slot;
+            int x, y; // position of the slot in the grid
             int slot_distance;
             QPixmap icon;
-#ifdef HAVE_OPENGL
+#ifdef KWIN_HAVE_OPENGL_COMPOSITING
             GLTexture iconTexture;
 #endif
-#ifdef HAVE_XRENDER
+#ifdef KWIN_HAVE_XRENDER_COMPOSITING
             Picture iconPicture;
 #endif
             };
         typedef QHash<EffectWindow*, WindowData> DataHash;
         DataHash mWindowData;
-        EffectWindow* mHoverWindow;
+        EffectWindow* mHighlightedWindow;
         
         QString windowFilter;
-#ifdef HAVE_OPENGL
+#ifdef KWIN_HAVE_OPENGL_COMPOSITING
         GLTexture* filterTexture;
         QRect filterTextureRect;
         QRect filterFrameRect;
@@ -123,7 +138,7 @@ class PresentWindowsEffect
         ElectricBorder borderActivate;
         ElectricBorder borderActivateAll;
 
-#ifdef HAVE_XRENDER
+#ifdef KWIN_HAVE_XRENDER_COMPOSITING
         XRenderPictFormat* alphaFormat;
 #endif
     };
