@@ -1,17 +1,29 @@
-/*****************************************************************
+/********************************************************************
  KWin - the KDE window manager
  This file is part of the KDE project.
 
 Copyright (C) 2007 Lubos Lunak <l.lunak@kde.org>
+Copyright (C) 2007 Christian Nitschkowski <christian.nitschkowski@kdemail.net>
 
-You can Freely distribute this program under the GNU General Public
-License. See the file "COPYING" for the exact licensing terms.
-******************************************************************/
+This program is free software; you can redistribute it and/or modify
+it under the terms of the GNU General Public License as published by
+the Free Software Foundation; either version 2 of the License, or
+(at your option) any later version.
+
+This program is distributed in the hope that it will be useful,
+but WITHOUT ANY WARRANTY; without even the implied warranty of
+MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+GNU General Public License for more details.
+
+You should have received a copy of the GNU General Public License
+along with this program.  If not, see <http://www.gnu.org/licenses/>.
+*********************************************************************/
 
 #include "thumbnailaside.h"
 
 #include <kactioncollection.h>
 #include <kaction.h>
+#include <kconfiggroup.h>
 #include <klocale.h>
 
 namespace KWin
@@ -21,14 +33,18 @@ KWIN_EFFECT( thumbnailaside, ThumbnailAsideEffect )
 
 ThumbnailAsideEffect::ThumbnailAsideEffect()
     {
+    KConfigGroup conf = EffectsHandler::effectConfig("ThumbnailAside");
+
     KActionCollection* actionCollection = new KActionCollection( this );
     KAction* a = (KAction*)actionCollection->addAction( "ToggleCurrentThumbnail" );
     a->setText( i18n("Toggle Thumbnail for Current Window" ));
-    a->setGlobalShortcut(KShortcut(Qt::META + Qt::Key_F9));
+    a->setGlobalShortcut(KShortcut(Qt::CTRL + Qt::META + Qt::Key_T));
     connect(a, SIGNAL(triggered(bool)), this, SLOT(toggleCurrentThumbnail()));
-    maxwidth = 200;
-    spacing = 10; // TODO config options?
-    opacity = 0.5;
+
+    maxwidth = conf.readEntry("MaxWidth", 200);
+    spacing = conf.readEntry("Spacing", 10);
+    opacity = conf.readEntry("Opacity", 50) / 100.0;
+    screen = conf.readEntry("Screen",-1); // Xinerama screen TODO add gui option
     }
 
 void ThumbnailAsideEffect::paintScreen( int mask, QRegion region, ScreenPaintData& data )
@@ -126,7 +142,7 @@ void ThumbnailAsideEffect::arrange()
         mwidth = qMax( mwidth, d.window->width());
         pos[ d.index ] = d.window->height();
         }
-    QRect area = effects->clientArea( WorkArea, QPoint(), effects->currentDesktop());
+    QRect area = effects->clientArea( WorkArea, screen, effects->currentDesktop());
     double scale = area.height() / double( height );
     scale = qMin( scale, maxwidth / double( mwidth )); // don't be wider than maxwidth pixels
     int add = 0;
