@@ -1,13 +1,23 @@
-/*****************************************************************
+/********************************************************************
  KWin - the KDE window manager
  This file is part of the KDE project.
 
 Copyright (C) 1999, 2000 Matthias Ettrich <ettrich@kde.org>
 Copyright (C) 2003 Lubos Lunak <l.lunak@kde.org>
 
-You can Freely distribute this program under the GNU General Public
-License. See the file "COPYING" for the exact licensing terms.
-******************************************************************/
+This program is free software; you can redistribute it and/or modify
+it under the terms of the GNU General Public License as published by
+the Free Software Foundation; either version 2 of the License, or
+(at your option) any later version.
+
+This program is distributed in the hope that it will be useful,
+but WITHOUT ANY WARRANTY; without even the implied warranty of
+MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+GNU General Public License for more details.
+
+You should have received a copy of the GNU General Public License
+along with this program.  If not, see <http://www.gnu.org/licenses/>.
+*********************************************************************/
 
 /*
 
@@ -130,14 +140,16 @@ void RootInfo::changeActiveWindow( Window w, NET::RequestSource src, Time timest
         else // NET::FromApplication
             {
             Client* c2;
-            if( workspace->allowClientActivation( c, timestamp ))
+            if( workspace->allowClientActivation( c, timestamp, false, true ))
                 workspace->activateClient( c );
             // if activation of the requestor's window would be allowed, allow activation too
             else if( active_window != None
                 && ( c2 = workspace->findClient( WindowMatchPredicate( active_window ))) != NULL
                 && workspace->allowClientActivation( c2,
-                    timestampCompare( timestamp, c2->userTime() > 0 ? timestamp : c2->userTime())))
+                    timestampCompare( timestamp, c2->userTime() > 0 ? timestamp : c2->userTime()), false, true ))
+                {
                 workspace->activateClient( c );
+                }
             else
                 c->demandAttention();
             }
@@ -246,7 +258,7 @@ bool Workspace::workspaceEvent( XEvent * e )
             was_user_interaction = true;
             int keyQt;
             KKeyServer::xEventToQt(e, &keyQt);
-            kDebug(125) << "Workspace::keyPress( " << keyQt << " )";
+//            kDebug(125) << "Workspace::keyPress( " << keyQt << " )";
             if (movingClient)
                 {
                 movingClient->keyPressEvent(keyQt);
@@ -455,8 +467,16 @@ bool Workspace::workspaceEvent( XEvent * e )
                 return true;
             break;
         case Expose:
-            if( e->xexpose.window == rootWindow() && compositing())  // root window needs repainting
+            if( compositing()
+                && ( e->xexpose.window == rootWindow()  // root window needs repainting
+                    || overlay != None && e->xexpose.window == overlay )) // overlay needs repainting
+                {
                 addRepaint( e->xexpose.x, e->xexpose.y, e->xexpose.width, e->xexpose.height );
+                }
+            break;
+        case VisibilityNotify:
+            if( compositing() && overlay != None && e->xvisibility.window == overlay )
+                overlay_visible = ( e->xvisibility.state != VisibilityFullyObscured );
             break;
         default:
             if( e->type == Extensions::randrNotifyEvent() && Extensions::randrAvailable() )
@@ -1704,6 +1724,8 @@ void Toplevel::propertyNotifyEvent( XPropertyEvent* e )
                 getWindowRole();
             break;
         }
+    if( effects )
+        static_cast< EffectsHandlerImpl* >( effects )->propertyNotify( effectWindow(), e->atom );
     }
 
 // ****************************************
