@@ -326,7 +326,7 @@ NET::WindowType KDecorationPreviewBridge::windowType( unsigned long ) const
 
 QIcon KDecorationPreviewBridge::icon() const
     {
-    return KIcon( "xapp" );
+    return KIcon( "xorg" );
     }
 
 QString KDecorationPreviewBridge::caption() const
@@ -375,11 +375,6 @@ QRegion KDecorationPreviewBridge::unobscuredRegion( const QRegion& r ) const
     return preview->unobscuredRegion( active, r );
     }
 
-QWidget* KDecorationPreviewBridge::workspaceWidget() const
-    {
-    return preview;
-    }
-
 WId KDecorationPreviewBridge::windowId() const
     {
     return 0; // no decorated window
@@ -430,10 +425,6 @@ int KDecorationPreviewBridge::currentDesktop() const
     return 1;
     }
 
-void KDecorationPreviewBridge::helperShowHide( bool )
-    {
-    }
-
 void KDecorationPreviewBridge::grabXServer( bool )
     {
     }
@@ -473,8 +464,8 @@ unsigned long KDecorationPreviewOptions::updateSettings()
         if (!customTitleButtonsRight.isNull() )
             d->title_buttons_right = customTitleButtonsRight;
     } else {
-        d->title_buttons_left = "MS";
-        d->title_buttons_right = "HIAX";
+        d->title_buttons_left = KDecorationOptions::defaultTitleButtonsLeft();
+        d->title_buttons_right = KDecorationOptions::defaultTitleButtonsRight();
     }
 
     return changed;
