@@ -48,8 +48,8 @@ namespace KWin
     rule_##var->setWhatsThis( type##RuleDesc );
 
 RulesWidget::RulesWidget( QWidget* parent )
-: RulesWidgetBase( parent )
-, detect_dlg( NULL )
+    : RulesWidgetBase( parent )
+    , detect_dlg( NULL )
     {
     QString enableDesc =
         i18n( "Enable this checkbox to alter this window property for the specified window(s)." );
@@ -630,7 +630,7 @@ void RulesWidget::prefillUnusedValues( const KWindowInfo& info )
     //LINEEDIT_PREFILL( shortcut, );
     //COMBOBOX_PREFILL( fsplevel, );
     //COMBOBOX_PREFILL( moveresizemode, moveresizeToCombo );
-    COMBOBOX_PREFILL( type, typeToCombo, info.windowType( SUPPORTED_WINDOW_TYPES_MASK ) );
+    COMBOBOX_PREFILL( type, typeToCombo, info.windowType( SUPPORTED_MANAGED_WINDOW_TYPES_MASK ) );
     //CHECKBOX_PREFILL( ignoreposition, );
     LINEEDIT_PREFILL( minsize, sizeToStr, info.frameGeometry().size() );
     LINEEDIT_PREFILL( maxsize, sizeToStr, info.frameGeometry().size() );
@@ -679,15 +679,10 @@ void RulesWidget::prepareWindowSpecific( WId window )
 
 void RulesWidget::shortcutEditClicked()
     {
-#ifdef __GNUC__
-#warning KShortcutDialog is gone, and it is a good opportunity to clean up here
-#endif
-#if 0
     EditShortcutDialog dlg( topLevelWidget());
     dlg.setShortcut( shortcut->text());
     if( dlg.exec() == QDialog::Accepted )
         shortcut->setText( dlg.shortcut());
-#endif
     }
 
 RulesDialog::RulesDialog( QWidget* parent, const char* name )
@@ -737,18 +732,15 @@ void RulesDialog::accept()
     KDialog::accept();
     }
 
-#ifdef __GNUC__
-#warning KShortcutDialog is gone
-#endif
-#if 0
 EditShortcut::EditShortcut( QWidget* parent )
-: EditShortcutBase( parent )
+    : QWidget( parent )
     {
+    setupUi( this );
     }
 
 void EditShortcut::editShortcut()
     {
-    ShortcutDialog dlg( KShortcut( shortcut->text()), topLevelWidget());
+    ShortcutDialog dlg( QKeySequence( shortcut->text()), topLevelWidget());
     if( dlg.exec() == QDialog::Accepted )
         shortcut->setText( dlg.shortcut().toString());
     }
@@ -761,13 +753,13 @@ void EditShortcut::clearShortcut()
 EditShortcutDialog::EditShortcutDialog( QWidget* parent, const char* name )
 : KDialog( parent )
     {
-      setObjectName( name );
-      setModal( true );
-      setCaption( i18n( "Edit Shortcut" ) );
-      setButtons( Ok | Cancel );
+    setObjectName( name );
+    setModal( true );
+    setCaption( i18n( "Edit Shortcut" ) );
+    setButtons( Ok | Cancel );
 
-      widget = new EditShortcut( this );
-      setMainWidget( widget );
+    widget = new EditShortcut( this );
+    setMainWidget( widget );
     }
 
 void EditShortcutDialog::setShortcut( const QString& cut )
@@ -780,39 +772,40 @@ QString EditShortcutDialog::shortcut() const
     return widget->shortcut->text();
     }
 
-ShortcutDialog::ShortcutDialog( const KShortcut& cut, QWidget* parent )
-    : KShortcutDialog( cut, parent )
+ShortcutDialog::ShortcutDialog( const QKeySequence& cut, QWidget* parent )
+    : KDialog( parent )
+    , widget( new KKeySequenceWidget( this ))
     {
+    widget->setKeySequence( cut );
+    setMainWidget( widget );
     }
 
 void ShortcutDialog::accept()
     {
-    foreach( const QKeySequence &seq, shortcut() )
+    QKeySequence seq = shortcut();
+    if( !seq.isEmpty())
         {
-        if( seq.isEmpty())
-            break;
         if( seq[0] == Qt::Key_Escape )
             {
             reject();
             return;
             }
-        if( seq[0] == Qt::Key_Space )
+        if( seq[0] == Qt::Key_Space
+            || (seq[0] & Qt::KeyboardModifierMask) == 0 )
             { // clear
-            setShortcut( KShortcut());
-            KShortcutDialog::accept();
-            return;
-            }
-        if( (seq[0] & Qt::KeyboardModifierMask) == 0 )
-            { // no shortcuts without modifiers
-            KShortcut cut = shortcut();
-            cut.remove( seq );
-            setShortcut( cut );
+            widget->clearKeySequence();
+            KDialog::accept();
             return;
             }
         }
-    KShortcutDialog::accept();
+    KDialog::accept();
     }
-#endif
+
+QKeySequence ShortcutDialog::shortcut() const
+    {
+    return widget->keySequence();
+    }
+
 } // namespace
 
 #include "ruleswidget.moc"
