@@ -1,13 +1,23 @@
-/*****************************************************************
+/********************************************************************
  KWin - the KDE window manager
  This file is part of the KDE project.
 
 Copyright (C) 1999, 2000 Matthias Ettrich <ettrich@kde.org>
 Copyright (C) 2003 Lubos Lunak <l.lunak@kde.org>
 
-You can Freely distribute this program under the GNU General Public
-License. See the file "COPYING" for the exact licensing terms.
-******************************************************************/
+This program is free software; you can redistribute it and/or modify
+it under the terms of the GNU General Public License as published by
+the Free Software Foundation; either version 2 of the License, or
+(at your option) any later version.
+
+This program is distributed in the hope that it will be useful,
+but WITHOUT ANY WARRANTY; without even the implied warranty of
+MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+GNU General Public License for more details.
+
+You should have received a copy of the GNU General Public License
+along with this program.  If not, see <http://www.gnu.org/licenses/>.
+*********************************************************************/
 
 // SELI zmenit doc
 
@@ -427,7 +437,7 @@ void Workspace::restackClientUnderActive( Client* c )
     if( Client::belongToSameApplication( active_client, c ))
         { // put it below the active window if it's the same app
         unconstrained_stacking_order.removeAll( c );
-        unconstrained_stacking_order.insert( unconstrained_stacking_order.find( active_client ), c );
+        unconstrained_stacking_order.insert( unconstrained_stacking_order.indexOf( active_client ), c );
         }
     else
         { // put in the stacking order below _all_ windows belonging to the active application
@@ -456,7 +466,7 @@ void Workspace::restackClientUnderActive( Client* c )
             if( Client::belongToSameApplication( active_client, c ))
                 { // put it after the active window if it's the same app
                 focus_chain[ desktop ].removeAll( c );
-                focus_chain[ desktop ].insert( focus_chain[ desktop ].find( active_client ), c );
+                focus_chain[ desktop ].insert( focus_chain[ desktop ].indexOf( active_client ), c );
                 }
             else
                 { // put it in focus_chain[currentDesktop()] after all windows belonging to the active applicationa
@@ -480,7 +490,7 @@ void Workspace::restackClientUnderActive( Client* c )
         if( Client::belongToSameApplication( active_client, c ))
             {
             global_focus_chain.removeAll( c );
-            global_focus_chain.insert( global_focus_chain.find( active_client ), c );
+            global_focus_chain.insert( global_focus_chain.indexOf( active_client ), c );
             }
         else
             {
