@@ -30,7 +30,6 @@ DEALINGS IN THE SOFTWARE.
 #include <QtCore/QObject>
 #include <QtGui/QIcon>
 #include <netwm_def.h>
-#include <kdeversion.h>
 #include <QtGui/QMouseEvent>
 
 class KDecorationOptionsPrivate;
@@ -72,7 +71,7 @@ public:
 	MaximizeVertical   = 1, ///< The window is maximized vertically.
 	MaximizeHorizontal = 2, ///< The window is maximized horizontally.
 	/// Equal to @p MaximizeVertical | @p MaximizeHorizontal
-	MaximizeFull = MaximizeVertical | MaximizeHorizontal 
+	MaximizeFull = MaximizeVertical | MaximizeHorizontal
         };
 
     enum WindowOperation
@@ -114,9 +113,9 @@ public:
 	ColorButtonBg,   ///< The color to use for the titlebar buttons
 	ColorFrame,      ///< The color for the window frame (border)
 	ColorHandle,     ///< The color for the resize handle
-	NUM_COLORS
+	NUM_COLORS       ///< @internal This value may change, do not use
 	};
-    
+
     /**
      * These flags specify which settings changed when rereading settings.
      * Each setting in class KDecorationOptions specifies its matching flag.
@@ -130,7 +129,7 @@ public:
         SettingTooltips   = 1 << 4, ///< The tooltip setting was changed
         SettingBorder     = 1 << 5  ///< The border size setting was changed
         };
-        
+
     /**
      * Border size. KDecorationOptions::preferredBorderSize() returns
      * one of these values.
@@ -153,7 +152,10 @@ public:
      */
     enum Ability
         {
+        // announce
         AbilityAnnounceButtons = 0, ///< decoration supports AbilityButton* values (always use)
+        AbilityAnnounceColors = 1, ///< decoration supports AbilityColor* values (always use)
+        // buttons
         AbilityButtonMenu = 1000,   ///< decoration supports the menu button
         AbilityButtonOnAllDesktops = 1001, ///< decoration supports the on all desktops button
         AbilityButtonSpacer = 1002, ///< decoration supports inserting spacers between buttons
@@ -165,18 +167,27 @@ public:
         AbilityButtonBelowOthers = 1008, ///< decoration supports a below button
         AbilityButtonShade = 1009, ///< decoration supports a shade button
         AbilityButtonResize = 1010, ///< decoration supports a resize button
+        // colors
+        AbilityColorTitleBack = 2000, ///< decoration supports titlebar background color
+        AbilityColorTitleFore = 2001, ///< decoration supports titlebar foreground color
+        AbilityColorTitleBlend = 2002, ///< decoration supports second titlebar background color
+        AbilityColorFrame = 2010, ///< decoration supports frame color
+        AbilityColorHandle = 2011, ///< decoration supports resize handle color
+        AbilityColorButtonBack = 2020, ///< decoration supports button background color
+        AbilityColorButtonFore = 2021, ///< decoration supports button foreground color
+        // TODO colors for individual button types
         ABILITY_DUMMY = 10000000
         };
 
     enum Requirement { REQUIREMENT_DUMMY = 1000000 };
-};    
+};
 
 class KDecorationProvides
     : public KDecorationDefines
     {
     public:
-        virtual  ~KDecorationProvides(){}
-		virtual bool provides( Requirement req ) = 0;
+        virtual ~KDecorationProvides(){}
+        virtual bool provides( Requirement req ) = 0;
     };
 
 /**
@@ -243,6 +254,10 @@ public:
     * The changed flags for this setting is SettingButtons.
     */
     QString titleButtonsLeft() const;
+    /**
+     * Returns the default left button sequence
+     */
+    static QString defaultTitleButtonsLeft();
     /**
     * If customButtonPositions() returns true, titleButtonsRight
     * returns which buttons should be on the right side of the titlebar from left
@@ -255,13 +270,16 @@ public:
     * The changed flags for this setting is SettingButtons.
     */
     QString titleButtonsRight() const;
-
+    /**
+     * Returns the default right button sequence.
+     */
+    static QString defaultTitleButtonsRight();
     /**
     * @returns true if the style should use tooltips for window buttons
     * The changed flags for this setting is SettingTooltips.
     */
     bool showTooltips() const;
-    
+
     /**
      * The preferred border size selected by the user, e.g. for accessibility
      * reasons, or when using high resolution displays. It's up to the decoration
@@ -285,7 +303,7 @@ public:
     /**
      * @internal
      */
-		WindowOperation operationMaxButtonClick( Qt::MouseButtons button ) const;
+    WindowOperation operationMaxButtonClick( Qt::MouseButtons button ) const;
 
     /**
      * @internal
@@ -321,9 +339,9 @@ class KWIN_EXPORT KDecoration
 	 * Destroys the KDecoration.
 	 */
 	virtual ~KDecoration();
-	
+
 	// requests from decoration
-	
+
 	/**
 	 * Returns the KDecorationOptions object, which is used to access
 	 * configuration settings for the decoration.
@@ -416,7 +434,7 @@ class KWIN_EXPORT KDecoration
 	 * to support older code). For a description of all window types,
 	 * see the definition of the NET::WindowType type. Note that
 	 * some window types never have decorated windows.
-	 * 
+	 *
 	 * An example of usage:
 	 * @code
 	 * const unsigned long supported_types = NET::NormalMask | NET::DesktopMask
@@ -441,12 +459,12 @@ class KWIN_EXPORT KDecoration
 	 */
 	QString caption() const;
 	/**
-	 * This function invokes the window operations menu. 
+	 * This function invokes the window operations menu.
 	 * \param pos specifies the place on the screen where the menu should
 	 * show up. The menu pops up at the bottom-left corner of the specified
 	 * rectangle, unless there is no space, in which case the menu is
 	 * displayed above the rectangle.
-	 * 
+	 *
 	 * \note Decorations that enable a double-click operation for the menu
 	 * button must ensure to call \a showWindowMenu() with the \a pos
 	 * rectangle set to the menu button geometry.
@@ -469,7 +487,7 @@ class KWIN_EXPORT KDecoration
 	void showWindowMenu( const QRect &pos );
 
 	/**
-	 * Overloaded version of the above. 
+	 * Overloaded version of the above.
 	 */
 	void showWindowMenu( QPoint pos );
         /**
@@ -515,11 +533,6 @@ class KWIN_EXPORT KDecoration
          * @param r The region you want to check for holes
          */
         QRegion unobscuredRegion( const QRegion& r ) const;
-	/**
-	 * Returns the main workspace widget. The main purpose of this function is to
-	 * allow painting the minimize animation or the transparent move bound on it.
-	 */
-        QWidget* workspaceWidget() const;
         /**
          * Returns the handle of the window that is being decorated. It is possible
          * the returned value will be 0.
@@ -556,7 +569,7 @@ class KWIN_EXPORT KDecoration
 	 * instead of in the constructor.
 	 */
         virtual void init() = 0; // called once right after created
-	
+
         /**
          * This function should return mouse cursor position in the decoration.
          * Positions at the edge will result in window resizing with mouse button
@@ -638,24 +651,28 @@ class KWIN_EXPORT KDecoration
          * Note that if you e.g. paint the outline using a 5 pixels wide line,
          * you should compensate for the 2 pixels that would make the window
          * look larger.
+         * It is the decoration's responsibility to do the painting, using
+         * e.g. code like:
+         * @code
+         *     Display* dpy = QX11Info::display();
+         *     XGCValues xgc;
+         *     xgc.function = GXxor;
+         *     xgc.foreground = WhitePixel( dpy, DefaultScreen( dpy ));
+         *     xgc.line_width = width;
+         *     xgc.subwindow_mode = IncludeInferiors;
+         *     GC gc = XCreateGC( dpy, DefaultRootWindow( dpy ),
+         *         GCFunction | GCForeground | GCLineWidth | GCSubwindowMode, &xgc );
+         *     XDrawRectangle( dpy, DefaultRootWindow( dpy ), gc, r.x(), r.y(), r.width(), r.height());
+         *     XFreeGC( dpy, gc );
+         * @endcode
 	 *
 	 * @param geom  The geometry at this the bound should be drawn
-	 * @param clear @a true if the bound should be cleared
+	 * @param clear @a true if the bound should be cleared (when doing the usual XOR
+         *              painting this argument can be simply ignored)
 	 *
-	 * @see workspaceWidget() and geometry().
+	 * @see geometry()
 	 */
         virtual bool drawbound( const QRect& geom, bool clear );
-	/**
-	 * This function may be reimplemented to provide custom minimize/restore animations
-	 * The reimplementation is allowed to perform X server grabs if necessary
-         * (only using the functions provided by this API, no direct Xlib calls), but no
-	 * futher event processing is allowed (i.e. no kapp->processEvents()).
-	 * @a False should be returned if the default implementation should be used.
-	 * Note that you should not use this function to force disabling of the animation.
-	 *
-	 * @see workspaceWidget(), geometry() and helperShowHide().
-	 */
-        virtual bool animateMinimize( bool minimize );
         /**
          * @internal Reserved.
          */
@@ -664,7 +681,7 @@ class KWIN_EXPORT KDecoration
 	/**
 	 * This function is called to reset the decoration on settings changes.
 	 * It is usually invoked by calling KDecorationFactory::resetDecorations().
-	 * 
+	 *
 	 * @param changed Specifies which settings were changed, given by the SettingXXX masks
 	 */
         virtual void reset( unsigned long changed );
@@ -676,7 +693,7 @@ class KWIN_EXPORT KDecoration
 	 * the main widget of the decoration. The widget should be created
 	 * with parent specified by initialParentWidget() and flags
 	 * specified by initialWFlags().
-	 */	
+	 */
         void setMainWidget( QWidget* );
 	/**
 	 * Convenience functions that creates and sets a main widget as necessary.
@@ -698,14 +715,7 @@ class KWIN_EXPORT KDecoration
 	 * that affect widget drawing are allowed. Window type flags like WX11BypassWM
 	 * or WStyle_NoBorder are forbidden.
 	 */
-		Qt::WFlags initialWFlags() const;
-	/**
-	 * This function is only allowed to be called once from animateMinimize().
-	 * It can be used if the window should be shown or hidden at a specific
-	 * time during the animation. It is forbidden to use this function
-	 * for other purposes.
-	 */
-        void helperShowHide( bool show );
+	Qt::WFlags initialWFlags() const;
 	/**
 	 * Returns the main widget for the decoration.
 	 */
@@ -730,7 +740,7 @@ class KWIN_EXPORT KDecoration
 	// requests from decoration
 
 	/**
-	 * This function can be called by the decoration to request 
+	 * This function can be called by the decoration to request
 	 * closing of the decorated window. Note that closing the window
 	 * also involves destroying the decoration.
          * IMPORTANT: This function may destroy the current decoration object,
@@ -851,10 +861,10 @@ inline int KDecoration::width() const
     {
     return geometry().width();
     }
-    
+
 inline int KDecoration::height() const
     {
     return geometry().height();
     }
-    
+
 #endif
