@@ -88,7 +88,6 @@ class KDecorationBridge : public KDecorationDefines
         virtual QRect geometry() const = 0;
         virtual QRect iconGeometry() const = 0;
         virtual QRegion unobscuredRegion( const QRegion& r ) const = 0;
-        virtual QWidget* workspaceWidget() const = 0;
         virtual WId windowId() const = 0;
 	virtual void closeWindow() = 0;
 	virtual void maximize( MaximizeMode mode ) = 0;
@@ -104,7 +103,6 @@ class KDecorationBridge : public KDecorationDefines
         virtual int currentDesktop() const = 0;
         virtual QWidget* initialParentWidget() const = 0;
         virtual Qt::WFlags initialWFlags() const = 0;
-        virtual void helperShowHide( bool ) = 0;
         virtual void grabXServer( bool grab ) = 0;
     };
 
