@@ -1,18 +1,27 @@
-/*****************************************************************
+/********************************************************************
  KWin - the KDE window manager
  This file is part of the KDE project.
 
 Copyright (C) 2006 Lubos Lunak <l.lunak@kde.org>
 
-You can Freely distribute this program under the GNU General Public
-License. See the file "COPYING" for the exact licensing terms.
-******************************************************************/
+This program is free software; you can redistribute it and/or modify
+it under the terms of the GNU General Public License as published by
+the Free Software Foundation; either version 2 of the License, or
+(at your option) any later version.
 
-// TODO MIT or some other license, perhaps move to some lib
+This program is distributed in the hope that it will be useful,
+but WITHOUT ANY WARRANTY; without even the implied warranty of
+MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+GNU General Public License for more details.
+
+You should have received a copy of the GNU General Public License
+along with this program.  If not, see <http://www.gnu.org/licenses/>.
+*********************************************************************/
 
 #ifndef KWINEFFECTS_H
 #define KWINEFFECTS_H
 
+#include <kwinconfig.h>
 #include <kwinglobals.h>
 
 #include <QtCore/QPair>
@@ -24,12 +33,20 @@ License. See the file "COPYING" for the exact licensing terms.
 #include <QtCore/QList>
 #include <QtCore/QHash>
 
-#include <KDE/KPluginFactory> 
+#include <KDE/KPluginFactory>
+#include <KDE/KShortcutsEditor>
 
 #include <assert.h>
 
+#define KWIN_EFFECT_API_MAKE_VERSION( major, minor ) (( major ) << 8 | ( minor )) 
+#define KWIN_EFFECT_API_VERSION_MAJOR 0
+#define KWIN_EFFECT_API_VERSION_MINOR 6
+#define KWIN_EFFECT_API_VERSION KWIN_EFFECT_API_MAKE_VERSION( \
+    KWIN_EFFECT_API_VERSION_MAJOR, KWIN_EFFECT_API_VERSION_MINOR )
+
 class KLibrary;
 class KConfigGroup;
+class KActionCollection;
 class QKeyEvent;
 
 namespace KWin
@@ -53,51 +70,168 @@ typedef QPair< Effect*, Window > InputWindowPair;
 typedef QList< EffectWindow* > EffectWindowList;
 
 
+/**
+ * @short Base class for all KWin effects
+ *
+ * This is the base class for all effects. By reimplementing virtual methods
+ *  of this class, you can customize how the windows are painted.
+ *
+ * The virtual methods of this class can broadly be divided into two
+ *  categories: the methods used for painting and those you can use to be
+ *  notified and react to certain events, e.g. that a window was closed.
+ *
+ * @section Chaining
+ * Most methods of this class are called in chain style. This means that when
+ *  effects A and B area active then first e.g. A::paintWindow() is called and
+ *  then from within that method B::paintWindow() is called (although
+ *  indirectly). To achieve this, you need to make sure to call corresponding
+ *  method in EffectsHandler class from each such method (using @ref effects
+ *  pointer):
+ * @code
+ *  void MyEffect::postPaintScreen()
+ *  {
+ *      // Do your own processing here
+ *      ...
+ *      // Call corresponding EffectsHandler method
+ *      effects->postPaintScreen();
+ *  }
+ * @endcode
+ *
+ * @section Effectsptr Effects pointer
+ * @ref effects pointer points to the global EffectsHandler object that you can
+ *  use to interact with the windows.
+ *
+ * @section painting Painting stages
+ * Painting of windows is done in three stages:
+ * @li First, the prepaint pass.<br>
+ *  Here you can specify how the windows will be painted, e.g. that they will
+ *  be translucent and transformed.
+ * @li Second, the paint pass.<br>
+ *  Here the actual painting takes place. You can change attributes such as
+ *  opacity of windows as well as apply transformations to them. You can also
+ *  paint something onto the screen yourself.
+ * @li Finally, the postpaint pass.<br>
+ *  Here you can mark windows, part of windows or even the entire screen for
+ *  repainting to create animations.
+ *
+ * For each stage there are *Screen() and *Window() methods. The window method
+ *  is called for every window which the screen method is usually called just
+ *  once.
+ **/
 class KWIN_EXPORT Effect
     {
     public:
-        // Flags controlling how painting is done.
+        /** Flags controlling how painting is done. */
         // TODO: is that ok here?
         enum
         {
-            // Window (or at least part of it) will be painted opaque.
+            /**
+             * Window (or at least part of it) will be painted opaque.
+             **/
             PAINT_WINDOW_OPAQUE         = 1 << 0,
-            // Window (or at least part of it) will be painted translucent.
+            /**
+             * Window (or at least part of it) will be painted translucent.
+             **/
             PAINT_WINDOW_TRANSLUCENT    = 1 << 1,
-            // Window will be painted with transformed geometry.
+            /**
+             * Window will be painted with transformed geometry.
+             **/
             PAINT_WINDOW_TRANSFORMED    = 1 << 2,
-            // Paint only a region of the screen (can be optimized, cannot
-            // be used together with TRANSFORMED flags).
+            /**
+             * Paint only a region of the screen (can be optimized, cannot
+             * be used together with TRANSFORMED flags).
+             **/
             PAINT_SCREEN_REGION         = 1 << 3,
-            // Whole screen will be painted with transformed geometry.
+            /**
+             * The whole screen will be painted with transformed geometry.
+             * Forces the entire screen to be painted.
+             **/
             PAINT_SCREEN_TRANSFORMED    = 1 << 4,
-            // At least one window will be painted with transformed geometry.
+            /**
+             * At least one window will be painted with transformed geometry.
+             * Forces the entire screen to be painted.
+             **/
             PAINT_SCREEN_WITH_TRANSFORMED_WINDOWS = 1 << 5,
-            // Clear whole background as the very first step, without optimizing it
+            /**
+             * Clear whole background as the very first step, without optimizing it
+             **/
             PAINT_SCREEN_BACKGROUND_FIRST = 1 << 6
         };
 
+        /**
+         * Constructs new Effect object.
+         **/
         Effect();
+        /**
+         * Destructs the Effect object.
+         **/
         virtual ~Effect();
 
+        /**
+         * Called before starting to paint the screen.
+         * In this method you can:
+         * @li set whether the windows or the entire screen will be transformed
+         * @li change the region of the screen that will be painted
+         * @li do various housekeeping tasks such as initing your effect's variables
+                for the upcoming paint pass or updating animation's progress
+        **/
         virtual void prePaintScreen( ScreenPrePaintData& data, int time );
+        /**
+         * In this method you can:
+         * @li paint something on top of the windows (by painting after calling
+         *      effects->paintScreen())
+         * @li paint multiple desktops and/or multiple copies of the same desktop
+         *      by calling effects->paintScreen() multiple times
+         **/
         virtual void paintScreen( int mask, QRegion region, ScreenPaintData& data );
+        /**
+         * Called after all the painting has been finished.
+         * In this method you can:
+         * @li schedule next repaint in case of animations
+         * You shouldn't paint anything here.
+         **/
         virtual void postPaintScreen();
+
+        /**
+         * Called for every window before the actual paint pass
+         * In this method you can:
+         * @li enable or disable painting of the window (e.g. enable paiting of minimized window)
+         * @li set window to be painted with translucency
+         * @li set window to be transformed
+         * @li request the window to be divided into multiple parts
+         **/
         virtual void prePaintWindow( EffectWindow* w, WindowPrePaintData& data, int time );
-        // paintWindow() can do various transformations
+        /**
+         * This is the main method for painting windows.
+         * In this method you can:
+         * @li do various transformations
+         * @li change opacity of the window
+         * @li change brightness and/or saturation, if it's supported
+         **/
         virtual void paintWindow( EffectWindow* w, int mask, QRegion region, WindowPaintData& data );
+        /**
+         * Called for every window after all painting has been finished.
+         * In this method you can:
+         * @li schedule next repaint for individual window(s) in case of animations
+         * You shouldn't paint anything here.
+         **/
         virtual void postPaintWindow( EffectWindow* w );
 
-        // drawWindow() is used even for thumbnails etc. - it can alter the window itself where it
-        // makes sense (e.g.darkening out unresponsive windows), but it cannot do transformations
+        /**
+         * Can be called to draw multiple copies (e.g. thumbnails) of a window.
+         * You can change window's opacity/brightness/etc here, but you can't
+         *  do any transformations
+         **/
         virtual void drawWindow( EffectWindow* w, int mask, QRegion region, WindowPaintData& data );
-        // This function is used e.g. by the shadow effect which adds area around windows
-        // that needs to be painted as well - e.g. when a window is hidden and the workspace needs
-        // to be repainted at that area, shadow's transformWindowDamage() adds the shadow area
-        // to it, so that it is repainted as well.
+        /**
+         * This function is used e.g. by the shadow effect which adds area around windows
+         * that needs to be painted as well - e.g. when a window is hidden and the workspace needs
+         * to be repainted at that area, shadow's transformWindowDamage() adds the shadow area
+         * to it, so that it is repainted as well.
+         **/
         virtual QRect transformWindowDamage( EffectWindow* w, const QRect& r );
 
-        // called when moved/resized or once after it's finished
+        /** called when moved/resized or once after it's finished */
         virtual void windowUserMovedResized( EffectWindow* c, bool first, bool last );
         virtual void windowOpacityChanged( EffectWindow* c, double old_opacity );
         virtual void windowAdded( EffectWindow* c );
@@ -110,9 +244,18 @@ class KWIN_EXPORT Effect
         virtual void desktopChanged( int old );
         virtual void windowDamaged( EffectWindow* w, const QRect& r );
         virtual void windowGeometryShapeChanged( EffectWindow* w, const QRect& old );
-        virtual void mouseChanged( const QPoint& pos, const QPoint& old,
-            Qt::MouseButtons buttons, Qt::KeyboardModifiers modifiers );
+        virtual void mouseChanged( const QPoint& pos, const QPoint& oldpos,
+            Qt::MouseButtons buttons, Qt::MouseButtons oldbuttons,
+            Qt::KeyboardModifiers modifiers, Qt::KeyboardModifiers oldmodifiers );
         virtual void grabbedKeyboardEvent( QKeyEvent* e );
+        /**
+         Receives events registered for using EffectsHandler::registerPropertyType().
+         Use readProperty() to get the property data.
+         Note that the property may be already set on the window, so doing the same
+         processing from windowAdded() (e.g. simply calling propertyNotify() from it)
+         is usually needed.
+         */
+        virtual void propertyNotify( EffectWindow* w, long atom );
 
         virtual void tabBoxAdded( int mode );
         virtual void tabBoxClosed();
@@ -123,13 +266,18 @@ class KWIN_EXPORT Effect
         static int displayHeight();
         static QPoint cursorPos();
 
-        // Interpolates between x and y
+        /**
+         * Linearly interpolates between @p x and @p y.
+         *
+         * Returns @p x when @p a = 0; returns @p y when @p a = 1.
+         **/
         static double interpolate(double x, double y, double a)
             {
             return x * (1 - a) + y * a;
             }
-        // helper to set WindowPaintData and QRegion to necessary transformations so that
-        // a following drawWindow() would put the window at the requested geometry (useful for thumbnails)
+        /** Helper to set WindowPaintData and QRegion to necessary transformations so that
+         * a following drawWindow() would put the window at the requested geometry (useful for thumbnails)
+         **/
         static void setPositionTransformations( WindowPaintData& data, QRect& region, EffectWindow* w,
             const QRect& r, Qt::AspectRatioMode aspect );
     };
@@ -147,6 +295,7 @@ class KWIN_EXPORT Effect
 #define KWIN_EFFECT( name, classname ) \
     extern "C" { \
         KWIN_EXPORT Effect* effect_create_kwin4_effect_##name() { return new classname; } \
+        KWIN_EXPORT int effect_version_kwin4_effect_##name() { return KWIN_EFFECT_API_VERSION; } \
     }
 /**
  * Defines the function used to check whether an effect is supported
@@ -170,6 +319,16 @@ class KWIN_EXPORT Effect
 #define KWIN_EFFECT_CONFIG_FACTORY K_PLUGIN_FACTORY_DECLARATION(EffectFactory)
 
 
+/**
+ * @short Manager class that handles all the effects.
+ *
+ * This class creates Effect objects and calls it's appropriate methods.
+ *
+ * Effect objects can call methods of this class to interact with the
+ *  workspace, e.g. to activate or move a specific window, change current
+ *  desktop or create a special input window to receive mouse and keyboard
+ *  events.
+ **/
 class KWIN_EXPORT EffectsHandler
     {
     friend class Effect;
@@ -189,7 +348,7 @@ class KWIN_EXPORT EffectsHandler
         // covering the whole screen is created and all mouse events will be intercepted by it.
         // The effect's windowInputMouseEvent() will get called with such events.
         virtual Window createInputWindow( Effect* e, int x, int y, int w, int h, const QCursor& cursor ) = 0;
-        virtual Window createInputWindow( Effect* e, const QRect& r, const QCursor& cursor );
+        Window createInputWindow( Effect* e, const QRect& r, const QCursor& cursor );
         virtual Window createFullScreenInputWindow( Effect* e, const QCursor& cursor );
         virtual void destroyInputWindow( Window w ) = 0;
         virtual QPoint cursorPos() const = 0;
@@ -211,10 +370,18 @@ class KWIN_EXPORT EffectsHandler
         virtual int numberOfDesktops() const = 0;
         virtual void setCurrentDesktop( int desktop ) = 0;
         virtual QString desktopName( int desktop ) const = 0;
+        virtual int activeScreen() const = 0; // Xinerama
+        virtual QRect clientArea( clientAreaOption, int screen, int desktop ) const = 0;
+        virtual QRect clientArea( clientAreaOption, const EffectWindow* c ) const = 0;
         virtual QRect clientArea( clientAreaOption, const QPoint& p, int desktop ) const = 0;
         virtual void calcDesktopLayout(int* x, int* y, Qt::Orientation* orientation) const = 0;
         virtual bool optionRollOverDesktops() const = 0;
+        virtual int desktopToLeft( int desktop, bool wrap ) const = 0;
+        virtual int desktopToRight( int desktop, bool wrap ) const = 0;
+        virtual int desktopUp( int desktop, bool wrap ) const = 0;
+        virtual int desktopDown( int desktop, bool wrap ) const = 0;
 
+        virtual EffectWindow* findWindow( WId id ) const = 0;
         virtual EffectWindowList stackingOrder() const = 0;
         // window will be temporarily painted as if being at the top of the stack
         virtual void setElevatedWindow( EffectWindow* w, bool set ) = 0;
@@ -229,10 +396,17 @@ class KWIN_EXPORT EffectsHandler
         virtual int currentTabBoxDesktop() const = 0;
         virtual EffectWindow* currentTabBoxWindow() const = 0;
 
+        virtual void setActiveFullScreenEffect( Effect* e ) = 0;
+        virtual Effect* activeFullScreenEffect() const = 0;
+
         virtual void pushRenderTarget(GLRenderTarget* target) = 0;
         virtual GLRenderTarget* popRenderTarget() = 0;
 
-        // Repaints the entire workspace
+        /**
+         * Schedules the entire workspace to be repainted next time.
+         * If you call it during painting (including prepaint) then it does not
+         *  affect the current painting.
+         **/
         virtual void addRepaintFull() = 0;
         virtual void addRepaint( const QRect& r ) = 0;
         virtual void addRepaint( int x, int y, int w, int h ) = 0;
@@ -241,6 +415,14 @@ class KWIN_EXPORT EffectsHandler
         virtual unsigned long xrenderBufferPicture() = 0;
         virtual void reconfigure() = 0;
 
+        /**
+         Makes KWin core watch PropertyNotify events for the given atom,
+         or stops watching if reg is false (must be called the same number
+         of times as registering). Events are sent using Effect::propertyNotify().
+         Note that even events that haven't been registered for can be received.
+        */
+        virtual void registerPropertyType( long atom, bool reg ) = 0;
+
         /**
          * Paints given text onto screen, possibly in elided form
          * @param text
@@ -281,22 +463,25 @@ class KWIN_EXPORT EffectsHandler
     };
 
 
-// This class is a representation of a window used by/for Effect classes.
-// The purpose is to hide internal data and also to serve as a single
-// representation for the case when Client/Unmanaged becomes Deleted.
+/**
+ * @short Representation of a window used by/for Effect classes.
+ *
+ * The purpose is to hide internal data and also to serve as a single
+ *  representation for the case when Client/Unmanaged becomes Deleted.
+ **/
 class KWIN_EXPORT EffectWindow
     {
     public:
-        // Flags explaining why painting should be disabled
+        /**  Flags explaining why painting should be disabled  */
         enum
         {
-            // Window will not be painted
+            /**  Window will not be painted  */
             PAINT_DISABLED              = 1 << 0,
-            // Window will not be painted because it is deleted
+            /**  Window will not be painted because it is deleted  */
             PAINT_DISABLED_BY_DELETE    = 1 << 1,
-            // Window will not be painted because of which desktop it's on
+            /**  Window will not be painted because of which desktop it's on  */
             PAINT_DISABLED_BY_DESKTOP   = 1 << 2,
-            // Window will not be painted because it is minimized
+            /**  Window will not be painted because it is minimized  */
             PAINT_DISABLED_BY_MINIMIZE  = 1 << 3
         };
 
@@ -327,6 +512,9 @@ class KWIN_EXPORT EffectWindow
         virtual int width() const = 0;
         virtual int height() const = 0;
         virtual QRect geometry() const = 0;
+        virtual QRegion shape() const = 0;
+        /** @internal Do not use */
+        virtual bool hasOwnShape() const = 0; // only for shadow effect, for now
         virtual QPoint pos() const = 0;
         virtual QSize size() const = 0;
         virtual QRect rect() const = 0;
@@ -339,10 +527,12 @@ class KWIN_EXPORT EffectWindow
          */
         virtual QRect contentsRect() const = 0;
         bool hasDecoration() const;
+        virtual QByteArray readProperty( long atom, long type, int format ) const = 0;
 
         virtual QString caption() const = 0;
         virtual QPixmap icon() const = 0;
         virtual QString windowClass() const = 0;
+        virtual QString windowRole() const = 0;
         virtual const EffectWindowGroup* group() const = 0;
 
         virtual bool isDesktop() const = 0;
@@ -361,7 +551,8 @@ class KWIN_EXPORT EffectWindow
         virtual bool isNotification() const = 0;
         virtual bool isComboBox() const = 0;
         virtual bool isDNDIcon() const = 0;
-        
+        virtual bool isManaged() const = 0; // whether it's managed or override-redirect
+
         virtual bool isModal() const = 0;
         virtual EffectWindow* findModal() = 0;
         virtual EffectWindowList mainWindows() const = 0;
@@ -377,8 +568,15 @@ class KWIN_EXPORT EffectWindowGroup
         virtual EffectWindowList members() const = 0;
     };
 
+class KWIN_EXPORT GlobalShortcutsEditor : public KShortcutsEditor
+    {
+    public:
+        GlobalShortcutsEditor( QWidget *parent );
+    };
+
 /**
  * @short Vertex class
+ *
  * A vertex is one position in a window. WindowQuad consists of four WindowVertex objects
  * and represents one part of a window.
  **/
@@ -411,6 +609,7 @@ enum WindowQuadType
 
 /**
  * @short Class representing one area of a window.
+ *
  * WindowQuads consists of four WindowVertex objects and represents one part of a window.
  */
 // NOTE: This class expects the (original) vertices to be in the clockwise order starting from topleft.
@@ -455,7 +654,14 @@ class KWIN_EXPORT WindowPrePaintData
     {
     public:
         int mask;
+        /**
+         * Region that will be painted, in screen coordinates.
+         **/
         QRegion paint;
+        /**
+         * The clip region will be substracted from paint region of following windows.
+         * I.e. window will definitely cover it's clip region
+         **/
         QRegion clip;
         WindowQuadList quads;
         /**
@@ -522,6 +728,9 @@ class KWIN_EXPORT ScreenPrePaintData
         QRegion paint;
     };
 
+/**
+ * Pointer to the global EffectsHandler object.
+ **/
 extern KWIN_EXPORT EffectsHandler* effects;
 
 /***************************************************************
