@@ -1,16 +1,26 @@
-/*****************************************************************
+/********************************************************************
  KWin - the KDE window manager
  This file is part of the KDE project.
 
 Copyright (C) 2006-2007 Rivo Laks <rivolaks@hot.ee>
 
-You can Freely distribute this program under the GNU General Public
-License. See the file "COPYING" for the exact licensing terms.
-******************************************************************/
+This program is free software; you can redistribute it and/or modify
+it under the terms of the GNU General Public License as published by
+the Free Software Foundation; either version 2 of the License, or
+(at your option) any later version.
+
+This program is distributed in the hope that it will be useful,
+but WITHOUT ANY WARRANTY; without even the implied warranty of
+MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+GNU General Public License for more details.
+
+You should have received a copy of the GNU General Public License
+along with this program.  If not, see <http://www.gnu.org/licenses/>.
+*********************************************************************/
 
 #include "kwinglutils.h"
 
-#ifdef HAVE_OPENGL
+#ifdef KWIN_HAVE_OPENGL
 
 #include "kwinglobals.h"
 #include "kwineffects.h"
@@ -279,6 +289,8 @@ void renderRoundBox( const QRect& area, float roundness, GLTexture* texture )
     glPushMatrix();
 
     QVector<float> verts, texcoords;
+    verts.reserve(80);
+    texcoords.reserve(80);
     // center
     addQuadVertices(verts, area.left() + roundness, area.top() + roundness, area.right() - roundness, area.bottom() - roundness);
     addQuadVertices(texcoords, 0.5, 0.5, 0.5, 0.5);
