@@ -1,13 +1,23 @@
-/*****************************************************************
+/********************************************************************
  KWin - the KDE window manager
  This file is part of the KDE project.
 
 Copyright (C) 1999, 2000 Matthias Ettrich <ettrich@kde.org>
 Copyright (C) 2003 Lubos Lunak <l.lunak@kde.org>
 
-You can Freely distribute this program under the GNU General Public
-License. See the file "COPYING" for the exact licensing terms.
-******************************************************************/
+This program is free software; you can redistribute it and/or modify
+it under the terms of the GNU General Public License as published by
+the Free Software Foundation; either version 2 of the License, or
+(at your option) any later version.
+
+This program is distributed in the hope that it will be useful,
+but WITHOUT ANY WARRANTY; without even the implied warranty of
+MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+GNU General Public License for more details.
+
+You should have received a copy of the GNU General Public License
+along with this program.  If not, see <http://www.gnu.org/licenses/>.
+*********************************************************************/
 
 #include "options.h"
 
@@ -73,7 +83,7 @@ unsigned long Options::updateSettings()
     if ( val == "CDE" )
         altTabStyle = CDE;
         
-    separateScreenFocus = config.readEntry( "SeparateScreenFocus", true );
+    separateScreenFocus = config.readEntry( "SeparateScreenFocus", false );
     activeMouseScreen = config.readEntry( "ActiveMouseScreen", focusPolicy != ClickToFocus );
 
     rollOverDesktops = config.readEntry("RollOverDesktops", true);
@@ -85,7 +95,7 @@ unsigned long Options::updateSettings()
     if( !focusPolicyIsReasonable()) // #48786, comments #7 and later
         focusStealingPreventionLevel = 0;
 
-    KConfig *gc = new KConfig("kdeglobals", KConfig::CascadeConfig);
+    KConfig *gc = new KConfig("kdeglobals", KConfig::NoGlobals);
     bool isVirtual = KApplication::desktop()->isVirtualDesktop();
     KConfigGroup gWindowsConfig(gc, "Windows");
     xineramaEnabled = gWindowsConfig.readEntry ("XineramaEnabled", isVirtual) &&
@@ -107,10 +117,6 @@ unsigned long Options::updateSettings()
     xineramaPlacementScreen = qBound( -1, config.readEntry( "XineramaPlacementScreen", -1 ),
         qApp->desktop()->numScreens() - 1 );
 
-    animateShade = config.readEntry("AnimateShade", true);
-    animateMinimize = config.readEntry("AnimateMinimize", true);
-    animateMinimizeSpeed = config.readEntry("AnimateMinimizeSpeed", 5 );
-
     if( focusPolicy == ClickToFocus )
         {
         autoRaise = false;
@@ -161,7 +167,7 @@ unsigned long Options::updateSettings()
     showDesktopIsMinimizeAll = config.readEntry( "ShowDesktopIsMinimizeAll", false );
 
     // Mouse bindings
-    config.changeGroup("MouseBindings");
+    config = KConfigGroup(_config,"MouseBindings");
     CmdActiveTitlebar1 = mouseCommand(config.readEntry("CommandActiveTitlebar1","Raise"), true );
     CmdActiveTitlebar2 = mouseCommand(config.readEntry("CommandActiveTitlebar2","Lower"), true );
     CmdActiveTitlebar3 = mouseCommand(config.readEntry("CommandActiveTitlebar3","Operations menu"), true );
@@ -178,11 +184,8 @@ unsigned long Options::updateSettings()
     CmdAll3 = mouseCommand(config.readEntry("CommandAll3","Resize"), false );
     CmdAllWheel = mouseWheelCommand(config.readEntry("CommandAllWheel","Nothing"));
 
-    config.changeGroup("Translucency");
+    config=KConfigGroup(_config,"Compositing");
     refreshRate = config.readEntry( "RefreshRate", 0 );
-    glStrictBinding = config.readEntry( "GLStrictBinding", false );
-    const HiddenPreviews hps[] = { HiddenPreviewsNever, HiddenPreviewsKeep, HiddenPreviewUpdate, HiddenPreviewsActive };
-    hiddenPreviews = hps[ qBound( 0, config.readEntry( "HiddenPreviews", 3 ), 3 ) ];
 
     // Read button tooltip animation effect from kdeglobals
     // Since we want to allow users to enable window decoration tooltips
@@ -231,8 +234,12 @@ void Options::reloadCompositingSettings(const CompositingPrefs& prefs)
     glDirect = config.readEntry("GLDirect", prefs.enableDirectRendering() );
     glVSync = config.readEntry("GLVSync", prefs.enableVSync() );
     smoothScale = qBound( -1, config.readEntry( "GLTextureFilter", -1 ), 2 );
+    glStrictBinding = config.readEntry( "GLStrictBinding", prefs.strictBinding());
 
     xrenderSmoothScale = config.readEntry("XRenderSmoothScale", false );
+
+    const HiddenPreviews hps[] = { HiddenPreviewsNever, HiddenPreviewsKeep, HiddenPreviewUpdate, HiddenPreviewsActive };
+    hiddenPreviews = hps[ qBound( 0, config.readEntry( "HiddenPreviews", 0 ), 3 ) ];
     }
 
 
