@@ -1,15 +1,26 @@
-/*****************************************************************
+/********************************************************************
  KWin - the KDE window manager
  This file is part of the KDE project.
 
 Copyright (C) 2006 Lubos Lunak <l.lunak@kde.org>
 
-You can Freely distribute this program under the GNU General Public
-License. See the file "COPYING" for the exact licensing terms.
-
 Based on glcompmgr code by Felix Bellaby.
 Using code from Compiz and Beryl.
-******************************************************************/
+
+This program is free software; you can redistribute it and/or modify
+it under the terms of the GNU General Public License as published by
+the Free Software Foundation; either version 2 of the License, or
+(at your option) any later version.
+
+This program is distributed in the hope that it will be useful,
+but WITHOUT ANY WARRANTY; without even the implied warranty of
+MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+GNU General Public License for more details.
+
+You should have received a copy of the GNU General Public License
+along with this program.  If not, see <http://www.gnu.org/licenses/>.
+*********************************************************************/
+
 
 /*
  This is the OpenGL-based compositing code. It is the primary and most powerful
@@ -71,7 +82,7 @@ Sources and other compositing managers:
 // normally only few of them are enabled
 //#define CHECK_GL_ERROR
 
-#ifdef HAVE_OPENGL
+#ifdef KWIN_HAVE_OPENGL_COMPOSITING
 
 namespace KWin
 {
@@ -189,6 +200,7 @@ SceneOpenGL::~SceneOpenGL()
             glXDestroyPixmap( display(), last_pixmap );
         glXDestroyContext( display(), ctxdrawable );
         }
+    glXMakeCurrent( display(), None, NULL );
     glXDestroyContext( display(), ctxbuffer );
     checkGLError( "Cleanup" );
     }
@@ -297,10 +309,11 @@ bool SceneOpenGL::initRenderingContext()
             kDebug( 1212 ) << "Couldn't initialize rendering context";
             return false;
             }
+	glXMakeCurrent( display(), None, NULL );
         glXDestroyContext( display(), ctxbuffer );
         direct_rendering = false; // try again
         ctxbuffer = glXCreateNewContext( display(), fbcbuffer, GLX_RGBA_TYPE, NULL, GL_FALSE );
-        if( ctxbuffer == NULL || !glXMakeCurrent( display(), glxbuffer, ctxbuffer ))
+        if( ctxbuffer == NULL || !glXMakeCurrent( display(), glxbuffer, ctxbuffer ) || errs.error( true ))
             {
             kDebug( 1212 ) << "Couldn't initialize rendering context";
             return false;
@@ -548,6 +561,11 @@ bool SceneOpenGL::initDrawableConfigs()
         kDebug( 1212 ) << "Couldn't find framebuffer configuration for default depth!";
         return false;
         }
+    if( fbcdrawableinfo[ 32 ].fbconfig == NULL )
+        {
+        kDebug( 1212 ) << "Couldn't find framebuffer configuration for depth 32!";
+        return false;
+        }
     return true;
     }
 
@@ -672,12 +690,12 @@ void SceneOpenGL::paintBackground( QRegion region )
     {
     if( region == infiniteRegion())
         {
-        glClearColor( 1, 1, 1, 1 ); // white
+        glClearColor( 0, 0, 0, 1 ); // black
         glClear( GL_COLOR_BUFFER_BIT );
         }
     else
         {
-        glColor4f( 1, 1, 1, 1 ); // white
+        glColor4f( 0, 0, 0, 1 ); // black
         glBegin( GL_QUADS );
         foreach( QRect r, region.rects())
             {
@@ -694,6 +712,7 @@ void SceneOpenGL::windowAdded( Toplevel* c )
     {
     assert( !windows.contains( c ));
     windows[ c ] = new Window( c );
+    c->effectWindow()->setSceneWindow( windows[ c ]);
     }
 
 void SceneOpenGL::windowClosed( Toplevel* c, Deleted* deleted )
@@ -870,7 +889,9 @@ bool SceneOpenGL::Texture::load( const Pixmap& pix, const QSize& size,
             createTexture();
         // when the pixmap is bound to the texture, they share the same data, so the texture
         // updates automatically - no need to do anything in such case
-        if( bound_glxpixmap == None )
+        if( bound_glxpixmap != None )
+            glBindTexture( mTarget, mTexture );
+        else
             {
             int attrs[] =
                 {
@@ -911,11 +932,12 @@ bool SceneOpenGL::Texture::load( const Pixmap& pix, const QSize& size,
             }
         findTarget();
 #ifdef CHECK_GL_ERROR
-            checkGLError( "TextureLoadSHM1" );
+        checkGLError( "TextureLoadSHM1" );
 #endif
         if( mTexture == None )
             {
             createTexture();
+            glBindTexture( mTarget, mTexture );
             y_inverted = false;
             glTexImage2D( mTarget, 0, depth == 32 ? GL_RGBA : GL_RGB,
                 mSize.width(), mSize.height(), 0,
@@ -970,6 +992,7 @@ bool SceneOpenGL::Texture::load( const Pixmap& pix, const QSize& size,
         if( mTexture == None )
             {
             createTexture();
+            glBindTexture( mTarget, mTexture );
             y_inverted = false;
             glCopyTexImage2D( mTarget, 0,
                 depth == 32 ? GL_RGBA : GL_RGB,
