@@ -1,12 +1,22 @@
-/*****************************************************************
+/********************************************************************
  KWin - the KDE window manager
  This file is part of the KDE project.
 
 Copyright (C) 2006 Lubos Lunak <l.lunak@kde.org>
 
-You can Freely distribute this program under the GNU General Public
-License. See the file "COPYING" for the exact licensing terms.
-******************************************************************/
+This program is free software; you can redistribute it and/or modify
+it under the terms of the GNU General Public License as published by
+the Free Software Foundation; either version 2 of the License, or
+(at your option) any later version.
+
+This program is distributed in the hope that it will be useful,
+but WITHOUT ANY WARRANTY; without even the implied warranty of
+MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+GNU General Public License for more details.
+
+You should have received a copy of the GNU General Public License
+along with this program.  If not, see <http://www.gnu.org/licenses/>.
+*********************************************************************/
 
 /*
  This is the XRender-based compositing code. The primary compositing
@@ -29,7 +39,7 @@ License. See the file "COPYING" for the exact licensing terms.
 
 #include "scene_xrender.h"
 
-#if defined(HAVE_XRENDER) && defined(HAVE_XFIXES)
+#ifdef KWIN_HAVE_XRENDER_COMPOSITING
 
 #include "toplevel.h"
 #include "client.h"
@@ -52,10 +62,6 @@ struct RegionDebug
    XserverRegion rr;   
    };   
 
-#ifdef NDEBUG
-inline
-kndbgstream& operator<<( kndbgstream& stream, RegionDebug ) { return stream; }
-#else
 kdbgstream& operator<<( kdbgstream& stream, RegionDebug r )
     {       
     if( r.rr == None )
@@ -70,7 +76,6 @@ kdbgstream& operator<<( kdbgstream& stream, RegionDebug r )
        stream << "[" << rects[ i ].x << "+" << rects[ i ].y << " " << rects[ i ].width << "x" << rects[ i ].height << "]";
     return stream;
     }
-#endif
 
 Picture SceneXrender::buffer = None;
 ScreenPaintData SceneXrender::screen_paint;
@@ -232,7 +237,7 @@ void SceneXrender::paintTransformedScreen( int orig_mask )
         // If the window is transparent, the transparent part will be done
         // in the 2nd pass.
         if( data.mask & PAINT_WINDOW_TRANSLUCENT )
-            phase2.prepend( Phase2Data( w, data.paint, data.mask, data.quads ));
+            phase2.prepend( Phase2Data( w, data.paint, data.clip, data.mask, data.quads ));
         if( data.mask & PAINT_WINDOW_OPAQUE )
             {
             w->setTransformedShape( QRegion());
@@ -266,7 +271,7 @@ void SceneXrender::paintBackground( QRegion region )
         XFixesSetPictureClipRegion( display(), buffer, 0, 0, background_region );
         XFixesDestroyRegion( display(), background_region );
         }
-    XRenderColor col = { 0xffff, 0xffff, 0xffff, 0xffff };
+    XRenderColor col = { 0, 0, 0, 0xffff }; // black
     XRenderFillRectangle( display(), PictOpSrc, buffer, &col, 0, 0, displayWidth(), displayHeight());
     XFixesSetPictureClipRegion( display(), buffer, 0, 0, None );
     }
@@ -316,6 +321,7 @@ void SceneXrender::windowAdded( Toplevel* c )
     {
     assert( !windows.contains( c ));
     windows[ c ] = new Window( c );
+    c->effectWindow()->setSceneWindow( windows[ c ]);
     }
 
 // Convert QRegion to XserverRegion. This code uses XserverRegion
