@@ -279,7 +279,7 @@ NET::WindowType KDecorationPreviewBridge::windowType( unsigned long ) const
 
 QIcon KDecorationPreviewBridge::icon() const
     {
-		return QIcon( KGlobal::iconLoader()->loadIcon( "x", KIconLoader::NoGroup, 32 ));
+		return QIcon( KGlobal::iconLoader()->loadIcon( "xorg", KIconLoader::NoGroup, 32 ));
     }
 
 QString KDecorationPreviewBridge::caption() const
@@ -328,11 +328,6 @@ QRegion KDecorationPreviewBridge::unobscuredRegion( const QRegion& r ) const
     return preview->unobscuredRegion( active, r );
     }
 
-QWidget* KDecorationPreviewBridge::workspaceWidget() const
-    {
-    return preview;
-    }
-
 WId KDecorationPreviewBridge::windowId() const
     {
     return 0; // no decorated window
@@ -383,10 +378,6 @@ int KDecorationPreviewBridge::currentDesktop() const
     return 1;
     }
 
-void KDecorationPreviewBridge::helperShowHide( bool )
-    {
-    }
-
 void KDecorationPreviewBridge::grabXServer( bool )
     {
     }
