@@ -87,7 +87,6 @@ class KDecorationPreviewBridge
         virtual QRect geometry() const;
         virtual QRect iconGeometry() const;
         virtual QRegion unobscuredRegion( const QRegion& r ) const;
-        virtual QWidget* workspaceWidget() const;
         virtual WId windowId() const;
 	virtual void closeWindow();
 	virtual void maximize( MaximizeMode mode );
@@ -102,7 +101,6 @@ class KDecorationPreviewBridge
         virtual int currentDesktop() const;
         virtual QWidget* initialParentWidget() const;
         virtual Qt::WFlags initialWFlags() const;
-        virtual void helperShowHide( bool show );
         virtual void grabXServer( bool grab );
     private:
         KDecorationPreview* preview;
