@@ -1,13 +1,23 @@
-/*****************************************************************
+/********************************************************************
  KWin - the KDE window manager
  This file is part of the KDE project.
 
 Copyright (C) 1999, 2000 Matthias Ettrich <ettrich@kde.org>
 Copyright (C) 2003 Lubos Lunak <l.lunak@kde.org>
 
-You can Freely distribute this program under the GNU General Public
-License. See the file "COPYING" for the exact licensing terms.
-******************************************************************/
+This program is free software; you can redistribute it and/or modify
+it under the terms of the GNU General Public License as published by
+the Free Software Foundation; either version 2 of the License, or
+(at your option) any later version.
+
+This program is distributed in the hope that it will be useful,
+but WITHOUT ANY WARRANTY; without even the implied warranty of
+MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+GNU General Public License for more details.
+
+You should have received a copy of the GNU General Public License
+along with this program.  If not, see <http://www.gnu.org/licenses/>.
+*********************************************************************/
 
 /*
 
@@ -109,7 +119,8 @@ QMenu* Workspace::clientPopup()
         action = popup->addMenu( advanced_popup );
         action->setText( i18n("Ad&vanced") );
 
-        if (options->useCompositing){
+        trans_popup = 0;
+        if (compositing()){
             trans_popup = new QMenu( popup );
             trans_popup->setFont(KGlobalSettings::menuFont());
             connect( trans_popup, SIGNAL( triggered(QAction*) ), this, SLOT( setPopupClientOpacity(QAction*)));
@@ -222,7 +233,7 @@ void Workspace::clientPopupAboutToShow()
     mNoBorderOpAction->setChecked( active_popup_client->noBorder() );
     mMinimizeOpAction->setEnabled( active_popup_client->isMinimizable() );
     mCloseOpAction->setEnabled( active_popup_client->isCloseable() );
-    if (options->useCompositing)
+    if( trans_popup != NULL )
         {
         foreach( QAction* action, trans_popup->actions())
             {
@@ -308,6 +319,9 @@ void Workspace::initShortcuts()
     // a separate KActionCollection is needed for the shortcut for disabling global shortcuts,
     // otherwise it would also disable itself
     disable_shortcuts_keys = new KActionCollection( this );
+#ifdef __GNUC__
+#warning TODO PORT ME (KGlobalAccel related)
+#endif
     // FIXME KAccel port... needed?
     //disable_shortcuts_keys->disableBlocking( true );
 #define IN_KWIN
@@ -354,15 +368,14 @@ void Workspace::readShortcuts()
 
 void Workspace::setupWindowShortcut( Client* c )
     {
-#ifdef __GNUC__
-#warning ifdefed out, KShortcutDialog is 0xDEAD
-#endif //__GNUC__
-#if 0
     assert( client_keys_dialog == NULL );
-    keys->setEnabled( false );
-    disable_shortcuts_keys->setEnabled( false );
-    client_keys->setEnabled( false );
-    client_keys_dialog = new ShortcutDialog( c->shortcut());
+#ifdef __GNUC__
+#warning TODO PORT ME (KGlobalAccel related)
+#endif
+//    keys->setEnabled( false );
+//    disable_shortcuts_keys->setEnabled( false );
+//    client_keys->setEnabled( false );
+    client_keys_dialog = new ShortcutDialog( c->shortcut().primary());
     client_keys_client = c;
     connect( client_keys_dialog, SIGNAL( dialogDone( bool )), SLOT( setupWindowShortcutDone( bool )));
     QRect r = clientArea( ScreenArea, c );
@@ -376,24 +389,19 @@ void Workspace::setupWindowShortcut( Client* c )
     client_keys_dialog->show();
     active_popup = client_keys_dialog;
     active_popup_client = c;
-#endif
     }
 
 void Workspace::setupWindowShortcutDone( bool ok )
     {
-#if 0 //see above
-    keys->setEnabled( true );
-    disable_shortcuts_keys->setEnabled( true );
-    client_keys->setEnabled( true );
+//    keys->setEnabled( true );
+//    disable_shortcuts_keys->setEnabled( true );
+//    client_keys->setEnabled( true );
     if( ok )
-        {
         client_keys_client->setShortcut( KShortcut( client_keys_dialog->shortcut()).toString());
-        }
     closeActivePopup();
     delete client_keys_dialog;
     client_keys_dialog = NULL;
     client_keys_client = NULL;
-#endif
     }
 
 void Workspace::clientShortcutUpdated( Client* c )
@@ -757,7 +765,7 @@ void Workspace::slotSwitchDesktopPrevious()
 
 void Workspace::slotSwitchDesktopRight()
     {
-    int desktop = desktopToRight( currentDesktop());
+    int desktop = desktopToRight( currentDesktop(), options->rollOverDesktops);
     if( desktop == currentDesktop())
         return;
     setCurrentDesktop( desktop );
@@ -765,7 +773,7 @@ void Workspace::slotSwitchDesktopRight()
 
 void Workspace::slotSwitchDesktopLeft()
     {
-    int desktop = desktopToLeft( currentDesktop());
+    int desktop = desktopToLeft( currentDesktop(), options->rollOverDesktops);
     if( desktop == currentDesktop())
         return;
     setCurrentDesktop( desktop );
@@ -773,7 +781,7 @@ void Workspace::slotSwitchDesktopLeft()
 
 void Workspace::slotSwitchDesktopUp()
     {
-    int desktop = desktopUp( currentDesktop());
+    int desktop = desktopUp( currentDesktop(), options->rollOverDesktops);
     if( desktop == currentDesktop())
         return;
     setCurrentDesktop( desktop );
@@ -781,7 +789,7 @@ void Workspace::slotSwitchDesktopUp()
 
 void Workspace::slotSwitchDesktopDown()
     {
-    int desktop = desktopDown( currentDesktop());
+    int desktop = desktopDown( currentDesktop(), options->rollOverDesktops);
     if( desktop == currentDesktop())
         return;
     setCurrentDesktop( desktop );
@@ -1003,7 +1011,7 @@ void Workspace::windowToPreviousDesktop( Client* c )
 
 void Workspace::slotWindowToDesktopRight()
     {
-    int d = desktopToRight( currentDesktop());
+    int d = desktopToRight( currentDesktop(), options->rollOverDesktops);
     if( d == currentDesktop())
         return;
     Client* c = active_popup_client ? active_popup_client : active_client;
@@ -1018,7 +1026,7 @@ void Workspace::slotWindowToDesktopRight()
 
 void Workspace::slotWindowToDesktopLeft()
     {
-    int d = desktopToLeft( currentDesktop());
+    int d = desktopToLeft( currentDesktop(), options->rollOverDesktops);
     if( d == currentDesktop())
         return;
     Client* c = active_popup_client ? active_popup_client : active_client;
@@ -1033,7 +1041,7 @@ void Workspace::slotWindowToDesktopLeft()
 
 void Workspace::slotWindowToDesktopUp()
     {
-    int d = desktopUp( currentDesktop());
+    int d = desktopUp( currentDesktop(), options->rollOverDesktops);
     if( d == currentDesktop())
         return;
     Client* c = active_popup_client ? active_popup_client : active_client;
@@ -1048,7 +1056,7 @@ void Workspace::slotWindowToDesktopUp()
 
 void Workspace::slotWindowToDesktopDown()
     {
-    int d = desktopDown( currentDesktop());
+    int d = desktopDown( currentDesktop(), options->rollOverDesktops);
     if( d == currentDesktop())
         return;
     Client* c = active_popup_client ? active_popup_client : active_client;
