@@ -1,13 +1,23 @@
-/*****************************************************************
+/********************************************************************
  KWin - the KDE window manager
  This file is part of the KDE project.
 
 Copyright (C) 1999, 2000 Matthias Ettrich <ettrich@kde.org>
 Copyright (C) 2003 Lubos Lunak <l.lunak@kde.org>
 
-You can Freely distribute this program under the GNU General Public
-License. See the file "COPYING" for the exact licensing terms.
-******************************************************************/
+This program is free software; you can redistribute it and/or modify
+it under the terms of the GNU General Public License as published by
+the Free Software Foundation; either version 2 of the License, or
+(at your option) any later version.
+
+This program is distributed in the hope that it will be useful,
+but WITHOUT ANY WARRANTY; without even the implied warranty of
+MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+GNU General Public License for more details.
+
+You should have received a copy of the GNU General Public License
+along with this program.  If not, see <http://www.gnu.org/licenses/>.
+*********************************************************************/
 
 /*
 
@@ -32,28 +42,6 @@ License. See the file "COPYING" for the exact licensing terms.
 #include <X11/Xatom.h>
 #include <QX11Info>
 
-#ifdef HAVE_XRENDER
-#include <X11/extensions/Xrender.h>
-#endif
-#ifdef HAVE_XFIXES
-#include <X11/extensions/Xfixes.h>
-#endif
-#ifdef HAVE_XDAMAGE
-#include <X11/extensions/Xdamage.h>
-#endif
-#ifdef HAVE_XRANDR
-#include <X11/extensions/Xrandr.h>
-#endif
-#ifdef HAVE_XCOMPOSITE
-#include <X11/extensions/Xcomposite.h>
-#endif
-#ifdef HAVE_OPENGL
-#include <GL/glx.h>
-#endif
-#ifdef HAVE_XSYNC
-#include <X11/extensions/sync.h>
-#endif
-
 #include <stdio.h>
 
 #include "atoms.h"
@@ -415,13 +403,10 @@ bool isLocalMachine( const QByteArray& host )
     }
 
 #ifndef KCMRULES
-#ifdef __GNUC__
-#warning KShortcutDialog is gone
-#endif //__GNUC__
-#if 0
-ShortcutDialog::ShortcutDialog( const KShortcut& cut )
-    : KShortcutDialog( cut, false /*TODO: ???*/ )
+ShortcutDialog::ShortcutDialog( const QKeySequence& cut )
+    : widget( new KKeySequenceWidget( this ))
     {
+    widget->setKeySequence( cut );
     // make it a popup, so that it has the grab
     XSetWindowAttributes attrs;
     attrs.override_redirect = True;
@@ -431,32 +416,36 @@ ShortcutDialog::ShortcutDialog( const KShortcut& cut )
 
 void ShortcutDialog::accept()
     {
-    foreach( const QKeySequence &seq, shortcut() )
+    QKeySequence seq = shortcut();
+    if( !seq.isEmpty())
         {
-        if( seq.isEmpty())
-            break;
         if( seq[0] == Qt::Key_Escape )
             {
             reject();
             return;
             }
-        if( seq[0] == Qt::Key_Space )
+        if( seq[0] == Qt::Key_Space
+            || (seq[0] & Qt::KeyboardModifierMask) == 0 )
             { // clear
-            setShortcut( KShortcut());
-            KShortcutDialog::accept();
-            return;
-            }
-        if( (seq[0] & Qt::KeyboardModifierMask) == 0 )
-            { // no shortcuts without modifiers
-            KShortcut cut = shortcut();
-            cut.remove( seq );
-            setShortcut( cut );
+            widget->clearKeySequence();
+            KDialog::accept();
             return;
             }
         }
-    KShortcutDialog::accept();
+    KDialog::accept();
     }
-#endif //0
+
+void ShortcutDialog::done( int r )
+    {
+    KDialog::done( r );
+    emit dialogDone( r == Accepted );
+    }
+
+QKeySequence ShortcutDialog::shortcut() const
+    {
+    return widget->keySequence();
+    }
+
 #endif //KCMRULES
 } // namespace
 
