@@ -1,13 +1,23 @@
-/*****************************************************************
+/********************************************************************
  KWin - the KDE window manager
  This file is part of the KDE project.
 
 Copyright (C) 1999, 2000 Matthias Ettrich <ettrich@kde.org>
 Copyright (C) 2003 Lubos Lunak <l.lunak@kde.org>
 
-You can Freely distribute this program under the GNU General Public
-License. See the file "COPYING" for the exact licensing terms.
-******************************************************************/
+This program is free software; you can redistribute it and/or modify
+it under the terms of the GNU General Public License as published by
+the Free Software Foundation; either version 2 of the License, or
+(at your option) any later version.
+
+This program is distributed in the hope that it will be useful,
+but WITHOUT ANY WARRANTY; without even the implied warranty of
+MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+GNU General Public License for more details.
+
+You should have received a copy of the GNU General Public License
+along with this program.  If not, see <http://www.gnu.org/licenses/>.
+*********************************************************************/
 
 //#define QT_CLEAN_NAMESPACE
 
@@ -78,7 +88,6 @@ Workspace::Workspace( bool restore )
     number_of_desktops(0),
     active_popup( NULL ),
     active_popup_client( NULL ),
-    desktop_widget    (0),
     temporaryRulesMessages( "_KDE_NET_WM_TEMPORARY_RULES", NULL, false ),
     rules_updates_disabled( false ),
     active_client     (0),
@@ -105,7 +114,7 @@ Workspace::Workspace( bool restore )
     desk_popup        (0),
     keys              (0),
     client_keys       ( NULL ),
-//    client_keys_dialog ( NULL ),
+    client_keys_dialog ( NULL ),
     client_keys_client ( NULL ),
     disable_shortcuts_keys ( NULL ),
     global_shortcuts_disabled( false ),
@@ -128,6 +137,7 @@ Workspace::Workspace( bool restore )
     cm_selection( NULL ),
     compositeRate( 0 ),
     overlay( None ),
+    overlay_visible( true ),
     transSlider( NULL ),
     transButton( NULL )
     {
@@ -165,10 +175,6 @@ Workspace::Workspace( bool restore )
 
     (void) QApplication::desktop(); // trigger creation of desktop widget
 
-    desktop_widget = new QWidget( 0, Qt::Desktop );
-    desktop_widget->setObjectName( "desktop_widget" );
-    desktop_widget->setAttribute( Qt::WA_PaintUnclipped );
-
     // call this before XSelectInput() on the root window
     startup = new KStartupInfo(
         KStartupInfo::DisableKWinModule | KStartupInfo::AnnounceSilenceChanges, this );
@@ -266,6 +272,7 @@ void Workspace::init()
         NET::TopMenuMask |
         NET::UtilityMask |
         NET::SplashMask |
+    // No compositing window types here unless we support them also as managed window types
         0
         ,
         NET::Modal |
@@ -449,7 +456,6 @@ Workspace::~Workspace()
          it != unmanaged.end();
          ++it )
         (*it)->release();
-    delete desktop_widget;
     delete tab_box;
     delete popupinfo;
     delete popup;
@@ -468,7 +474,7 @@ Workspace::~Workspace()
     delete topmenu_watcher;
     delete topmenu_selection;
     delete topmenu_space;
-//    delete client_keys_dialog;
+    delete client_keys_dialog;
     while( !rules.isEmpty())
         {
         delete rules.front();
@@ -1140,7 +1146,7 @@ QStringList Workspace::configModules(bool controlCenter)
 
 void Workspace::configureWM()
     {
-	KToolInvocation::kdeinitExec( "kcmshell", configModules(false) );
+	KToolInvocation::kdeinitExec( "kcmshell4", configModules(false) );
     }
 
 /*!
@@ -1392,7 +1398,7 @@ void Workspace::previousDesktop()
     setCurrentDesktop(desktop > 0 ? desktop : numberOfDesktops());
     }
 
-int Workspace::desktopToRight( int desktop ) const
+int Workspace::desktopToRight( int desktop, bool wrap ) const
     {
     int x,y;
     Qt::Orientation orientation;
@@ -1403,7 +1409,7 @@ int Workspace::desktopToRight( int desktop ) const
         dt += y;
         if ( dt >= numberOfDesktops() )
             {
-            if ( options->rollOverDesktops )
+            if ( wrap )
               dt -= numberOfDesktops();
             else
               return desktop;
@@ -1414,7 +1420,7 @@ int Workspace::desktopToRight( int desktop ) const
         int d = (dt % x) + 1;
         if ( d >= x )
             {
-            if ( options->rollOverDesktops )
+            if ( wrap )
               d -= x;
             else
               return desktop;
@@ -1424,7 +1430,7 @@ int Workspace::desktopToRight( int desktop ) const
     return dt+1;
     }
 
-int Workspace::desktopToLeft( int desktop ) const
+int Workspace::desktopToLeft( int desktop, bool wrap ) const
     {
     int x,y;
     Qt::Orientation orientation;
@@ -1435,7 +1441,7 @@ int Workspace::desktopToLeft( int desktop ) const
         dt -= y;
         if ( dt < 0 )
             {
-            if ( options->rollOverDesktops )
+            if ( wrap )
               dt += numberOfDesktops();
             else
               return desktop;
@@ -1446,7 +1452,7 @@ int Workspace::desktopToLeft( int desktop ) const
         int d = (dt % x) - 1;
         if ( d < 0 )
             {
-            if ( options->rollOverDesktops )
+            if ( wrap )
               d += x;
             else
               return desktop;
@@ -1456,7 +1462,7 @@ int Workspace::desktopToLeft( int desktop ) const
     return dt+1;
     }
 
-int Workspace::desktopUp( int desktop ) const
+int Workspace::desktopUp( int desktop, bool wrap ) const
     {
     int x,y;
     Qt::Orientation orientation;
@@ -1467,7 +1473,7 @@ int Workspace::desktopUp( int desktop ) const
         dt -= x;
         if ( dt < 0 )
             {
-            if ( options->rollOverDesktops )
+            if ( wrap )
               dt += numberOfDesktops();
             else
               return desktop;
@@ -1478,7 +1484,7 @@ int Workspace::desktopUp( int desktop ) const
         int d = (dt % y) - 1;
         if ( d < 0 )
             {
-            if ( options->rollOverDesktops )
+            if ( wrap )
               d += y;
             else
               return desktop;
@@ -1488,7 +1494,7 @@ int Workspace::desktopUp( int desktop ) const
     return dt+1;
     }
 
-int Workspace::desktopDown( int desktop ) const
+int Workspace::desktopDown( int desktop, bool wrap ) const
     {
     int x,y;
     Qt::Orientation orientation;
@@ -1499,7 +1505,7 @@ int Workspace::desktopDown( int desktop ) const
         dt += x;
         if ( dt >= numberOfDesktops() )
             {
-            if ( options->rollOverDesktops )
+            if ( wrap )
               dt -= numberOfDesktops();
             else
               return desktop;
@@ -1510,7 +1516,7 @@ int Workspace::desktopDown( int desktop ) const
         int d = (dt % y) + 1;
         if ( d >= y )
             {
-            if ( options->rollOverDesktops )
+            if ( wrap )
               d -= y;
             else
               return desktop;
@@ -2041,16 +2047,6 @@ bool Workspace::keyPressMouseEmulation( XKeyEvent& ev )
 
     }
 
-/*!
-  Returns the workspace's desktop widget. The desktop widget is
-  sometimes required by clients to draw on it, for example outlines on
-  moving or resizing.
- */
-QWidget* Workspace::desktopWidget()
-    {
-    return desktop_widget;
-    }
-
 //Delayed focus functions
 void Workspace::delayFocus()
     {
@@ -2252,22 +2248,22 @@ void Workspace::electricBorderSwitchDesktop( ElectricBorder border, const QPoint
     const int OFFSET = 2;
     if( border == ElectricLeft || border == ElectricTopLeft || border == ElectricBottomLeft )
         {
-        desk = desktopToLeft( desk );
+        desk = desktopToLeft( desk, options->rollOverDesktops );
         pos.setX( displayWidth() - 1 - OFFSET );
         }
     if( border == ElectricRight || border == ElectricTopRight || border == ElectricBottomRight )
         {
-        desk = desktopToRight( desk );
+        desk = desktopToRight( desk, options->rollOverDesktops );
         pos.setX( OFFSET );
         }
     if( border == ElectricTop || border == ElectricTopLeft || border == ElectricTopRight )
         {
-        desk = desktopUp( desk );
+        desk = desktopUp( desk, options->rollOverDesktops );
         pos.setY( displayHeight() - 1 - OFFSET );
         }
     if( border == ElectricBottom || border == ElectricBottomLeft || border == ElectricBottomRight )
         {
-        desk = desktopDown( desk );
+        desk = desktopDown( desk, options->rollOverDesktops );
         pos.setY( OFFSET );
         }
     int desk_before = currentDesktop();
@@ -2636,7 +2632,8 @@ void Workspace::checkCursorPos()
     cursorPos(); // update if needed
     if( last != last_cursor_pos || lastb != last_buttons )
         static_cast< EffectsHandlerImpl* >( effects )->mouseChanged( cursorPos(), last,
-            x11ToQtMouseButtons( last_buttons ), x11ToQtKeyboardModifiers( last_buttons ));
+            x11ToQtMouseButtons( last_buttons ), x11ToQtMouseButtons( lastb ),
+            x11ToQtKeyboardModifiers( last_buttons ), x11ToQtKeyboardModifiers( lastb ));
     }
 
 } // namespace
