@@ -1,13 +1,23 @@
-/*****************************************************************
+/********************************************************************
  KWin - the KDE window manager
  This file is part of the KDE project.
 
 Copyright (C) 1999, 2000 Matthias Ettrich <ettrich@kde.org>
 Copyright (C) 2003 Lubos Lunak <l.lunak@kde.org>
 
-You can Freely distribute this program under the GNU General Public
-License. See the file "COPYING" for the exact licensing terms.
-******************************************************************/
+This program is free software; you can redistribute it and/or modify
+it under the terms of the GNU General Public License as published by
+the Free Software Foundation; either version 2 of the License, or
+(at your option) any later version.
+
+This program is distributed in the hope that it will be useful,
+but WITHOUT ANY WARRANTY; without even the implied warranty of
+MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+GNU General Public License for more details.
+
+You should have received a copy of the GNU General Public License
+along with this program.  If not, see <http://www.gnu.org/licenses/>.
+*********************************************************************/
 
 #ifndef KWIN_WORKSPACE_H
 #define KWIN_WORKSPACE_H
@@ -98,7 +108,7 @@ class Workspace : public QObject, public KDecorationDefines
         void requestFocus( Client* c, bool force = false );
         void takeActivity( Client* c, int flags, bool handled ); // flags are ActivityFlags
         void handleTakeActivity( Client* c, Time timestamp, int flags ); // flags are ActivityFlags
-        bool allowClientActivation( const Client* c, Time time = -1U, bool focus_in = false );
+        bool allowClientActivation( const Client* c, Time time = -1U, bool focus_in = false, bool ignore_desktop = false );
         void restoreFocus();
         void gotFocusIn( const Client* );
         void setShouldGetFocus( Client* );
@@ -149,6 +159,10 @@ class Workspace : public QObject, public KDecorationDefines
         int numberOfDesktops() const;
         void setNumberOfDesktops( int n );
         void calcDesktopLayout(int* x, int* y, Qt::Orientation* orientation) const;
+        int desktopToRight( int desktop, bool wrap ) const;
+        int desktopToLeft( int desktop, bool wrap ) const;
+        int desktopUp( int desktop, bool wrap ) const;
+        int desktopDown( int desktop, bool wrap ) const;
 
         int activeScreen() const;
         int numScreens() const;
@@ -156,7 +170,6 @@ class Workspace : public QObject, public KDecorationDefines
         void setActiveScreenMouse( const QPoint &mousepos );
         QRect screenGeometry( int screen ) const;
         int screenNumber( const QPoint &pos ) const;
-        QWidget* desktopWidget();
 
     // for TabBox
         Client* currentTabBoxClient() const;
@@ -492,10 +505,6 @@ class Workspace : public QObject, public KDecorationDefines
         void oneStepThroughDesktopList( bool forward );
         bool establishTabBoxGrab();
         void removeTabBoxGrab();
-        int desktopToRight( int desktop ) const;
-        int desktopToLeft( int desktop ) const;
-        int desktopUp( int desktop ) const;
-        int desktopDown( int desktop ) const;
 
         void updateStackingOrder( bool propagate_new_clients = false );
         void propagateClients( bool propagate_new_clients ); // called only from updateStackingOrder
@@ -559,8 +568,6 @@ class Workspace : public QObject, public KDecorationDefines
         QWidget* active_popup;
         Client* active_popup_client;
 
-        QWidget* desktop_widget;
-
         void loadSessionInfo();
         void loadWindowRules();
         void editWindowRules( Client* c, bool whole_app );
@@ -646,7 +653,7 @@ class Workspace : public QObject, public KDecorationDefines
         QAction *mNoBorderOpAction;
         QAction *mMinimizeOpAction;
         QAction *mCloseOpAction;
-//        ShortcutDialog* client_keys_dialog;
+        ShortcutDialog* client_keys_dialog;
         Client* client_keys_client;
         KActionCollection *disable_shortcuts_keys;
         bool global_shortcuts_disabled;
@@ -715,6 +722,7 @@ class Workspace : public QObject, public KDecorationDefines
         int compositeRate;
         QRegion repaints_region;
         Window overlay; // XComposite overlay window
+        bool overlay_visible;
         QSlider *transSlider;
         QPushButton *transButton;
 
