@@ -30,10 +30,12 @@
 #include <QPainter>
 #include <QSize>
 #include <QStyleOptionGraphicsItem>
+#include <QTextDocument>
 #include <QTimer>
 #include <QUiLoader>
 
 #include <KIcon>
+#include <KColorScheme>
 #include <KConfigDialog>
 #include <KDialog>
 #include <KPluginInfo>
@@ -42,6 +44,8 @@
 #include <KServiceTypeTrader>
 #include <KIconLoader>
 
+#include <Solid/PowerManagement>
+
 #include "plasma/configxml.h"
 #include "plasma/containment.h"
 #include "plasma/corona.h"
@@ -52,11 +56,13 @@
 #include "plasma/scriptengine.h"
 #include "plasma/shadowitem_p.h"
 #include "plasma/svg.h"
+#include "plasma/theme.h"
+#include "plasma/view.h"
 
+#include "plasma/layouts/boxlayout.h"
 #include "plasma/widgets/widget.h"
 #include "plasma/widgets/lineedit.h"
 #include "plasma/widgets/pushbutton.h"
-#include "plasma/widgets/boxlayout.h"
 
 //#define DYNAMIC_SHADOWS
 namespace Plasma
@@ -67,8 +73,6 @@ class Applet::Private
 public:
     Private(KService::Ptr service, int uniqueID)
         : appletId(uniqueID),
-          globalConfig(0),
-          appletConfig(0),
           appletDescription(service),
           package(0),
           background(0),
@@ -77,11 +81,14 @@ public:
           configXml(0),
           shadow(0),
           cachedBackground(0),
+          mainConfig(0),
+          pendingConstraints(NoConstraint),
           kioskImmutable(false),
           immutable(false),
           hasConfigurationInterface(false),
           failed(false),
-          needsConfig(false)
+          needsConfig(false),
+          isContainment(false)
     {
         if (appletId == 0) {
             appletId = nextId();
@@ -102,14 +109,15 @@ public:
         delete configXml;
         delete shadow;
         delete cachedBackground;
+        delete mainConfig;
     }
 
     void init(Applet* applet)
     {
+        // WARNING: do not access config() OR globalConfig() in this method!
+        //          that requires a scene, which is not available at this point
+        applet->setAcceptsHoverEvents(true);
         applet->setZValue(100);
-        kioskImmutable = applet->globalConfig().isImmutable() ||
-                         applet->config().isImmutable();
-        applet->setImmutable(kioskImmutable);
 
         if (!appletDescription.isValid()) {
             applet->setFailedToLaunch(true);
@@ -122,14 +130,15 @@ public:
         if (!language.isEmpty()) {
             // find where the Package is
             QString path = KStandardDirs::locate("appdata",
-                                                 "plasmoids/" +
-                                                 appletDescription.pluginName());
+                                                 "plasmoids/" + appletDescription.pluginName() + '/');
 
-            if (!path.isEmpty()) {
+            if (path.isEmpty()) {
+                applet->setFailedToLaunch(true, i18n("Could not locate the %1 package required for the %2 widget.",
+                                                     appletDescription.pluginName(), appletDescription.name()));
+            } else {
                 // create the package and see if we have something real
-                package = new Package(path,
-                                      appletDescription.pluginName(),
-                                      PlasmoidStructure());
+                //kDebug() << "trying for" << path;
+                package = new Package(path, PlasmoidStructure());
                 if (package->isValid()) {
                     // now we try and set up the script engine.
                     // it will be parented to this applet and so will get
@@ -139,19 +148,22 @@ public:
                     if (!scriptEngine) {
                         delete package;
                         package = 0;
+                        applet->setFailedToLaunch(true, i18n("Could not create a %1 ScriptEngine for the %2 widget.",
+                                                             language, appletDescription.name()));
                     }
                 } else {
+                    applet->setFailedToLaunch(true, i18n("Could not open the %1 package required for the %2 widget.",
+                                                         appletDescription.pluginName(), appletDescription.name()));
                     delete package;
                     package = 0;
                 }
 
-                if (!package) {
-                    applet->setFailedToLaunch(true);
-                } else {
+                if (package) {
                     setupScriptSupport(applet);
                 }
             }
         }
+        applet->setDrawStandardBackground(true);
     }
 
     // put all setup routines for script here. at this point we can assume that
@@ -162,7 +174,9 @@ public:
         QString xmlPath = package->filePath("mainconfigxml");
         if (!xmlPath.isEmpty()) {
             QFile file(xmlPath);
-            configXml = new ConfigXml(config(), &file);
+            // FIXME: KConfigSkeleton doesn't play well with KConfigGroup =/
+            KConfigGroup config = applet->config();
+            configXml = new ConfigXml(&config, &file);
         }
 
         if (!package->filePath("mainconfigui").isEmpty()) {
@@ -170,7 +184,7 @@ public:
         }
     }
 
-    void paintBackground(QPainter* p2, Applet* q)
+    void paintBackground(QPainter* p2, Applet* q, QRectF paintRect)
     {
         if (q->formFactor() != Plasma::Planar) {
             // we don't paint special backgrounds for other form factors
@@ -179,31 +193,31 @@ public:
             return;
         }
 
-        QSize contents = contentSize(q).toSize();
-        const int contentWidth = contents.width();
-        const int contentHeight = contents.height();
-
-        background->resize();
-
+        // TODO: caching these values might be an idea
         const int topHeight = background->elementSize("top").height();
-        const int topWidth = background->elementSize("top").width();
         const int leftWidth = background->elementSize("left").width();
-        const int leftHeight = background->elementSize("left").height();
-        const int rightWidth = background->elementSize("right").width();
-        const int bottomHeight = background->elementSize("bottom").height();
-        //const int lrWidths = leftWidth + rightWidth;
-        //const int tbHeights = topHeight + bottomHeight;
-
-        // contents top-left corner is (0,0).  We need to draw up and left of that
         const int topOffset = 0 - topHeight;
         const int leftOffset = 0 - leftWidth;
-        const int rightOffset = contentWidth;
-        const int bottomOffset = contentHeight;
-        const int contentTop = 0;
-        const int contentLeft = 0;
-        QSize s = QSize(leftWidth + contentWidth + rightWidth,
-                        topHeight + contentHeight + bottomHeight);
-        if (!cachedBackground || cachedBackground->size() != s) {
+
+        if (!cachedBackground || cachedBackground->size() != q->boundingRect().toRect().size()) {
+            QSize contents = contentSize(q).toSize();
+
+            const int contentWidth = contents.width();
+            const int contentHeight = contents.height();
+
+            background->resize();
+
+            const int topWidth = background->elementSize("top").width();
+            const int leftHeight = background->elementSize("left").height();
+            const int rightWidth = background->elementSize("right").width();
+            const int bottomHeight = background->elementSize("bottom").height();
+
+            // contents top-left corner is (0,0).  We need to draw up and left of that
+            const int rightOffset = contentWidth;
+            const int bottomOffset = contentHeight;
+            const int contentTop = 0;
+            const int contentLeft = 0;
+
             delete cachedBackground;
             cachedBackground = new QPixmap(leftWidth + contentWidth + rightWidth, topHeight + contentHeight + bottomHeight);
             cachedBackground->fill(Qt::transparent);
@@ -213,8 +227,8 @@ public:
             p.setRenderHint(QPainter::SmoothPixmapTransform);
 
             //FIXME: This is a hack to fix a drawing problems with svg files where a thin transparent border is drawn around the svg image.
-            //       the transparent border around the svg seems to vary in size depending on the size of the svg and as a result increasing the 
-	    //	     svn image by 2 all around didn't resolve the issue. For now it resizes based on the border size.
+            //       the transparent border around the svg seems to vary in size depending on the size of the svg and as a result increasing the
+            //       svn image by 2 all around didn't resolve the issue. For now it resizes based on the border size.
 
             background->resize(contentWidth, contentHeight);
             background->paint(&p, QRect(contentLeft-leftWidth, contentTop-topHeight, contentWidth+leftWidth*2, contentHeight+topHeight*2), "center");
@@ -225,7 +239,7 @@ public:
             background->paint(&p, QRect(leftOffset, bottomOffset, leftWidth, bottomHeight), "bottomleft");
             background->paint(&p, QRect(rightOffset, bottomOffset, rightWidth, bottomHeight), "bottomright");
 
-            if (stretchBackgroundBorders) {
+            if (background->elementExists("hint-stretch-borders")) {
                 background->paint(&p, QRect(leftOffset, contentTop, leftWidth, contentHeight), "left");
                 background->paint(&p, QRect(rightOffset, contentTop, rightWidth, contentHeight), "right");
                 background->paint(&p, QRect(contentLeft, topOffset, contentWidth, topHeight), "top");
@@ -268,22 +282,16 @@ public:
                 p.drawTiledPixmap(QRect(contentLeft, bottomOffset, contentWidth, bottomHeight), bottom);
             }
 
+            // re-enable this once Qt's svg rendering is un-buggered
+            //background->resize(contentWidth, contentHeight);
+            //background->paint(&p, QRect(contentLeft, contentTop, contentWidth, contentHeight), "center");
         }
 
-        p2->drawPixmap(leftOffset, topOffset, *cachedBackground);
-    }
-
-    void paintHover(QPainter* , Applet* )
-    {
-        //TODO draw hover interface for close, configure, info and move
+        p2->drawPixmap(paintRect, *cachedBackground, paintRect.translated(-leftOffset,-topOffset));
     }
 
     QSizeF contentSize(const Applet* q)
     {
-        if (scriptEngine) {
-            return scriptEngine->size();
-        }
-
         if (failureText) {
             return failureText->geometry().size();
         }
@@ -297,17 +305,6 @@ public:
         return s_maxAppletId;
     }
 
-    KSharedConfig::Ptr config() {
-        if (!appletConfig) {
-            QString file = KStandardDirs::locateLocal("appdata",
-                                                      "applets/" + instanceName() + "rc",
-                                                      true);
-            appletConfig = KSharedConfig::openConfig(file);
-        }
-
-        return appletConfig;
-    }
-
     QString instanceName()
     {
         if (!appletDescription.isValid()) {
@@ -319,38 +316,97 @@ public:
 
     void getBorderSize(int& left , int& top, int &right, int& bottom)
     {
-        if (!background) {
-            top = left = right = bottom = 0;
-        } else {
+        if (background) {
             top = background->elementSize("top").height();
             left = background->elementSize("left").width();
             right = background->elementSize("right").width();
             bottom = background->elementSize("bottom").height();
+        } else {
+            top = left = right = bottom = 0;
+        }
+    }
+
+    void scheduleConstraintsUpdate(Plasma::Constraints c, Applet* applet)
+    {
+        if (pendingConstraints == NoConstraint) {
+            QTimer::singleShot(0, applet, SLOT(flushUpdatedConstraints()));
+        }
+        pendingConstraints |= c;
+    }
+
+    KConfigGroup* mainConfigGroup(const Applet* q)
+    {
+        if (mainConfig) {
+            return mainConfig;
+        }
+
+        if (isContainment) {
+            const Containment *asContainment = qobject_cast<Containment*>(const_cast<Applet*>(q));
+            Q_ASSERT(asContainment);
+
+            KConfigGroup containmentConfig;
+            //kDebug() << "got a corona, baby?" << (QObject*)asContainment->corona();
+            if (asContainment->corona()) {
+                containmentConfig = KConfigGroup(asContainment->corona()->config(), "Containments");
+            } else {
+                containmentConfig =  KConfigGroup(KGlobal::config(), "Containments");
+            }
+
+            mainConfig = new KConfigGroup(&containmentConfig, QString::number(appletId));
+        } else {
+            KConfigGroup appletConfig;
+            if (q->containment()) {
+                appletConfig = q->containment()->config();
+                appletConfig = KConfigGroup(&appletConfig, "Applets");
+            } else {
+                kWarning() << "requesting config for" << q->name() << "without a containment!";
+                appletConfig = KConfigGroup(KGlobal::config(), "Applets");
+            }
+
+            mainConfig = new KConfigGroup(&appletConfig, QString::number(appletId));
+        }
+
+        return mainConfig;
+    }
+
+    void copyEntries(KConfigGroup *source, KConfigGroup *destination)
+    {
+        foreach (const QString &group, source->groupList()) {
+            KConfigGroup subSource(source, group);
+            KConfigGroup subDest(destination, group);
+            copyEntries(&subSource, &subDest);
+        }
+
+        QMap<QString, QString> entries = source->entryMap();
+        QMapIterator<QString, QString> it(entries);
+        while (it.hasNext()) {
+            it.next();
+            destination->writeEntry(it.key(), it.value());
         }
     }
 
     //TODO: examine the usage of memory here; there's a pretty large
     //      number of members at this point.
     uint appletId;
-    KSharedConfig::Ptr globalConfig;
-    KSharedConfig::Ptr appletConfig;
     KPluginInfo appletDescription;
     Package* package;
     QList<QObject*> watchedForFocus;
     QStringList loadedEngines;
     static uint s_maxAppletId;
     Plasma::Svg *background;
-    bool stretchBackgroundBorders;
     Plasma::LineEdit *failureText;
     ScriptEngine* scriptEngine;
     ConfigXml* configXml;
     ShadowItem* shadow;
     QPixmap* cachedBackground;
+    KConfigGroup *mainConfig;
+    Plasma::Constraints pendingConstraints;
     bool kioskImmutable : 1;
     bool immutable : 1;
     bool hasConfigurationInterface : 1;
     bool failed : 1;
     bool needsConfig : 1;
+    bool isContainment : 1;
 };
 
 uint Applet::Private::s_maxAppletId = 0;
@@ -361,6 +417,8 @@ Applet::Applet(QGraphicsItem *parent,
     :  Widget(parent),
        d(new Private(KService::serviceByStorageId(serviceID), appletId))
 {
+    // WARNING: do not access config() OR globalConfig() in this method!
+    //          that requires a scene, which is not available at this point
     d->init(this);
 }
 
@@ -369,6 +427,8 @@ Applet::Applet(QObject* parentObject, const QVariantList& args)
        d(new Private(KService::serviceByStorageId(args.count() > 0 ? args[0].toString() : QString()),
                      args.count() > 1 ? args[1].toInt() : 0))
 {
+    // WARNING: do not access config() OR globalConfig() in this method!
+    //          that requires a scene, which is not available at this point
     d->init(this);
     // the brain damage seen in the initialization list is due to the
     // inflexibility of KService::createInstance
@@ -376,16 +436,7 @@ Applet::Applet(QObject* parentObject, const QVariantList& args)
 
 Applet::~Applet()
 {
-    needsFocus( false );
-
-    if (d->appletConfig) {
-        d->appletConfig->sync();
-    }
-
-    if (d->globalConfig) {
-        d->globalConfig->sync();
-    }
-
+    needsFocus(false);
     delete d;
 }
 
@@ -398,62 +449,90 @@ uint Applet::id() const
     return d->appletId;
 }
 
-KConfigGroup Applet::config() const
-{
-    return KConfigGroup(d->config(), "General");
-}
-
 void Applet::save(KConfigGroup* group) const
 {
+    // we call the dptr member directly for locked since isImmutable()
+    // also checks kiosk and parent containers
+    group->writeEntry("locked", d->immutable);
     group->writeEntry("plugin", pluginName());
-    group->writeEntry("geometry", QRect(pos().toPoint(), boundingRect().size().toSize()));
+    //FIXME: for containments, we need to have some special values here w/regards to
+    //       screen affinity (e.g. "bottom of screen 0")
+    //kDebug() << pluginName() << "geometry is" << geometry() << "pos is" << pos() << "bounding rect is" << boundingRect();
+    group->writeEntry("geometry", geometry());
 
-    Containment* c = containment();
-    if (c) {
-        group->writeEntry("containment", c->id());
+    if (transform() == QTransform()) {
+        group->deleteEntry("transform");
+    } else {
+        QList<qreal> m;
+        QTransform t = transform();
+        m << t.m11() << t.m12() << t.m13() << t.m21() << t.m22() << t.m23() << t.m31() << t.m32() << t.m33();
+        group->writeEntry("transform", m);
+        //group->writeEntry("transform", transformToString(transform()));
     }
 
-    saveState(group);
+    KConfigGroup appletConfigGroup(group, "Configuration");
+    //FIXME: we need a global save state too
+    saveState(&appletConfigGroup);
 }
 
 void Applet::saveState(KConfigGroup* group) const
 {
+    if (group->config()->name() != config().config()->name()) {
+        // we're being saved to a different file!
+        // let's just copy the current values in our configuration over
+        KConfigGroup c = config();
+        d->copyEntries(&c, group);
+    }
+
     Q_UNUSED(group)
 }
 
-KConfigGroup Applet::config(const QString& group) const
+KConfigGroup Applet::config(const QString &group) const
 {
     KConfigGroup cg = config();
-    cg.changeGroup(instanceName() + '-' + group);
-    return cg;
+    return KConfigGroup(&cg, group);
+}
+
+KConfigGroup Applet::config() const
+{
+    if (d->isContainment) {
+        return *(d->mainConfigGroup(this));
+    }
+
+    return KConfigGroup(d->mainConfigGroup(this), "Configuration");
 }
 
 KConfigGroup Applet::globalConfig() const
 {
-    if ( !d->globalConfig ) {
-        QString file = KStandardDirs::locateLocal( "config", "plasma_" + globalName() + "rc" );
-        d->globalConfig = KSharedConfig::openConfig( file );
+    KConfigGroup globalAppletConfig;
+    if (containment() && containment()->corona()) {
+        KSharedConfig::Ptr coronaConfig = containment()->corona()->config();
+        globalAppletConfig = KConfigGroup(coronaConfig, "AppletGlobals");
+    } else {
+        globalAppletConfig = KConfigGroup(KGlobal::config(), "AppletGlobals");
     }
 
-    return KConfigGroup(d->globalConfig, "General");
+    return KConfigGroup(&globalAppletConfig, globalName());
 }
 
 void Applet::destroy()
 {
+    //kDebug() << "???????????????? DESTROYING APPLET" << name() << " ???????????????????????????";
     if (d->configXml) {
         d->configXml->setDefaults();
     }
 
-    if (d->appletConfig) {
-        foreach (const QString& group, d->appletConfig->groupList()) {
-            d->appletConfig->deleteGroup(group);
-        }
-        d->appletConfig = 0;
-    }
-
+    resetConfigurationObject();
     deleteLater();
 }
 
+void Applet::resetConfigurationObject()
+{
+    d->mainConfigGroup(this)->deleteGroup();
+    delete d->mainConfig;
+    d->mainConfig = 0;
+}
+
 ConfigXml* Applet::configXml() const
 {
     return d->configXml;
@@ -481,13 +560,13 @@ const Package* Applet::package() const
 
 void Applet::updateConstraints(Plasma::Constraints constraints)
 {
-    constraintsUpdated(constraints);
-    setShadowShown(formFactor() == Planar);
+    d->scheduleConstraintsUpdate(constraints, this);
 }
 
 void Applet::constraintsUpdated(Plasma::Constraints constraints)
 {
-    kDebug() << constraints << "constraints are FormFactor: " << formFactor() << ", Location: " << location();
+    Q_UNUSED(constraints)
+    //kDebug() << constraints << "constraints are FormFactor: " << formFactor() << ", Location: " << location();
 }
 
 QString Applet::name() const
@@ -517,6 +596,11 @@ QString Applet::pluginName() const
     return d->appletDescription.pluginName();
 }
 
+bool Applet::shouldConserveResources() const
+{
+    return Solid::PowerManagement::appShouldConserveResources();
+}
+
 QString Applet::category() const
 {
     if (!d->appletDescription.isValid()) {
@@ -549,14 +633,25 @@ QString Applet::category(const QString& appletName)
 
 bool Applet::isImmutable() const
 {
-    return d->immutable || d->kioskImmutable;
+    return  d->immutable || d->kioskImmutable ||
+            (containment() && containment()->isImmutable()) ||
+            (dynamic_cast<Corona*>( scene() ) && static_cast<Corona*>(scene())->isImmutable());
+}
+
+bool Applet::isKioskImmutable() const
+{
+    return d->kioskImmutable;
 }
 
 void Applet::setImmutable(bool immutable)
 {
+    if (d->immutable == immutable) {
+        return;
+    }
+
     d->immutable = immutable;
-    setFlag(QGraphicsItem::ItemIsMovable, d->immutable || d->kioskImmutable ||
-                                          !scene() || !static_cast<Corona*>(scene())->isImmutable());
+    // TODO: should we tell the applets too?
+    updateConstraints(ImmutableConstraint);
 }
 
 bool Applet::drawStandardBackground()
@@ -568,14 +663,13 @@ void Applet::setDrawStandardBackground(bool drawBackground)
 {
     if (drawBackground) {
         if (!d->background) {
-            prepareGeometryChange();
             d->background = new Plasma::Svg("widgets/background");
-            d->stretchBackgroundBorders = d->background->elementExists("hint-stretch-borders");
+            updateGeometry();
         }
     } else if (d->background) {
-        prepareGeometryChange();
         delete d->background;
         d->background = 0;
+        updateGeometry();
     }
 }
 
@@ -591,7 +685,7 @@ QString visibleFailureText(const QString& reason)
     if (reason.isEmpty()) {
         text = i18n("This object could not be created.");
     } else {
-        text = i18n("This object could not be created for the following reason:<p>%1</p>", reason);
+        text = i18n("This object could not be created for the following reason:<p><b>%1</b></p>", reason);
     }
 
     return text;
@@ -602,24 +696,33 @@ void Applet::setFailedToLaunch(bool failed, const QString& reason)
     if (d->failed == failed) {
         if (d->failureText) {
             d->failureText->setHtml(visibleFailureText(reason));
+            setGeometry(QRectF(geometry().topLeft(), d->failureText->sizeHint()));
         }
         return;
     }
 
     d->failed = failed;
     prepareGeometryChange();
+
+    d->failureText = 0;
     qDeleteAll(QGraphicsItem::children());
     delete layout();
 
     if (failed) {
         setDrawStandardBackground(true);
-        Layout* failureLayout = new BoxLayout(BoxLayout::TopToBottom,this);
-        d->failureText = new LineEdit(this, scene());
-        d->failureText->setFlags(0);
+        Layout* failureLayout = new BoxLayout(BoxLayout::TopToBottom, this);
+        failureLayout->setMargin(0);
+        d->failureText = new LineEdit(this);
+        d->failureText->setStyled(false);
+        d->failureText->document()->setTextWidth(200);
         d->failureText->setHtml(visibleFailureText(reason));
+        //FIXME: this needs to get the colour from the theme's colour scheme
+        d->failureText->setDefaultTextColor(KStatefulBrush(KColorScheme::Window,
+                                                           KColorScheme::NormalText,
+                                                           Theme::self()->colors())
+                                                        .brush(QPalette::Normal).color());
         failureLayout->addItem(d->failureText);
-    } else {
-        d->failureText = 0;
+        setGeometry(QRectF(geometry().topLeft(), d->failureText->sizeHint()));
     }
 
     update();
@@ -658,6 +761,42 @@ void Applet::performSetupConfig()
     showConfigurationInterface();
 }
 
+void Applet::checkImmutability()
+{
+    d->kioskImmutable = globalConfig().isImmutable() || config().isImmutable() ||
+                        (containment() && containment()->isKioskImmutable()) ||
+                        (dynamic_cast<Corona*>( scene() ) && static_cast<Corona*>(scene())->isKioskImmutable());
+    setImmutable(d->kioskImmutable);
+}
+
+void Applet::flushUpdatedConstraints()
+{
+    if (d->pendingConstraints == NoConstraint) {
+        return;
+    }
+
+    //kDebug() << "fushing constraints: " << d->pendingConstraints << "!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!";
+    Plasma::Constraints c = d->pendingConstraints;
+    d->pendingConstraints = NoConstraint;
+
+    Containment* containment = qobject_cast<Plasma::Containment*>(this);
+    if (c & Plasma::FormFactorConstraint) {
+        FormFactor f = formFactor();
+        setShadowShown(f == Planar);
+        setDrawStandardBackground(!containment && f != Vertical && f != Horizontal);
+    }
+
+    if (isContainment() && containment) {
+        containment->containmentConstraintsUpdated(c);
+    }
+
+    constraintsUpdated(c);
+
+    if (layout()) {
+        layout()->updateGeometry();
+    }
+}
+
 int Applet::type() const
 {
     return Type;
@@ -675,27 +814,29 @@ QRectF Applet::boundingRect() const
     d->getBorderSize(left,top,right,bottom);
 
 
-    //qDebug() << "Background , Border size" << d->background << left << top << right << bottom;
+    //kDebug() << "Background , Border size" << d->background << left << top << right << bottom;
 
     return rect.adjusted(-left,-top,right,bottom);
 }
 
 QSizeF Applet::sizeHint() const
 {
-    int left;
-    int right;
-    int top;
-    int bottom;
+    int left = 0;
+    int right = 0;
+    int top = 0;
+    int bottom = 0;
 
-    d->getBorderSize(left,top,right,bottom);
+    d->getBorderSize(left, top, right, bottom);
+    QSizeF borderSize = QSizeF(left + right, top + bottom);
 
-    //qDebug() << "Applet content size hint: " << contentSizeHint();
+    //kDebug() << "Applet content size hint: " << contentSizeHint() << "plus our borders" << left << right << top << bottom;
 
-    return contentSizeHint() + QSizeF(left+right,top+bottom);
+    return contentSizeHint() + QSizeF(left + right, top + bottom);
 }
 
 QList<QAction*> Applet::contextActions()
 {
+    kDebug() << "empty context actions";
     return QList<QAction*>();
 }
 
@@ -739,30 +880,32 @@ void Applet::paintWidget(QPainter *painter, const QStyleOptionGraphicsItem *opti
         d->shadow->generate();
     }
 
-    //qreal zoomLevel = painter->transform().m11() / transform().m11();
-    //kDebug() << "qreal " << zoomLevel << " = " << painter->transform().m11() << " / " << transform().m11();
-    //if (fabs(zoomLevel - scalingFactor(Plasma::DesktopZoom)) < std::numeric_limits<double>::epsilon()) { // Show Desktop
-        if (d->background) {
-            d->paintBackground(painter, this);
+    painter->save();
+    if (transform().isRotating()) {
+        painter->setRenderHint(QPainter::SmoothPixmapTransform);
+        painter->setRenderHint(QPainter::Antialiasing);
+    }
+
+    if (d->background) {
+        //kDebug() << "option rect is" << option->rect;
+        d->paintBackground(painter, this, option->rect);
+    }
+
+    if (!d->failed && !d->needsConfig) {
+        if (widget && isContainment()) {
+            // note that the widget we get is actually the viewport of the view, not the view itself
+            View* v = qobject_cast<Plasma::View*>(widget->parent());
+            if (v && !v->drawWallpaper()) {
+                painter->restore();
+                return;
+            }
         }
 
-        if (!d->failed && !d->needsConfig) {
-            paintInterface(painter, option, QRect(QPoint(0,0), d->contentSize(this).toSize()));
-        }
+        //kDebug() << "paint interface of" << (QObject*) this;
+        paintInterface(painter, option, QRect(QPoint(0,0), d->contentSize(this).toSize()));
+    }
 
-        d->paintHover(painter, this);
-    /*} else if (fabs(zoomLevel - scalingFactor(Plasma::GroupZoom)) < std::numeric_limits<double>::epsilon()) {
-        // Show Groups + Applet outline
-        //TODO: make pretty.
-        painter->setBrush(QBrush(color(), Qt::SolidPattern));
-        painter->drawRoundRect(boundingRect());
-        int iconDim = KIconLoader::global()->currentSize(KIconLoader::Desktop);
-        qreal midX = (boundingRect().width() / 2) - (iconDim / 2);
-        qreal midY = (boundingRect().height() / 2) - (iconDim / 2);
-        KIcon ico(icon());
-        ico.paint(painter, (int)midX, (int)midY, iconDim, iconDim);
-    }  else if (zoomLevel == scalingFactor(Plasma::OverviewZoom)) { //Show Groups only
-    } */
+    painter->restore();
 }
 
 void Applet::paintInterface(QPainter *painter, const QStyleOptionGraphicsItem *option,
@@ -771,7 +914,7 @@ void Applet::paintInterface(QPainter *painter, const QStyleOptionGraphicsItem *o
     Q_UNUSED(contentsRect)
 
     if (d->scriptEngine) {
-        d->scriptEngine->paintInterface(painter, option);
+        d->scriptEngine->paintInterface(painter, option, contentsRect);
     } else {
         //kDebug() << "Applet::paintInterface() default impl";
     }
@@ -780,17 +923,20 @@ void Applet::paintInterface(QPainter *painter, const QStyleOptionGraphicsItem *o
 FormFactor Applet::formFactor() const
 {
     Containment* c = containment();
-
-    if (!c) {
-        return Plasma::Planar;
-    }
-
-    return c->formFactor();
+    return c ? c->formFactor() : Plasma::Planar;
 }
 
 Containment* Applet::containment() const
 {
-    return dynamic_cast<Containment*>(parentItem());
+    QGraphicsItem *parent = parentItem();
+    Containment *c = 0;
+    while (parent) {
+        if ((c = dynamic_cast<Containment*>(parent))) {
+            break;
+        }
+        parent = parent->parentItem();
+    }
+    return c;
 }
 
 Location Applet::location() const
@@ -804,15 +950,33 @@ Location Applet::location() const
     return c->location();
 }
 
+QRectF Applet::contentRect() const
+{
+    return QRectF(QPointF(0, 0), contentSize());
+}
+
 QSizeF Applet::contentSize() const
 {
-    int top , left , right , bottom;
-    d->getBorderSize(left,top,right,bottom);
+    int top, left, right, bottom;
+    d->getBorderSize(left, top, right, bottom);
 
-    // qDebug() << "Geometry size: " << geometry().size();
-    // qDebug() << "Borders: " << left << top << right << bottom;
+    // kDebug() << "Geometry size: " << geometry().size();
+    // kDebug() << "Borders: " << left << top << right << bottom;
 
-    return geometry().size() - QSizeF(left+right,top+bottom);
+    return (geometry().size() - QSizeF(left + right, top + bottom)).expandedTo(QSizeF(0, 0));
+}
+
+void Applet::setContentSize(const QSizeF &size)
+{
+    int top, left, right, bottom;
+    d->getBorderSize(left, top, right, bottom);
+
+    resize(size + QSizeF(left + right, top + bottom));
+}
+
+void Applet::setContentSize(int width, int height)
+{
+    setContentSize(QSizeF(width, height));
 }
 
 QSizeF Applet::contentSizeHint() const
@@ -821,7 +985,49 @@ QSizeF Applet::contentSizeHint() const
         return layout()->sizeHint();
     }
 
-    return QSizeF(0, 0);
+    return contentSize();
+}
+
+void Applet::setMinimumContentSize(const QSizeF &minSize)
+{
+    int top, left, right, bottom;
+    d->getBorderSize(left, top, right, bottom);
+
+    setMinimumSize(minSize + QSizeF(left + right, top + bottom));
+}
+
+void Applet::setMinimumContentSize(int minWidth, int minHeight)
+{
+    setMinimumContentSize(QSizeF(minWidth, minHeight));
+}
+
+QSizeF Applet::minimumContentSize() const
+{
+    int top, left, right, bottom;
+    d->getBorderSize(left, top, right, bottom);
+
+    return minimumSize() - QSizeF(left + right, top + bottom);
+}
+
+void Applet::setMaximumContentSize(const QSizeF &maxSize)
+{
+    int top, left, right, bottom;
+    d->getBorderSize(left, top, right, bottom);
+
+    setMaximumSize(maxSize + QSizeF(left + right, top + bottom));
+}
+
+void Applet::setMaximumContentSize(int maxWidth, int maxHeight)
+{
+    setMaximumContentSize(QSizeF(maxWidth, maxHeight));
+}
+
+QSizeF Applet::maximumContentSize() const
+{
+    int top, left, right, bottom;
+    d->getBorderSize(left, top, right, bottom);
+
+    return maximumSize() - QSizeF(left + right, top + bottom);
 }
 
 QString Applet::globalName() const
@@ -953,7 +1159,7 @@ KPluginInfo::List Applet::knownAppletsForMimetype(const QString &mimetype)
     return KPluginInfo::fromServices(offers);
 }
 
-QStringList Applet::knownCategories(const QString &parentApp)
+QStringList Applet::knownCategories(const QString &parentApp, bool visibleOnly)
 {
     QString constraint = "exist [X-KDE-PluginInfo-Category]";
 
@@ -965,8 +1171,13 @@ QStringList Applet::knownCategories(const QString &parentApp)
 
     KService::List offers = KServiceTypeTrader::self()->query("Plasma/Applet", constraint);
     QStringList categories;
-    foreach (KService::Ptr applet, offers) {
+    foreach (const KService::Ptr applet, offers) {
         QString appletCategory = applet->property("X-KDE-PluginInfo-Category").toString();
+        if (visibleOnly && applet->noDisplay()) {
+            // we don't want to show the hidden category
+            continue;
+        }
+
         //kDebug() << "   and we have " << appletCategory;
         if (appletCategory.isEmpty()) {
             if (!categories.contains(i18n("Miscellaneous"))) {
@@ -995,16 +1206,26 @@ Applet* Applet::loadApplet(const QString& appletName, uint appletId, const QVari
         //      from the network at this point
         kDebug() << "Applet::loadApplet: offers is empty for \"" << appletName << "\"";
         return 0;
-    }
+    } /* else if (offers.count() > 1) {
+        kDebug() << "hey! we got more than one! let's blindly take the first one";
+    } */
+
+    KService::Ptr offer = offers.first();
 
     if (appletId == 0) {
         appletId = Private::nextId();
     }
 
+    if (!offer->property("X-Plasma-Language").toString().isEmpty()) {
+        kDebug() << "we have a script in the language of" << offer->property("X-Plasma-Language").toString();
+        Applet *applet = new Applet(0, offer->storageId(), appletId);
+        return applet;
+    }
+
     QVariantList allArgs;
-    allArgs << offers.first()->storageId() << appletId << args;
+    allArgs << offer->storageId() << appletId << args;
     QString error;
-    Applet* applet = offers.first()->createInstance<Plasma::Applet>(0, allArgs, &error);
+    Applet* applet = offer->createInstance<Plasma::Applet>(0, allArgs, &error);
 
     if (!applet) {
         kDebug() << "Couldn't load applet \"" << appletName << "\"! reason given: " << error;
@@ -1055,41 +1276,57 @@ bool Applet::isShadowShown() const
     return d->shadow && d->shadow->isVisible();
 }
 
+
+
 QVariant Applet::itemChange(GraphicsItemChange change, const QVariant &value)
 {
-    if (!d->shadow) {
-        return QGraphicsItem::itemChange(change, value);
-    }
-
     switch (change) {
     case ItemPositionChange:
-        d->shadow->adjustPosition();
+        if (d->shadow) {
+            d->shadow->adjustPosition();
+        }
         break;
     case ItemSceneChange: {
         QGraphicsScene *newScene = qvariant_cast<QGraphicsScene*>(value);
-        if (d->shadow->scene())
-            d->shadow->scene()->removeItem(d->shadow);
         if (newScene) {
-            newScene->addItem(d->shadow);
-            d->shadow->generate();
-            d->shadow->adjustPosition();
-            d->shadow->show();
+            // checking immutability requires having a working config object
+            // Applet relies on having a Corona scene to be able to get the
+            // correct config. so we have to wait until we have the scene,
+            // otherwise we trigger premature creation of the config objects
+            QTimer::singleShot(0, this, SLOT(checkImmutability()));
+        }
+
+        if (d->shadow) {
+            if (d->shadow->scene()) {
+                d->shadow->scene()->removeItem(d->shadow);
+            }
+
+            if (newScene) {
+                newScene->addItem(d->shadow);
+                d->shadow->generate();
+                d->shadow->adjustPosition();
+                d->shadow->show();
+            }
         }
     }
         break;
     case ItemVisibleChange:
-        d->shadow->setVisible(isVisible());
+        if (d->shadow) {
+            d->shadow->setVisible(isVisible());
+        }
         break;
     default:
         break;
     };
 
-    return QGraphicsItem::itemChange(change, value);
+    return Widget::itemChange(change, value);
 }
 
 void Applet::setGeometry(const QRectF& geometry)
 {
-    if (size() != geometry.size()) {
+    Plasma::Constraints updatedConstraints(0);
+
+    if (geometry.size().width() > 0 && geometry.size().height() > 0 && size() != geometry.size()) {
         prepareGeometryChange();
         qreal width = qBound(minimumSize().width(), geometry.size().width(), maximumSize().width());
         qreal height = qBound(minimumSize().height(), geometry.size().height(), maximumSize().height());
@@ -1099,30 +1336,34 @@ void Applet::setGeometry(const QRectF& geometry)
         if (layout()) {
             layout()->setGeometry(QRectF(QPoint(0, 0), contentSize()));
         }
+
+        if (managingLayout()) {
+            managingLayout()->invalidate();
+        }
+
+        updatedConstraints |= Plasma::SizeConstraint;
     }
 
-    setPos(geometry.topLeft());
-    update();
+    if (geometry.topLeft() != pos()) {
+        setPos(geometry.topLeft());
+        updatedConstraints |= Plasma::LocationConstraint;
+    }
+
+    if (updatedConstraints) {
+        updateConstraints(updatedConstraints);
+        emit geometryChanged();
+        update();
+    }
 }
 
-void Applet::contextMenuEvent(QGraphicsSceneContextMenuEvent *event)
+void Applet::setIsContainment(bool isContainment)
 {
-    //kDebug() << "context menu event!";
-    if (!scene()) {
-        return;
-    }
+    d->isContainment = isContainment;
+}
 
-    Applet* containment = dynamic_cast<Plasma::Applet*>(topLevelItem());
-
-    if (!containment) {
-        Widget::contextMenuEvent(event);
-        return;
-    }
-
-    // we want to pass up the context menu event to the Containment at
-    // this point
-    containment->contextMenuEvent(event);
-    return;
+bool Applet::isContainment() const
+{
+    return d->isContainment;
 }
 
 } // Plasma namespace
