@@ -1,4 +1,5 @@
 /*
+ *   Copyright 2007 Aaron Seigo <aseigo@kde.org>
  *   Copyright 2007 Matt Broadstone <mbroadst@gmail.com>
  *
  *   This program is free software; you can redistribute it and/or modify
@@ -43,7 +44,7 @@ class PLASMA_EXPORT Corona : public QGraphicsScene
 public:
     explicit Corona(QObject * parent = 0);
     explicit Corona(const QRectF & sceneRect, QObject * parent = 0);
-    explicit Corona(qreal x, qreal y, qreal width, qreal height, QObject * parent = 0);
+    Corona(qreal x, qreal y, qreal width, qreal height, QObject * parent = 0);
     ~Corona();
 
     /**
@@ -51,6 +52,11 @@ public:
      **/
     bool isImmutable() const;
 
+    /**
+     * @return true if this Corona is immutable due to Kiosk settings
+     */
+    bool isKioskImmutable() const;
+
     /**
      * A rect containing the maximum size a plasmoid on this corona should
      * consider being.
@@ -92,6 +98,11 @@ public:
      */
     void clearApplets();
 
+    /**
+     * Returns the the config file used to store the configuration for this Corona
+     */
+    KSharedConfig::Ptr config() const;
+
 public Q_SLOTS:
     /**
      * Load applets from the default config file
@@ -104,25 +115,19 @@ public Q_SLOTS:
     void saveApplets() const;
 
     /**
-     * Adds an applet to the default Containment
-     *
-     * @param name the plugin name for the applet, as given by 
-     *        KPluginInfo::pluginName()
-     * @param args argument list to pass to the plasmoid
-     * @param id to assign to this applet, or 0 to auto-assign it a new id
-     * @param geometry where to place the applet, or to auto-place it if an invalid
-     *                 is provided
-     *
-     * @return a pointer to the applet on success, or 0 on failure
+     * Called when there have been changes made to configuration that should be saved
+     * to disk at the next convenient moment
      */
-    Applet* addApplet(const QString& name, const QVariantList& args = QVariantList(),
-                      uint id = 0, const QRectF &geometry = QRectF(-1, -1, -1, -1));
+    void scheduleConfigSync() const;
 
     /**
      * Adds a Containment to the Corona
      *
      * @param name the plugin name for the containment, as given by
-     *        KPluginInfo::pluginName()
+     *        KPluginInfo::pluginName(). If an empty string is passed in, the defalt
+     *        containment plugin will be used (usually DesktopContainment). If the
+     *        string literal "null" is passed in, then no plugin will be loaded and
+     *        a simple Containment object will be created instead.
      * @param args argument list to pass to the containment
      * @param id to assign to this containment, or 0 to auto-assign it a new id
      * @param geometry where to place the containment, or to auto-place it if an invalid
@@ -133,6 +138,11 @@ public Q_SLOTS:
     Containment* addContainment(const QString& name, const QVariantList& args = QVariantList(),
                                 uint id = 0, bool delayInit = false);
 
+    /**
+     * Removes a given containment from the corona
+     */
+    void destroyContainment(Containment *containment);
+
     /**
      * Returns the Containment, if any, for a given physical screen
      *
@@ -145,18 +155,25 @@ public Q_SLOTS:
      */
     QList<Containment*> containments() const;
 
-    /**
-     * Adds a SuperKaramba theme to the scene
-     *
-     * @param path the path to the theme file
-     */
-    void addKaramba(const KUrl& path);
-
     /**
      * Sets if the applets are Immutable
      */
     void setImmutable(bool immutable_);
 
+Q_SIGNALS:
+    /**
+     * This signal indicates that a application launch was triggered
+     */
+    void launchActivated();
+
+    /**
+     * This signal indicates that a new screen is being handled
+     * at runtime (e.g. after first creation)
+     *
+     * @param screen the xinerama number of the new screen
+     */
+    void newScreen(int screen);
+
 protected:
     void dragEnterEvent(QGraphicsSceneDragDropEvent* event);
     void dragLeaveEvent(QGraphicsSceneDragDropEvent* event);
@@ -165,6 +182,8 @@ protected:
 
 protected Q_SLOTS:
     void containmentDestroyed(QObject*);
+    void screenResized(int);
+    void syncConfig();
 
 private:
     class Private;
