@@ -28,10 +28,12 @@
 #include <KServiceTypeTrader>
 
 #include "animator.h"
+#include "widgets/widget.h"
 
 namespace Plasma
 {
 
+static const qreal MIN_TICK_RATE = 40;
 
 struct AnimationState
 {
@@ -66,9 +68,22 @@ struct MovementState
     int currentInterval;
     int frames;
     int currentFrame;
+    QPoint start;
     QPoint destination;
 };
 
+struct CustomAnimationState
+{
+    Phase::CurveShape curve;
+    int frames;
+    int currentFrame;
+    int interval;
+    int currentInterval;
+    Phase::AnimId id;
+    QObject* receiver;
+    char* slot;
+};
+
 class Phase::Private
 {
     public:
@@ -84,10 +99,27 @@ class Phase::Private
         {
             qDeleteAll(animatedItems);
             qDeleteAll(animatedElements);
+            qDeleteAll(movingItems);
+
+            QMutableMapIterator<AnimId, CustomAnimationState*> it(customAnims);
+            while (it.hasNext()) {
+                delete it.value()->slot;
+                delete it.value();
+                it.remove();
+            }
+
             // Animator is a QObject
             // and we don't own the items
         }
 
+        qreal calculateProgress(int frames, int currentFrame)
+        {
+            qreal progress = frames;
+            progress = currentFrame / progress;
+            progress = qMin(qreal(1.0), qMax(qreal(0.0), progress));
+            return progress;
+        }
+
         void performAnimation(qreal amount, const AnimationState* state)
         {
             switch (state->animation) {
@@ -113,10 +145,12 @@ class Phase::Private
         {
             switch (state->movement) {
                 case Phase::SlideIn:
-                    animator->slideIn(amount, state->item, state->destination);
+                    //kDebug() << "performMovement, SlideIn";
+                    animator->slideIn(amount, state->item, state->start, state->destination);
                     break;
                 case Phase::SlideOut:
-                    animator->slideOut(amount, state->item, state->destination);
+                    //kDebug() << "performMovement, SlideOut";
+                    animator->slideOut(amount, state->item, state->start, state->destination);
                     break;
             }
         }
@@ -131,6 +165,7 @@ class Phase::Private
         QMap<QGraphicsItem*, AnimationState*> animatedItems;
         QMap<QGraphicsItem*, MovementState*> movingItems;
         QMap<Phase::AnimId, ElementAnimationState*> animatedElements;
+        QMap<AnimId, CustomAnimationState*> customAnims;
 };
 
 class PhaseSingleton
@@ -181,6 +216,18 @@ void Phase::appletDestroyed(QObject* o)
     }
 }
 
+void Phase::customAnimReceiverDestroyed(QObject* o)
+{
+    QMutableMapIterator<AnimId, CustomAnimationState*> it(d->customAnims);
+    while (it.hasNext()) {
+        if (it.next().value()->receiver == o) {
+            delete it.value()->slot;
+            delete it.value();
+            it.remove();
+        }
+    }
+}
+
 void Phase::animateItem(QGraphicsItem* item, Animation animation)
 {
      //kDebug();
@@ -208,14 +255,14 @@ void Phase::animateItem(QGraphicsItem* item, Animation animation)
     state->frames = frames / 3;
     state->currentFrame = 0;
     state->interval = 333 / state->frames;
-    state->interval = (state->interval / 40) * 40;
+    state->interval = (state->interval / MIN_TICK_RATE) * MIN_TICK_RATE;
     state->currentInterval = state->interval;
 
     d->animatedItems[item] = state;
     d->performAnimation(0, state);
 
     if (!d->timerId) {
-        d->timerId = startTimer(40);
+        d->timerId = startTimer(MIN_TICK_RATE);
         d->time.restart();
     }
 }
@@ -237,34 +284,70 @@ void Phase::moveItem(QGraphicsItem* item, Movement movement, const QPoint &desti
      }
 
      MovementState* state = new MovementState;
-     state->destination=destination;
+     state->destination = destination;
+     state->start = item->pos().toPoint();
      state->item = item;
      state->movement = movement;
      state->curve = d->animator->curve(movement);
      //TODO: variance in times based on the value of animation
      state->frames = frames / 2;
      state->currentFrame = 0;
-     state->interval = 500 / state->frames;
-     state->interval = (state->interval / 40) * 40;
+     state->interval = 250 / state->frames;
+     state->interval = (state->interval / MIN_TICK_RATE) * MIN_TICK_RATE;
      state->currentInterval = state->interval;
 
      d->movingItems[item] = state;
      d->performMovement(0, state);
 
      if (!d->timerId) {
-          d->timerId = startTimer(40);
+          d->timerId = startTimer(MIN_TICK_RATE);
           d->time.restart();
      }
 }
 
-void Phase::render(QGraphicsItem* item, QImage& image, RenderOp op)
+Phase::AnimId Phase::customAnimation(int frames, int duration, Phase::CurveShape curve,
+                                     QObject* receiver, const char* slot)
 {
-    Q_UNUSED(item);
-    switch (op) {
-        case RenderBackground:
-            d->animator->renderBackground(image);
-            break;
+    if (frames < 1 || duration < 1 || !receiver || !slot) {
+        return -1;
     }
+
+    CustomAnimationState *state = new CustomAnimationState;
+    state->id = ++d->animId;
+    state->frames = frames;
+    state->currentFrame = 0;
+    state->curve = curve;
+    state->interval = duration / qreal(state->frames);
+    state->interval = qMax( 1, state->interval );
+    state->interval = (state->interval / MIN_TICK_RATE) * MIN_TICK_RATE;
+    state->currentInterval = state->interval;
+    state->receiver = receiver;
+    state->slot = qstrdup(slot);
+
+    d->customAnims[state->id] = state;
+
+    connect(receiver, SIGNAL(destroyed(QObject*)),
+            this, SLOT(customAnimReceiverDestroyed(QObject*)));
+
+    QMetaObject::invokeMethod(receiver, slot, Q_ARG(qreal, 0));
+
+    if (!d->timerId) {
+        d->timerId = startTimer(MIN_TICK_RATE);
+        d->time.restart();
+    }
+
+    return state->id;
+}
+
+void Phase::stopCustomAnimation(AnimId id)
+{
+    QMap<AnimId, CustomAnimationState*>::iterator it = d->customAnims.find(id);
+    if (it != d->customAnims.end()) {
+        delete [] it.value()->slot;
+        delete it.value();
+        d->customAnims.erase(it);
+    }
+    //kDebug() << "stopCustomAnimation(AnimId " << id << ") done";
 }
 
 Phase::AnimId Phase::animateElement(QGraphicsItem *item, ElementAnimation animation)
@@ -278,7 +361,7 @@ Phase::AnimId Phase::animateElement(QGraphicsItem *item, ElementAnimation animat
     state->frames = d->animator->framesPerSecond(animation) / 5;
     state->currentFrame = 0;
     state->interval = 200 / state->frames;
-    state->interval = (state->interval / 40) * 40;
+    state->interval = (state->interval / MIN_TICK_RATE) * MIN_TICK_RATE;
     state->currentInterval = state->interval;
     state->id = ++d->animId;
 
@@ -291,13 +374,20 @@ Phase::AnimId Phase::animateElement(QGraphicsItem *item, ElementAnimation animat
     }
 
     d->animatedElements[state->id] = state;
-    state->item->update();
+    // nasty hack because QGraphicsItem::update isn't virtual!
+    // FIXME: remove in 4.1 as we will no longer need the caching in Plasma::Widget with Qt 4.4
+    Plasma::Widget *widget = dynamic_cast<Plasma::Widget*>(state->item);
+    if (widget) {
+        widget->update();
+    } else {
+        state->item->update();
+    }
 
     //kDebug() << "startElementAnimation(AnimId " << animation << ") returning " << state->id;
     if (needTimer && !d->timerId) {
         // start a 20fps timer;
         //TODO: should be started at the maximum frame rate needed only?
-        d->timerId = startTimer(40);
+        d->timerId = startTimer(MIN_TICK_RATE);
         d->time.restart();
     }
     return state->id;
@@ -338,7 +428,7 @@ QPixmap Phase::animationResult(AnimId id)
     qreal progress = state->frames;
     //kDebug() << "Phase::animationResult(" << id <<   " at " << progress;
     progress = state->currentFrame / progress;
-    progress = qMin(1.0, qMax(0.0, progress));
+    progress = qMin(qreal(1.0), qMax(qreal(0.0), progress));
     //kDebug() << "Phase::animationResult(" << id <<   " at " << progress;
 
     switch (state->animation) {
@@ -357,7 +447,7 @@ void Phase::timerEvent(QTimerEvent *event)
 {
     Q_UNUSED(event)
     bool animationsRemain = false;
-    int elapsed = 40;
+    int elapsed = MIN_TICK_RATE;
     if (d->time.elapsed() > elapsed) {
         elapsed = d->time.elapsed();
     }
@@ -370,9 +460,7 @@ void Phase::timerEvent(QTimerEvent *event)
             state->currentFrame += qMax(1, elapsed / state->interval);
 
             if (state->currentFrame < state->frames) {
-                qreal progress = state->frames;
-                progress = state->currentFrame / progress;
-                progress = qMin(1.0, qMax(0.0, progress));
+                qreal progress = d->calculateProgress(state->frames, state->currentFrame);
                 d->performAnimation(progress, state);
                 state->currentInterval = state->interval;
                 //TODO: calculate a proper interval based on the curve
@@ -396,11 +484,8 @@ void Phase::timerEvent(QTimerEvent *event)
             state->currentFrame += qMax(1, elapsed / state->interval);
 
             if (state->currentFrame < state->frames) {
-                qreal progress = state->frames;
-                progress = state->currentFrame / progress;
-                progress = qMin(1.0, qMax(0.0, progress));
-                //kDebug()<<progress;
-                d->performMovement(progress, state);
+                d->performMovement(d->calculateProgress(state->frames, state->currentFrame), state);
+                //TODO: calculate a proper interval based on the curve
                 state->currentInterval = state->interval;
                 animationsRemain = true;
             } else {
@@ -417,6 +502,7 @@ void Phase::timerEvent(QTimerEvent *event)
 
     foreach (ElementAnimationState* state, d->animatedElements) {
         if (state->currentFrame == state->frames) {
+            //kDebug() << "skipping" << state->id << "as its already at frame" << state->currentFrame << "of" << state->frames;
             // since we keep element animations around until they are
             // removed, we will end up with finished animations in the queue;
             // just skip them
@@ -426,19 +512,23 @@ void Phase::timerEvent(QTimerEvent *event)
 
         if (state->currentInterval <= elapsed) {
             // we need to step forward!
-/*            kDebug() << "stepping forwards element anim " << state->id << " from " << state->currentFrame
+            /*kDebug() << "stepping forwards element anim " << state->id << " from " << state->currentFrame
                     << " by " << qMax(1, elapsed / state->interval) << " to "
                     << state->currentFrame + qMax(1, elapsed / state->interval) << endl;*/
             state->currentFrame += qMax(1, elapsed / state->interval);
-            state->item->update();
-
+            // nasty hack because QGraphicsItem::update isn't virtual!
+            // FIXME: remove in 4.1 as we will no longer need the caching in Plasma::Widget with Qt 4.4
+            Plasma::Widget *widget = dynamic_cast<Plasma::Widget*>(state->item);
+            if (widget) {
+                widget->update();
+            } else {
+                state->item->update();
+            }
+            
             if (state->currentFrame < state->frames) {
                 state->currentInterval = state->interval;
                 //TODO: calculate a proper interval based on the curve
-                qreal progress = state->frames;
-                progress = state->currentFrame / progress;
-                progress = qMin(1.0, qMax(0.0, progress));
-                state->interval *= 1 - progress;
+                state->interval *= 1 - d->calculateProgress(state->frames, state->currentFrame);
                 animationsRemain = true;
             }
         } else {
@@ -447,6 +537,34 @@ void Phase::timerEvent(QTimerEvent *event)
         }
     }
 
+    foreach (CustomAnimationState *state, d->customAnims) {
+        if (state->currentInterval <= elapsed) {
+            // advance the frame
+            state->currentFrame += qMax(1, elapsed / state->interval);
+            /*kDebug() << "custom anim for" << state->receiver << "to slot" << state->slot
+                     << "with interval of" << state->interval << "at frame" << state->currentFrame;*/
+
+            if (state->currentFrame < state->frames) {
+                //kDebug () << "not the final frame";
+                //TODO: calculate a proper interval based on the curve
+                state->currentInterval = state->interval;
+                animationsRemain = true;
+                // signal the object
+                QMetaObject::invokeMethod(state->receiver, state->slot,
+                                          Q_ARG(qreal,
+                                                d->calculateProgress(state->frames, state->currentFrame)));
+            } else {
+                QMetaObject::invokeMethod(state->receiver, state->slot, Q_ARG(qreal, 1));
+                d->customAnims.erase(d->customAnims.find(state->id));
+                delete [] state->slot;
+                delete state;
+            }
+        } else {
+            state->currentInterval -= elapsed;
+            animationsRemain = true;
+        }
+    }
+
     if (!animationsRemain && d->timerId) {
         killTimer(d->timerId);
         d->timerId = 0;
