@@ -21,6 +21,10 @@
 #define PLASMA_THEME_H
 
 #include <QtCore/QObject>
+#include <QtGui/QFont>
+#include <QtGui/QFontMetrics>
+
+#include <KDE/KSharedConfig>
 
 #include <plasma/plasma_export.h>
 
@@ -69,7 +73,12 @@ class PLASMA_EXPORT Theme : public QObject
         void setApplication(const QString &appname);
 
         /**
-         * @return the name of the theme. "default" is none set.
+         * Sets the current theme being used.
+         */
+        void setThemeName(const QString &themeName);
+
+        /**
+         * @return the name of the theme.
          */
         QString themeName() const;
 
@@ -82,6 +91,34 @@ class PLASMA_EXPORT Theme : public QObject
          */
         Q_INVOKABLE QString image( const QString& name ) const;
 
+        /**
+         * Returns the color scheme configurationthat goes along this theme.
+         * This can be used with KStatefulBrush and KColorScheme to determine
+         * the proper colours to use along with the visual elements in this theme.
+         */
+        Q_INVOKABLE KSharedConfigPtr colors() const;
+
+        /**
+         * Returns the text color to be used by items resting on the background
+         */
+        Q_INVOKABLE QColor textColor() const;
+
+        /**
+         * Sets the default font to be used with themed items. Defaults to
+         * the application wide default font.
+         */
+        Q_INVOKABLE void setFont(const QFont &font);
+
+        /**
+         * Returns the font to be used by themed items
+         */
+        Q_INVOKABLE QFont font() const;
+
+        /**
+         * Returns the font metrics for the font to be used by themed items
+         */
+        Q_INVOKABLE QFontMetrics fontMetrics() const;
+
     Q_SIGNALS:
         /**
          * Emitted when the user changes the theme. SVGs should be reloaded at
