@@ -119,7 +119,7 @@ void CheckBox::paintWidget(QPainter *painter, const QStyleOptionGraphicsItem *op
     widget-> style()->drawControl(QStyle::CE_CheckBox, &options, painter, widget);
 }
 
-void CheckBox::updated(const QString&, const DataEngine::Data& data)
+void CheckBox::dataUpdated(const QString&, const DataEngine::Data& data)
 {
     foreach (const QVariant& variant, data) {
         if (variant.canConvert(QVariant::Bool)) {
@@ -192,8 +192,12 @@ void CheckBox::mouseReleaseEvent(QGraphicsSceneMouseEvent * event)
             d->state = Qt::Checked;
         }
     }
+
     update();
-    emit clicked();
+
+    if (sceneBoundingRect().contains(event->scenePos())) {
+        emit clicked();
+    }
 }
 
 
@@ -210,6 +214,7 @@ void CheckBox::hoverLeaveEvent ( QGraphicsSceneHoverEvent * event )
     event->accept();
     d->hasMouse= false;
     update();
+    Widget::hoverEnterEvent(event);
 }
 
 void  CheckBox::hoverEnterEvent ( QGraphicsSceneHoverEvent * event)
@@ -217,6 +222,7 @@ void  CheckBox::hoverEnterEvent ( QGraphicsSceneHoverEvent * event)
     event->accept();
     d->hasMouse = true;
     update();
+    Widget::hoverLeaveEvent(event);
 }
 
 /*
