@@ -3,6 +3,7 @@
  *   Copyright 2007 by Riccardo Iaconelli <riccardo@kde.org>
  *   Copyright 2007 by Matt Broadstone <mbroadst@gmail.com>
  *   Copyright 2006-2007 Fredrik Höglund <fredrik@kde.org>
+ *   Copyright 2007 by Marco Martin <notmart@gmail.com>
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU Library General Public License as
@@ -35,6 +36,7 @@
 
 #ifdef BACKINGSTORE_BLUR_HACK
 #include <private/qwindowsurface_p.h>
+#include "effects/blur.cpp"
 #endif
 
 #include <KIconEffect>
@@ -44,76 +46,36 @@
 #include <KRun>
 #include <KMimeType>
 #include <KDebug>
+#include <KColorScheme>
+
+#include <plasma/theme.h>
 
 #include "phase.h"
 #include "svg.h"
-#include "effects/blur.cpp"
 
 namespace Plasma
 {
 
 Icon::Private::Private()
-    : svg("widgets/iconbutton"),
-      svgElements(0),
-      size(48*1.1, 48*1.1),
+    : iconSvg(0),
       iconSize(48, 48),
-      state(Private::NoState),
+      states(Private::NoState),
       orientation(Qt::Vertical),
-      calculateSizeRequested(true)          // First time always true
+      invertLayout(false),
+      drawBg(false)
 {
-    svg.setContentType(Plasma::Svg::ImageSet);
-    svg.resize(size);
-
-    //TODO: recheck when svg changes
-    checkSvgElements();
+    textColor = KColorScheme(QPalette::Active, KColorScheme::Window,
+                             Plasma::Theme::self()->colors()).foreground().color();
+    shadowColor = KColorScheme(QPalette::Active, KColorScheme::Window,
+                               Plasma::Theme::self()->colors()).background().color();
 }
 
 Icon::Private::~Private()
 {
     qDeleteAll(cornerActions);
+    delete iconSvg;
 }
 
-void Icon::Private::checkSvgElements()
-{
-    if (svg.elementExists("background")) {
-        svgElements |= SvgBackground;
-    }
-
-    if (svg.elementExists("background-hover")) {
-        svgElements |= SvgBackgroundHover;
-    }
-
-    if (svg.elementExists("background-pressed")) {
-        svgElements |= SvgBackgroundPressed;
-    }
-
-    if (svg.elementExists("foreground")) {
-        svgElements |= SvgForeground;
-    }
-
-    if (svg.elementExists("foreground-hover")) {
-        svgElements |= SvgForegroundHover;
-    }
-
-    if (svg.elementExists("foreground-pressed")) {
-        svgElements |= SvgForegroundPressed;
-    }
-
-    if (svg.elementExists("minibutton")) {
-        svgElements |= SvgMinibutton;
-    }
-
-    if (svg.elementExists("minibutton-hover")) {
-        svgElements |= SvgMinibuttonHover;
-    }
-
-    if (svg.elementExists("minibutton-pressed")) {
-        svgElements |= SvgMinibuttonPressed;
-    }
-}
-
-
-
 IconAction::IconAction(Icon* icon, QAction *action)
     : m_icon(icon),
       m_action(action),
@@ -197,11 +159,11 @@ void IconAction::rebuildPixmap()
     m_pixmap = QPixmap(26, 26);
     m_pixmap.fill(Qt::transparent);
 
-    int element = Icon::Private::SvgMinibutton;
+    int element = Icon::Private::Minibutton;
     if (m_pressed) {
-        element = Icon::Private::SvgMinibuttonPressed;
+        element = Icon::Private::MinibuttonPressed;
     } else if (m_hovered) {
-        element = Icon::Private::SvgMinibuttonHover;
+        element = Icon::Private::MinibuttonHover;
     }
 
     QPainter painter(&m_pixmap);
@@ -212,13 +174,13 @@ void IconAction::rebuildPixmap()
 bool IconAction::event(QEvent::Type type, const QPointF &pos)
 {
     switch (type) {
-    case QEvent::MouseButtonPress: {
+    case QEvent::GraphicsSceneMousePress: {
         setSelected(m_rect.contains(pos));
         return isSelected();
         }
         break;
 
-    case QEvent::MouseMove: {
+    case QEvent::GraphicsSceneMouseMove: {
         bool wasSelected = isSelected();
         bool active = m_rect.contains(pos);
         setSelected(wasSelected && active);
@@ -226,7 +188,7 @@ bool IconAction::event(QEvent::Type type, const QPointF &pos)
         }
         break;
 
-    case QEvent::MouseButtonRelease: {
+    case QEvent::GraphicsSceneMouseRelease: {
         // kDebug() << "IconAction::event got a QEvent::MouseButtonRelease, " << isSelected();
         bool wasSelected = isSelected();
         setSelected(false);
@@ -317,7 +279,8 @@ void Icon::init()
     d->setVerticalMargin(Private::IconMargin, focusHMargin, focusVMargin);
     d->setVerticalMargin(Private::ItemMargin, 0, 0);
 
-    calculateSize();
+    d->setActiveMargins();
+    d->currentSize = QSizeF(-1,-1);
 }
 
 void Icon::addAction(QAction *action)
@@ -338,13 +301,13 @@ void Icon::addAction(QAction *action)
         iconAction->setRect(QRectF(6, 6, 32, 32));
         break;
     case Private::TopRight:
-        iconAction->setRect(QRectF(d->size.width() - 38, 6, 32, 32));
+        iconAction->setRect(QRectF(size().width() - 38, 6, 32, 32));
         break;
     case Private::BottomLeft:
-        iconAction->setRect(QRectF(6, d->size.height() - 38, 32, 32));
+        iconAction->setRect(QRectF(6, size().height() - 38, 32, 32));
         break;
     case Private::BottomRight:
-        iconAction->setRect(QRectF(d->size.width() - 38, d->size.height() - 38, 32, 32));
+        iconAction->setRect(QRectF(size().width() - 38, size().height() - 38, 32, 32));
         break;
     }
 }
@@ -363,68 +326,142 @@ void Icon::actionDestroyed(QObject* action)
     update();   // redraw since an action has been deleted.
 }
 
-QSizeF Icon::Private::displaySizeHint(const QStyleOptionGraphicsItem *option) const
+int Icon::numDisplayLines()
 {
+    return d->numDisplayLines;
+}
+
+void Icon::setNumDisplayLines(int numLines)
+{
+    if(numLines > d->maxDisplayLines) {
+        d->numDisplayLines = d->maxDisplayLines;
+    }
+    else {
+        d->numDisplayLines = numLines;
+    }
+}
+
+QSizeF Icon::sizeHint() const
+{
+    return d->currentSize;
+}
+
+void Icon::setDrawBackground(bool draw)
+{
+    if (d->drawBg != draw) {
+        d->drawBg = draw;
+        update();
+    }
+}
+
+bool Icon::drawBackground() const
+{
+    return d->drawBg;
+}
+
+QSizeF Icon::Private::displaySizeHint(const QStyleOptionGraphicsItem *option, const qreal width) const
+{
+    if (text.isEmpty() && infoText.isEmpty()) {
+      return QSizeF( .0, .0 );
+    }
     QString label = text;
     // const qreal maxWidth = (orientation == Qt::Vertical) ? iconSize.width() + 10 : 32757;
     // NOTE: find a way to use the other layoutText, it currently returns nominal width, when
     //       we actually need the actual width.
 
 
+    qreal textWidth = width -
+                      horizontalMargin[Private::TextMargin].left -
+                      horizontalMargin[Private::TextMargin].right;
+
+    //allow only five lines of text
+    const qreal maxHeight = numDisplayLines*Plasma::Theme::self()->fontMetrics().lineSpacing();
+
     // To compute the nominal size for the label + info, we'll just append
     // the information string to the label
-    const QString info = infoText;
-    if (!info.isEmpty())
-        label += QString(QChar::LineSeparator) + info;
+    if (!infoText.isEmpty()) {
+        label += QString(QChar::LineSeparator) + infoText;
+    }
 
     QTextLayout layout;
     setLayoutOptions(layout, option);
-    QSizeF size = layoutText(layout, option, label, QSizeF(iconSize.width() + 10, 32757));
+    QSizeF size = layoutText(layout, option, label, QSizeF(textWidth, maxHeight));
 
     return addMargin(size, TextMargin);
 }
 
-void Icon::calculateSize()
+void Icon::layoutIcons(const QStyleOptionGraphicsItem *option)
 {
-    if (d->calculateSizeRequested = true) {
-        // We do this to get size hint information before the icon has been drawn, as
-        // we have no access to the style option before that time. So we create a dummy.
-        QStyleOptionGraphicsItem option;
-        option.state = QStyle::State_None;
-        option.rect = boundingRect().toRect();
-        calculateSize(&option);
-    }
-    else {
-        d->calculateSizeRequested = true;
-    }
-}
-
-void Icon::calculateSize(const QStyleOptionGraphicsItem *option)
-{
-    if (!d->calculateSizeRequested)
+    if (size() == d->currentSize) {
         return;
+    }
 
-    prepareGeometryChange();
+    d->currentSize = size();
     d->setActiveMargins();
 
-    const QSizeF displaySize    = d->displaySizeHint(option);
-    const QSizeF decorationSize = d->addMargin(d->iconSize, Private::IconMargin);
+    //calculate icon size based on the available space
+    qreal iconWidth;
 
-    QSizeF newSize;
-    if (d->orientation == Qt::Vertical)
-    {
-        newSize.rwidth()  = qMax(decorationSize.width(), displaySize.width());
-        newSize.rheight() = decorationSize.height() + displaySize.height() + 1;
-    }
-    else
-    {
-        newSize.rwidth()  = decorationSize.width() + displaySize.width() + 1;
-        newSize.rheight() = qMax(decorationSize.height(), displaySize.height());
+    if (d->orientation == Qt::Vertical) {
+        qreal heightAvail;
+        //if there is text resize the icon in order to make room for the text
+        if (!d->text.isEmpty() || !d->infoText.isEmpty()) {
+            heightAvail = d->currentSize.height() -
+                          d->displaySizeHint(option, d->currentSize.width()).height() -
+                          d->verticalMargin[Private::TextMargin].top -
+                          d->verticalMargin[Private::TextMargin].bottom;
+            //never make a label higher than half the total height
+            heightAvail = qMax(heightAvail, d->currentSize.height()/2);
+        }else{
+            heightAvail = d->currentSize.height();
+        }
+
+        //aspect ratio very "tall"
+        if (d->currentSize.width() < heightAvail) {
+            iconWidth = d->currentSize.width() -
+                        d->horizontalMargin[Private::IconMargin].left -
+                        d->horizontalMargin[Private::IconMargin].right;
+        }else{
+            iconWidth = heightAvail -
+                        d->verticalMargin[Private::IconMargin].top -
+                        d->verticalMargin[Private::IconMargin].bottom;
+        }
+    //Horizontal layout
+    }else{
+        qreal widthAvail;
+        QFontMetricsF fm(font());
+
+        //make room for at most 14 characters
+        qreal textWidth = qMax(fm.width(d->text.left(12)),
+                          fm.width(d->infoText.left(12))) +
+                          fm.width("xx") +
+                          d->horizontalMargin[Private::TextMargin].left +
+                          d->horizontalMargin[Private::TextMargin].right;
+
+        //if there is text resize the icon in order to make room for the text
+        if (!d->text.isEmpty() || !d->infoText.isEmpty()) {
+            widthAvail = d->currentSize.width() -
+                         //FIXME: fontmetrics
+                         textWidth -
+                         d->horizontalMargin[Private::TextMargin].left -
+                         d->horizontalMargin[Private::TextMargin].right;
+        }else{
+            widthAvail = d->currentSize.width();
+        }
+
+        //aspect ratio very "wide"
+        if (d->currentSize.height() < widthAvail) {
+            iconWidth = d->currentSize.height() -
+                        d->verticalMargin[Private::IconMargin].top -
+                        d->verticalMargin[Private::IconMargin].bottom;
+        }else{
+            iconWidth = widthAvail -
+                        d->horizontalMargin[Private::IconMargin].left -
+                        d->horizontalMargin[Private::IconMargin].right;
+        }
     }
 
-    d->size = d->addMargin(newSize, Private::ItemMargin);
-    d->svg.resize(d->size);
-    d->calculateSizeRequested = false;
+    d->iconSize = QSizeF(iconWidth, iconWidth);
 
     int count = 0;
     foreach (IconAction* iconAction, d->cornerActions) {
@@ -436,107 +473,100 @@ void Icon::calculateSize(const QStyleOptionGraphicsItem *option)
             //iconAction->setRect(QRectF(6, 6, 32, 32));
             break;
         case Private::TopRight:
-            iconAction->setRect(QRectF(d->size.width() - 38, 6, 32, 32));
+            iconAction->setRect(QRectF(d->currentSize.width() - 38, 6, 32, 32));
             break;
         case Private::BottomLeft:
-            iconAction->setRect(QRectF(6, d->size.height() - 38, 32, 32));
+            iconAction->setRect(QRectF(6, d->currentSize.height() - 38, 32, 32));
             break;
         case Private::BottomRight:
-            iconAction->setRect(QRectF(d->size.width() - 38, d->size.height() - 38, 32, 32));
+            iconAction->setRect(QRectF(d->currentSize.width() - 38, d->currentSize.height() - 38, 32, 32));
             break;
         }
 
         ++count;
     }
 
-    update();
 }
 
-QRectF Icon::boundingRect() const
+void Icon::setSvg(const QString &svgFilePath, const QString &elementId)
 {
-    return QRectF(QPointF(0, 0), d->size);
-}
-
-void Icon::Private::drawBackground(QPainter *painter)
-{
-    QString element;
-    if (svgElements & Private::SvgBackground) {
-        element = "background";
+    if (!d->iconSvg) {
+        d->iconSvg = new Plasma::Svg(svgFilePath);
+    } else {
+        d->iconSvg->setFile(svgFilePath);
     }
 
+    d->iconSvgElement = elementId;
+}
+
+void Icon::Private::drawBackground(QPainter *painter, IconState state)
+{
+    if (!drawBg) {
+        return;
+    }
+
+    bool darkShadow = shadowColor.value() < 128;
+    QColor shadow = shadowColor;
+
+    shadow.setAlphaF(.35);
+
     switch (state) {
-        case Private::NoState:
-            break;
         case Private::HoverState:
-            if (svgElements & Private::SvgBackgroundHover) {
-                element = "background-hover";
-            }
+            shadow.setHsv(shadow.hue(),
+                          shadow.saturation(),
+                          shadow.value()+(darkShadow?50:-50),
+                          100);
             break;
         case Private::PressedState:
-            if (svgElements & Private::SvgBackgroundPressed) {
-                element = "background-pressed";
-            } else if (svgElements & Private::SvgBackgroundHover) {
-                element = "background-hover";
-            }
+            shadow.setHsv(shadow.hue(),
+                          shadow.saturation(),
+                          shadow.value()+(darkShadow?100:-100),
+                          128);
+            break;
+        default:
             break;
     }
 
-    if (!element.isEmpty()) {
-        svg.paint(painter, 0, 0, element);
-    }
+    painter->save();
+    painter->setRenderHint(QPainter::Antialiasing);
+    painter->setBrush(shadow);
+    painter->setPen(QPen(shadow, 1.0));
+    painter->drawPath(roundedRectangle(QRectF(QPointF(0.0, 0.0), currentSize), 10.0));
+    painter->restore();
 }
 
-void Icon::Private::drawForeground(QPainter *painter)
-{
-    QString element;
-    if (svgElements & Private::SvgForeground) {
-        element = "foreground";
-    }
-
-    switch (state) {
-        case Private::NoState:
-            break;
-        case Private::HoverState:
-            if (svgElements & Private::SvgForegroundHover) {
-                element = "foreground-hover";
-            }
-            break;
-        case Private::PressedState:
-            if (svgElements & Private::SvgForegroundPressed) {
-                element = "foreground-pressed";
-            } else if (svgElements & Private::SvgForegroundHover) {
-                element = "foreground-hover";
-            }
-            break;
-    }
-
-    if (!element.isEmpty()) {
-        svg.paint(painter, 0, 0, element);
-    }
-}
-
-QPixmap Icon::Private::decoration(const QStyleOptionGraphicsItem *option) const
+QPixmap Icon::Private::decoration(const QStyleOptionGraphicsItem *option, bool useHoverEffect)
 {
     QPixmap result;
 
     QIcon::Mode mode   = option->state & QStyle::State_Enabled ? QIcon::Normal : QIcon::Disabled;
     QIcon::State state = option->state & QStyle::State_Open ? QIcon::On : QIcon::Off;
-    const QSize size = icon.actualSize(iconSize.toSize(), mode, state);
-    result = icon.pixmap(size, mode, state);
 
-    if (!result.isNull())
-    {
-        // Apply the configured hover effect    NOTE: This has no visible effect because we 
-        //                                            do drawForeground() -MB
-        if (option->state & QStyle::State_MouseOver) {
-            KIconEffect *effect = KIconLoader::global()->iconEffect();
-
-            // Note that in KIconLoader terminology, active = hover.
-            // ### We're assuming that the icon group is desktop/filemanager, since this
-            //     is KFileItemDelegate.
-            if (effect->hasEffect(KIconLoader::Desktop, KIconLoader::ActiveState)) {
-                result = effect->apply(result, KIconLoader::Desktop, KIconLoader::ActiveState);
+    if (iconSvg) {
+        if (iconSvgPixmap.size() != iconSize.toSize()) {
+            QImage img(iconSize.toSize(), QImage::Format_ARGB32_Premultiplied);
+            {
+                img.fill(0);
+                QPainter p(&img);
+                iconSvg->resize(iconSize);
+                iconSvg->paint(&p, img.rect(), iconSvgElement);
             }
+            iconSvgPixmap = QPixmap::fromImage(img);
+        }
+        result = iconSvgPixmap;
+    } else {
+        const QSize size = icon.actualSize(iconSize.toSize(), mode, state);
+        result = icon.pixmap(size, mode, state);
+    }
+
+    if (!result.isNull() && useHoverEffect) {
+        KIconEffect *effect = KIconLoader::global()->iconEffect();
+
+        // Note that in KIconLoader terminology, active = hover.
+        // ### We're assuming that the icon group is desktop/filemanager, since this
+        //     is KFileItemDelegate.
+        if (effect->hasEffect(KIconLoader::Desktop, KIconLoader::ActiveState)) {
+            result = effect->apply(result, KIconLoader::Desktop, KIconLoader::ActiveState);
         }
     }
 
@@ -546,11 +576,24 @@ QPixmap Icon::Private::decoration(const QStyleOptionGraphicsItem *option) const
 QPointF Icon::Private::iconPosition(const QStyleOptionGraphicsItem *option, const QPixmap &pixmap) const
 {
     const QRectF itemRect = subtractMargin(option->rect, Private::ItemMargin);
-    Qt::Alignment alignment = Qt::AlignHCenter | Qt::AlignTop;  // NOTE: This assumes always top-center -MB
 
     // Compute the nominal decoration rectangle
     const QSizeF size = addMargin(iconSize, Private::IconMargin);
-    const QRect iconRect = QStyle::alignedRect(option->direction, alignment, size.toSize(), itemRect.toRect());
+
+    Qt::LayoutDirection direction = iconDirection(option);
+
+    //alignment depends from orientation and option->direction
+    Qt::Alignment alignment;
+    if (text.isEmpty() && infoText.isEmpty()) {
+        alignment = Qt::AlignCenter;
+    }else if (orientation == Qt::Vertical) {
+        alignment = Qt::Alignment(Qt::AlignHCenter | Qt::AlignTop);
+    //Horizontal
+    }else{
+        alignment = QStyle::visualAlignment(direction, Qt::Alignment(Qt::AlignLeft | Qt::AlignVCenter));
+    }
+
+    const QRect iconRect = QStyle::alignedRect(direction, alignment, size.toSize(), itemRect.toRect());
 
     // Position the pixmap in the center of the rectangle
     QRect pixmapRect = pixmap.rect();
@@ -567,17 +610,26 @@ QRectF Icon::Private::labelRectangle(const QStyleOptionGraphicsItem *option, con
 {
     Q_UNUSED(string)
 
-    if (icon.isNull())
+    if (icon.isNull()) {
         return option->rect;
+    }
 
     const QSizeF decoSize = addMargin(iconSize, Private::IconMargin); 
 
     const QRectF itemRect = subtractMargin(option->rect, Private::ItemMargin);
     QRectF textArea(QPointF(0, 0), itemRect.size());
 
-    textArea.setTop(decoSize.height() + 1);
+
+    if (orientation == Qt::Vertical) {
+        textArea.setTop(decoSize.height() + 1);
+    //Horizontal
+    }else{
+       textArea.setLeft(decoSize.width() + 1);
+    }
+
     textArea.translate(itemRect.topLeft());
-    return QRectF(QStyle::visualRect(option->direction, option->rect, textArea.toRect()));
+
+    return QRectF(QStyle::visualRect(iconDirection(option), option->rect, textArea.toRect()));
 }
 
 // Lays the text out in a rectangle no larger than constraints, eliding it as necessary
@@ -636,10 +688,6 @@ QString Icon::Private::elidedText(QTextLayout &layout, const QStyleOptionGraphic
     qreal maxHeight      = size.height();
     qreal height         = 0;
 
-    // If the string contains a single line of text it shouldn't be word wrapped
-    if (text.indexOf(QChar::LineSeparator) == -1)
-        return metrics.elidedText(text, Qt::ElideRight, maxWidth);
-
     // Elide each line that has already been laid out in the layout.
     QString elided;
     elided.reserve(text.length());
@@ -655,7 +703,7 @@ QString Icon::Private::elidedText(QTextLayout &layout, const QStyleOptionGraphic
         {
             // Unfortunately, if the line ends because of a line separator, elidedText() will be too
             // clever and keep adding lines until it finds one that's too wide.
-            if (line.naturalTextWidth() < maxWidth && text[start + length - 1] == QChar::LineSeparator)
+            if (line.naturalTextWidth() < maxWidth && start+length > 0 && text[start + length - 1] == QChar::LineSeparator)
                 elided += text.mid(start, length - 1);
             else
                 elided += metrics.elidedText(text.mid(start), Qt::ElideRight, maxWidth);
@@ -714,7 +762,7 @@ void Icon::Private::layoutTextItems(const QStyleOptionGraphicsItem *option,
     // Compute the bounding rect of the text
     const Qt::Alignment alignment = labelLayout->textOption().alignment();
     const QSizeF size(qMax(labelSize.width(), infoSize.width()), labelSize.height() + infoSize.height());
-    *textBoundingRect = QStyle::alignedRect(option->direction, alignment, size.toSize(), textRect.toRect());
+    *textBoundingRect = QStyle::alignedRect(iconDirection(option), alignment, size.toSize(), textRect.toRect());
 
     // Compute the positions where we should draw the layouts
     labelLayout->setPosition(QPointF(textRect.x(), textBoundingRect->y()));
@@ -750,21 +798,13 @@ QBrush Icon::Private::backgroundBrush(const QStyleOptionGraphicsItem *option) co
 void Icon::Private::drawTextItems(QPainter *painter, const QStyleOptionGraphicsItem *option,
                                   const QTextLayout &labelLayout, const QTextLayout &infoLayout) const
 {
-    QPen pen(foregroundBrush(option), 0);
-    painter->setPen(pen);
+    Q_UNUSED(option)
+
+    painter->setPen(textColor);
     labelLayout.draw(painter, QPointF());
 
-    if (!infoLayout.text().isEmpty())
-    {
-        QColor color;
-        if (option->state & QStyle::State_Selected)
-        {
-            color = option->palette.color(QPalette::HighlightedText);
-            color.setAlphaF(.5);
-        } else
-            color = option->palette.color(QPalette::Highlight);
-
-        painter->setPen(color);
+    if (!infoLayout.text().isEmpty()) {
+        painter->setPen(textColor);
         infoLayout.draw(painter, QPointF());
     }
 }
@@ -791,26 +831,34 @@ void Icon::paintWidget(QPainter *painter, const QStyleOptionGraphicsItem *option
      }
 #endif
 
-    calculateSize(option);
-    d->setActiveMargins();
+    //Lay out the main icon and action icons
+    layoutIcons(option);
 
     // Compute the metrics, and lay out the text items
     // ========================================================================
-    QPixmap icon         = d->decoration(option);
+    Private::IconState state = Private::NoState;
+    if (d->states & Private::ManualPressedState) {
+        state = Private::PressedState;
+    } else if (d->states & Private::PressedState) {
+        if (d->states & Private::HoverState) {
+            state = Private::PressedState;
+        }
+    } else if (d->states & Private::HoverState) {
+        state = Private::HoverState;
+    }
+
+    QPixmap icon          = d->decoration(option, state != Private::NoState);
     const QPointF iconPos = d->iconPosition(option, icon);
 
     QTextLayout labelLayout, infoLayout;
     QRectF textBoundingRect;
     d->layoutTextItems(option, icon, &labelLayout, &infoLayout, &textBoundingRect);
 
-    d->svg.resize(d->size);
-    d->drawBackground(painter);
+    d->drawBackground(painter, state);
 
     // draw icon
     painter->drawPixmap(iconPos, icon);
 
-    d->drawForeground(painter);
-
     // Draw corner actions
     foreach (IconAction *action, d->cornerActions) {
         if (action->animationId()) {
@@ -820,39 +868,37 @@ void Icon::paintWidget(QPainter *painter, const QStyleOptionGraphicsItem *option
 
     // Draw text last because its overlayed
     d->drawTextItems(painter, option, labelLayout, infoLayout);
-
 }
 
 void Icon::drawActionButtonBase(QPainter* painter, const QSize &size, int element)
 {
-    // Determine proper svg element
-    QString id;
-    if (d->svgElements & Private::SvgMinibutton) {
-        id = "minibutton";
-    }
+    qreal radius = size.width()/2;
+    QRadialGradient gradient(radius, radius, radius, radius, radius);
+    int alpha;
 
-    if (element == Private::SvgMinibuttonPressed) {
-        if (d->svgElements & Private::SvgMinibuttonPressed) {
-            id = "minibutton-pressed";
-        } else if (d->svgElements & Private::SvgMinibuttonHover) {
-            id = "minibutton-hover";
-        }
-    } else if (element == Icon::Private::SvgMinibuttonHover &&
-               d->svgElements & Private::SvgMinibuttonHover) {
-        id = "minibutton-hover";
+    if (element == Private::MinibuttonPressed) {
+        alpha = 255;
+    } else if (element == Private::MinibuttonHover) {
+        alpha = 200;
+    } else {
+        alpha = 160;
     }
+    gradient.setColorAt(0, QColor::fromRgb(d->textColor.red(),
+                                           d->textColor.green(),
+                                           d->textColor.blue(), alpha));
+    gradient.setColorAt(1, QColor::fromRgb(d->textColor.red(),
+                                           d->textColor.green(),
+                                           d->textColor.blue(), 0));
 
-    if (!id.isEmpty()) {
-        d->svg.resize(size);
-        d->svg.paint(painter, 0, 0, id);
-    }
+    painter->setBrush(gradient);
+    painter->setPen(Qt::NoPen);
+    painter->drawEllipse(QRectF(QPointF(.0, .0), size));
 }
 
 
 void Icon::setText(const QString& text)
 {
     d->text = text;
-    calculateSize();
 }
 
 QString Icon::text() const
@@ -863,7 +909,6 @@ QString Icon::text() const
 void Icon::setInfoText(const QString& text)
 {
     d->infoText = text;
-    calculateSize();
 }
 
 QString Icon::infoText() const
@@ -889,7 +934,6 @@ void Icon::setIcon(const QString& icon)
 void Icon::setIcon(const QIcon& icon)
 {
     d->icon = icon;
-    calculateSize();
 }
 
 QSizeF Icon::iconSize() const
@@ -897,62 +941,86 @@ QSizeF Icon::iconSize() const
     return d->iconSize;
 }
 
-void Icon::setIconSize(const QSizeF& s)
-{
-    d->iconSize = s;
-    calculateSize();
-}
-
-void Icon::setIconSize(int w, int h)
-{
-    setIconSize(QSizeF(w, h));
-}
-
 bool Icon::isDown()
 {
-    return d->state == Private::PressedState;
+    return d->states & Private::PressedState;
 }
 
 void Icon::mousePressEvent(QGraphicsSceneMouseEvent *event)
 {
-    foreach (IconAction *action, d->cornerActions) {
-        action->event(event->type(), event->pos());
+    if (event->button() != Qt::LeftButton) {
+        Widget::mousePressEvent(event);
+        return;
     }
 
-    d->state = Private::PressedState;
+    d->states |= Private::PressedState;
+
+    bool handled = false;
+    foreach (IconAction *action, d->cornerActions) {
+        handled = action->event(event->type(), event->pos());
+        if (handled) {
+            break;
+        }
+    }
+
+    if (!handled) {
+        emit pressed(true);
+    }
 
-    QGraphicsItem::mousePressEvent(event);
     update();
 }
 
+void Icon::mouseMoveEvent(QGraphicsSceneMouseEvent *event)
+{
+    if (~d->states & Private::PressedState) {
+        Widget::mouseMoveEvent(event);
+        return;
+    }
+
+    if (boundingRect().contains(event->pos())) {
+        if (~d->states & Private::HoverState) {
+            d->states |= Private::HoverState;
+            update();
+        }
+    } else {
+        if (d->states & Private::HoverState) {
+            d->states &= ~Private::HoverState;
+            update();
+        }
+    }
+    
+    d->states |= Private::MovedState;
+}
+
 void Icon::mouseReleaseEvent(QGraphicsSceneMouseEvent *event)
 {
-    bool inside = boundingRect().contains(event->pos());
-    Private::ButtonState was = d->state;
+    if (~d->states & Private::PressedState) {
+        Widget::mouseMoveEvent(event);
+        return;
+    }
 
-    if (inside) {
-        d->state = Private::HoverState;
+    d->states &= ~Private::PressedState;
 
-        foreach (IconAction *action, d->cornerActions) {
-            if (action->event(event->type(), event->pos())) {
-                inside = false;
-                break;
-            }
+    bool handled = false;
+    foreach (IconAction *action, d->cornerActions) {
+        if (action->event(event->type(), event->pos())) {
+            handled = true;
+            break;
         }
     }
-    else if (was == Private::PressedState) {
-        emit pressed(false);
 
-        if (inside) {
+    if (d->states & Private::MovedState) {
+        d->states &= ~Private::MovedState;
+        handled = true;
+    }
+
+    if (!handled) {
+        if (boundingRect().contains(event->pos())) {
             emit clicked();
         }
-        d->state = Private::NoState;
-    }
-    else {
-        d->state = Private::NoState;
+        emit pressed(false);
     }
 
-    QGraphicsItem::mouseReleaseEvent(event);
     update();
 }
 
@@ -963,8 +1031,10 @@ void Icon::hoverEnterEvent(QGraphicsSceneHoverEvent *event)
         action->event(event->type(), event->pos());
     }
 
-    d->state = Private::HoverState;
-    QGraphicsItem::hoverEnterEvent(event);
+    d->states |= Private::HoverState;
+    update();
+
+    Widget::hoverEnterEvent(event);
 }
 
 void Icon::hoverLeaveEvent(QGraphicsSceneHoverEvent *event)
@@ -974,83 +1044,96 @@ void Icon::hoverLeaveEvent(QGraphicsSceneHoverEvent *event)
         action->event(event->type(), event->pos());
     }
 
-    d->state = Private::NoState;
-    QGraphicsItem::hoverLeaveEvent(event);
+    d->states &= ~Private::HoverState;
+    update();
+
+    Widget::hoverLeaveEvent(event);
 }
 
-void Icon::mouseMoveEvent(QGraphicsSceneMouseEvent *event)
+void Icon::setPressed(bool pressed)
 {
-    if (d->state == Private::PressedState) {
-        d->state = Private::HoverState;
+    if (pressed) {
+        d->states |= Private::ManualPressedState;
+    } else {
+        d->states &= ~Private::ManualPressedState;
+    }
+    update();
+}
+
+void Icon::setUnpressed()
+{
+    setPressed(false);
+}
+
+void Icon::setOrientation(Qt::Orientation orientation)
+{
+    d->orientation = orientation;
+}
+
+void Icon::invertLayout(bool invert)
+{
+    d->invertLayout = invert;
+}
+
+bool Icon::invertedLayout() const
+{
+    return d->invertLayout;
+}
+
+QSizeF Icon::sizeFromIconSize(const qreal iconWidth) const
+{
+    //no text, less calculations
+    if (d->text.isEmpty() && d->infoText.isEmpty()) {
+        return d->addMargin(d->addMargin(QSizeF(iconWidth, iconWidth),
+                                         Private::IconMargin),
+                            Private::ItemMargin);
     }
 
-    QGraphicsItem::mouseMoveEvent(event);
-}
+    QFontMetricsF fm(font());
+    //make room for at most 14 characters
+    qreal width = qMax(fm.width(d->text.left(12)),
+                       fm.width(d->infoText.left(12))) +
+                  fm.width("xx") +
+                  d->horizontalMargin[Private::TextMargin].left +
+                  d->horizontalMargin[Private::TextMargin].right;
 
-QSizeF Icon::sizeHint() const
-{
-    return d->size;
-}
+    if (d->orientation == Qt::Vertical) {
+        width = qMax(width,
+                     iconWidth +
+                     d->horizontalMargin[Private::IconMargin].left +
+                     d->horizontalMargin[Private::IconMargin].right);
+    }
 
-Qt::Orientations Icon::expandingDirections() const
-{
-    return 0;
-}
+    qreal height;
+    qreal textHeight;
 
-/*
-QSizeF Icon::sizeHint() const
-{
-    return d->size;
-}
+    QStyleOptionGraphicsItem option;
+    option.state = QStyle::State_None;
+    option.rect = boundingRect().toRect();
+    textHeight = d->displaySizeHint(&option, width).height();
 
-QSizeF Icon::minimumSize() const
-{
-    return d->size; // probably should be more flexible on this =)
-}
+    if (d->orientation == Qt::Vertical) {
+        height = iconWidth + textHeight +
+                 d->verticalMargin[Private::TextMargin].top +
+                 d->verticalMargin[Private::TextMargin].bottom +
+                 d->verticalMargin[Private::IconMargin].top +
+                 d->verticalMargin[Private::IconMargin].bottom;
+    //Horizontal
+    }else{
+        height = qMax(iconWidth +
+                      d->verticalMargin[Private::IconMargin].top +
+                      d->verticalMargin[Private::IconMargin].bottom,
+                      textHeight +
+                      d->verticalMargin[Private::TextMargin].top +
+                      d->verticalMargin[Private::IconMargin].bottom);
+        width = width + iconWidth +
+                d->horizontalMargin[Private::IconMargin].left +
+                d->horizontalMargin[Private::IconMargin].right;
+    }
 
-QSizeF Icon::maximumSize() const
-{
-    return d->size;
+    return d->addMargin(QSizeF(width, height), Private::ItemMargin);
 }
 
-Qt::Orientations Icon::expandingDirections() const
-{
-    return Qt::Horizontal;
-}
-
-bool Icon::hasHeightForWidth() const
-{
-    return true;
-}
-
-qreal Icon::heightForWidth(qreal w) const
-{
-    return w; //FIXME: we shouldn't assume squareness but actually calculate based on text and what not
-}
-
-bool Icon::hasWidthForHeight() const
-{
-    return true;
-}
-
-qreal Icon::widthForHeight(qreal h) const
-{
-    return h; //FIXME: we shouldn't assume squareness but actually calculate based on text and what not
-}
-
-QRectF Icon::geometry() const
-{
-    return boundingRect();
-}
-
-void Icon::setGeometry(const QRectF &r)
-{
-    // TODO: this is wrong, but we should probably never call setGeometry anyway!
-    setIconSize(r.size());
-    setPos(r.x(),r.y());
-}
-*/
-
 } // namespace Plasma
 
 #include "icon.moc"
