@@ -51,7 +51,8 @@ class PLASMA_EXPORT Icon : public Plasma::Widget
     Q_PROPERTY( QString text READ text WRITE setText )
     Q_PROPERTY( QString infoText READ infoText WRITE setInfoText )
     Q_PROPERTY( QIcon icon READ icon WRITE setIcon )
-    Q_PROPERTY( QSizeF iconSize READ iconSize WRITE setIconSize )
+    Q_PROPERTY( QSizeF iconSize READ iconSize )
+    Q_PROPERTY( QString svg WRITE setSvg )
 public:
     /**
     * Creates a new Plasma::Icon.
@@ -90,6 +91,13 @@ public:
     */
     void setText(const QString &text);
 
+    /**
+    * Convenience method to set the svg image to use when given the filepath and name of svg.
+    * @param svgFilePath the svg filepath including name of the svg.
+    * @param svgIconElement the svg element to use when displaying the svg. Defaults to all of them.
+    */
+    void setSvg(const QString &svgFilePath, const QString &svgIconElement = QString());
+
     /**
     * Returns the meta text associated with this icon.
     */
@@ -125,19 +133,6 @@ public:
     */
     QSizeF iconSize() const;
 
-    /**
-    * Sets the size of the graphical icon for this Plasma::Icon.
-    * @param size the size of the icon.
-    */
-    void setIconSize(const QSizeF& size);
-
-    /**
-    * Convenience method to set the icon size without a QSizeF.
-    * @param height the height of the icon.
-    * @param width the width of the icon.
-    */
-    Q_INVOKABLE void setIconSize(int height, int width);
-
     /**
     * Plasma::Icon allows the user to specify a number of actions
     * (current four) to be displayed around the widget. This method
@@ -147,27 +142,72 @@ public:
     */
     void addAction(QAction* action);
 
-    // Layout stuff - no need to document
-/*
-    Qt::Orientations expandingDirections() const;
+    /**
+    * let set the orientation of the icon
+    * Qt::Vertical: text under the icon
+    * Qt::Horizontal text at a side of the icon depending
+    * by the direction of the language
+    * @param orientation the orientation we want
+    */
+    void setOrientation(Qt::Orientation orientation);
 
-    QSizeF minimumSize() const;
-    QSizeF maximumSize() const;
+    /**
+    * inverts the layout of the icons if the orientation is horizontal,
+    * normally we get icon on the left with left-to-right languages
+    * @param invert if we want to invert the layout of icons
+    */
+    void invertLayout(bool invert);
 
-    bool hasHeightForWidth() const;
-    qreal heightForWidth(qreal w) const;
+    /**
+    * @return if the layout of the icons should appear inverted or not
+    */
+    bool invertedLayout() const;
 
-    bool hasWidthForHeight() const;
-    qreal widthForHeight(qreal h) const;
+    /**
+    * @return optimal size given a size for the icon
+    * @param  iconWidth desidered width of the icon
+    */
+    QSizeF sizeFromIconSize(const qreal iconWidth) const;
 
-    QRectF geometry() const;
-    void setGeometry(const QRectF& r);
+    /**
+    * @return the number of lines allowed to display
+    */
+    int numDisplayLines();
 
+    /**
+    * @param numLines the number of lines to show in the display.
+    */
+    void setNumDisplayLines(int numLines);
+
+    /**
+    * @return the recommended size of the widget
+    */
     QSizeF sizeHint() const;
-*/
-    Qt::Orientations expandingDirections() const;
-    QSizeF sizeHint() const;
-    QRectF boundingRect() const;
+
+    /**
+     * Sets whether or not to draw a background area for the icon
+     *
+     * @arg draw true if a background should be drawn or not
+     */
+    void setDrawBackground(bool draw);
+
+    /**
+     * @return true if a background area is to be drawn for the icon
+     */
+    bool drawBackground() const;
+
+public Q_SLOTS:
+    /**
+    * Sets the appearance of the icon to pressed or restores the appearance
+    * to normal. This does not simulate a mouse button press.
+    * @param pressed whether to appear as pressed (true) or as normal (false)
+    */
+    void setPressed(bool pressed = true);
+
+    /**
+    * Shortcut for setPressed(false)
+    */
+    void setUnpressed();
 
 protected:
     void paintWidget(QPainter *painter, const QStyleOptionGraphicsItem *option, QWidget *widget = 0);
@@ -202,14 +242,12 @@ public:
 
 private:
     void init();
-    void calculateSize();
-    void calculateSize(const QStyleOptionGraphicsItem *option);
+    void layoutIcons(const QStyleOptionGraphicsItem *option);
 
     Private * const d;
 
 private Q_SLOTS:
     void actionDestroyed(QObject* obj);
-
 };
 
 } // namespace Plasma
