@@ -6,7 +6,6 @@
  *   it under the terms of the GNU Library General Public License as
  *   published by the Free Software Foundation; either version 2, or
  *   (at your option) any later version.
-
  *
  *   This program is distributed in the hope that it will be useful,
  *   but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -26,16 +25,20 @@
 #include <limits>
 
 #include <QApplication>
+#include <QGraphicsScene>
+#include <QGraphicsSceneHoverEvent>
+#include <QGraphicsView>
+#include <QHelpEvent>
 #include <QList>
 #include <QPainter>
 #include <QPixmapCache>
 #include <QStyleOptionGraphicsItem>
-#include <QGraphicsSceneContextMenuEvent>
 
 #include <KDebug>
 
-#include "layout.h"
+#include "layouts/freelayout.h"
 #include "plasma/plasma.h"
+#include "tooltip_p.h"
 
 namespace Plasma
 {
@@ -44,12 +47,12 @@ class Widget::Private
 {
     public:
         Private()
-            : minimumSize(0,0)
-            , maximumSize(std::numeric_limits<qreal>::infinity(),
-                          std::numeric_limits<qreal>::infinity())
-            , parent(0)
-            , opacity(1.0)
-            , cachePaintMode(Widget::NoCacheMode)
+            : minimumSize(0,0),
+              maximumSize(std::numeric_limits<qreal>::infinity(),
+                          std::numeric_limits<qreal>::infinity()),
+              opacity(1.0),
+              cachePaintMode(Widget::NoCacheMode),
+              wasMovable(false)
         { }
         ~Private() { }
 
@@ -57,9 +60,6 @@ class Widget::Private
         QSizeF minimumSize;
         QSizeF maximumSize;
 
-        Widget *parent;
-        QList<Widget *> childList;
-
         qreal opacity;
 
         // Replace with CacheMode in 4.4
@@ -71,7 +71,10 @@ class Widget::Private
         QString cacheKey;
         QRectF cacheInvalidated;
 
+        bool wasMovable;
+
         bool shouldPaint(QPainter *painter, const QTransform &transform);
+        ToolTipData toolTip;
 };
 
 QGraphicsItem* Widget::graphicsItem()
@@ -79,6 +82,38 @@ QGraphicsItem* Widget::graphicsItem()
     return this;
 }
 
+QGraphicsView *Widget::view() const
+{
+    // It's assumed that we won't be visible on more than one view here.
+    // Anything that actually needs view() should only really care about
+    // one of them anyway though.
+    if (!scene()) {
+        return 0;
+    }
+
+    foreach (QGraphicsView *view, scene()->views()) {
+        if (view->sceneRect().intersects(sceneBoundingRect()) ||
+                view->sceneRect().contains(scenePos())) {
+            return view;
+        }
+    }
+    return 0;
+}
+
+QRectF Widget::mapFromView(const QGraphicsView *view, const QRect &rect) const
+{
+    // TODO: Confirm that adjusted() is needed and is not covering for some
+    // issue elsewhere
+    return mapFromScene(view->mapToScene(rect)).boundingRect().adjusted(0, 0, 1, 1);
+}
+
+QRect Widget::mapToView(const QGraphicsView *view, const QRectF &rect) const
+{
+    // TODO: Confirm that adjusted() is needed and is not covering for some
+    // issue elsewhere
+    return view->mapFromScene(mapToScene(rect)).boundingRect().adjusted(0, 0, -1, -1);
+}
+
 bool Widget::Private::shouldPaint(QPainter *painter, const QTransform &transform)
 {
     Q_UNUSED(painter)
@@ -88,7 +123,7 @@ bool Widget::Private::shouldPaint(QPainter *painter, const QTransform &transform
     return true;
 }
 
-Widget::Widget(QGraphicsItem *parent,QObject* parentObject)
+Widget::Widget(QGraphicsItem *parent, QObject* parentObject)
   : QObject(parentObject),
     QGraphicsItem(parent),
     d(new Private)
@@ -96,16 +131,18 @@ Widget::Widget(QGraphicsItem *parent,QObject* parentObject)
     setFlag(QGraphicsItem::ItemClipsToShape, true);
     setFlag(QGraphicsItem::ItemClipsChildrenToShape, true);
     setCachePaintMode(DeviceCoordinateCacheMode);
-    d->parent = dynamic_cast<Widget *>(parent);
 
-    if (d->parent) {
-        d->parent->addChild(this);
-        d->parent->updateGeometry();
+    Widget *w = dynamic_cast<Widget *>(parent);
+    if (w) {
+        w->addChild(this);
     }
 }
 
 Widget::~Widget()
 {
+    if (ToolTip::instance()->currentWidget() == this) {
+        ToolTip::instance()->hide();
+    }
     delete d;
 }
 
@@ -113,6 +150,7 @@ void Widget::setOpacity(qreal opacity)
 {
     d->opacity = opacity;
 }
+
 qreal Widget::opacity() const
 {
     return d->opacity;
@@ -153,6 +191,10 @@ Qt::Orientations Widget::expandingDirections() const
 void Widget::setMinimumSize(const QSizeF& size)
 {
     d->minimumSize = size;
+    if (d->size != d->size.expandedTo(size)) {
+        d->size = d->size.expandedTo(size);
+        updateGeometry();
+    }
 }
 
 QSizeF Widget::minimumSize() const
@@ -163,6 +205,10 @@ QSizeF Widget::minimumSize() const
 void Widget::setMaximumSize(const QSizeF& size)
 {
     d->maximumSize = size;
+    if (d->size != d->size.boundedTo(size)) {
+        d->size = d->size.boundedTo(size);
+        updateGeometry();
+    }
 }
 
 QSizeF Widget::maximumSize() const
@@ -208,7 +254,7 @@ QRectF Widget::localGeometry() const
 
 void Widget::setGeometry(const QRectF& geometry)
 {
-    if (d->size != geometry.size()) {
+    if (geometry.size().width() > 0 && geometry.size().height() > 0 && d->size != geometry.size()) {
         prepareGeometryChange();
         qreal width = qBound(d->minimumSize.width(), geometry.size().width(), d->maximumSize.width());
         qreal height = qBound(d->minimumSize.height(), geometry.size().height(), d->maximumSize.height());
@@ -216,7 +262,11 @@ void Widget::setGeometry(const QRectF& geometry)
         d->size = QSizeF(width, height);
 
         if (layout()) {
-            layout()->setGeometry(boundingRect());
+            layout()->setGeometry(QRectF(QPointF(0, 0), d->size));
+        }
+
+        if (managingLayout()) {
+            managingLayout()->invalidate();
         }
     }
 
@@ -224,6 +274,11 @@ void Widget::setGeometry(const QRectF& geometry)
     update();
 }
 
+void Widget::setSize(qreal width, qreal height)
+{
+    setSize(QSizeF(width, height));
+}
+
 void Widget::setSize(const QSizeF& size)
 {
     d->size = size;
@@ -233,6 +288,8 @@ void Widget::updateGeometry()
 {
     if (managingLayout()) {
         managingLayout()->invalidate();
+    } else {
+        setGeometry(QRectF(pos(), sizeHint()));
     }
 }
 
@@ -241,13 +298,13 @@ QSizeF Widget::sizeHint() const
     if (layout()) {
         return layout()->sizeHint();
     } else {
-        return QSizeF();
+        return d->size;
     }
 }
 
 QSizeF Widget::size() const
 {
-    return geometry().size();
+    return d->size;
 }
 
 QFont Widget::font() const
@@ -268,6 +325,7 @@ QRectF Widget::boundingRect() const
 
 void Widget::resize(const QSizeF& size)
 {
+    //FIXME: we need to respect minimum size here i think
     setGeometry(QRectF(pos(), size));
 }
 
@@ -278,33 +336,50 @@ void Widget::resize(qreal w, qreal h)
 
 Widget *Widget::parent() const
 {
-    return d->parent;
+    QGraphicsItem *parent = parentItem();
+
+    while (parent) {
+        Widget *parentWidget = dynamic_cast<Widget *>(parent);
+
+        if (parentWidget) {
+            return parentWidget;
+        }
+
+        parent = parent->parentItem();
+    }
+    return 0;
 }
 
 void Widget::addChild(Widget *w)
 {
-    if (!w) {
+    if (!w || QGraphicsItem::children().contains(w)) {
         return;
     }
 
-    w->reparent(this);
-    d->childList.append(w);
+    w->setParentItem(this);
 
-    qDebug("Added Child Widget : %p", (void*)w);
+    //kDebug() << "Added Child Widget" <<  (QObject*)w << "our geom is" << geometry();
 
     if (layout()) {
         layout()->addItem(w);
-        updateGeometry();
     }
+
+    updateGeometry();
+    //kDebug() << "after the item is added our geom is now" << geometry();
 }
 
 void Widget::paint(QPainter *painter, const QStyleOptionGraphicsItem *option, QWidget *widget)
 {
-    painter->setOpacity(d->opacity);
+    if (d->opacity < 1.0) {
+        painter->setOpacity(painter->opacity() * d->opacity);
+    }
 
+    /*
+NOTE: put this back if we end up needing to control when things paint due to, e.g. zooming.
     if (!d->shouldPaint(painter, transform())) {
         return;
     }
+    */
 
     if (d->cachePaintMode == NoCacheMode) {
         paintWidget(painter, option, widget);
@@ -313,7 +388,6 @@ void Widget::paint(QPainter *painter, const QStyleOptionGraphicsItem *option, QW
 
     // Cached painting
     QRectF brect = boundingRect();
-    QRect boundingRectInt = brect.toRect();
 
     // Fetch the off-screen transparent buffer and exposed area info.
     QPixmap pix;
@@ -325,7 +399,7 @@ void Widget::paint(QPainter *painter, const QStyleOptionGraphicsItem *option, QW
         // Recreate the pixmap if it's gone.
         if (pix.isNull()) {
             pix = QPixmap(d->cacheSize);
- 	    pix.fill(Qt::transparent);
+            pix.fill(Qt::transparent);
             exposed = brect;
         }
 
@@ -370,10 +444,16 @@ void Widget::paint(QPainter *painter, const QStyleOptionGraphicsItem *option, QW
         QTransform transform = painter->worldTransform();
         QRect deviceRect = transform.mapRect(brect).toRect();
 
+        if (deviceRect.width() < 1 ||
+            deviceRect.height() < 1) {
+            // we have nothing to paint!
+            return;
+        }
+
         // Auto-adjust the pixmap size.
         if (deviceRect.size() != pix.size()) {
             pix = QPixmap(deviceRect.size());
-	    pix.fill(Qt::transparent);
+            pix.fill(Qt::transparent);
             exposed = brect;
         }
 
@@ -415,6 +495,21 @@ void Widget::paint(QPainter *painter, const QStyleOptionGraphicsItem *option, QW
     }
 }
 
+ToolTipData Widget::toolTip() const
+{
+    return d->toolTip;
+}
+
+void Widget::setToolTip(const ToolTipData &tip)
+{
+    d->toolTip = tip;
+    if (ToolTip::instance()->currentWidget() == this) {
+        QPoint viewPos = view()->mapFromScene(scenePos());
+        QPoint globalPos = view()->mapToGlobal(viewPos);
+        ToolTip::instance()->show(globalPos, this);
+    }
+}
+
 void Widget::paintWidget(QPainter *painter, const QStyleOptionGraphicsItem *option, QWidget *widget)
 {
     Q_UNUSED(painter);
@@ -424,11 +519,83 @@ void Widget::paintWidget(QPainter *painter, const QStyleOptionGraphicsItem *opti
     // Replaced by widget's own function
 }
 
-void Widget::reparent(Widget *w)
+QVariant Widget::itemChange(GraphicsItemChange change, const QVariant &value)
 {
-    d->parent = w;
-    setParentItem(w);
-    update();
+    if (change == QGraphicsItem::ItemChildRemovedChange) {
+        if (layout()) {
+            layout()->removeItem(dynamic_cast<Plasma::LayoutItem*>(value.value<QGraphicsItem*>()));
+            updateGeometry();
+        }
+    }
+
+    return QGraphicsItem::itemChange(change, value);
+}
+
+void Widget::managingLayoutChanged()
+{
+    if (managingLayout()) {
+        d->wasMovable = flags() & ItemIsMovable;
+        if (!dynamic_cast<FreeLayout*>(managingLayout())) {
+            setFlag(ItemIsMovable, false);
+        }
+    } else {
+        setFlag(ItemIsMovable, d->wasMovable);
+    }
+}
+
+void Widget::contextMenuEvent(QGraphicsSceneContextMenuEvent *event)
+{
+    // HACK: QGraphicsItem's documentation says that the event will be passed
+    // to the parent if it's not handled, but it isn't passed. This can be
+    // removed when Qt4.4 becomes a requirement. See Qt bug #176902.
+    Widget *parentWidget = parent();
+    if (parentWidget) {
+        parentWidget->contextMenuEvent(event);
+    }
+}
+
+bool Widget::sceneEvent(QEvent *event)
+{
+    switch (event->type()) {
+    case QEvent::GraphicsSceneHoverMove:
+        // If the tooltip isn't visible, run through showing the tooltip again
+        // so that it only becomes visible after a stationary hover
+        if (ToolTip::instance()->isVisible()) {
+            break;
+        }
+
+    case QEvent::GraphicsSceneHoverEnter:
+    {
+        // Check that there is a tooltip to show
+        if (d->toolTip.image.isNull() &&
+                d->toolTip.subText.isEmpty() &&
+                d->toolTip.mainText.isEmpty()) {
+            break;
+        }
+
+        // If the mouse is in the widget's area at the time that it is being
+        // created the widget can receive a hover event before it is fully
+        // initialized, in which case view() will return 0.
+        QGraphicsView *parentView = view();
+        if (parentView) {
+            QPoint viewPos = parentView->mapFromScene(scenePos());
+            QPoint globalPos = parentView->mapToGlobal(viewPos);
+            ToolTip::instance()->show(globalPos, this);
+        }
+
+        break;
+    }
+
+    case QEvent::GraphicsSceneHoverLeave:
+    case QEvent::GraphicsSceneMousePress:
+    case QEvent::GraphicsSceneWheel:
+        ToolTip::instance()->hide();
+
+    default:
+        break;
+    }
+
+    return QGraphicsItem::sceneEvent(event);
 }
 
 } // Plasma namespace
