@@ -38,10 +38,15 @@ public:
 
     void paintInterface(QPainter *painter,
                         const QStyleOptionGraphicsItem *option,
-                        const QRect& contentsRect);
-
+                        const QRect &contentsRect);
+    void paintBackground(QPainter *painter, const QRect &contentsRect);
 private:
     Plasma::Svg *m_background;
+    QPixmap* m_cachedBackground;
+    bool m_drawTop : 1;
+    bool m_drawLeft : 1;
+    bool m_drawRight : 1;
+    bool m_drawBottom : 1;
 };
 
 
