@@ -20,9 +20,8 @@
 #ifndef DESKTOPVIEW_H
 #define DESKTOPVIEW_H
 
-#include <QGraphicsView>
-
 #include "plasma/plasma.h"
+#include "plasma/view.h"
 
 class QPixmap;
 
@@ -30,28 +29,34 @@ namespace Plasma
 {
     class Svg;
     class Corona;
+    class AppletBrowser;
 }
 
-class DesktopView : public QGraphicsView
+class DashboardView;
+
+class DesktopView : public Plasma::View
 {
     Q_OBJECT
 
 public:
-    DesktopView(QWidget *parent, int screen = -1);
+    DesktopView(int screen, QWidget *parent);
     ~DesktopView();
 
 public slots:
     void zoomIn();
     void zoomOut();
+    void showAppletBrowser();
+    void appletBrowserDestroyed();
+    void toggleDashboard();
+    void adjustSize();
 
 protected:
     void wheelEvent(QWheelEvent *event);
 
 private:
-    QAction *m_zoomInAction;
-    QAction *m_zoomOutAction;
-    int m_screen;
     Plasma::ZoomLevel m_zoomLevel;
+    Plasma::AppletBrowser *m_appletBrowser;
+    DashboardView *m_dashboard;
 };
 
 #endif // multiple inclusion guard
