@@ -25,6 +25,8 @@
 
 #include <unistd.h>
 
+#include <QApplication>
+#include <QDesktopWidget>
 #include <QPixmapCache>
 #include <QTimer>
 #include <QtDBus/QtDBus>
@@ -32,6 +34,7 @@
 #include <KCrash>
 #include <KDebug>
 #include <KCmdLineArgs>
+#include <KWindowSystem>
 
 #include <ksmserver_interface.h>
 
@@ -43,21 +46,70 @@
 #include "desktopview.h"
 #include "panelview.h"
 
+#include <X11/Xlib.h>
+#include <X11/extensions/Xrender.h>
+
+Display* dpy = 0;
+Colormap colormap = 0;
+Visual *visual = 0;
+
+void checkComposite()
+{
+    dpy = XOpenDisplay(0); // open default display
+    if (!dpy) {
+        kError() << "Cannot connect to the X server" << endl;
+        return;
+    }
+
+    int screen = DefaultScreen(dpy);
+    int eventBase, errorBase;
+
+    if (XRenderQueryExtension(dpy, &eventBase, &errorBase)) {
+        int nvi;
+        XVisualInfo templ;
+        templ.screen  = screen;
+        templ.depth   = 32;
+        templ.c_class = TrueColor;
+        XVisualInfo *xvi = XGetVisualInfo(dpy,
+                                          VisualScreenMask | VisualDepthMask | VisualClassMask,
+                                          &templ, &nvi);
+        for (int i = 0; i < nvi; ++i) {
+            XRenderPictFormat *format = XRenderFindVisualFormat(dpy, xvi[i].visual);
+            if (format->type == PictTypeDirect && format->direct.alphaMask) {
+                visual = xvi[i].visual;
+                colormap = XCreateColormap(dpy, RootWindow(dpy, screen), visual, AllocNone);
+                break;
+            }
+        }
+    }
+
+    kDebug() << (colormap ? "Plasma has an argb visual" : "Plasma lacks an argb visual") << visual << colormap;
+    kDebug() << ((KWindowSystem::compositingActive() && colormap) ? "Plasma can use COMPOSITE for effects"
+                                                                    : "Plasma is COMPOSITE-less") << "on" << dpy;
+}
+
 PlasmaApp* PlasmaApp::self()
 {
+    if (!kapp) {
+        checkComposite();
+        return new PlasmaApp(dpy, visual ? Qt::HANDLE(visual) : 0, colormap ? Qt::HANDLE(colormap) : 0);
+    }
+
     return qobject_cast<PlasmaApp*>(kapp);
 }
 
-PlasmaApp::PlasmaApp()
-    : m_root(0),
+PlasmaApp::PlasmaApp(Display* display, Qt::HANDLE visual, Qt::HANDLE colormap)
+    : KUniqueApplication(display, visual, colormap),
+      m_root(0),
       m_corona(0)
 {
+    KGlobal::locale()->insertCatalog("libplasma");
+
     new AppAdaptor(this); 
     QDBusConnection::sessionBus().registerObject("/App", this);
     notifyStartup(false);
 
-
-    // this same pattern is in KRunner (see workspace/krunner/restartingapplication.h)
+    // TODO: this same pattern is in KRunner (see workspace/krunner/restartingapplication.h)
     // would be interesting to see if this could be shared.
     if (!KCrash::crashHandler())
     {
@@ -74,30 +126,54 @@ PlasmaApp::PlasmaApp()
         setCrashHandler();
     }
 
-    // enlarge application pixmap cache
-    // TODO: make this dependand on system
-    // memory and screen resolution. 8MB is ok for caching the background up
-    // to 1600x1200 resolution
-    if (QPixmapCache::cacheLimit() < 8192) {
-        QPixmapCache::setCacheLimit(8192);
+    // Enlarge application pixmap cache
+    
+    // Calculate the size required to hold background pixmaps for all screens.
+    // Add 10% so that other (smaller) pixmaps can also be cached.
+    int cacheSize = 0;
+    QDesktopWidget *desktop = QApplication::desktop();
+    for (int i = 0; i < desktop->numScreens(); i++) {
+        QRect geometry = desktop->screenGeometry(i);
+        cacheSize += 4 * geometry.width() * geometry.height() / 1024;
+    }
+    cacheSize += cacheSize / 10;
+
+    // Calculate the size of physical system memory
+    int memorySize = sysconf(_SC_PHYS_PAGES);
+    memorySize *= sysconf(_SC_PAGESIZE) / 1024;
+
+    // Increase the pixmap cache size to 1% of system memory if it isn't already
+    // larger so as to maximize cache usage. 1% of 1GB ~= 10MB.
+    if (cacheSize < memorySize / 100) {
+        cacheSize = memorySize / 100;
     }
 
-    m_root = new RootWidget();
-    //TODO: the line below is just inane. get rid of it before release. seriously.
-    m_root->setAsDesktop(KCmdLineArgs::parsedArgs()->isSet("desktop"));
+    kDebug() << "Setting the pixmap cache size to" << cacheSize << "kilobytes";
+    QPixmapCache::setCacheLimit(cacheSize);
 
+    m_root = new RootWidget();
+    m_root->setAsDesktop(KCmdLineArgs::parsedArgs()->isSet("desktop"));
     m_root->show();
-    connect(this, SIGNAL(aboutToQuit()), corona(), SLOT(saveApplets()));
 
     createPanels();
-
     notifyStartup(true);
+
+    connect(this, SIGNAL(aboutToQuit()), this, SLOT(cleanup()));
 }
 
 PlasmaApp::~PlasmaApp()
 {
+
+}
+
+void PlasmaApp::cleanup()
+{
+    corona()->saveApplets();
     delete m_root;
+    m_root = 0;
     qDeleteAll(m_panels);
+    m_panels.clear();
+    delete corona();
 }
 
 void PlasmaApp::initializeWallpaper()
@@ -110,6 +186,13 @@ void PlasmaApp::initializeWallpaper()
     //m_root->desktop()->initializeWallpaper();
 }
 
+void PlasmaApp::toggleDashboard()
+{
+    if (m_root) {
+        m_root->toggleDashboard();
+    }
+}
+
 void PlasmaApp::setCrashHandler()
 {
     KCrash::setEmergencySaveFunction(PlasmaApp::crashHandler);
@@ -137,6 +220,11 @@ Plasma::Corona* PlasmaApp::corona()
     return m_corona;
 }
 
+bool PlasmaApp::hasComposite()
+{
+    return colormap && KWindowSystem::compositingActive();
+}
+
 void PlasmaApp::notifyStartup(bool completed)
 {
     org::kde::KSMServerInterface ksmserver("org.kde.ksmserver", "/KSMServer", QDBusConnection::sessionBus());
@@ -153,11 +241,13 @@ void PlasmaApp::createPanels()
 {
     foreach (Plasma::Containment *containment, corona()->containments()) {
         kDebug() << "Containment name:" << containment->name()
+                 << "| type" << containment->containmentType()
                  << "| screen:" << containment->screen()
                  << "| geometry:" << containment->geometry()
                  << "| zValue:" << containment->zValue();
-        if (containment->name() == "Panel") {
-            Plasma::PanelView *panelView = new Plasma::PanelView(containment);
+        if (containment->containmentType() == Plasma::Containment::PanelContainment) {
+            kDebug() << "we have a panel!";
+            PanelView *panelView = new PanelView(containment);
             m_panels << panelView;
             panelView->show();
         }
