@@ -28,7 +28,6 @@ namespace Plasma
 {
     class Corona;
     class Containment;
-    class PanelView;
 } // namespace Plasma
 
 class RootWidget;
@@ -38,10 +37,11 @@ class PlasmaApp : public KUniqueApplication
 {
     Q_OBJECT
 public:
-    PlasmaApp();
     ~PlasmaApp();
 
     static PlasmaApp* self();
+    static bool hasComposite();
+
     void notifyStartup(bool completed);
     Plasma::Corona* corona();
 
@@ -49,17 +49,20 @@ public Q_SLOTS:
     // DBUS interface. if you change these methods, you MUST run:
     // qdbuscpp2xml plasmaapp.h -o org.kde.plasma.App.xml
     void initializeWallpaper();
+    void toggleDashboard();
 
 private Q_SLOTS:
     void setCrashHandler();
+    void cleanup();
 
 private:
+    PlasmaApp(Display* display, Qt::HANDLE visual, Qt::HANDLE colormap);
     static void crashHandler(int signal);
     void createPanels();
 
     RootWidget *m_root;
     Plasma::Corona *m_corona;
-    QList<Plasma::PanelView*> m_panels;
+    QList<PanelView*> m_panels;
 };
 
 #endif // multiple inclusion guard
