@@ -399,7 +399,7 @@ int main(int argc, char **argv)
 
       cout << "  solidshell network set network 'device-uni' 'network-uni' [authentication 'key']" << endl;
       cout << i18n("             # Activate the network 'network-uni' on 'device-uni'.\n"
-                    "             # Optionally, use WEP128, open-system encryption with hex key 'key'. (Hardcoded)"
+                    "             # Optionally, use WEP128, open-system encryption with hex key 'key'. (Hardcoded)\n"
                     "             # Where 'authentication' is one of:\n"
                     "             # wep hex64|ascii64|hex128|ascii128|passphrase64|passphrase128 'key' [open|shared]\n"
                     "             # wpapsk wpa|wpa2 tkip|ccmp-aes password\n"
@@ -976,21 +976,21 @@ bool SolidShell::hwVolumeCall(SolidShell::VolumeCallType type, const QString &ud
     {
     case Mount:
         connect(device.as<Solid::StorageAccess>(),
-                SIGNAL(setupDone(Solid::ErrorType, QVariant)),
+                SIGNAL(setupDone(Solid::ErrorType, QVariant, const QString &)),
                 this,
                 SLOT(slotStorageResult(Solid::ErrorType, QVariant)));
         device.as<Solid::StorageAccess>()->setup();
         break;
     case Unmount:
         connect(device.as<Solid::StorageAccess>(),
-                SIGNAL(teardownDone(Solid::ErrorType, QVariant)),
+                SIGNAL(teardownDone(Solid::ErrorType, QVariant, const QString &)),
                 this,
                 SLOT(slotStorageResult(Solid::ErrorType, QVariant)));
         device.as<Solid::StorageAccess>()->teardown();
         break;
     case Eject:
         connect(device.as<Solid::OpticalDrive>(),
-                SIGNAL(ejectDone(Solid::ErrorType, QVariant)),
+                SIGNAL(ejectDone(Solid::ErrorType, QVariant, const QString &)),
                 this,
                 SLOT(slotStorageResult(Solid::ErrorType, QVariant)));
         device.as<Solid::OpticalDrive>()->eject();
