@@ -25,6 +25,7 @@
 
 #include <qlayout.h>
 #include <QProcess>
+#include <QScrollArea>
 
 #include <kdebug.h>
 #include <kiconloader.h>
@@ -35,6 +36,7 @@
 #include <kstandardguiitem.h>
 #include <kuser.h>
 #include <kauthorized.h>
+#include <ktoolinvocation.h>
 
 #include "kcmoduleloader.h"
 #include "kcmoduleproxy.h"
@@ -42,37 +44,26 @@
 /*
 Button usage:
 
-    User1 => Reset
-    User2 => Close
-    User3 => Admin (dead in KDE 4)
+    User1 => Close
+    User2 => Admin (dead in KDE 4)
 */
 
 class KCMultiWidget::KCMultiWidgetPrivate
 {
 	public:
 		KCMultiWidgetPrivate()
-			: hasRootKCM( false ), currentModule( 0 )
+			: hasRootKCM( false )
 		{}
 
 		bool hasRootKCM;
-		KCModuleProxy* currentModule;
 };
 
  
 KCMultiWidget::KCMultiWidget(QWidget *parent, Qt::WindowModality modality)
-	: KPageDialog( parent ),
-    d( new KCMultiWidgetPrivate )
+	: KPageDialog( parent )
+	, d( new KCMultiWidgetPrivate )
 {
-  InitKIconDialog(i18n("Configure"), modality);
-	init();
-}
-
-KCMultiWidget::KCMultiWidget( int dialogFace, QWidget * parent, Qt::WindowModality modality )
-	: KPageDialog( parent ),
-    dialogface( dialogFace ),
-    d( new KCMultiWidgetPrivate )
-{
-  InitKIconDialog(QString(), modality);
+	InitKIconDialog(i18n("Configure"), modality);
 	init();
 }
 
@@ -86,27 +77,30 @@ void KCMultiWidget::InitKIconDialog(const QString& caption,
              KDialog::Cancel |
              KDialog::Apply |
              KDialog::Ok |
-             KDialog::User1 |
-             KDialog::User2);
+             KDialog::Reset |
+             KDialog::User1);
   setDefaultButton(KDialog::Ok);
-  setButtonGuiItem(KDialog::User1, KStandardGuiItem::reset());
-  setButtonGuiItem(KDialog::User2, KStandardGuiItem::close());
+  setButtonGuiItem(KDialog::User1, KStandardGuiItem::close());
 
   setWindowModality(modality);
-
-//   iconPage.setParent(this);
 }
 
 inline void KCMultiWidget::init()
 {
+	// A bit hackish: KCMultiWidget inherits from KPageDialog, but it really is
+	// a widget...
+	setWindowFlags(Qt::Widget);
+
 	connect( this, SIGNAL( finished()), SLOT( dialogClosed()));
 	showButton( Ok, false );
 	showButton( Cancel, false );
-	showButton( User1, true );     // Reset button
-	showButton( User2, false );    // Close button.
+	showButton( Reset, true );     // Reset button
+	showButton( User1, false );    // Close button.
 
 	enableButton(Apply, false);
-	enableButton(User1, false);
+	enableButton(Reset, false);
+	enableButton(Default, false);
+	enableButton(Help, false);
 
 	connect( this, SIGNAL(currentPageChanged(KPageWidgetItem*, KPageWidgetItem*)), this, SLOT(slotAboutToShow(KPageWidgetItem*, KPageWidgetItem* )) );
 	setInitialSize(QSize(640,480));
@@ -114,15 +108,13 @@ inline void KCMultiWidget::init()
 	connect( this, SIGNAL(helpClicked()), this, SLOT(slotHelp()) );
 	connect( this, SIGNAL(defaultClicked()), this, SLOT(slotDefault()) );
 	connect( this, SIGNAL(applyClicked()), this, SLOT(slotApply()) );
-	connect( this, SIGNAL(user1Clicked()), this, SLOT(slotReset()) );
-	connect( this, SIGNAL(user2Clicked()), this, SLOT(slotClose()) );
+	connect( this, SIGNAL(resetClicked()), this, SLOT(slotReset()) );
+	connect( this, SIGNAL(user1Clicked()), this, SLOT(slotClose()) );
 }
 
 KCMultiWidget::~KCMultiWidget()
 {
-	OrphanMap::Iterator end2 = m_orphanModules.end();
-	for( OrphanMap::Iterator it = m_orphanModules.begin(); it != end2; ++it )
-		delete ( *it );
+	qDeleteAll( m_orphanModules );
 	delete d;
 }
 
@@ -143,26 +135,25 @@ void KCMultiWidget::slotReset()
 void KCMultiWidget::apply()
 {
 	QStringList updatedModules;
-	ModuleList::Iterator end = m_modules.end();
-	for( ModuleList::Iterator it = m_modules.begin(); it != end; ++it )
+	foreach( const CreatedModule &it, m_modules )
 	{
-		KCModuleProxy * m = ( *it ).kcm;
+		KCModuleProxy * m = it.kcm;
 		if( m && m->changed() )
 		{
 			m->save();
 			QStringList names = moduleParentComponents[ m ];
 
-            foreach ( const QString &name , names )
-            {
-                if ( updatedModules.indexOf(name) == -1 )
-                    updatedModules.append(name);
-            }
+			foreach ( const QString &name , names )
+			{
+				if ( updatedModules.indexOf(name) == -1 )
+					updatedModules.append(name);
+			}
 		}
 	}
-	for( QStringList::const_iterator it = updatedModules.begin(); it != updatedModules.end(); ++it )
+	foreach( const QString &it, updatedModules )
 	{
-		kDebug() << *it << " " << ( *it );
-		emit configCommitted( ( *it ).toLatin1() );
+		kDebug() << it << " " << ( it );
+		emit configCommitted( it.toLatin1() );
 	}
 	emit configCommitted();
 }
@@ -186,10 +177,9 @@ void KCMultiWidget::slotHelp()
 	KUrl url( KUrl("help:/"), docPath );
 
 	if (url.protocol() == "help" || url.protocol() == "man" || url.protocol() == "info") {
-		QProcess process(this);
-		process.start("khelpcenter", QStringList() << url.url());
+		QProcess::startDetached("khelpcenter", QStringList() << url.url());
 	} else {
-		new KRun(url, this);
+		KToolInvocation::invokeBrowser( url.url() );
 	}
 }
 
@@ -201,31 +191,17 @@ void KCMultiWidget::slotClose() {
 void KCMultiWidget::clientChanged(bool state)
 {
 	kDebug( 710 ) << state;
-	ModuleList::Iterator end = m_modules.end();
-	for( ModuleList::Iterator it = m_modules.begin(); it != end; ++it )
-		if( ( *it ).kcm->changed() ) {
+	foreach( const CreatedModule &it, m_modules )
+		if( it.kcm->changed() ) {
 			enableButton( Apply, true );
-                        enableButton( User1, true);
+                        enableButton( Reset, true);
 			return;
 		}
 	enableButton( Apply, false );
-        enableButton( User1, false);
+        enableButton( Reset, false);
 }
 
-void KCMultiWidget::addModule(const QString& path, bool withfallback)
-{
-	QString complete = path;
-
-	if( !path.endsWith( ".desktop" ))
-		complete += ".desktop";
-
-	KService::Ptr service = KService::serviceByStorageId( complete );
-
-	addModule( KCModuleInfo( service ), QStringList(), withfallback);
-}
-
-void KCMultiWidget::addModule(const KCModuleInfo& moduleinfo,
-		QStringList parentmodulenames, bool withfallback)
+void KCMultiWidget::addModule(const KCModuleInfo& moduleinfo)
 {
 	if( !moduleinfo.service() ) {
 		kWarning() << "ModuleInfo has no associated KService" ;
@@ -241,6 +217,7 @@ void KCMultiWidget::addModule(const KCModuleInfo& moduleinfo,
 		return;
 	}
 	KCModuleProxy * module;
+	QScrollArea * moduleScrollArea;
 	if( m_orphanModules.contains( moduleinfo.service() ) )
 	{
 		// the KCModule already exists - it was removed from the dialog in
@@ -254,11 +231,17 @@ void KCMultiWidget::addModule(const KCModuleInfo& moduleinfo,
 			clientChanged( true );
 		}
 
+		moduleScrollArea = static_cast<QScrollArea*>( module->parentWidget() );
 	}
 	else
 	{
-		module = new KCModuleProxy( moduleinfo, this );
-
+		moduleScrollArea = new QScrollArea( this );
+		module = new KCModuleProxy( moduleinfo, moduleScrollArea );
+		moduleScrollArea->setWidget( module );
+		moduleScrollArea->setWidgetResizable( true );
+		moduleScrollArea->setFrameStyle( QFrame::NoFrame );
+                moduleScrollArea->viewport()->setAutoFillBackground(false);
+                module->setAutoFillBackground(false);
 		QStringList parentComponents = moduleinfo.service()->property(
 				"X-KDE-System-Settings-Parent-Category" ).toStringList();
 		moduleParentComponents.insert( module,
@@ -289,19 +272,20 @@ void KCMultiWidget::addModule(const KCModuleInfo& moduleinfo,
 	if( m_modules.count() == 1 ) {
 		slotAboutToShow( module );
 	}
-	KPageWidgetItem* page = addPage(module, moduleinfo.moduleName());
+	KPageWidgetItem* page = addPage(moduleScrollArea, moduleinfo.moduleName());
 	page->setIcon( KIcon(moduleinfo.icon()) );
 	page->setHeader(moduleinfo.comment());
 }
 
 KCModuleProxy* KCMultiWidget::currentModule() {
-	QWidget* pageWidget = currentPage()->widget();
-	KCModuleProxy* module = qobject_cast<KCModuleProxy*>(pageWidget);
-	if (!module) {
-		return NULL;
-	} else {
-		return module;
-	}
+	KPageWidgetItem *pageWidget = currentPage();
+	if ( pageWidget == 0 )
+		return 0;
+
+	QScrollArea *scrollArea = qobject_cast<QScrollArea*>( pageWidget->widget() );
+	KCModuleProxy *module = qobject_cast<KCModuleProxy*>( scrollArea->widget() );
+
+	return module;
 }
 
 void KCMultiWidget::applyOrRevert(KCModuleProxy * module){
@@ -309,11 +293,11 @@ void KCMultiWidget::applyOrRevert(KCModuleProxy * module){
 		return;
 	
 	int res = KMessageBox::warningYesNo(this,
-				i18n("There are unsaved changes in the active module.\n"
-             "Do you want to apply the changes or discard them?"),
-                                      i18n("Unsaved Changes"),
-                                      KStandardGuiItem::apply(),
-                                      KStandardGuiItem::discard());
+	                                    i18n("There are unsaved changes in the active module.\n"
+	                                         "Do you want to apply the changes or discard them?"),
+	                                    i18n("Unsaved Changes"),
+	                                    KStandardGuiItem::apply(),
+	                                    KStandardGuiItem::discard());
 	if (res == KMessageBox::Yes) {
 		slotApply();
 	} else {
@@ -322,7 +306,7 @@ void KCMultiWidget::applyOrRevert(KCModuleProxy * module){
 	}
 }
 
-void KCMultiWidget::slotAboutToShow(KPageWidgetItem* current, KPageWidgetItem* before)
+void KCMultiWidget::slotAboutToShow(KPageWidgetItem* current, KPageWidgetItem* /*before*/)
 {
 	QWidget* sendingWidget = current->widget();
 	slotAboutToShow(sendingWidget);
@@ -332,60 +316,64 @@ void KCMultiWidget::slotAboutToShow(QWidget *page)
 {
 	QList<KCModuleProxy*> objects = page->findChildren<KCModuleProxy*>();
 
-    // add fall back
-    objects.append( qobject_cast<KCModuleProxy*>(page) );
+	// add fall back
+	objects.append( qobject_cast<KCModuleProxy*>(page) );
 
 	KCModuleProxy *module = objects.first();
 	if( ! module ) {
 		return;
 	}
 
-	if( d && d->currentModule ) {
-		applyOrRevert( d->currentModule );
+	if( currentModule() != 0 ) {
+		applyOrRevert( currentModule() );
 	}
-	d->currentModule = module;
-	emit ( aboutToShow( d->currentModule ) );
+	emit ( aboutToShow( module ) );
 
-	ModuleList::Iterator end = m_modules.end();
 	int buttons = 0;
-	for( ModuleList::Iterator it = m_modules.begin(); it != end; ++it ) {
-		if( ( *it ).kcm==d->currentModule) {
-			showButton(User3, ( *it ).adminmode);
-			buttons = ( *it ).buttons;
+        bool found = false;
+	foreach( const CreatedModule &it, m_modules ) {
+		if( it.kcm==module) {
+			showButton(User2, it.adminmode);
+			buttons = it.buttons;
+                        found = true;
 		}
 	}
+        if (!found) {
+            buttons = module->buttons();
+        }
+        //Q_ASSERT(found);
 
         showButton(Apply, buttons & KCModule::Apply);
-        showButton(User1, buttons & KCModule::Apply);   // Reset button.
+        showButton(Reset, buttons & KCModule::Apply);
 
         // Close button. No Apply button implies a Close button.
-        showButton(User2, (buttons & KCModule::Apply)==0);
+        showButton(User1, (buttons & KCModule::Apply)==0);
 
 	enableButton( KDialog::Help, buttons & KCModule::Help );
 	enableButton( KDialog::Default, buttons & KCModule::Default );
 
 	disconnect( this, SIGNAL(user3Clicked()), 0, 0 );
 
-// 	if (d->currentModule->moduleInfo().needsRootPrivileges() &&
-// 			!d->currentModule->rootMode() )
+// 	if (module->moduleInfo().needsRootPrivileges() &&
+// 			!module->rootMode() )
 // 	{ /* Enable the Admin Mode button */
-// 		enableButton( User3, true );
-// 		connect( this, SIGNAL(user3Clicked()), d->currentModule, SLOT( runAsRoot() ));
+// 		enableButton( User2, true );
+// 		connect( this, SIGNAL(user3Clicked()), module, SLOT( runAsRoot() ));
 // 		connect( this, SIGNAL(user3Clicked()), SLOT( disableRModeButton() ));
 // 	} else {
-// 		enableButton( User3, false );
+// 		enableButton( User2, false );
 // 	}
 }
 
 void KCMultiWidget::rootExit()
 {
-	enableButton( User3, true);
+	enableButton( User2, true);
 }
 
 void KCMultiWidget::disableRModeButton()
 {
-	enableButton( User3, false );
-	connect ( d->currentModule, SIGNAL( childClosed() ), SLOT( rootExit() ) );
+	enableButton( User2, false );
+	connect ( currentModule(), SIGNAL( childClosed() ), SLOT( rootExit() ) );
 }
 
 void KCMultiWidget::slotCancel() {
@@ -394,10 +382,7 @@ void KCMultiWidget::slotCancel() {
 
 void KCMultiWidget::dialogClosed()
 {
-	if(d)
-	{
-		applyOrRevert(d->currentModule);
-	}
+	applyOrRevert( currentModule() );
 }
 
 #include "kcmultiwidget.moc"
